@@ -9,6 +9,7 @@ import ij.util.Tools;
 import ij.plugin.frame.Recorder;
 import ij.macro.Interpreter;
 import ij.measure.Calibration;
+import ij.measure.ResultsTable;
 import java.awt.event.*;
 import java.text.*;
 import java.util.*;	
@@ -16,11 +17,12 @@ import java.awt.*;
 import java.applet.Applet;
 import java.io.*;
 import java.lang.reflect.*;
+import java.net.*;
 
 
 /** This class consists of static utility methods. */
 public class IJ {
-	public static final String URL = "http://rsb.info.nih.gov/ij";
+	public static final String URL = "http://imagej.nih.gov/ij";
 	public static final int ALL_KEYS = -1;
 	
 	public static boolean debugMode;
@@ -45,18 +47,17 @@ public class IJ {
 	private static boolean memMessageDisplayed;
 	private static long maxMemory;
 	private static boolean escapePressed;
-	private static boolean redirectErrorMessages;
+	private static boolean redirectErrorMessages, redirectErrorMessages2;
 	private static boolean suppressPluginNotFoundError;
-	private static Dimension screenSize;
 	private static Hashtable commandTable;
-
+	private static Vector eventListeners = new Vector();
 			
 	static {
 		osname = System.getProperty("os.name");
 		isWin = osname.startsWith("Windows");
 		isMac = !isWin && osname.startsWith("Mac");
 		isLinux = osname.startsWith("Linux");
-		isVista = isWin && osname.indexOf("Vista")!=-1;
+		isVista = isWin && (osname.indexOf("Vista")!=-1||osname.indexOf(" 7")!=-1);
 		String version = System.getProperty("java.version").substring(0,3);
 		if (version.compareTo("2.9")<=0) {  // JVM on Sharp Zaurus PDA claims to be "3.1"!
 			isJava2 = version.compareTo("1.1")>0;
@@ -73,14 +74,19 @@ public class IJ {
 		progressBar = ij.getProgressBar();
 	}
 
+	static void cleanup() {
+		ij=null; applet=null; progressBar=null; textPanel=null;
+	}
+
 	/**Returns a reference to the "ImageJ" frame.*/
 	public static ImageJ getInstance() {
 		return ij;
 	}
 	
 	/** Runs the macro contained in the string <code>macro</code>.
-		Returns any string value returned by the macro, or null. 
-		The equivalent macro function is eval(). */
+		Returns any string value returned by the macro, null if the macro
+		does not return a value, or "[aborted]" if the macro was aborted
+		due to an error. The equivalent macro function is eval(). */
 	public static String runMacro(String macro) {
 		return runMacro(macro, "");
 	}
@@ -88,7 +94,9 @@ public class IJ {
 	/** Runs the macro contained in the string <code>macro</code>.
 		The optional string argument can be retrieved in the
 		called macro using the getArgument() macro function. 
-		Returns any string value returned by the macro, or null. */
+		Returns any string value returned by the macro, null if the macro
+		does not return a value, or "[aborted]" if the macro was aborted
+		due to an error.  */
 	public static String runMacro(String macro, String arg) {
 		Macro_Runner mr = new Macro_Runner();
 		return mr.runMacro(macro, arg);
@@ -189,7 +197,8 @@ public class IJ {
 		}
 		catch (InstantiationException e) {error("Unable to load plugin (ins)");}
 		catch (IllegalAccessException e) {error("Unable to load plugin, possibly \nbecause it is not public.");}
- 		redirectErrorMessages = false;
+		if (redirectErrorMessages && !"HandleExtraFileTypes".equals(className))
+ 			redirectErrorMessages = false;
 		suppressPluginNotFoundError = false;
 		return thePlugIn;
 	} 
@@ -264,6 +273,13 @@ public class IJ {
 			commandTable.put("RGB Merge...", "Merge Channels...");
 			commandTable.put("Channels...", "Channels Tool...");
 			commandTable.put("New... ", "Table...");
+			commandTable.put("Arbitrarily...", "Rotate... ");
+			commandTable.put("Measurements...", "Results... ");
+			commandTable.put("List Commands...", "Find Commands...");
+			commandTable.put("Capture Screen ", "Capture Screen");
+			commandTable.put("Add to Manager ", "Add to Manager");
+			commandTable.put("In", "In [+]");
+			commandTable.put("Out", "Out [-]");
 		}
 		String command2 = (String)commandTable.get(command);
 		if (command2!=null)
@@ -319,8 +335,12 @@ public class IJ {
 			ic.setShowCursorStatus(s.length()==0?true:false);
 	}
 
-	/** Displays a line of text in the "Results" window. Writes to
-		System.out.println if the "ImageJ" frame is not present. */
+	/**
+	* @deprecated
+	* replaced by IJ.log(), ResultsTable.setResult() and TextWindow.append().
+	* There are examples at
+	*   http://imagej.nih.gov/ij/plugins/sine-cosine.html
+	*/
 	public static void write(String s) {
 		if (textPanel==null && ij!=null)
 			showResults();
@@ -331,17 +351,15 @@ public class IJ {
 	}
 
 	private static void showResults() {
-		TextWindow resultsWindow = new TextWindow("Results", "", 300, 200);
+		TextWindow resultsWindow = new TextWindow("Results", "", 400, 250);
 		textPanel = resultsWindow.getTextPanel();
 		textPanel.setResultsTable(Analyzer.getResultsTable());
-		if (ij!=null)
-			textPanel.addKeyListener(ij);
 	}
 
 	public static synchronized void log(String s) {
 		if (s==null) return;
 		if (logPanel==null && ij!=null) {
-			TextWindow logWindow = new TextWindow("Log", "", 350, 250);
+			TextWindow logWindow = new TextWindow("Log", "", 400, 250);
 			logPanel = logWindow.getTextPanel();
 			logPanel.setFont(new Font("SansSerif", Font.PLAIN, 16));
 		}
@@ -379,13 +397,25 @@ public class IJ {
 			}
 			String s2 = s.substring(cindex+1, s.length());
 			logPanel.setLine(line, s2);
-		} else if (s.equals("\\Clear"))
+		} else if (s.equals("\\Clear")) {
 			logPanel.clear();
-		else
+		} else if (s.equals("\\Close")) {
+			Frame f = WindowManager.getFrame("Log");
+			if (f!=null && (f instanceof TextWindow))
+				((TextWindow)f).close();
+		} else
 			logPanel.append(s);
 	}
 	
-	/** Clears the "Results" window and sets the column headings to
+	/** Returns the contents of the Log window or null if the Log window is not open. */
+	public static synchronized String getLog() { 
+		if (logPanel==null || ij==null)
+			return null;
+		else
+			return logPanel.getText(); 
+	} 
+
+/** Clears the "Results" window and sets the column headings to
 		those in the tab-delimited 'headings' String. Writes to
 		System.out.println if the "ImageJ" frame is not present.*/
 	public static void setColumnHeadings(String headings) {
@@ -402,10 +432,38 @@ public class IJ {
 		return textPanel!=null;
 	}
 	
+	/** Renames a results window. */
+	public static void renameResults(String title) {
+		Frame frame = WindowManager.getFrontWindow();
+		if (frame!=null && (frame instanceof TextWindow)) {
+			TextWindow tw = (TextWindow)frame;
+			if (tw.getTextPanel().getResultsTable()==null) {
+				IJ.error("Rename", "\""+tw.getTitle()+"\" is not a results table");
+				return;
+			}
+			tw.rename(title);
+		} else if (isResultsWindow()) {
+			TextPanel tp = getTextPanel();
+			TextWindow tw = (TextWindow)tp.getParent();
+			tw.rename(title);
+		}
+	}
+
+	/** Deletes 'row1' through 'row2' of the "Results" window. Arguments
+	     must be in the range 0-Analyzer.getCounter()-1. */
+	public static void deleteRows(int row1, int row2) {
+		int n = row2 - row1 + 1;
+		ResultsTable rt = Analyzer.getResultsTable();
+		for (int i=row1; i<row1+n; i++)
+			rt.deleteRow(row1);
+		rt.show("Results");
+	}
+
 	/** Returns a reference to the "Results" window TextPanel.
-		Opens the "Results" window if it is currently not open. */
+		Opens the "Results" window if it is currently not open.
+		Returns null if the "ImageJ" window is not open. */
 	public static TextPanel getTextPanel() {
-		if (textPanel==null)
+		if (textPanel==null && ij!=null)
 			showResults();
 		return textPanel;
 	}
@@ -454,7 +512,11 @@ public class IJ {
     The bar is updated only if more than 90 ms have passed since the last call.
     Does nothing if the ImageJ window is not present. */
     public static void showProgress(int currentIndex, int finalIndex) {
-		if (progressBar!=null) progressBar.show(currentIndex, finalIndex);
+		if (progressBar!=null) {
+			progressBar.show(currentIndex, finalIndex);
+			if (currentIndex==finalIndex)
+				progressBar.setBatchMode(false);
+		}
 	}
 
 	/** Displays a message in a dialog box titled "Message".
@@ -463,14 +525,12 @@ public class IJ {
 		showMessage("Message", msg);
 	}
 
-	/**	Displays a message in a dialog box with the specified title.
-		Writes the Java console if ImageJ is not present. */
+	/** Displays a message in a dialog box with the specified title.
+		Displays HTML formatted text if 'msg' starts with "<html>".
+		There are examples at
+		"http://imagej.nih.gov/ij/macros/HtmlDialogDemo.txt".
+		Writes to the Java console if ImageJ is not present. */
 	public static void showMessage(String title, String msg) {
-		if (redirectErrorMessages) {
-			IJ.log(title + ": " + msg);
-			redirectErrorMessages = false;
-			return;
-		}
 		if (ij!=null) {
 			if (msg!=null && msg.startsWith("<html>"))
 				new HTMLDialog(title, msg);
@@ -484,19 +544,27 @@ public class IJ {
 		macro is running, it is aborted. Writes to the Java console
 		if the ImageJ window is not present.*/
 	public static void error(String msg) {
-		showMessage("ImageJ", msg);
+		error(null, msg);
 		if (Thread.currentThread().getName().endsWith("JavaScript"))
 			throw new RuntimeException(Macro.MACRO_CANCELED);
 		else
 			Macro.abort();
 	}
 	
-	/**	Displays a message in a dialog box with the specified title.
+	/**Displays a message in a dialog box with the specified title.
 		If a macro is running, it is aborted. Writes to the Java  
 		console if ImageJ is not present. */
-	public static synchronized void error(String title, String msg) {
-		showMessage(title, msg);
-		Macro.abort();
+	public static void error(String title, String msg) {
+		String title2 = title!=null?title:"ImageJ";
+		boolean abortMacro = title!=null;
+		if (redirectErrorMessages || redirectErrorMessages2) {
+			IJ.log(title2 + ": " + msg);
+			if (abortMacro && (title.equals("Opener")||title.equals("Open URL")||title.equals("DicomDecoder")))
+				abortMacro = false;
+		} else
+			showMessage(title2, msg);
+		redirectErrorMessages = false;
+		if (abortMacro) Macro.abort();
 	}
 
 	/** Displays a message in a dialog box with the specified title.
@@ -589,20 +657,28 @@ public class IJ {
 	
 	public static void showTime(ImagePlus imp, long start, String str, int nslices) {
 		if (Interpreter.isBatchMode()) return;
-	    long elapsedTime = System.currentTimeMillis() - start;
-		double seconds = elapsedTime / 1000.0;
-		long pixels = imp.getWidth() * imp.getHeight();
-		int rate = (int)((double)pixels*nslices/seconds);
+	    double seconds = (System.currentTimeMillis()-start)/1000.0;
+		double pixels = (double)imp.getWidth() * imp.getHeight();
+		double rate = pixels*nslices/seconds;
 		String str2;
-		if (rate>1000000000)
+		if (rate>1000000000.0)
 			str2 = "";
-		else if (rate<1000000)
-			str2 = ", "+rate+" pixels/second";
+		else if (rate<1000000.0)
+			str2 = ", "+d2s(rate,0)+" pixels/second";
 		else
 			str2 = ", "+d2s(rate/1000000.0,1)+" million pixels/second";
 		showStatus(str+seconds+" seconds"+str2);
 	}
-
+	
+	/** Experimental */
+	public static  String time(ImagePlus imp, long startNanoTime) {
+		double planes = imp.getStackSize();
+		double seconds = (System.nanoTime()-startNanoTime)/1000000000.0;
+		double mpixels = imp.getWidth()*imp.getHeight()*planes/1000000.0;
+		String time = seconds<1.0?d2s(seconds*1000.0,0)+" ms":d2s(seconds,1)+" seconds";
+		return time+", "+d2s(mpixels/seconds,1)+" million pixels/second";
+	}
+		
 	/** Converts a number to a formatted string using
 		2 digits to the right of the decimal point. */
 	public static String d2s(double n) {
@@ -615,10 +691,11 @@ public class IJ {
 
 	/** Converts a number to a rounded formatted string.
 		The 'decimalPlaces' argument specifies the number of
-		digits to the right of the decimal point (0-9). */
+		digits to the right of the decimal point (0-9). Uses
+		scientific notation if 'decimalPlaces is negative. */
 	public static String d2s(double n, int decimalPlaces) {
-		if (Double.isNaN(n))
-			return "NaN";
+		if (Double.isNaN(n)||Double.isInfinite(n))
+			return ""+n;
 		if (n==Float.MAX_VALUE) // divide by 0 in FloatProcessor
 			return "3.4e38";
 		double np = n;
@@ -637,12 +714,9 @@ public class IJ {
 			df[8] = new DecimalFormat("0.00000000", dfs);
 			df[9] = new DecimalFormat("0.000000000", dfs);
 		}
-		if ((np<0.001 && np!=0.0 && np<1.0/Math.pow(10,decimalPlaces)) || np>999999999999d || decimalPlaces<0) {
-			if (decimalPlaces<0) {
-				decimalPlaces = -decimalPlaces;
-				if (decimalPlaces>9) decimalPlaces=9;
-			} else
-				decimalPlaces = 3;
+		if (decimalPlaces<0) {
+			decimalPlaces = -decimalPlaces;
+			if (decimalPlaces>9) decimalPlaces=9;
 			if (sf==null) {
 				sf = new DecimalFormat[10];
 				sf[1] = new DecimalFormat("0.0E0",dfs);
@@ -655,16 +729,21 @@ public class IJ {
 				sf[8] = new DecimalFormat("0.00000000E0",dfs);
 				sf[9] = new DecimalFormat("0.000000000E0",dfs);
 			}
-			if (Double.isInfinite(n))
-				return ""+n;
-			else
-				return sf[decimalPlaces].format(n); // use scientific notation
+			return sf[decimalPlaces].format(n); // use scientific notation
 		}
 		if (decimalPlaces<0) decimalPlaces = 0;
 		if (decimalPlaces>9) decimalPlaces = 9;
 		return df[decimalPlaces].format(n);
 	}
 
+	/** Pad 'n' with leading zeros to the specified number of digits. */
+	public static String pad(int n, int digits) {
+		String str = ""+n;
+		while (str.length()<digits)
+			str = "0"+str;
+		return str;
+	}
+
 	/** Adds the specified class to a Vector to keep it from being garbage
 	collected, which would cause the classes static fields to be reset. 
 	Probably not needed with Java 1.2 or later. */
@@ -806,7 +885,7 @@ public class IJ {
 	public static boolean versionLessThan(String version) {
 		boolean lessThan = ImageJ.VERSION.compareTo(version)<0;
 		if (lessThan)
-			error("This plugin or macro requires ImageJ "+version+" or later.");
+			error("This plugin or macro requires ImageJ "+version+" or later. Use\nHelp>Update ImageJ to upgrade to the latest version.");
 		return lessThan;
 	}
 	
@@ -831,19 +910,19 @@ public class IJ {
 			}
 			if (hideProcessStackDialog)
 				return flags;
+			String note = ((flags&PlugInFilter.NO_CHANGES)==0)?" There is\nno Undo if you select \"Yes\".":"";
  			YesNoCancelDialog d = new YesNoCancelDialog(getInstance(),
-				"Process Stack?", "Process all "+stackSize+" images?  There is\n"
-				+"no Undo if you select \"Yes\".");
+				"Process Stack?", "Process all "+stackSize+" images?"+note);
 			if (d.cancelPressed())
 				return PlugInFilter.DONE;
 			else if (d.yesPressed()) {
-		    	if (imp.getStack().isVirtual()) {
+		    	if (imp.getStack().isVirtual() && ((flags&PlugInFilter.NO_CHANGES)==0)) {
 		    		int size = (stackSize*imp.getWidth()*imp.getHeight()*imp.getBytesPerPixel()+524288)/1048576;
 		    		String msg =
-						"Custom code is required to process this virtual stack\n"+
-						"(e.g., \"Process Virtual Stack\" macro) or it must be\n"+
-						"converted to a normal stack using Image>Duplicate,\n"+
-						"which will require "+size+"MB of additional memory.";
+						"Use the Process>Batch>Virtual Stack command\n"+
+						"to process a virtual stack ike this one or convert\n"+
+						"it to a normal stack using Image>Duplicate, which\n"+
+						"will require "+size+"MB of additional memory.";
 		    		error(msg);
 					return PlugInFilter.DONE;
 		    	}
@@ -864,11 +943,14 @@ public class IJ {
 			getImage().killRoi();
 		else {
 			ImagePlus img = getImage();
-			img.setRoi(x, y, width, height);
+			if (Interpreter.isBatchMode())
+				img.setRoi(new Roi(x,y,width,height), false);
+			else
+				img.setRoi(x, y, width, height);
 		}
 	}
 	
-	/** Creates an elliptical selection. Removes any existing 
+	/** Creates an oval selection. Removes any existing 
 		selection if width or height are less than 1. */
 	public static void makeOval(int x, int y, int width, int height) {
 		if (width<=0 || height<0)
@@ -892,6 +974,10 @@ public class IJ {
 			Polygon p = roi.getPolygon();
 			p.addPoint(x, y);
 			img.setRoi(new PointRoi(p.xpoints, p.ypoints, p.npoints));
+			IJ.setKeyUp(KeyEvent.VK_SHIFT);
+		} else if (altKeyDown() && roi!=null && roi.getType()==Roi.POINT) {
+			((PolygonRoi)roi).deleteHandle(x, y);
+			IJ.setKeyUp(KeyEvent.VK_ALT);
 		} else
 			img.setRoi(new PointRoi(x, y));
 	}
@@ -901,15 +987,23 @@ public class IJ {
 		getImage().setRoi(new Line(x1, y1, x2, y2));
 	}
 
-	/** Sets the minimum and maximum displayed pixel values. */
+	/** Sets the display range (minimum and maximum displayed pixel values) of the current image. */
 	public static void setMinAndMax(double min, double max) {
-		setMinAndMax(min, max, 7);
+		setMinAndMax(getImage(), min, max, 7);
+	}
+
+	/** Sets the display range (minimum and maximum displayed pixel values) of the specified image. */
+	public static void setMinAndMax(ImagePlus img, double min, double max) {
+		setMinAndMax(img, min, max, 7);
 	}
 
 	/** Sets the minimum and maximum displayed pixel values on the specified RGB
 	channels, where 4=red, 2=green and 1=blue. */
 	public static void setMinAndMax(double min, double max, int channels) {
-		ImagePlus img = getImage();
+		setMinAndMax(getImage(), min, max, channels);
+	}
+
+	private static void setMinAndMax(ImagePlus img, double min, double max, int channels) {
 		Calibration cal = img.getCalibration();
 		min = cal.getRawValue(min); 
 		max = cal.getRawValue(max);
@@ -920,10 +1014,15 @@ public class IJ {
 		img.updateAndDraw();
 	}
 
-	/** Resets the minimum and maximum displayed pixel values
-		to be the same as the min and max pixel values. */
+	/** Resets the minimum and maximum displayed pixel values of the
+		current image to be the same as the min and max pixel values. */
 	public static void resetMinAndMax() {
-		ImagePlus img = getImage();
+		resetMinAndMax(getImage());
+	}
+
+	/** Resets the minimum and maximum displayed pixel values of the
+		specified image to be the same as the min and max pixel values. */
+	public static void resetMinAndMax(ImagePlus img) {
 		img.resetDisplayRange();
 		img.updateAndDraw();
 	}
@@ -939,6 +1038,17 @@ public class IJ {
 	/** Sets the lower and upper threshold levels and displays the image using
 		the specified <code>displayMode</code> ("Red", "Black & White", "Over/Under" or "No Update"). */
 	public static void setThreshold(double lowerThreshold, double upperThreshold, String displayMode) {
+		setThreshold(getImage(), lowerThreshold, upperThreshold, displayMode);
+	}
+
+	/** Sets the lower and upper threshold levels of the specified image. */
+	public static void setThreshold(ImagePlus img, double lowerThreshold, double upperThreshold) {
+		setThreshold(img, lowerThreshold, upperThreshold, "Red");
+	}
+
+	/** Sets the lower and upper threshold levels of the specified image and updates the display using
+		the specified <code>displayMode</code> ("Red", "Black & White", "Over/Under" or "No Update"). */
+	public static void setThreshold(ImagePlus img, double lowerThreshold, double upperThreshold, String displayMode) {
 		int mode = ImageProcessor.RED_LUT;
 		if (displayMode!=null) {
 			displayMode = displayMode.toLowerCase(Locale.US);
@@ -949,7 +1059,6 @@ public class IJ {
 			else if (displayMode.indexOf("no")!=-1)
 				mode = ImageProcessor.NO_LUT_UPDATE;
 		}
-		ImagePlus img = getImage();
 		Calibration cal = img.getCalibration();
 		lowerThreshold = cal.getRawValue(lowerThreshold); 
 		upperThreshold = cal.getRawValue(upperThreshold); 
@@ -960,9 +1069,65 @@ public class IJ {
 		}
 	}
 
-	/** Disables thresholding. */
+	public static void setAutoThreshold(ImagePlus imp, String method) {
+		ImageProcessor ip = imp.getProcessor();
+		if (ip instanceof ColorProcessor)
+			throw new IllegalArgumentException("Non-RGB image required");
+		ip.setRoi(imp.getRoi());
+		if (method!=null) {
+			try {
+				if (method.indexOf("stack")!=-1)
+					setStackThreshold(imp, ip, method);
+				else
+					ip.setAutoThreshold(method);
+			} catch (Exception e) {
+				log(e.getMessage());
+			}
+		} else
+			ip.setAutoThreshold(ImageProcessor.ISODATA2, ImageProcessor.RED_LUT);
+		imp.updateAndDraw();
+	}
+	
+	private static void setStackThreshold(ImagePlus imp, ImageProcessor ip, String method) {
+		boolean darkBackground = method.indexOf("dark")!=-1;
+		ImageStatistics stats = new StackStatistics(imp);
+		AutoThresholder thresholder = new AutoThresholder();
+		double min=0.0, max=255.0;
+		if (imp.getBitDepth()!=8) {
+			min = stats.min;
+			max = stats.max;
+		}
+		int threshold = thresholder.getThreshold(method, stats.histogram);
+		double lower, upper;
+		if (darkBackground) {
+			if (ip.isInvertedLut())
+				{lower=0.0; upper=threshold;}
+			else
+				{lower=threshold+1; upper=255.0;}
+		} else {
+			if (ip.isInvertedLut())
+				{lower=threshold+1; upper=255.0;}
+			else
+				{lower=0.0; upper=threshold;}
+		}
+		if (lower>255) lower = 255;
+		if (max>min) {
+			lower = min + (lower/255.0)*(max-min);
+			upper = min + (upper/255.0)*(max-min);
+		} else
+			lower = upper = min;
+		ip.setMinAndMax(min, max);
+		ip.setThreshold(lower, upper, ImageProcessor.RED_LUT);
+		imp.updateAndDraw();
+	}
+
+	/** Disables thresholding on the current image. */
 	public static void resetThreshold() {
-		ImagePlus img = getImage();
+		resetThreshold(getImage());
+	}
+	
+	/** Disables thresholding on the specified image. */
+	public static void resetThreshold(ImagePlus img) {
 		ImageProcessor ip = img.getProcessor();
 		ip.resetThreshold();
 		ip.setLutAnimation(true);
@@ -1005,6 +1170,8 @@ public class IJ {
 
 	/** Activates the window with the specified title. */
 	public static void selectWindow(String title) {
+		if (title.equals("ImageJ")&&ij!=null)
+			{ij.toFront(); return;}
 		long start = System.currentTimeMillis();
 		while (System.currentTimeMillis()-start<3000) { // 3 sec timeout
 			Frame frame = WindowManager.getFrame(title);
@@ -1077,10 +1244,15 @@ public class IJ {
 		return Toolbar.getInstance().setTool(name);
 	}
 
+	/** Returns the name of the current tool. */
+	public static String getToolName() {
+		return Toolbar.getToolName();
+	}
+
 	/** Equivalent to clicking on the current image at (x,y) with the
 		wand tool. Returns the number of points in the resulting ROI. */
 	public static int doWand(int x, int y) {
-		return doWand(x, y, 0, null);
+		return doWand(getImage(), x, y, 0, null);
 	}
 
 	/** Traces the boundary of the area with pixel values within
@@ -1091,7 +1263,11 @@ public class IJ {
 	* it is ignored if 'tolerance' > 0.
 	*/
 	public static int doWand(int x, int y, double tolerance, String mode) {
-		ImagePlus img = getImage();
+		return doWand(getImage(), x, y, tolerance, mode);
+	}
+
+	/** This version of doWand adds an ImagePlus argument. */
+	public static int doWand(ImagePlus img, int x, int y, double tolerance, String mode) {
 		ImageProcessor ip = img.getProcessor();
 		if ((img.getType()==ImagePlus.GRAY32) && Double.isNaN(ip.getPixelValue(x,y)))
 			return 0;
@@ -1161,7 +1337,10 @@ public class IJ {
 		ImagePlus img = WindowManager.getCurrentImage();
 		if (img==null) {
 			IJ.noImage();
-			abort();
+			if (ij==null)
+				System.exit(0);
+			else
+				abort();
 		}
 		return img;
 	}
@@ -1176,10 +1355,10 @@ public class IJ {
 		return ImageJ.VERSION;
 	}
 	
-	/** Returns the path to the home ("user.home"), startup (ImageJ directory), plugins, macros, 
-		temp, current or image directory if <code>title</code> is "home", "startup", 
-		"plugins", "macros", "temp", "current" or "image", otherwise, displays a dialog 
-		and returns the path to the directory selected by the user. 
+	/** Returns the path to the home ("user.home"), startup, ImageJ, plugins, macros, 
+		luts, temp, current or image directory if <code>title</code> is "home", "startup", 
+		"imagej", "plugins", "macros", "luts", "temp", "current" or "image", otherwise, 
+		displays a dialog and returns the path to the directory selected by the user. 
 		Returns null if the specified directory is not found or the user
 		cancels the dialog box. Also aborts the macro if the user cancels
 		the dialog box.*/
@@ -1188,16 +1367,23 @@ public class IJ {
 			return Menus.getPlugInsPath();
 		else if (title.equals("macros"))
 			return Menus.getMacrosPath();
-		else if (title.equals("luts"))
-			return Prefs.getHomeDir()+File.separator+"luts"+File.separator;
-		else if (title.equals("home"))
+		else if (title.equals("luts")) {
+			String ijdir = getIJDir();
+			if (ijdir!=null)
+				return ijdir + "luts" + File.separator;
+			else
+				return null;
+		} else if (title.equals("home"))
 			return System.getProperty("user.home") + File.separator;
-		else if (title.equals("startup")||title.equals("imagej"))
+		else if (title.equals("startup"))
 			return Prefs.getHomeDir() + File.separator;
+		else if (title.equals("imagej"))
+			return getIJDir();
 		else if (title.equals("current"))
 			return OpenDialog.getDefaultDirectory();
 		else if (title.equals("temp")) {
 			String dir = System.getProperty("java.io.tmpdir");
+			if (isMacintosh()) dir = "/tmp/";
 			if (dir!=null && !dir.endsWith(File.separator)) dir += File.separator;
 			return dir;
 		} else if (title.equals("image")) {
@@ -1215,6 +1401,14 @@ public class IJ {
 		}
 	}
 	
+	private static String getIJDir() {
+		String path = Menus.getPlugInsPath();
+		if (path==null) return null;
+		String ijdir = (new File(path)).getParent();
+		if (ijdir!=null) ijdir += File.separator;
+		return ijdir;
+	}
+	
 	/** Displays a file open dialog box and then opens the tiff, dicom, 
 		fits, pgm, jpeg, bmp, gif, lut, roi, or text file selected by 
 		the user. Displays an error message if the selected file is not
@@ -1239,20 +1433,61 @@ public class IJ {
 		macroRunning = false;
 	}
 		
-	/** Open the specified file as a tiff, bmp, dicom, fits, pgm, gif 
+	/** Opens and displays the nth image in the specified tiff stack. */
+	public static void open(String path, int n) {
+		if (ij==null && Menus.getCommands()==null) init();
+		ImagePlus imp = openImage(path, n);
+		if (imp!=null) imp.show();
+	}
+
+	/** Opens the specified file as a tiff, bmp, dicom, fits, pgm, gif 
 		or jpeg image and returns an ImagePlus object if successful.
 		Calls HandleExtraFileTypes plugin if the file type is not recognised.
 		Displays a file open dialog if 'path' is null or an empty string.
-		Note that 'path' can also be a URL. */
+		Note that 'path' can also be a URL. Some reader plugins, including
+		the Bio-Formats plugin, display the image and return null. */
 	public static ImagePlus openImage(String path) {
 		return (new Opener()).openImage(path);
 	}
 
+	/** Opens the nth image of the specified tiff stack. */
+	public static ImagePlus openImage(String path, int n) {
+		return (new Opener()).openImage(path, n);
+	}
+
 	/** Opens an image using a file open dialog and returns it as an ImagePlus object. */
 	public static ImagePlus openImage() {
 		return openImage(null);
 	}
 
+	/** Opens a URL and returns the contents as a string.
+		Returns "<Error: message>" if there an error, including
+		host or file not found. */
+	public static String openUrlAsString(String url) {
+		StringBuffer sb = null;
+		url = url.replaceAll(" ", "%20");
+		try {
+			URL u = new URL(url);
+			URLConnection uc = u.openConnection();
+			long len = uc.getContentLength();
+			if (len>1048576L)
+				return "<Error: file is larger than 1MB>";
+			InputStream in = u.openStream();
+			BufferedReader br = new BufferedReader(new InputStreamReader(in));
+			sb = new StringBuffer() ;
+			String line;
+			while ((line=br.readLine()) != null)
+				sb.append (line + "\n");
+			in.close ();
+		} catch (Exception e) {
+			return("<Error: "+e+">");
+		}
+		if (sb!=null)
+			return new String(sb);
+		else
+			return "";
+	}
+
 	/** Saves the current image, lookup table, selection or text window to the specified file path. 
 		The path must end in ".tif", ".jpg", ".gif", ".zip", ".raw", ".avi", ".bmp", ".fits", ".pgm", ".png", ".lut", ".roi" or ".txt".  */
 	public static void save(String path) {
@@ -1260,13 +1495,18 @@ public class IJ {
 	}
 
 	/** Saves the specified image, lookup table or selection to the specified file path. 
-		The path must end in ".tif", ".jpg", ".gif", ".zip", ".raw", ".avi", ".bmp", ".fits", ".pgm", ".png", ".lut", ".roi" or ".txt".  */
+		The file path must end with ".tif", ".jpg", ".gif", ".zip", ".raw", ".avi", ".bmp", 
+		".fits", ".pgm", ".png", ".lut", ".roi" or ".txt". */
 	public static void save(ImagePlus imp, String path) {
 		int dotLoc = path.lastIndexOf('.');
-		if (dotLoc!=-1)
+		if (dotLoc!=-1) {
+			ImagePlus imp2 = imp;
+			if (imp2==null) imp2 = WindowManager.getCurrentImage();
+			String title = imp2!=null?imp2.getTitle():null;
 			saveAs(imp, path.substring(dotLoc+1), path);
-		else
-			error("The save() macro function requires a file name extension.\n \n"+path);
+			if (title!=null) imp2.setTitle(title);
+		} else
+			error("The file path passed to IJ.save() method or save()\nmacro function is missing the required extension.\n \n\""+path+"\"");
 	}
 
 	/* Saves the active image, lookup table, selection, measurement results, selection XY 
@@ -1286,7 +1526,8 @@ public class IJ {
 		if (path!=null && path.length()==0) path = null;
 		format = format.toLowerCase(Locale.US);
 		if (format.indexOf("tif")!=-1) {
-			path = updateExtension(path, ".tif");
+			if (path!=null&&!path.endsWith(".tiff"))
+				path = updateExtension(path, ".tif");
 			format = "Tiff...";
 		} else if (format.indexOf("jpeg")!=-1  || format.indexOf("jpg")!=-1) {
 			path = updateExtension(path, ".jpg");
@@ -1305,7 +1546,7 @@ public class IJ {
 			path = updateExtension(path, ".zip");
 			format = "ZIP...";
 		} else if (format.indexOf("raw")!=-1) {
-			path = updateExtension(path, ".raw");
+			//path = updateExtension(path, ".raw");
 			format = "Raw Data...";
 		} else if (format.indexOf("avi")!=-1) {
 			path = updateExtension(path, ".avi");
@@ -1325,8 +1566,8 @@ public class IJ {
 		} else if (format.indexOf("lut")!=-1) {
 			path = updateExtension(path, ".lut");
 			format = "LUT...";
-		} else if (format.indexOf("measurements")!=-1) {
-			format = "Measurements...";
+		} else if (format.indexOf("results")!=-1 || format.indexOf("measurements")!=-1) {
+			format = "Results...";
 		} else if (format.indexOf("selection")!=-1 || format.indexOf("roi")!=-1) {
 			path = updateExtension(path, ".roi");
 			format = "Selection...";
@@ -1334,7 +1575,7 @@ public class IJ {
 			path = updateExtension(path, ".txt");
 			format = "XY Coordinates...";
 		} else
-			error("Unrecognized format: "+format);
+			error("Unsupported save() or saveAs() file format: \""+format+"\"\n \n\""+path+"\"");
 		if (path==null)
 			run(format);
 		else
@@ -1354,9 +1595,10 @@ public class IJ {
 			path += extension;
 		return path;
 	}
-
-	/** Saves a string as a file. Returns an error message 
-		if there is  an exception, otherwise returns null. */
+	
+	/** Saves a string as a file. Displays a file save dialog if
+		'path' is null or blank. Returns an error message 
+		if there is an exception, otherwise returns null. */
 	public static String saveString(String string, String path) {
 		return write(string, path, false);
 	}
@@ -1369,6 +1611,13 @@ public class IJ {
 	}
 
 	private static String write(String string, String path, boolean append) {
+		if (path==null || path.equals("")) {
+			String msg = append?"Append String...":"Save String...";
+			SaveDialog sd = new SaveDialog(msg, "Untitled", ".txt");
+			String name = sd.getFileName();
+			if (name==null) return null;
+			path = sd.getDirectory() + name;
+		}
 		try {
 			BufferedWriter out = new BufferedWriter(new FileWriter(path, append));
 			out.write(string);
@@ -1379,6 +1628,41 @@ public class IJ {
 		return null;
 	}
 
+	/** Opens a text file as a string. Displays a file open dialog
+		if path is null or blank. Returns null if the user cancels
+		the file open dialog. If there is an error, returns a 
+		 message in the form "Error: message". */
+	public static String openAsString(String path) {
+		if (path==null || path.equals("")) {
+			OpenDialog od = new OpenDialog("Open Text File", "");
+			String directory = od.getDirectory();
+			String name = od.getFileName();
+			if (name==null) return null;
+			path = directory + name;
+		}
+		String str = "";
+		File file = new File(path);
+		if (!file.exists())
+			return "Error: file not found";
+		try {
+			StringBuffer sb = new StringBuffer(5000);
+			BufferedReader r = new BufferedReader(new FileReader(file));
+			while (true) {
+				String s=r.readLine();
+				if (s==null)
+					break;
+				else
+					sb.append(s+"\n");
+			}
+			r.close();
+			str = new String(sb);
+		}
+		catch (Exception e) {
+			str = "Error: "+e.getMessage();
+		}
+		return str;
+	}
+
 	 /** Creates a new imagePlus. <code>Type</code> should contain "8-bit", "16-bit", "32-bit" or "RGB". 
 		 In addition, it can contain "white", "black" or "ramp" (the default is "white"). <code>Width</code> 
 	 	and <code>height</code> specify the width and height of the image in pixels.  
@@ -1428,14 +1712,19 @@ public class IJ {
 		escapePressed = false;
 	}
 	
-	/** Causes IJ.error() and IJ.showMessage() output to be temporarily redirected to the "Log" window. */
+	/** Causes IJ.error() output to be temporarily redirected to the "Log" window. */
 	public static void redirectErrorMessages() {
 		redirectErrorMessages = true;
 	}
 	
+	/** Set 'true' and IJ.error() output will be redirected to the "Log" window. */
+	public static void redirectErrorMessages(boolean redirect) {
+		redirectErrorMessages2 = redirect;
+	}
+
 	/** Returns the state of the  'redirectErrorMessages' flag. The File/Import/Image Sequence command sets this flag.*/
 	public static boolean redirectingErrorMessages() {
-		return redirectErrorMessages;
+		return redirectErrorMessages || redirectErrorMessages2;
 	}
 
 	/** Temporarily suppress "plugin not found" errors. */
@@ -1443,7 +1732,8 @@ public class IJ {
 		suppressPluginNotFoundError = true;
 	}
 
-	/** Returns an instance of the class loader ImageJ uses to run plugins. */
+	/** Returns the class loader ImageJ uses to run plugins or the
+		system class loader if Menus.getPlugInsPath() returns null. */
 	public static ClassLoader getClassLoader() {
 		if (classLoader==null) {
 			String pluginsDir = Menus.getPlugInsPath();
@@ -1469,27 +1759,20 @@ public class IJ {
 	
 	/** Returns the size, in pixels, of the primary display. */
 	public static Dimension getScreenSize() {
-		if (screenSize==null) {
-			if (isWindows()) { // GraphicsEnvironment.getConfigurations is *very* slow on Windows
-				screenSize = Toolkit.getDefaultToolkit().getScreenSize();
-				return screenSize;
-			}
-			if (GraphicsEnvironment.isHeadless())
-				screenSize = new Dimension(0, 0);
-			else {
-				// Can't use Toolkit.getScreenSize() on Linux because it returns 
-				// size of all displays rather than just the primary display.
-				GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
-				GraphicsDevice[] gd = ge.getScreenDevices();
-				GraphicsConfiguration[] gc = gd[0].getConfigurations();
-				Rectangle bounds = gc[0].getBounds();
-				if (bounds.x==0&&bounds.y==0)
-					screenSize = new Dimension(bounds.width, bounds.height);
-				else
-					screenSize = Toolkit.getDefaultToolkit().getScreenSize();
-			}
-		}
-		return screenSize;
+		if (isWindows())  // GraphicsEnvironment.getConfigurations is *very* slow on Windows
+			return Toolkit.getDefaultToolkit().getScreenSize();
+		if (GraphicsEnvironment.isHeadless())
+			return new Dimension(0, 0);
+		// Can't use Toolkit.getScreenSize() on Linux because it returns 
+		// size of all displays rather than just the primary display.
+		GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
+		GraphicsDevice[] gd = ge.getScreenDevices();
+		GraphicsConfiguration[] gc = gd[0].getConfigurations();
+		Rectangle bounds = gc[0].getBounds();
+		if (bounds.x==0&&bounds.y==0)
+			return new Dimension(bounds.width, bounds.height);
+		else
+			return Toolkit.getDefaultToolkit().getScreenSize();
 	}
 	
 	static void abort() {
@@ -1501,4 +1784,50 @@ public class IJ {
 		classLoader = loader;
 	}
 
-}
\ No newline at end of file
+	/** Displays a stack trace. Use the setExceptionHandler 
+		method() to override with a custom exception handler. */
+	public static void handleException(Throwable e) {
+		if (exceptionHandler!=null) {
+			exceptionHandler.handle(e);
+			return;
+		}
+		CharArrayWriter caw = new CharArrayWriter();
+		PrintWriter pw = new PrintWriter(caw);
+		e.printStackTrace(pw);
+		String s = caw.toString();
+		if (getInstance()!=null)
+			new TextWindow("Exception", s, 500, 300);
+		else
+			log(s);
+	}
+
+	/** Installs a custom exception handler that 
+		overrides the handleException() method. */
+	public static void setExceptionHandler(ExceptionHandler handler) {
+		exceptionHandler = handler;
+	}
+
+	public interface ExceptionHandler {
+		public void handle(Throwable e);
+	}
+
+	static ExceptionHandler exceptionHandler;
+
+	public static void addEventListener(IJEventListener listener) {
+		eventListeners.addElement(listener);
+	}
+	
+	public static void removeEventListener(IJEventListener listener) {
+		eventListeners.removeElement(listener);
+	}
+	
+	public static void notifyEventListeners(int eventID) {
+		synchronized (eventListeners) {
+			for (int i=0; i<eventListeners.size(); i++) {
+				IJEventListener listener = (IJEventListener)eventListeners.elementAt(i);
+				listener.eventOccurred(eventID);
+			}
+		}
+	}
+
+}
