@@ -11,19 +11,25 @@ import ij.plugin.filter.Analyzer;
 import ij.util.Tools;
 import ij.macro.Interpreter;
 import ij.plugin.frame.ContrastAdjuster;
+import ij.plugin.frame.Recorder;
 import ij.plugin.Converter;
+import ij.plugin.Duplicator;
+import ij.plugin.RectToolOptions;
+
 
 /**
-This is an extended image class that supports 8-bit, 16-bit,
-32-bit (real) and RGB images. It also provides support for
-3D image stacks.
+An ImagePlus contain an ImageProcessor (2D image) or an ImageStack (3D, 4D or 5D image).
+It also includes metadata (spatial calibration and possibly the directory/file where
+it was read from). The ImageProcessor contains the pixel data (8-bit, 16-bit, float or RGB) 
+of the 2D image and some basic methods to manipulate it. An ImageStack is essentually 
+a list ImageProcessors of same type and size.
 @see ij.process.ImageProcessor
 @see ij.ImageStack
 @see ij.gui.ImageWindow
 @see ij.gui.ImageCanvas
 */
    
-public class ImagePlus implements ImageObserver, Measurements {
+public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 
 	/** 8-bit grayscale (unsigned)*/
 	public static final int GRAY8 = 0;
@@ -43,15 +49,6 @@ public class ImagePlus implements ImageObserver, Measurements {
 	/** True if any changes have been made to this image. */
 	public boolean changes;
 	
-	/** Obsolete. Use GetCalibration(). */
-	public double pixelWidth=1.0, pixelHeight=1.0;
-	/** Obsolete. Use GetCalibration(). */
-	public String unit="pixel";
-	/** Obsolete. Use GetCalibration(). */
-	public String units=unit;
-	/** Obsolete. Use GetCalibration(). */
-	public boolean sCalibrated;
-
 	protected Image img;
 	protected ImageProcessor ip;
 	protected ImageWindow win;
@@ -62,14 +59,14 @@ public class ImagePlus implements ImageObserver, Measurements {
 	protected int width;
 	protected int height;
 	protected boolean locked = false;
+	protected int nChannels = 1;
+	protected int nSlices = 1;
+	protected int nFrames = 1;
 
 	private ImageJ ij = IJ.getInstance();
 	private String title;
 	private	String url;
 	private FileInfo fileInfo;
-	private int nSlices = 1;
-	private int nChannels = 1;
-	private int nFrames = 1;
 	private int imageType = GRAY8;
 	private ImageStack stack;
 	private static int currentID = -1;
@@ -89,6 +86,11 @@ public class ImagePlus implements ImageObserver, Measurements {
 	private boolean openAsHyperStack;
 	private int[] position = {1,1,1};
 	private boolean noUpdateMode;
+	private ImageCanvas flatteningCanvas;
+	private Overlay overlay;
+	private boolean hideOverlay;
+	private static int default16bitDisplayRange;
+
 
     /** Constructs an uninitialized ImagePlus. */
     public ImagePlus() {
@@ -152,10 +154,10 @@ public class ImagePlus implements ImageObserver, Measurements {
 		if (locked) {
 			IJ.beep();
 			IJ.showStatus("\"" + title + "\" is locked");
-			if (IJ.macroRunning()) {
-				IJ.error("Image is locked");
-				Macro.abort();
-			}
+			//if (IJ.macroRunning()) {
+			//	IJ.error("Image is locked");
+			//	Macro.abort();
+			//}
 			return false;
         } else {
         	locked = true;
@@ -348,6 +350,7 @@ public class ImagePlus implements ImageObserver, Measurements {
 	public void show(String statusMessage) {
 		if (win!=null) return;
 		if ((IJ.isMacro() && ij==null) || Interpreter.isBatchMode()) {
+			if (isComposite()) ((CompositeImage)this).reset();
 			ImagePlus img = WindowManager.getCurrentImage();
 			if (img!=null) img.saveRoi();
 			WindowManager.setTempCurrentImage(this);
@@ -366,6 +369,8 @@ public class ImagePlus implements ImageObserver, Measurements {
 			else
 				win = new ImageWindow(this);
 			if (roi!=null) roi.setImage(this);
+			if (overlay!=null && getCanvas()!=null)
+				getCanvas().setOverlay(overlay);
 			draw();
 			IJ.showStatus(statusMessage);
 			if (IJ.isMacro()) { // wait for window to be activated
@@ -378,6 +383,17 @@ public class ImagePlus implements ImageObserver, Measurements {
 					}
 				}
 			}
+			if (imageType==GRAY16 && default16bitDisplayRange!=0) {
+				resetDisplayRange();
+				updateAndDraw();
+			}
+			if (stackSize>1) {
+				int c = getChannel();
+				int z = getSlice();
+				int t = getFrame();
+				if (c>1 || z>1 || t>1)
+					setPosition(c, z, t);
+			}
 			notifyListeners(OPENED);
 		}
 	}
@@ -467,7 +483,24 @@ public class ImagePlus implements ImageObserver, Measurements {
 		}
 	}
 	
-	/** Replaces the ImageProcessor, if any, with the one specified.
+	/** Replaces this image with the specified ImagePlus. May
+		not work as expected if 'imp' is a CompositeImage
+		and this image is not. */
+	public void setImage(ImagePlus imp) {
+		if (imp.getWindow()!=null)
+			imp = imp.duplicate();
+		ImageStack stack2 = imp.getStack();
+		if (imp.isHyperStack())
+			setOpenAsHyperStack(true);
+		setStack(stack2, imp.getNChannels(), imp.getNSlices(), imp.getNFrames());
+	}
+	
+	/** Replaces the ImageProcessor with the one specified and updates the display. */
+	public void setProcessor(ImageProcessor ip) {
+		setProcessor(null, ip);
+	}
+
+	/** Replaces the ImageProcessor with the one specified and updates the display.
 		Set 'title' to null to leave the image title unchanged. */
 	public void setProcessor(String title, ImageProcessor ip) {
         if (ip==null || ip.getPixels()==null)
@@ -518,40 +551,53 @@ public class ImagePlus implements ImageObserver, Measurements {
 		}
 	}
 
-	/** Replaces the stack, if any, with the one specified.
-		Set 'title' to null to leave the title unchanged. */
-    public void setStack(String title, ImageStack stack) {
-		int stackSize = stack.getSize();
-		if (stackSize==0)
+	/** Replaces the image with the specified stack and updates the display. */
+	public void setStack(ImageStack stack) {
+    	setStack(null, stack);
+    }
+
+	/** Replaces the image with the specified stack and updates 
+		the display. Set 'title' to null to leave the title unchanged. */
+    public void setStack(String title, ImageStack newStack) {
+		int newStackSize = newStack.getSize();
+		if (newStackSize==0)
 			throw new IllegalArgumentException("Stack is empty");
-		if (!stack.isVirtual()) {
-			Object[] arrays = stack.getImageArray();
+		if (!newStack.isVirtual()) {
+			Object[] arrays = newStack.getImageArray();
 			if (arrays==null || (arrays.length>0&&arrays[0]==null))
 				throw new IllegalArgumentException("Stack pixel array null");
 		}
-    	boolean stackSizeChanged = this.stack!=null && stackSize!=getStackSize();
+    	boolean sliderChange = false;
+    	if (win!=null && (win instanceof StackWindow)) {
+    		int nScrollbars = ((StackWindow)win).getNScrollbars();
+    		if (nScrollbars>0 && newStackSize==1)
+    			sliderChange = true;
+    		else if (nScrollbars==0 && newStackSize>1)
+    			sliderChange = true;
+    	}
     	if (currentSlice<1) setCurrentSlice(1);
-    	boolean resetCurrentSlice = currentSlice>stackSize;
-    	if (resetCurrentSlice) setCurrentSlice(stackSize);
-    	ImageProcessor ip = stack.getProcessor(currentSlice);
+    	boolean resetCurrentSlice = currentSlice>newStackSize;
+    	if (resetCurrentSlice) setCurrentSlice(newStackSize);
+    	ImageProcessor ip = newStack.getProcessor(currentSlice);
     	boolean dimensionsChanged = width>0 && height>0 && (width!=ip.getWidth()||height!=ip.getHeight());
-    	this.stack = stack;
-    	setProcessor2(title, ip, stack);
-		if (win==null) return;
+    	this.stack = newStack;
+    	setProcessor2(title, ip, newStack);
+		if (win==null) {
+			if (resetCurrentSlice) setSlice(currentSlice);
+			return;
+		}
 		boolean invalidDimensions = isDisplayedHyperStack() && !((StackWindow)win).validDimensions();
-		if (stackSize==1 && win instanceof StackWindow)
-			win = new ImageWindow(this, getCanvas());   // replaces this window
-		else if (dimensionsChanged && !stackSizeChanged)
-			win.updateImage(this);
-		else if (stackSize>1 && !(win instanceof StackWindow)) {
+		if (newStackSize>1 && !(win instanceof StackWindow)) {
 			if (isDisplayedHyperStack()) setOpenAsHyperStack(true);
 			win = new StackWindow(this, getCanvas());   // replaces this window
 			setPosition(1, 1, 1);
-		} else if (stackSize>1 && (dimensionsChanged||invalidDimensions)) {
+		} else if (newStackSize>1 && invalidDimensions) {
 			if (isDisplayedHyperStack()) setOpenAsHyperStack(true);
 			win = new StackWindow(this);   // replaces this window
 			setPosition(1, 1, 1);
-		} else
+		} else if (dimensionsChanged || sliderChange)
+			win.updateImage(this);
+		else
 			repaintWindow();
 		if (resetCurrentSlice) setSlice(currentSlice);
     }
@@ -562,6 +608,8 @@ public class ImagePlus implements ImageObserver, Measurements {
 		this.nChannels = nChannels;
 		this.nSlices = nSlices;
 		this.nFrames = nFrames;
+		if (isComposite())
+			((CompositeImage)this).setChannelsUpdated();
 		setStack(null, stack);
 	}
 
@@ -595,7 +643,7 @@ public class ImagePlus implements ImageObserver, Measurements {
 	/** Returns the ImageCanvas being used to
 		display this image, or null. */
 	public ImageCanvas getCanvas() {
-		return win!=null?win.getCanvas():null;
+		return win!=null?win.getCanvas():flatteningCanvas;
 	}
 
 	/** Sets current foreground color. */
@@ -606,15 +654,10 @@ public class ImagePlus implements ImageObserver, Measurements {
 	
 	void setupProcessor() {
 		if (imageType==COLOR_RGB) {
-			if (ip == null || ip instanceof ByteProcessor) {
+			if (ip == null || ip instanceof ByteProcessor)
 				ip = new ColorProcessor(getImage());
-				if (IJ.debugMode) IJ.log(title + ": new ColorProcessor");
-			}
-		}
-		else if (ip==null || (ip instanceof ColorProcessor)) {
+		} else if (ip==null || (ip instanceof ColorProcessor))
 			ip = new ByteProcessor(getImage());
-			if (IJ.debugMode) IJ.log(title + ": new ByteProcessor");
-		}
 		if (roi!=null && roi.isArea())
 			ip.setRoi(roi.getBounds());
 		else
@@ -627,7 +670,9 @@ public class ImagePlus implements ImageObserver, Measurements {
 	
 	/** Returns a reference to the current ImageProcessor. If there
 	    is no ImageProcessor, it creates one. Returns null if this
-	    ImagePlus contains no ImageProcessor and no AWT Image. */
+	    ImagePlus contains no ImageProcessor and no AWT Image.
+		Sets the line width to the current line width and sets the
+		calibration table if the image is density calibrated. */
 	public ImageProcessor getProcessor() {
 		if (ip==null && img==null)
 			return null;
@@ -641,6 +686,10 @@ public class ImagePlus implements ImageObserver, Measurements {
 			ip.setCalibrationTable(cal.getCTable());
 		else
 			ip.setCalibrationTable(null);
+		if (Recorder.record) {
+			Recorder recorder = Recorder.getInstance();
+			if (recorder!=null) recorder.imageUpdated(this);
+		}
 		return ip;
 	}
 	
@@ -650,14 +699,13 @@ public class ImagePlus implements ImageObserver, Measurements {
 		ImageProcessor ip2 = ip;
 		if (!locked && ip2!=null) {
 			if (IJ.debugMode) IJ.log(title + ": trimProcessor");
+			Roi roi2 = getRoi();
+			if (roi2!=null && roi2.getPasteMode()!=Roi.NOT_PASTING)
+				roi2.endPaste();
 			ip2.setSnapshotPixels(null);
 		}
 	}
 	
-	/** Obsolete. */
-	public void killProcessor() {
-	}
-	
 	/** For images with irregular ROIs, returns a byte mask, otherwise, returns
 		null. Mask pixels have a non-zero value. */
 	public ImageProcessor getMask() {
@@ -668,7 +716,7 @@ public class ImagePlus implements ImageObserver, Measurements {
 		ImageProcessor mask = roi.getMask();
 		if (mask==null)
 			return null;
-		if (ip!=null) {
+		if (ip!=null && roi!=null) {
 			ip.setMask(mask);
 			ip.setRoi(roi.getBounds());
 		}
@@ -676,13 +724,44 @@ public class ImagePlus implements ImageObserver, Measurements {
 	}
 
 	/** Returns an ImageStatistics object generated using the standard
-		measurement options (area, mean, mode, min and max). */
+		measurement options (area, mean, mode, min and max).
+		This plugin demonstrates how get the area, mean and max of the
+		current image or selection:
+		<pre>
+   public class Get_Statistics implements PlugIn {
+      public void run(String arg) {
+         ImagePlus imp = IJ.getImage();
+         ImageStatistics stats = imp.getStatistics();
+         IJ.log("Area: "+stats.area);
+         IJ.log("Mean: "+stats.mean);
+         IJ.log("Max: "+stats.max);
+      }
+   }
+		</pre>
+		@see ij.process.ImageStatistics
+		@see ij.process.ImageStatistics#getStatistics
+		*/
 	public ImageStatistics getStatistics() {
 		return getStatistics(AREA+MEAN+MODE+MIN_MAX);
 	}
 	
 	/** Returns an ImageStatistics object generated using the
-		 specified measurement options. */
+		specified measurement options. This plugin demonstrates how
+		get the area and centroid of the current selection:
+		<pre>
+   public class Get_Statistics implements PlugIn, Measurements {
+      public void run(String arg) {
+         ImagePlus imp = IJ.getImage();
+         ImageStatistics stats = imp.getStatistics(MEDIAN+CENTROID);
+         IJ.log("Median: "+stats.median);
+         IJ.log("xCentroid: "+stats.xCentroid);
+         IJ.log("yCentroid: "+stats.yCentroid);
+      }
+   }
+		</pre>
+		@see ij.process.ImageStatistics
+		@see ij.measure.Measurements
+	*/
 	public ImageStatistics getStatistics(int mOptions) {
 		return getStatistics(mOptions, 256, 0.0, 0.0);
 	}
@@ -876,7 +955,6 @@ public class ImagePlus implements ImageObserver, Measurements {
 		return d;
 	}
 
-
 	void verifyDimensions() {
 		int stackSize = getImageStackSize();
 		if (nSlices==1) {
@@ -959,7 +1037,7 @@ public class ImagePlus implements ImageObserver, Measurements {
 		
 	/** Creates a LookUpTable object that corresponds to this image. */
     public LookUpTable createLut() {
-	ImageProcessor ip2 = getProcessor();
+		ImageProcessor ip2 = getProcessor();
 		if (ip2!=null)
 			return new LookUpTable(ip2.getColorModel());
 		else
@@ -987,8 +1065,6 @@ public class ImagePlus implements ImageObserver, Measurements {
 	*/
 	public int[] getPixel(int x, int y) {
 		pvalue[0]=pvalue[1]=pvalue[2]=pvalue[3]=0;
-		if (img == null)
-			return pvalue;
 		switch (imageType) {
 			case GRAY8: case COLOR_256:
 				int index;
@@ -996,6 +1072,7 @@ public class ImagePlus implements ImageObserver, Measurements {
 					index = ip.getPixel(x, y);
 				else {
 					byte[] pixels8;
+					if (img==null) return pvalue;
 					PixelGrabber pg = new PixelGrabber(img,x,y,1,1,false);
 					try {pg.grabPixels();}
 					catch (InterruptedException e){return pvalue;};
@@ -1009,12 +1086,17 @@ public class ImagePlus implements ImageObserver, Measurements {
 				pvalue[3] = index;
 				// fall through to get rgb values
 			case COLOR_RGB:
-				int[] pixels32 = new int[1];
-				if (win==null) break;
-				PixelGrabber pg = new PixelGrabber(img, x, y, 1, 1, pixels32, 0, width);
-				try {pg.grabPixels();}
-				catch (InterruptedException e){return pvalue;};
-				int c = pixels32[0];
+				int c = 0;
+				if (imageType==COLOR_RGB && ip!=null)
+					c = ip.getPixel(x, y);
+				else {
+					int[] pixels32 = new int[1];
+					if (img==null) return pvalue;
+					PixelGrabber pg = new PixelGrabber(img, x, y, 1, 1, pixels32, 0, width);
+					try {pg.grabPixels();}
+					catch (InterruptedException e) {return pvalue;};
+					c = pixels32[0];
+				}
 				int r = (c&0xff0000)>>16;
 				int g = (c&0xff00)>>8;
 				int b = c&0xff;
@@ -1059,6 +1141,13 @@ public class ImagePlus implements ImageObserver, Measurements {
 			s.update(ip2);
 		} else {
 			s = stack;
+			if (ip!=null) {
+				Calibration cal = getCalibration();
+				if (cal.calibrated())
+					ip.setCalibrationTable(cal.getCTable());
+				else
+					ip.setCalibrationTable(null);
+			}
 			s.update(ip);
 		}
 		if (roi!=null)
@@ -1111,6 +1200,8 @@ public class ImagePlus implements ImageObserver, Measurements {
 		trimProcessor();
 	}
 	
+	/** Sets the current hyperstack position and updates the display,
+		where 'channel', 'slice' and 'frame' are one-based indexes. */
 	public void setPosition(int channel, int slice, int frame) {
 		//IJ.log("setPosition: "+channel+"  "+slice+"  "+frame+"  "+noUpdateMode);
 		verifyDimensions();
@@ -1128,13 +1219,15 @@ public class ImagePlus implements ImageObserver, Measurements {
 		}
 	}
 	
+	/** Sets the current hyperstack position without updating the display,
+		where 'channel', 'slice' and 'frame' are one-based indexes. */
 	public void setPositionWithoutUpdate(int channel, int slice, int frame) {
 		noUpdateMode = true;
 		setPosition(channel, slice, frame);
 		noUpdateMode = false;
 	}
 	
-	/** Returns that stack index (1-based) corresponding to the specified position. */
+	/** Returns that stack index (one-based) corresponding to the specified position. */
 	public int getStackIndex(int channel, int slice, int frame) {	
    		if (channel<1) channel = 1;
     	if (channel>nChannels) channel = nChannels;
@@ -1157,19 +1250,30 @@ public class ImagePlus implements ImageObserver, Measurements {
 		}
 	}
 	
+	/** Set the current hyperstack position based on the stack index 'n' (one-based). */
 	public void setPosition(int n) {
-		int[] dim = getDimensions();
-		int c = ((n-1)%dim[2])+1;
-		int z = (((n-1)/dim[2])%dim[3])+1;
-		int t = (((n-1)/(dim[2]*dim[3]))%dim[4])+1;
-		setPosition(c, z, t);
+		int[] pos = convertIndexToPosition(n);
+		setPosition(pos[0], pos[1], pos[2]);
 	}
 			
+	/** Converts the stack index 'n' (one-based) into a hyperstack position (channel, slice, frame). */
+	public int[] convertIndexToPosition(int n) {
+		if (n<1 || n>getStackSize())
+			throw new IllegalArgumentException("n out of range: "+n);
+		int[] position = new int[3];
+		int[] dim = getDimensions();
+		position[0] = ((n-1)%dim[2])+1;
+		position[1] = (((n-1)/dim[2])%dim[3])+1;
+		position[2] = (((n-1)/(dim[2]*dim[3]))%dim[4])+1;
+		return position;
+	}
+
 	/** Displays the specified stack image, where 1<=n<=stackSize.
 		Does nothing if this image is not a stack. */
 	public synchronized void setSlice(int n) {
 		if (stack==null || (n==currentSlice&&ip!=null)) {
-	    	updateAndRepaintWindow();
+			if (!noUpdateMode)
+				updateAndRepaintWindow();
 			return;
 		}
 		if (n>=1 && n<=stack.getSize()) {
@@ -1188,16 +1292,16 @@ public class ImagePlus implements ImageObserver, Measurements {
 				ip = stack.getProcessor(n);
 			if (win!=null && win instanceof StackWindow)
 				((StackWindow)win).updateSliceSelector();
-			if (IJ.altKeyDown() && !IJ.isMacro()) {
-				if (imageType==GRAY16 || imageType==GRAY32) {
-					ip.resetMinAndMax();
-					IJ.showStatus(n+": min="+ip.getMin()+", max="+ip.getMax());
-				}
-				ContrastAdjuster.update();
-			}
+			//if (IJ.altKeyDown() && !IJ.isMacro()) {
+			//	if (imageType==GRAY16 || imageType==GRAY32) {
+			//		ip.resetMinAndMax();
+			//		IJ.showStatus(n+": min="+ip.getMin()+", max="+ip.getMax());
+			//	}
+			//	ContrastAdjuster.update();
+			//}
 			if (imageType==COLOR_RGB)
 				ContrastAdjuster.update();
-			if (!(Interpreter.isBatchMode()||noUpdateMode))
+			if (!noUpdateMode)
 				updateAndRepaintWindow();
 			else
 				img = null;
@@ -1212,14 +1316,6 @@ public class ImagePlus implements ImageObserver, Measurements {
 		noUpdateMode = false;
 	}
 
-	/** Obsolete */
-	void undoFilter() {
-		if (ip!=null) {
-			ip.reset();
-			updateAndDraw();
-		}
-	}
-
 	/** Returns the current selection, or null if there is no selection. */
 	public Roi getRoi() {
 		return roi;
@@ -1272,10 +1368,18 @@ public class ImagePlus implements ImageObserver, Measurements {
 		killRoi();
 		switch (Toolbar.getToolId()) {
 			case Toolbar.RECTANGLE:
-				roi = new Roi(sx, sy, this);
+				int cornerDiameter = Toolbar.getRoundRectArcSize();
+				roi = new Roi(sx, sy, this, cornerDiameter);
+				if (cornerDiameter>0) {
+					roi.setStrokeColor(Toolbar.getForegroundColor());
+					roi.setStrokeWidth(RectToolOptions.getDefaultStrokeWidth());
+				}
 				break;
 			case Toolbar.OVAL:
-				roi = new OvalRoi(sx, sy, this);
+				if (Toolbar.getOvalToolType()==Toolbar.ELLIPSE_ROI)
+					roi = new EllipseRoi(sx, sy, this);
+				else
+					roi = new OvalRoi(sx, sy, this);
 				break;
 			case Toolbar.POLYGON:
 			case Toolbar.POLYLINE:
@@ -1287,7 +1391,10 @@ public class ImagePlus implements ImageObserver, Measurements {
 				roi = new FreehandRoi(sx, sy, this);
 				break;
 			case Toolbar.LINE:
-				roi = new Line(sx, sy, this);
+				if ("arrow".equals(Toolbar.getToolName()))
+					roi = new Arrow(sx, sy, this);
+				else
+					roi = new Line(sx, sy, this);
 				break;
 			case Toolbar.TEXT:
 				roi = new TextRoi(sx, sy, this);
@@ -1311,7 +1418,7 @@ public class ImagePlus implements ImageObserver, Measurements {
 
 	/** Deletes the current region of interest. Makes a copy
 		of the current ROI so it can be recovered by the
-		Edit/Restore Selection command. */
+		Edit/Selection/Restore Selection command. */
 	public void killRoi() {
 		if (roi!=null) {
 			saveRoi();
@@ -1326,7 +1433,7 @@ public class ImagePlus implements ImageObserver, Measurements {
 		if (roi!=null) {
 			roi.endPaste();
 			Rectangle r = roi.getBounds();
-			if (r.width>0 && r.height>0) {
+			if ((r.width>0 || r.height>0)) {
 				Roi.previousRoi = (Roi)roi.clone();
 				if (IJ.debugMode) IJ.log("saveRoi: "+roi);
 			}
@@ -1359,14 +1466,12 @@ public class ImagePlus implements ImageObserver, Measurements {
 	
 	/** Implements the File/Revert command. */
 	public void revert() {
-		if (getStackSize()>1) // can't revert stacks
+		if (getStackSize()>1 && getStack().isVirtual())
 			return;
 		FileInfo fi = getOriginalFileInfo();
 		boolean isFileInfo = fi!=null && fi.fileFormat!=FileInfo.UNKNOWN;
 		if (!(isFileInfo || url!=null))
 			return;
-		if (getStackSize()>1 && (fi==null||fi.fileFormat!=FileInfo.TIFF||fi.compression!=FileInfo.COMPRESSION_NONE))
-			return;
 		if (ij!=null && changes && isFileInfo && !Interpreter.isBatchMode() && !IJ.isMacro() && !IJ.altKeyDown()) {
 			if (!IJ.showMessageWithCancel("Revert?", "Revert to saved version of\n\""+getTitle()+"\"?"))
 				return;
@@ -1376,6 +1481,12 @@ public class ImagePlus implements ImageObserver, Measurements {
 			roi.endPaste();
 			saveRoi = (Roi)roi.clone();
 		}
+		
+		if (getStackSize()>1) {
+			revertStack(fi);
+			return;
+		}
+
 		trimProcessor();
 		if (isFileInfo && !(url!=null&&(fi.directory==null||fi.directory.equals(""))))
 			new FileOpener(fi).revertToSaved(this);
@@ -1395,7 +1506,7 @@ public class ImagePlus implements ImageObserver, Measurements {
 		if (getProperty("FHT")!=null) {
 			properties.remove("FHT");
 			if (getTitle().startsWith("FFT of "))
-				setTitle(getTitle().substring(6));
+				setTitle(getTitle().substring(7));
 		}
 		ContrastAdjuster.update();
 		if (saveRoi!=null) setRoi(saveRoi);
@@ -1405,6 +1516,47 @@ public class ImagePlus implements ImageObserver, Measurements {
 		notifyListeners(UPDATED);
     }
     
+	void revertStack(FileInfo fi) {
+		String path = null;
+		String url2 = null;
+		if (url!=null && !url.equals("")) {
+			path = url;
+			url2 = url;
+		} else if (fi!=null && !((fi.directory==null||fi.directory.equals("")))) {
+			path = fi.directory+fi.fileName;
+		} else if (fi!=null && fi.url!=null && !fi.url.equals("")) {
+			path = fi.url;
+			url2 = fi.url;
+		} else
+			return;
+		//IJ.log("revert: "+path+"  "+fi);
+		IJ.showStatus("Loading: " + path);
+		ImagePlus imp = IJ.openImage(path);
+		if (imp!=null) {
+			int n = imp.getStackSize();
+			int c = imp.getNChannels();
+			int z = imp.getNSlices();
+			int t = imp.getNFrames();
+			if (z==n || t==n || (c==getNChannels()&&z==getNSlices()&&t==getNFrames())) {
+				setCalibration(imp.getCalibration());
+				setStack(imp.getStack(), c, z, t);
+			} else {
+				ImageWindow win = getWindow();
+				Point loc = null;
+				if (win!=null) loc = win.getLocation();
+				changes = false;
+				close();
+				FileInfo fi2 = imp.getOriginalFileInfo();
+				if (fi2!=null && (fi2.url==null || fi2.url.length()==0)) {
+					fi2.url = url2;
+					imp.setFileInfo(fi2);
+				}
+				ImageWindow.setNextLocation(loc);
+				imp.show();
+			}
+		}
+	}
+
     /** Returns a FileInfo object containing information, including the
 		pixel array, needed to save this image. Use getOriginalFileInfo()
 		to get a copy of the FileInfo object used to open the image.
@@ -1500,8 +1652,8 @@ public class ImagePlus implements ImageObserver, Measurements {
 	/** Sets the ImageProcessor, Roi, AWT Image and stack image
 		arrays to null. Does nothing if the image is locked. */
 	public synchronized void flush() {
-		if (locked || ignoreFlush) return;
 		notifyListeners(CLOSED);
+		if (locked || ignoreFlush) return;
 		ip = null;
 		if (roi!=null) roi.setImage(null);
 		roi = null;
@@ -1514,13 +1666,25 @@ public class ImagePlus implements ImageObserver, Measurements {
 		}
 		stack = null;
 		img = null;
+		win = null;
+		if (roi!=null) roi.setImage(null);
+		roi = null;
+		properties = null;
+		calibration = null;
+		overlay = null;
+		flatteningCanvas = null;
 	}
 	
-	/** Obsolete */
 	public void setIgnoreFlush(boolean ignoreFlush) {
 		this.ignoreFlush = ignoreFlush;
 	}
 	
+
+	/** Returns a copy (clone) of this ImagePlus. */
+	public ImagePlus duplicate() {
+		return (new Duplicator()).run(this);
+	}
+
 	/** Returns a new ImagePlus with this image's attributes
 		(e.g. spatial scale), but no image. */
 	public ImagePlus createImagePlus() {
@@ -1720,7 +1884,7 @@ public class ImagePlus implements ImageObserver, Measurements {
 			return;
 		}
 		boolean batchMode = Interpreter.isBatchMode();
-		String msg = (cut)?"Cut":"Copy";
+		String msg = (cut)?"Cutt":"Copy";
 		if (!batchMode) IJ.showStatus(msg+ "ing...");
 		ImageProcessor ip = getProcessor();
 		ImageProcessor ip2;	
@@ -1755,8 +1919,10 @@ public class ImagePlus implements ImageObserver, Measurements {
 		}
 		//Roi roi3 = clipboard.getRoi();
 		//IJ.log("copy: "+clipboard +" "+ "roi3="+(roi3!=null?""+roi3:""));
-		if (!batchMode) 
+		if (!batchMode) {
+			msg = (cut)?"Cut":"Copy";
 			IJ.showStatus(msg + ": " + (clipboard.getWidth()*clipboard.getHeight()*bytesPerPixel)/1024 + "k");
+		}
     }
                 
 
@@ -1775,11 +1941,11 @@ public class ImagePlus implements ImageObserver, Measurements {
 		Roi roi = getRoi();
 		if (roi!=null)
 			r = roi.getBounds();
-		if (w==width && h==height && (r==null||w!=r.width||h!=r.height)) {
-			setRoi(0, 0, width, height);
-			roi = getRoi();
-			r = roi.getBounds();
-		}
+		//if (w==width && h==height && (r==null||w!=r.width||h!=r.height)) {
+		//	setRoi(0, 0, width, height);
+		//	roi = getRoi();
+		//	r = roi.getBounds();
+		//}
 		if (r==null || (r!=null && (w!=r.width || h!=r.height))) {
 			// create a new roi centered on visible part of image
 			ImageCanvas ic = null;
@@ -1866,7 +2032,7 @@ public class ImagePlus implements ImageObserver, Measurements {
 	
 	/** Returns true if this is a CompositeImage. */
 	public boolean isComposite() {
-		return compositeImage && getNChannels()>1 && (this instanceof CompositeImage);
+		return compositeImage && nChannels>=1 && (this instanceof CompositeImage);
 	}
 
 	/** Sets the display range of the current channel. With non-composite
@@ -1896,9 +2062,25 @@ public class ImagePlus implements ImageObserver, Measurements {
 	}
 
 	public void resetDisplayRange() {
-		ip.resetMinAndMax();
+		if (imageType==GRAY16 && default16bitDisplayRange>=8 && default16bitDisplayRange<=16 && !(getCalibration().isSigned16Bit())) {
+			ip.setMinAndMax(0, Math.pow(2,default16bitDisplayRange)-1);
+		} else
+			ip.resetMinAndMax();
 	}
 	
+    /** Set the default 16-bit display range, where 'bitDepth' must be 0 (auto-scaling), 
+    	8 (0-255), 10 (0-1023), 12 (0-4095, 15 (0-32767) or 16 (0-65535). */
+    public static void setDefault16bitRange(int bitDepth) {
+    	if (!(bitDepth==8 || bitDepth==10 || bitDepth==12 || bitDepth==15 || bitDepth==16))
+    		bitDepth = 0;
+    	default16bitDisplayRange = bitDepth;
+    }
+    
+    /** Returns the default 16-bit display range, 0 (auto-scaling), 8, 10, 12, 15 or 16. */
+    public static int getDefault16bitRange() {
+    	return default16bitDisplayRange;
+    }
+
 	public void updatePosition(int c, int z, int t) {
 		//IJ.log("updatePosition: "+c+", "+z+", "+t);
 		position[0] = c;
@@ -1906,13 +2088,105 @@ public class ImagePlus implements ImageObserver, Measurements {
 		position[2] = t;
 	}
 	
-	public Object clone() {
-		try {return super.clone();}
-		catch (CloneNotSupportedException e) {return null;}
+	/** Returns a "flattened" version of this image, in RGB format. */
+	public ImagePlus flatten() {
+		ImagePlus imp2 = createImagePlus();
+		String title = "Flat_"+getTitle();
+		ImageCanvas ic2 = new ImageCanvas(imp2);
+		imp2.flatteningCanvas = ic2;
+		imp2.setRoi(getRoi());	
+		ImageCanvas ic = getCanvas();
+		Overlay overlay2 = getOverlay();
+		ic2.setOverlay(overlay2);
+		if (ic!=null) {
+			ic2.setShowAllROIs(ic.getShowAllROIs());
+			//double mag = ic.getMagnification();
+			//if (mag<1.0) ic2.setMagnification(mag);
+		}
+		BufferedImage bi = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
+		Graphics g = bi.getGraphics();
+		g.drawImage(getImage(), 0, 0, null);
+		ic2.paint(g);
+		imp2.flatteningCanvas = null;
+		if (Recorder.record) Recorder.recordCall("imp = IJ.getImage().flatten();");
+		return new ImagePlus(title, new ColorProcessor(bi));
+	}
+	
+	/** Installs a list of ROIs that will be drawn on this image as a non-destructive overlay.
+	 * @see ij.gui.Roi#setStrokeColor
+	 * @see ij.gui.Roi#setStrokeWidth
+	 * @see ij.gui.Roi#setFillColor
+	 * @see ij.gui.Roi#setLocation
+	 * @see ij.gui.Roi#setNonScalable
+	 */
+	public void setOverlay(Overlay overlay) {
+		ImageCanvas ic = getCanvas();
+		if (ic!=null) {
+			ic.setOverlay(overlay);
+			overlay = null;
+		} else
+			this.overlay = overlay;
+		setHideOverlay(false);
+	}
+	
+	/** Creates an Overlay from the specified Shape, Color 
+	 * and BasicStroke, and assigns it to this image.
+	 * @see #setOverlay(ij.gui.Overlay)
+	 * @see ij.gui.Roi#setStrokeColor
+	 * @see ij.gui.Roi#setStrokeWidth
+	 */
+	public void setOverlay(Shape shape, Color color, BasicStroke stroke) {
+		if (shape==null)
+			{setOverlay(null); return;}
+		Roi roi = new ShapeRoi(shape);
+		roi.setStrokeColor(color);
+		roi.setStroke(stroke);
+		setOverlay(new Overlay(roi));
+	}
+	
+	/** Creates an Overlay from the specified ROI, and assigns it to this image.
+	 * @see #setOverlay(ij.gui.Overlay)
+	 */
+	public void setOverlay(Roi roi, Color strokeColor, int strokeWidth, Color fillColor) {
+		roi.setStrokeColor(strokeColor);
+		roi.setStrokeWidth(strokeWidth);
+		roi.setFillColor(fillColor);
+		setOverlay(new Overlay(roi));
+	}
+
+	/** Returns the current overly, or null if this image does not have an overlay. */
+	public Overlay getOverlay() {
+		ImageCanvas ic = getCanvas();
+		if (ic!=null)
+			return ic.getOverlay();
+		else
+			return overlay;
+	}
+	
+	public void setHideOverlay(boolean hide) {
+		hideOverlay = hide;
+		ImageCanvas ic = getCanvas();
+		if (ic!=null && ic.getOverlay()!=null)
+			ic.repaint();
+	}
+
+	public boolean getHideOverlay() {
+		return hideOverlay;
+	}
+
+	/** Returns a shallow copy of this ImagePlus. */
+	public synchronized Object clone() {
+		try {
+			ImagePlus copy = (ImagePlus)super.clone();
+			copy.win = null;
+			return copy;
+		} catch (CloneNotSupportedException e) {
+			return null;
+		}
 	}
 
     public String toString() {
     	return "imp["+getTitle()+" "+width+"x"+height+"x"+getStackSize()+"]";
     }
-
+    
 }
