@@ -44,6 +44,7 @@ public class Menus {
 	
 	public static final int MAX_OPEN_RECENT_ITEMS = 15;
 
+	private static Menus instance;
 	private static MenuBar mbar;
 	private static CheckboxMenuItem gray8Item,gray16Item,gray32Item,
 			color256Item,colorRGBItem,RGBStackItem,HSBStackItem;
@@ -70,17 +71,20 @@ public class Menus {
     private boolean isJarErrorHeading;
 	private static boolean installingJars, duplicateCommand;
 	private static Vector jarFiles;  // JAR files in plugins folder with "_" in their name
+	private Map menuEntry2jarFile = new HashMap();
 	private static Vector macroFiles;  // Macro files in plugins folder with "_" in their name
 	private static int userPluginsIndex; // First user plugin or submenu in Plugins menu
 	private static boolean addSorted;
 	private static int defaultFontSize = IJ.isWindows()?14:0;
 	private static int fontSize = Prefs.getInt(Prefs.MENU_SIZE, defaultFontSize);
 	private static Font menuFont;
+
 	static boolean jnlp; // true when using Java WebStart
 		
 	Menus(ImageJ ijInstance, Applet appletInstance) {
 		ij = ijInstance;
 		applet = appletInstance;
+		instance = this;
 	}
 
 	String addMenuBar() {
@@ -93,6 +97,7 @@ public class Menus {
 		shortcuts = new Hashtable();
 		pluginsPrefs = new Vector();
 		macroShortcuts = null;
+		setupPluginsAndMacrosPaths();
 		Menu file = getMenu("File");
 		Menu newMenu = getMenu("File>New", true);
 		addPlugInItem(file, "Open...", "ij.plugin.Commands(\"open\")", KeyEvent.VK_O, false);
@@ -102,6 +107,7 @@ public class Menus {
 		Menu importMenu = getMenu("File>Import", true);
 		file.addSeparator();
 		addPlugInItem(file, "Close", "ij.plugin.Commands(\"close\")", KeyEvent.VK_W, false);
+		addPlugInItem(file, "Close All", "ij.plugin.Commands(\"close-all\")", 0, false);
 		addPlugInItem(file, "Save", "ij.plugin.Commands(\"save\")", KeyEvent.VK_S, false);
 		saveAsMenu = getMenu("File>Save As", true);
 		addPlugInItem(file, "Revert", "ij.plugin.Commands(\"revert\")", KeyEvent.VK_R,  false);
@@ -145,15 +151,16 @@ public class Menus {
 		addPlugInItem(image, "Properties...", "ij.plugin.filter.ImageProperties", KeyEvent.VK_P, true);
 		getMenu("Image>Color", true);
 		getMenu("Image>Stacks", true);
+		getMenu("Image>Stacks>Tools_", true);
 		Menu hyperstacksMenu = getMenu("Image>Hyperstacks", true);
 		image.addSeparator();
-		addPlugInItem(image, "Crop", "ij.plugin.filter.Resizer(\"crop\")", KeyEvent.VK_X, true);
-		addPlugInItem(image, "Duplicate...", "ij.plugin.filter.Duplicater", KeyEvent.VK_D, true);
+		addPlugInItem(image, "Crop", "ij.plugin.Resizer(\"crop\")", KeyEvent.VK_X, true);
+		addPlugInItem(image, "Duplicate...", "ij.plugin.Duplicator", KeyEvent.VK_D, true);
 		addPlugInItem(image, "Rename...", "ij.plugin.SimpleCommands(\"rename\")", 0, false);
 		addPlugInItem(image, "Scale...", "ij.plugin.Scaler", KeyEvent.VK_E, false);
-		addPlugInItem(image, "Translate...", "ij.plugin.filter.Translator", 0, false);
-		getMenu("Image>Rotate", true);
+		getMenu("Image>Transform", true);
 		getMenu("Image>Zoom", true);
+		getMenu("Image>Overlay", true);
 		image.addSeparator();
 		getMenu("Image>Lookup Tables", true);
 		
@@ -161,6 +168,7 @@ public class Menus {
 		addPlugInItem(process, "Smooth", "ij.plugin.filter.Filters(\"smooth\")", KeyEvent.VK_S, true);
 		addPlugInItem(process, "Sharpen", "ij.plugin.filter.Filters(\"sharpen\")", 0, false);
 		addPlugInItem(process, "Find Edges", "ij.plugin.filter.Filters(\"edge\")", 0, false);
+		addPlugInItem(process, "Find Maxima...", "ij.plugin.filter.MaximumFinder", 0, false);
 		addPlugInItem(process, "Enhance Contrast", "ij.plugin.ContrastEnhancer", 0, false);
 		getMenu("Process>Noise", true);
 		getMenu("Process>Shadows", true);
@@ -169,6 +177,7 @@ public class Menus {
 		getMenu("Process>FFT", true);
 		Menu filtersMenu = getMenu("Process>Filters", true);
 		process.addSeparator();
+		getMenu("Process>Batch", true);
 		addPlugInItem(process, "Image Calculator...", "ij.plugin.ImageCalculator", 0, false);
 		addPlugInItem(process, "Subtract Background...", "ij.plugin.filter.BackgroundSubtracter", 0, false);
 		addItem(process, "Repeat Command", KeyEvent.VK_R, true);
@@ -194,7 +203,7 @@ public class Menus {
 		addPluginsMenu();
 
 		Menu window = getMenu("Window");
-		addPlugInItem(window, "Show All", "ij.plugin.WindowOrganizer(\"show\")", KeyEvent.VK_F, true);
+		addPlugInItem(window, "Show All", "ij.plugin.WindowOrganizer(\"show\")", KeyEvent.VK_CLOSE_BRACKET, false);
 		addPlugInItem(window, "Put Behind [tab]", "ij.plugin.Commands(\"tab\")", 0, false);
 		addPlugInItem(window, "Cascade", "ij.plugin.WindowOrganizer(\"cascade\")", 0, false);
 		addPlugInItem(window, "Tile", "ij.plugin.WindowOrganizer(\"tile\")", 0, false);
@@ -205,8 +214,7 @@ public class Menus {
 		addPlugInItem(help, "ImageJ News...", "ij.plugin.BrowserLauncher(\""+IJ.URL+"/notes.html\")", 0, false);
 		addPlugInItem(help, "Documentation...", "ij.plugin.BrowserLauncher(\""+IJ.URL+"/docs\")", 0, false);
 		addPlugInItem(help, "Installation...", "ij.plugin.SimpleCommands(\"install\")", 0, false);
-		addPlugInItem(help, "Search Website...", "ij.plugin.BrowserLauncher(\""+IJ.URL+"/search.html\")", 0, false);
-		addPlugInItem(help, "List Archives...", "ij.plugin.BrowserLauncher(\"https://list.nih.gov/archives/imagej.html\")", 0, false);
+		addPlugInItem(help, "Mailing List...", "ij.plugin.BrowserLauncher(\"https://list.nih.gov/archives/imagej.html\")", 0, false);
 		help.addSeparator();
 		addPlugInItem(help, "Dev. Resources...", "ij.plugin.BrowserLauncher(\""+IJ.URL+"/developer/index.html\")", 0, false);
 		addPlugInItem(help, "Plugins...", "ij.plugin.BrowserLauncher(\""+IJ.URL+"/plugins\")", 0, false);
@@ -214,7 +222,7 @@ public class Menus {
 		addPlugInItem(help, "Macro Functions...", "ij.plugin.BrowserLauncher(\""+IJ.URL+"/developer/macro/functions.html\")", 0, false);
 		help.addSeparator();
 		addPlugInItem(help, "Update ImageJ...", "ij.plugin.ImageJ_Updater", 0, false);
-		addPlugInItem(help, "Update Menus", "ij.plugin.ImageJ_Updater(\"menus\")", 0, false);
+		addPlugInItem(help, "Refresh Menus", "ij.plugin.ImageJ_Updater(\"menus\")", 0, false);
 		help.addSeparator();
 		Menu aboutMenu = getMenu("Help>About Plugins", true);
 		addPlugInItem(help, "About ImageJ...", "ij.plugin.AboutBox", 0, false);
@@ -276,7 +284,7 @@ public class Menus {
 			menu.add(item);
 		item.addActionListener(ij);
 	}
-
+	
 	void addPlugInItem(Menu menu, String label, String className, int shortcut, boolean shift) {
 		pluginsTable.put(label, className);
 		nPlugins++;
@@ -318,8 +326,9 @@ public class Menus {
 	}
 	
 	static void addLuts(Menu submenu) {
-		String path = Prefs.getHomeDir()+File.separator;
-		File f = new File(path+"luts");
+		String path = IJ.getDirectory("luts");
+		if (path==null) return;
+		File f = new File(path);
 		String[] list = null;
 		if (applet==null && f.exists() && f.isDirectory())
 			list = f.list();
@@ -420,8 +429,8 @@ public class Menus {
 		String value, className;
 		char menuCode;
 		Menu menu;
-		String[] plugins = getPlugins();
-		String[] plugins2 = null;
+		String[] pluginList = getPlugins();
+		String[] pluginsList2 = null;
 		Hashtable skipList = new Hashtable();
  		for (int index=0; index<100; index++) {
 			value = Prefs.getString("plugin" + (index/10)%10 + index%10);
@@ -442,11 +451,11 @@ public class Menus {
 			value = value.substring(2,value.length()); //remove menu code and coma
 			className = value.substring(value.lastIndexOf(',')+1,value.length());
 			boolean found = className.startsWith("ij.");
-			if (!found && plugins!=null) { // does this plugin exist?
-				if (plugins2==null)
-					plugins2 = getStrippedPlugins(plugins);
-				for (int i=0; i<plugins2.length; i++) {
-					if (className.startsWith(plugins2[i])) {
+			if (!found && pluginList!=null) { // does this plugin exist?
+				if (pluginsList2==null)
+					pluginsList2 = getStrippedPlugins(pluginList);
+				for (int i=0; i<pluginsList2.length; i++) {
+					if (className.startsWith(pluginsList2[i])) {
 						found = true;
 						break;
 					}
@@ -463,10 +472,10 @@ public class Menus {
 				skipList.put(className, "");
 			}
 		}
-		if (plugins!=null) {
-			for (int i=0; i<plugins.length; i++) {
-				if (!skipList.containsKey(plugins[i]))
-					installUserPlugin(plugins[i]);
+		if (pluginList!=null) {
+			for (int i=0; i<pluginList.length; i++) {
+				if (!skipList.containsKey(pluginList[i]))
+					installUserPlugin(pluginList[i]);
 			}
 		}
 		installJarPlugins();
@@ -493,6 +502,14 @@ public class Menus {
 			dir = name.substring(0, slashIndex);
 			name = name.substring(slashIndex+1, name.length());
 			menu = getPluginsSubmenu(dir);
+			slashIndex = name.indexOf('/');
+			if (slashIndex>0) {
+				String dir2 = name.substring(0, slashIndex);
+				name = name.substring(slashIndex+1, name.length());
+				String menuName = "Plugins>"+dir+">"+dir2;
+				menu = getMenu(menuName);
+				dir += File.separator+dir2;
+			}
 		}
 		String command = name.replace('_',' ');
 		if (command.endsWith(".js"))
@@ -536,6 +553,12 @@ public class Menus {
 		menu.add(item);
 	}
 	
+	public static String getJarFileForMenuEntry(String menuEntry) {
+		if (instance == null)
+			return null;
+		return (String)instance.menuEntry2jarFile.get(menuEntry);
+	}
+
 	/** Install plugins located in JAR files. */
 	void installJarPlugins() {
 		if (jarFiles==null)
@@ -548,16 +571,14 @@ public class Menus {
             if (is==null) continue;
             int maxEntries = 100;
             String[] entries = new String[maxEntries];
-            int nEntries=0, nEntries2=0;
+            int nEntries=0;
             LineNumberReader lnr = new LineNumberReader(new InputStreamReader(is));
             try {
                 while(true) {
                     String s = lnr.readLine();
                     if (s==null || nEntries==maxEntries-1) break;
-					if (s.length()>=3 && !s.startsWith("#")) {
+					if (s.length()>=3 && !s.startsWith("#"))
 						entries[nEntries++] = s;
-						if (s.startsWith("Plugins>")) nEntries2++;
-					}
 	            }
             }
             catch (IOException e) {}
@@ -565,27 +586,8 @@ public class Menus {
 				try {if (lnr!=null) lnr.close();}
 				catch (IOException e) {}
 			}
-			for (int j=0; j<nEntries; j++) {
-				String s = entries[j];
-				//IJ.log(j+"  "+s);
-				if (nEntries2<=3) {
-					if (s.startsWith("Plugins>")) {
-						int firstComma = s.indexOf(',');
-						int firstQuote = s.indexOf('"');
-						boolean pluginsDir = (new File(jar)).getParent().endsWith("plugins");
-						if (firstComma>8 && firstQuote>firstComma && !pluginsDir) {
-							String submenuName = s.substring(8, firstComma);
-							String prefix = "";
-							// "Extended Depth of Field" and "Particle Detector & Tracker" plugins
-							if (submenuName.startsWith("Extend")||submenuName.startsWith("Particle D"))
-								prefix = submenuName+": ";
-							//IJ.log(nEntries+" "+nEntries2+" "+jar+" "+s+"  "+submenuName);
-							s = "Plugins, \""+prefix+s.substring(firstQuote+1, s.length());
-						}
-					}
-				}
-				installJarPlugin(jar, s);
-			}
+			for (int j=0; j<nEntries; j++)
+				installJarPlugin(jar, entries[j]);
 		}		
 	}
     
@@ -628,11 +630,26 @@ public class Menus {
             addPluginItem(menu, s);
             addSorted = false;
         }
+		String menuEntry = s;
+		if (s.startsWith("\"")) {
+			int quote = s.indexOf('"', 1);
+			menuEntry = quote<0?s.substring(1):s.substring(1, quote);
+		} else {
+			int comma = s.indexOf(',');
+			if (comma > 0)
+				menuEntry = s.substring(0, comma);
+		}
 		if (duplicateCommand) {
 			if (jarError==null) jarError = "";
             addJarErrorHeading(jar);
-			jarError += "    Duplicate command: " + s + "\n";
-		}
+			String jar2 = (String)menuEntry2jarFile.get(menuEntry);
+			if (jar2 != null && jar2.startsWith(pluginsPath))
+				jar2 = jar2.substring(pluginsPath.length());
+			jarError += "    Duplicate command: " + s
+				+ (jar2 != null ? " (already in " + jar2 + ")"
+				   : "") + "\n";
+		} else
+			menuEntry2jarFile.put(menuEntry, jar);
 		duplicateCommand = false;
     }
     
@@ -667,18 +684,14 @@ public class Menus {
 					window = result;
 				else if (menuName.equals("Plugins"))
 					pluginsMenu = result;
-			}
-			else {
-				String parentName =
-					menuName.substring(0, offset);
-				String menuItemName =
-					menuName.substring(offset + 1);
+			} else {
+				String parentName = menuName.substring(0, offset);
+				String menuItemName = menuName.substring(offset + 1);
 				Menu parentMenu = getMenu(parentName);
 				result = new Menu(menuItemName);
 				addPluginSeparatorIfNeeded(parentMenu);
 				if (readFromProps)
-					result = addSubMenu(parentMenu,
-							menuItemName);
+					result = addSubMenu(parentMenu, menuItemName);
 				else if (parentName.startsWith("Plugins") && menuSeparators != null)
 					addItemSorted(parentMenu, result, parentName.equals("Plugins")?userPluginsIndex:0);
 				else
@@ -732,7 +745,7 @@ public class Menus {
     	menu.addSeparator();
     }
 
-    /** Opens the configuration file ("plugins.txt") from a JAR file and returns it as an InputStream. */
+    /** Opens the configuration file ("plugins.config") from a JAR file and returns it as an InputStream. */
 	InputStream getConfigurationFile(String jar) {
 		try {
 			ZipFile jarFile = new ZipFile(jar);
@@ -743,7 +756,9 @@ public class Menus {
 					return jarFile.getInputStream(entry);
 			}
 		}
-    	catch (Exception e) {}
+    	catch (Throwable e) {
+    		IJ.log(jar+": "+e);
+    	}
 		return autoGenerateConfigFile(jar);
 	}
 	
@@ -758,6 +773,8 @@ public class Menus {
 				String name = entry.getName();
 				if (name.endsWith(".class") && name.indexOf("_")>0 && name.indexOf("$")==-1
 				&& name.indexOf("/_")==-1 && !name.startsWith("_")) {
+					if (Character.isLowerCase(name.charAt(0))&&name.indexOf("/")!=-1)
+						continue;
 					if (sb==null) sb = new StringBuffer();
 					String className = name.substring(0, name.length()-6);
 					int slashIndex = className.lastIndexOf('/');
@@ -769,12 +786,14 @@ public class Menus {
 						name = className;
 					name = name.replace('_', ' ');
 					className = className.replace('/', '.');
-					if (className.indexOf(".")==-1 || Character.isUpperCase(className.charAt(0)))
-						sb.append(plugins + ", \""+name+"\", "+className+"\n");
+					//if (className.indexOf(".")==-1 || Character.isUpperCase(className.charAt(0)))
+					sb.append(plugins + ", \""+name+"\", "+className+"\n");
 				}
 			}
 		}
-    	catch (Exception e) {}
+    	catch (Throwable e) {
+    		IJ.log(jar+": "+e);
+    	}
 		//IJ.log(""+(sb!=null?sb.toString():"null"));
 		if (sb==null)
 			return null;
@@ -794,12 +813,11 @@ public class Menus {
 		}
 		return plugins2;
 	}
-		
-	/** Returns a list of the plugins in the plugins menu. */
-	public static synchronized String[] getPlugins() {
+	
+	void setupPluginsAndMacrosPaths() {
+		pluginsPath = macrosPath = null;
 		String homeDir = Prefs.getHomeDir();
-		if (homeDir==null)
-			return null;
+		if (homeDir==null) return;
 		if (homeDir.endsWith("plugins"))
 			pluginsPath = homeDir+Prefs.separator;
 		else {
@@ -828,10 +846,16 @@ public class Menus {
 			macrosPath = null;
 		f = pluginsPath!=null?new File(pluginsPath):null;
 		if (f==null || (f!=null && !f.isDirectory())) {
-			//error = "Plugins folder not found at "+pluginsPath;
 			pluginsPath = null;
-			return null;
+			return;
 		}
+	}
+		
+	/** Returns a list of the plugins in the plugins menu. */
+	public static synchronized String[] getPlugins() {
+		File f = pluginsPath!=null?new File(pluginsPath):null;
+		if (f==null || (f!=null && !f.isDirectory()))
+			return null;
 		String[] list = f.list();
 		if (list==null)
 			return null;
@@ -892,6 +916,9 @@ public class Menus {
 				if (macroFiles==null) macroFiles = new Vector();
 				macroFiles.addElement(dir + name);
 				otherCount++;
+			} else {
+				File f2 = new File(f, name);
+				if (f2.isDirectory()) installSubdirectorMacros(f2, dir+name);
 			}
 		}
 		if (Prefs.moveToMisc && classCount==1 && otherCount==0 && dir.indexOf("_")==-1)
@@ -899,6 +926,21 @@ public class Menus {
 				v.size() - 1);
 	}
 	
+	/** Installs macros and scripts located in subdirectories. */
+	private static void installSubdirectorMacros(File f2, String dir) {
+		if (dir.endsWith("Launchers")) return;
+		String[] list = f2.list();
+		if (list==null) return;
+		for (int i=0; i<list.length; i++) {
+			String name = list[i];
+			boolean hasUnderscore = name.indexOf('_')>=0;
+			if ((hasUnderscore&&name.endsWith(".txt")) || name.endsWith(".ijm") || name.endsWith(".js")) {
+				if (macroFiles==null) macroFiles = new Vector();
+				macroFiles.addElement(dir+"/"+name);
+			}
+		}
+	}
+
 	/** Installs a plugin in the Plugins menu using the class name,
 		with underscores replaced by spaces, as the command. */
 	void installUserPlugin(String className) {
@@ -992,14 +1034,13 @@ public class Menus {
     		if (stack.isRGB()) type = RGB_STACK;
     		else if (stack.isHSB()) type = HSB_STACK;
     	}
-    	if (type==ImagePlus.GRAY8) {
+		if (type==ImagePlus.GRAY8) {
 			ImageProcessor ip = imp.getProcessor();
-    		if (ip!=null && ip.getMinThreshold()==ImageProcessor.NO_THRESHOLD && ip.isColorLut()) {
-    			type = ImagePlus.COLOR_256;
-	    		if (!ip.isPseudoColorLut())
-	    			imp.setType(ImagePlus.COLOR_256);
-    		}
-    	}
+			if (ip!=null && ip.getMinThreshold()==ImageProcessor.NO_THRESHOLD && ip.isColorLut() && !ip.isPseudoColorLut()) {
+				type = ImagePlus.COLOR_256;
+				imp.setType(ImagePlus.COLOR_256);
+			}
+		}
     	switch (type) {
     		case ImagePlus.GRAY8:
 				gray8Item.setState(true);
@@ -1033,7 +1074,7 @@ public class Menus {
 				CheckboxMenuItem item = (CheckboxMenuItem)window.getItem(i);
 				item.setState(i==index);
 			}
-		} catch (NullPointerException e) {}
+		} catch (Exception e) {}
 	}
 	
 	static boolean isColorLut(ImagePlus imp) {
@@ -1148,7 +1189,7 @@ public class Menus {
 
 	/** Changes the name of an item in the Window menu. */
 	public static synchronized void updateWindowMenuItem(String oldLabel, String newLabel) {
-		if (oldLabel.equals(newLabel))
+		if (oldLabel==null || oldLabel.equals(newLabel))
 			return;
 		int first = WINDOW_MENU_ITEMS;
 		int last = window.getItemCount()-1;
@@ -1326,14 +1367,14 @@ public class Menus {
 			code = KeyEvent.VK_A+c-97;
 		else if (c>=48&&c<=57) //0-9
 			code = KeyEvent.VK_0+c-48;
-		else {
-			switch (c) {
-				case 43: code = KeyEvent.VK_PLUS; break;
-				case 45: code = KeyEvent.VK_MINUS; break;
-				//case 92: code = KeyEvent.VK_BACK_SLASH; break;
-				default: return 0;
-			}
-		}
+		//else {
+		//	switch (c) {
+		//		case 43: code = KeyEvent.VK_PLUS; break;
+		//		case 45: code = KeyEvent.VK_MINUS; break;
+		//		//case 92: code = KeyEvent.VK_BACK_SLASH; break;
+		//		default: return 0;
+		//	}
+		//}
 		return code;
 	}
 	
@@ -1439,6 +1480,6 @@ public class Menus {
 		if (err!=null) IJ.error(err);
 		IJ.setClassLoader(null);
 		IJ.runPlugIn("ij.plugin.ClassChecker", "");
-		IJ.showStatus(m.nPlugins + " commands, " + m.nMacros + " macros");
+		IJ.showStatus("Menus updated: "+m.nPlugins + " commands, " + m.nMacros + " macros");
 	}
 }
