@@ -44,9 +44,16 @@ public class Prefs {
 		AUTO_MEASURE=1<<9, REQUIRE_CONTROL=1<<10, USE_INVERTING_LUT=1<<11, ANTIALIASED_TOOLS=1<<12,
 		INTEL_BYTE_ORDER=1<<13, DOUBLE_BUFFER=1<<14, NO_POINT_LABELS=1<<15, NO_BORDER=1<<16,
 		SHOW_ALL_SLICE_ONLY=1<<17, COPY_HEADERS=1<<18, NO_ROW_NUMBERS=1<<19,
-		MOVE_TO_MISC=1<<20, ADD_TO_MANAGER=1<<21, RUN_SOCKET_LISTENER=1<<22; 
+		MOVE_TO_MISC=1<<20, ADD_TO_MANAGER=1<<21, RUN_SOCKET_LISTENER=1<<22,
+		MULTI_POINT_MODE=1<<23, ROTATE_YZ=1<<24, FLIP_XZ=1<<25,
+		DONT_SAVE_HEADERS=1<<26, DONT_SAVE_ROW_NUMBERS=1<<27, NO_CLICK_TO_GC=1<<28,
+		AVOID_RESLICE_INTERPOLATION=1<<29, KEEP_UNDO_BUFFERS=1<<30; 
     public static final String OPTIONS = "prefs.options";
     
+	private static final int USE_SYSTEM_PROXIES=1<<0;
+	private static final int USE_FILE_CHOOSER=1<<1;
+	public static final String OPTIONS2 = "prefs.options2";
+    
 	public static final String vistaHint = "\n \nOn Windows Vista, ImageJ must be installed in a directory that\nthe user can write to, such as \"Desktop\" or \"Documents\"";
 
 	/** file.separator system property */
@@ -76,13 +83,13 @@ public class Prefs {
 	/** Open 8-bit images with inverting LUT so 0 is white and 255 is black. */
 	public static boolean useInvertingLut;
 	/** Draw tool icons using antialiasing. */
-	public static boolean antialiasedTools;
-	/** Export Raw using little-endian byte order. */
+	public static boolean antialiasedTools = true;
+	/** Export TIFF and Raw using little-endian byte order. */
 	public static boolean intelByteOrder;
-	/** Double buffer display of selections. */
-	public static boolean doubleBuffer;
+	/** Double buffer display of selections and overlays. */
+	public static boolean doubleBuffer = true;
 	/** Do not label multiple points created using point tool. */
-	public static boolean noPointLabels = true;
+	public static boolean noPointLabels;
 	/** Disable Edit/Undo command. */
 	public static boolean disableUndo;
 	/** Do not draw black border around image. */
@@ -101,6 +108,35 @@ public class Prefs {
 	public static boolean padEdges;
 	/** Run the SocketListener. */
 	public static boolean runSocketListener;
+	/** Use MultiPoint tool. */
+	public static boolean multiPointMode;
+	/** Open DICOMs as 32-bit float images */
+	public static boolean openDicomsAsFloat;
+	/** Plot rectangular selectons vertically */
+	public static boolean verticalProfile;
+	/** Rotate YZ orthogonal views 90 degrees */
+	public static boolean rotateYZ;
+	/** Rotate XZ orthogonal views 180 degrees */
+	public static boolean flipXZ;
+	/** Don't save Results table column headers */
+	public static boolean dontSaveHeaders;
+	/** Don't save Results table row numbers */
+	public static boolean dontSaveRowNumbers;
+	/** Don't run garbage collector when user clicks in status bar */
+	public static boolean noClickToGC;
+	/** Angle tool measures reflex angle */
+	public static boolean reflexAngle;
+	/** Avoid interpolation when re-slicing */
+	public static boolean avoidResliceInterpolation;
+	/** Preserve undo (snapshot) buffers when switching images */
+	public static boolean keepUndoBuffers;
+	/** Use ROI names as "show all" labels in the ROI Manager */
+	public static boolean useNamesAsLabels;
+	/** Set the "java.net.useSystemProxies" property */
+	public static boolean useSystemProxies;
+	/** Use the file chooser to import and export image sequences on Windows and Linux*/
+	public static boolean useFileChooser;
+
 
 	static Properties ijPrefs = new Properties();
 	static Properties props = new Properties(ijPrefs);
@@ -109,6 +145,7 @@ public class Prefs {
 	static String homeDir; // ImageJ folder
 	static int threads;
 	static int transparentIndex = -1;
+	static boolean commandLineMacro;
 
 	/** Finds and loads the ImageJ configuration file, "IJ_Props.txt".
 		@return	an error message if "IJ_Props.txt" not found.
@@ -202,6 +239,14 @@ public class Prefs {
 		homeDir = path;
 	}
 
+	/** Returns the default directory, if any, or null. */
+	public static String getDefaultDirectory() {
+		if (commandLineMacro)
+			return null;
+		else
+			return getString(DIR_IMAGE);
+	}
+
 	/** Finds an string in IJ_Props or IJ_Prefs.txt. */
 	public static String getString(String key) {
 		return props.getProperty(key);
@@ -317,7 +362,6 @@ public class Prefs {
 			Analyzer.savePreferences(prefs);
 			ImportDialog.savePreferences(prefs);
 			PlotWindow.savePreferences(prefs);
-			GelAnalyzer.savePreferences(prefs);
 			NewImage.savePreferences(prefs);
 			String path = prefsDir+separator+PREFS_NAME;
 			if (prefsDir.endsWith(".imagej")) {
@@ -341,7 +385,9 @@ public class Prefs {
 	}
 
 	static void loadOptions() {
-		int options = getInt(OPTIONS, ANTIALIASING);
+		int defaultOptions = ANTIALIASING+AVOID_RESLICE_INTERPOLATION
+			+(!IJ.isMacOSX()?RUN_SOCKET_LISTENER:0);
+		int options = getInt(OPTIONS, defaultOptions);
 		usePointerCursor = (options&USE_POINTER)!=0;
 		//antialiasedText = (options&ANTIALIASING)!=0;
 		antialiasedText = false;
@@ -359,8 +405,8 @@ public class Prefs {
 		useInvertingLut = (options&USE_INVERTING_LUT)!=0;
 		antialiasedTools = (options&ANTIALIASED_TOOLS)!=0;
 		intelByteOrder = (options&INTEL_BYTE_ORDER)!=0;
-		doubleBuffer = (options&DOUBLE_BUFFER)!=0;
-		noPointLabels = (options&NO_POINT_LABELS)!=0;
+		// doubleBuffer = (options&DOUBLE_BUFFER)!=0; // always double buffer
+		//noPointLabels = (options&NO_POINT_LABELS)!=0;
 		noBorder = (options&NO_BORDER)!=0;
 		showAllSliceOnly = (options&SHOW_ALL_SLICE_ONLY)!=0;
 		copyColumnHeaders = (options&COPY_HEADERS)!=0;
@@ -368,6 +414,19 @@ public class Prefs {
 		moveToMisc = (options&MOVE_TO_MISC)!=0;
 		pointAddToManager = (options&ADD_TO_MANAGER)!=0;
 		runSocketListener = (options&RUN_SOCKET_LISTENER)!=0;
+		multiPointMode = (options&MULTI_POINT_MODE)!=0;
+		rotateYZ = (options&ROTATE_YZ)!=0;
+		flipXZ = (options&FLIP_XZ)!=0;
+		dontSaveHeaders = (options&DONT_SAVE_HEADERS)!=0;
+		dontSaveRowNumbers = (options&DONT_SAVE_ROW_NUMBERS)!=0;
+		noClickToGC = (options&NO_CLICK_TO_GC)!=0;
+		avoidResliceInterpolation = (options&AVOID_RESLICE_INTERPOLATION)!=0;
+		keepUndoBuffers = (options&KEEP_UNDO_BUFFERS)!=0;
+		
+		defaultOptions = (!IJ.isMacOSX()?USE_FILE_CHOOSER:0);
+		int options2 = getInt(OPTIONS2, defaultOptions);
+		useSystemProxies = (options2&USE_SYSTEM_PROXIES)!=0;
+		useFileChooser = (options2&USE_FILE_CHOOSER)!=0;
 	}
 
 	static void saveOptions(Properties prefs) {
@@ -381,8 +440,17 @@ public class Prefs {
 			+ (noPointLabels?NO_POINT_LABELS:0) + (noBorder?NO_BORDER:0)
 			+ (showAllSliceOnly?SHOW_ALL_SLICE_ONLY:0) + (copyColumnHeaders?COPY_HEADERS:0)
 			+ (noRowNumbers?NO_ROW_NUMBERS:0) + (moveToMisc?MOVE_TO_MISC:0)
-			+ (pointAddToManager?ADD_TO_MANAGER:0) + (runSocketListener?RUN_SOCKET_LISTENER:0);
+			+ (pointAddToManager?ADD_TO_MANAGER:0) + (runSocketListener?RUN_SOCKET_LISTENER:0)
+			+ (multiPointMode?MULTI_POINT_MODE:0) + (rotateYZ?ROTATE_YZ:0)
+			+ (flipXZ?FLIP_XZ:0) + (dontSaveHeaders?DONT_SAVE_HEADERS:0)
+			+ (dontSaveRowNumbers?DONT_SAVE_ROW_NUMBERS:0) + (noClickToGC?NO_CLICK_TO_GC:0)
+			+ (avoidResliceInterpolation?AVOID_RESLICE_INTERPOLATION:0)
+			+ (keepUndoBuffers?KEEP_UNDO_BUFFERS:0);
 		prefs.put(OPTIONS, Integer.toString(options));
+
+		int options2 = (useSystemProxies?USE_SYSTEM_PROXIES:0)
+			+ (useFileChooser?USE_FILE_CHOOSER:0);
+		prefs.put(OPTIONS2, Integer.toString(options2));
 	}
 
 	/** Saves the value of the string <code>text</code> in the preferences
@@ -522,8 +590,10 @@ public class Prefs {
 	public static int getTransparentIndex() {
 		return transparentIndex;
 	}
-
-
-
+	
+	public static Properties getControlPanelProperties() {
+		return ijPrefs;
+	}
+	
 }
 
