@@ -8,7 +8,7 @@ import ij.plugin.ScreenGrabber;
 import ij.plugin.filter.PlugInFilter;
 import ij.plugin.filter.PlugInFilterRunner;
 import ij.util.Tools;
-import ij.macro.MacroRunner;
+import ij.macro.*;
 
 
 /**
@@ -47,6 +47,7 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 	protected Component theLabel;
 	private Button cancel, okay, no, help;
 	private String okLabel = "  OK  ";
+	private String helpLabel = "Help";
     private boolean wasCanceled, wasOKed;
     private int y;
     private int nfIndex, sfIndex, cbIndex, choiceIndex, textAreaIndex;
@@ -63,6 +64,7 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 	private int topInset, leftInset, bottomInset;
     private boolean customInsets;
     private int[] sliderIndexes;
+    private double[] sliderScales;
     private Checkbox previewCheckbox;    // the "Preview" Checkbox, if any
     private Vector dialogListeners;             // the Objects to notify on user input
     private PlugInFilterRunner pfr;      // the PlugInFilterRunner for automatic preview
@@ -74,6 +76,7 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
     private boolean hideCancelButton;
     private boolean centerDialog = true;
     private String helpURL;
+    private String yesLabel, noLabel;
 
     /** Creates a new GenericDialog with the specified title. Uses the current image
     	image window as the parent frame or the ImageJ frame if no image windows
@@ -343,29 +346,65 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 	* @param defaultValues	the initial states
 	*/
     public void addCheckboxGroup(int rows, int columns, String[] labels, boolean[] defaultValues) {
+    	addCheckboxGroup(rows, columns, labels, defaultValues, null);
+    }
+
+    /** Adds a group of checkboxs using a grid layout.
+	* @param rows			the number of rows
+	* @param columns		the number of columns
+	* @param labels			the labels
+	* @param defaultValues	the initial states
+	* @param headings	the column headings
+	* Example: http://imagej.nih.gov/ij/plugins/multi-column-dialog/index.html
+	*/
+    public void addCheckboxGroup(int rows, int columns, String[] labels, boolean[] defaultValues, String[] headings) {
     	Panel panel = new Panel();
-    	panel.setLayout(new GridLayout(rows,columns, 5, 0));
+    	int nRows = headings!=null?rows+1:rows;
+    	panel.setLayout(new GridLayout(nRows, columns, 6, 0));
     	int startCBIndex = cbIndex;
-    	int i1 = 0;
-    	int[] index = new int[labels.length];
     	if (checkbox==null)
     		checkbox = new Vector(12);
-    	boolean addListeners = labels.length<=4;
+    	if (headings!=null) {
+    		Font font = new Font("SansSerif", Font.BOLD, 12);
+			for (int i=0; i<columns; i++) {
+				if (i>headings.length-1 || headings[i]==null)
+					panel.add(new Label(""));
+				else {
+					Label label = new Label(headings[i]);
+					label.setFont(font);
+					panel.add(label);
+				}
+			}
+    	}
+    	int i1 = 0;
+    	int[] index = new int[labels.length];
     	for (int row=0; row<rows; row++) {
 			for (int col=0; col<columns; col++) {
 				int i2 = col*rows+row;
 				if (i2>=labels.length) break;
 				index[i1] = i2;
 				String label = labels[i1];
+				if (label==null || label.length()==0) {
+					Label lbl = new Label("");
+					panel.add(lbl);
+					i1++;
+					continue;
+				}
 				if (label.indexOf('_')!=-1)
    					label = label.replace('_', ' ');
 				Checkbox cb = new Checkbox(label);
 				checkbox.addElement(cb);
 				cb.setState(defaultValues[i1]);
-				if (addListeners) cb.addItemListener(this);
+				cb.addItemListener(this);
 				if (Recorder.record || macro)
 					saveLabel(cb, labels[i1]);
-				panel.add(cb);
+				if (IJ.isLinux()) {
+					Panel panel2 = new Panel();
+					panel2.setLayout(new BorderLayout());
+					panel2.add("West", cb);
+					panel.add(panel2);
+				} else
+					panel.add(cb);
  				i1++;
 			}
 		}
@@ -416,6 +455,13 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
     
     /** Adds a message consisting of one or more lines of text. */
     public void addMessage(String text) {
+    	addMessage(text, null);
+    }
+
+    /** Adds a message consisting of one or more lines of text,
+    	which will be displayed using the specified font. */
+    public void addMessage(String text, Font font) {
+    	theLabel = null;
     	if (text.indexOf('\n')>=0)
 			theLabel = new MultiLineLabel(text);
 		else
@@ -425,8 +471,12 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 		c.gridwidth = 2;
 		c.anchor = GridBagConstraints.WEST;
 		c.insets = getInsets(text.equals("")?0:10, 20, 0, 0);
+		c.fill = GridBagConstraints.HORIZONTAL;
 		grid.setConstraints(theLabel, c);
+		if (font!=null)
+			theLabel.setFont(font);
 		add(theLabel);
+		c.fill = GridBagConstraints.NONE;
 		y++;
     }
     
@@ -451,15 +501,32 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 		c.gridx = 0; c.gridy = y;
 		c.gridwidth = 2;
 		c.anchor = GridBagConstraints.WEST;
-		c.insets = new Insets(15, 20, 0, 0);
+		c.insets = getInsets(15, 20, 0, 0);
 		grid.setConstraints(panel, c);
 		add(panel);
 		y++;
     }
     
-   public void addSlider(String label, double minValue, double maxValue, double defaultValue) {
+	/**
+	* Adds a slider (scroll bar) to the dialog box.
+	* Floating point values will be used if (maxValue-minValue)<=5.0
+	* and either minValue or maxValue are non-integer.
+	* @param label	 the label
+	* @param minValue  the minimum value of the slider
+	* @param maxValue  the maximum value of the slider
+	* @param defaultValue  the initial value of the slider
+	*/
+	public void addSlider(String label, double minValue, double maxValue, double defaultValue) {
 		int columns = 4;
 		int digits = 0;
+		double scale = 1.0;
+		if ((maxValue-minValue)<=5.0 && (minValue!=(int)minValue||maxValue!=(int)maxValue||defaultValue!=(int)defaultValue)) {
+			scale = 20.0;
+			minValue *= scale;
+			maxValue *= scale;
+			defaultValue *= scale;
+			digits = 2;
+		}
    		String label2 = label;
    		if (label2.indexOf('_')!=-1)
    			label2 = label2.replace('_', ' ');
@@ -474,6 +541,7 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 		if (slider==null) {
 			slider = new Vector(5);
 			sliderIndexes = new int[MAX_SLIDERS];
+			sliderScales = new double[MAX_SLIDERS];
 		}
 		Scrollbar s = new Scrollbar(Scrollbar.HORIZONTAL, (int)defaultValue, 1, (int)minValue, (int)maxValue+1);
 		slider.addElement(s);
@@ -487,7 +555,7 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 		}
 		if (IJ.isWindows()) columns -= 2;
 		if (columns<1) columns = 1;
-		TextField tf = new TextField(IJ.d2s(defaultValue, digits), columns);
+		TextField tf = new TextField(IJ.d2s(defaultValue/scale, digits), columns);
 		if (IJ.isLinux()) tf.setBackground(Color.white);
 		tf.addActionListener(this);
 		tf.addTextListener(this);
@@ -495,7 +563,8 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 		tf.addKeyListener(this);
 		numberField.addElement(tf);
 		sliderIndexes[slider.size()-1] = numberField.size()-1;
-		defaultValues.addElement(new Double(defaultValue));
+		sliderScales[slider.size()-1] = scale;
+		defaultValues.addElement(new Double(defaultValue/scale));
 		defaultText.addElement(tf.getText());
 		tf.setEditable(true);
 		//if (firstNumericField && firstSlider) tf.selectAll();
@@ -581,11 +650,36 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
     	okLabel = label;
     }
 
+    /** Sets a replacement label for the "Help" button. */
+    public void setHelpLabel(String label) {
+    	helpLabel = label;
+    }
+
     /** Make this a "Yes No Cancel" dialog. */
     public void enableYesNoCancel() {
-    	yesNoCancel = true;
+    	enableYesNoCancel(" Yes ", " No ");
     }
     
+    /** Make this a "Yes No Cancel" dialog with custom labels. Here is an example:
+    	<pre>
+        GenericDialog gd = new GenericDialog("YesNoCancel Demo");
+        gd.addMessage("This is a custom YesNoCancel dialog");
+        gd.enableYesNoCancel("Do something", "Do something else");
+        gd.showDialog();
+        if (gd.wasCanceled())
+            IJ.log("User clicked 'Cancel'");
+        else if (gd.wasOKed())
+            IJ. log("User clicked 'Yes'");
+        else
+            IJ. log("User clicked 'No'");
+    	</pre>
+	*/
+    public void enableYesNoCancel(String yesLabel, String noLabel) {
+    	this.yesLabel = yesLabel;
+    	this.noLabel = noLabel;
+    	yesNoCancel = true;
+    }
+
     /** No not display "Cancel" button. */
     public void hideCancelButton() {
     	hideCancelButton = true;
@@ -627,7 +721,8 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
     	return wasOKed || macro;
     }
 
-	/** Returns the contents of the next numeric field. */
+	/** Returns the contents of the next numeric field,
+		or NaN if the field does not contain a number. */
    public double getNextNumber() {
 		if (numberField==null)
 			return -1.0;
@@ -649,14 +744,20 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 			if (d!=null)
 				value = d.doubleValue();
 			else {
-				invalidNumber = true;
-				errorMessage = "\""+theText+"\" is an invalid number";
-				value = 0.0;
-                if (macro) {
-                    IJ.error("Macro Error", "Numeric value expected in run() function\n \n"
-                        +"   Dialog: \""+getTitle()+"\"\n"
-                        +"   Label: \""+label+"\"\n"
-                        +"   Value: \""+theText+"\"");
+				// Is the value a macro variable?
+				if (theText.startsWith("&")) theText = theText.substring(1);
+				Interpreter interp = Interpreter.getInstance();
+				value = interp!=null?interp.getVariable2(theText):Double.NaN;
+				if (Double.isNaN(value)) {
+					invalidNumber = true;
+					errorMessage = "\""+theText+"\" is an invalid number";
+					value = Double.NaN;
+					if (macro) {
+						IJ.error("Macro Error", "Numeric value expected in run() function\n \n"
+							+"   Dialog box title: \""+getTitle()+"\"\n"
+							+"   Key: \""+label.toLowerCase(Locale.US)+"\"\n"
+							+"   Value or variable name: \""+theText+"\"");
+					}
                 }
 			}
 		}
@@ -699,6 +800,17 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 		return d;
 	}
 
+	public double parseDouble(String s) {
+		if (s==null) return Double.NaN;
+		double value = Tools.parseDouble(s);
+		if (Double.isNaN(value)) {
+			if (s.startsWith("&")) s = s.substring(1);
+			Interpreter interp = Interpreter.getInstance();
+			value = interp!=null?interp.getVariable2(s):Double.NaN;
+		}
+		return value;
+	}
+	
 	/** Returns true if one or more of the numeric fields contained an  
 		invalid number. Must be called after one or more calls to getNextNumber(). */
    public boolean invalidNumber() {
@@ -723,7 +835,13 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 		if (macro) {
 			String label = (String)labels.get((Object)tf);
 			theText = Macro.getValue(macroOptions, label, theText);
-			//IJ.write("getNextString: "+label+"  "+theText);
+			if (theText!=null && (theText.startsWith("&")||label.toLowerCase(Locale.US).startsWith(theText))) {
+				// Is the value a macro variable?
+				if (theText.startsWith("&")) theText = theText.substring(1);
+				Interpreter interp = Interpreter.getInstance();
+				String s = interp!=null?interp.getVariableAsString(theText):null;
+				if (s!=null) theText = s;
+			}
 		}	
 		if (recorderOn)
 			recordOption(tf, theText);
@@ -784,7 +902,8 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 		if (macro) {
 			String label = (String)labels.get((Object)thisChoice);
 			item = Macro.getValue(macroOptions, label, item);
-			//IJ.write("getNextChoice: "+label+"  "+item);
+			if (item!=null && item.startsWith("&")) // value is macro variable
+				item = getChoiceVariable(item);
 		}	
 		if (recorderOn)
 			recordOption(thisChoice, item);
@@ -803,10 +922,19 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 			String oldItem = thisChoice.getSelectedItem();
 			int oldIndex = thisChoice.getSelectedIndex();
 			String item = Macro.getValue(macroOptions, label, oldItem);
+			if (item!=null && item.startsWith("&")) // value is macro variable
+				item = getChoiceVariable(item);
 			thisChoice.select(item);
 			index = thisChoice.getSelectedIndex();
-			if (index==oldIndex && !item.equals(oldItem))
-				IJ.error(getTitle(), "\""+item+"\" is not a valid choice for \""+label+"\"");
+			if (index==oldIndex && !item.equals(oldItem)) {
+				// is value a macro variable?
+				Interpreter interp = Interpreter.getInstance();
+				String s = interp!=null?interp.getStringVariable(item):null;
+				if (s==null)
+					IJ.error(getTitle(), "\""+item+"\" is not a valid choice for \""+label+"\"");
+				else
+					item = s;
+			}
 		}	
 		if (recorderOn)
 			recordOption(thisChoice, thisChoice.getSelectedItem());
@@ -814,6 +942,14 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 		return index;
     }
     
+    private String getChoiceVariable(String item) {
+		item = item.substring(1);
+		Interpreter interp = Interpreter.getInstance();
+		String s = interp!=null?interp.getStringVariable(item):null;
+		if (s!=null) item = s;
+		return item;
+	}
+    
   	/** Returns the contents of the next textarea. */
 	public String getNextText() {
 		String text;
@@ -854,18 +990,14 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 		} else {
 			if (pfr!=null) // prepare preview (not in macro mode): tell the PlugInFilterRunner to listen
 			pfr.setDialog(this);
-			//if (stringField!=null&&numberField==null) {
-			//	TextField tf = (TextField)(stringField.elementAt(0));
-			//	tf.selectAll();
-			//}
 			Panel buttons = new Panel();
 			buttons.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 0));
 			cancel = new Button("Cancel");
 			cancel.addActionListener(this);
 			cancel.addKeyListener(this);
 			if (yesNoCancel) {
-				okLabel = " Yes ";
-				no = new Button(" No ");
+				okLabel = yesLabel;
+				no = new Button(noLabel);
 				no.addActionListener(this);
 				no.addKeyListener(this);
 			}
@@ -874,7 +1006,7 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 			okay.addKeyListener(this);
 			boolean addHelp = helpURL!=null;
 			if (addHelp) {
-				help = new Button("Help");
+				help = new Button(helpLabel);
 				help.addActionListener(this);
 				help.addKeyListener(this);
 			}
@@ -913,7 +1045,7 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 		}
 		resetCounters();
 	}
-
+	
     /** Reset the counters before reading the dialog parameters */
     private void resetCounters() {
         nfIndex = 0;        // prepare for readout
@@ -944,7 +1076,7 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
   		return choice;
   	}
 
-  	/** Returns the sliders (Scrollbars). */
+  	/** Returns the Vector containing the sliders (Scrollbars). */
   	public Vector getSliders() {
   		return slider;
   	}
@@ -970,6 +1102,16 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
         return previewCheckbox;
     }
     
+	/** Returns references to the "OK" ("Yes"), "Cancel", 
+		and if present, "No" buttons as an array. */
+	public Button[] getButtons() {
+  		Button[] buttons = new Button[3];
+  		buttons[0] = okay;
+  		buttons[1] = cancel;
+  		buttons[2] = no;
+		return buttons;
+  	}
+
     /** Used by PlugInFilterRunner to provide visable feedback whether preview
     	is running or not by switching from "Preview" to "wait..."
      */
@@ -994,9 +1136,13 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 			wasCanceled = source==cancel;
 			wasOKed = source==okay;
 			dispose();
-		} else if (source==help)
+		} else if (source==help) {
+			if (hideCancelButton) {
+				wasOKed = true;
+				dispose();
+			}
 			showHelp();
-		else
+		} else
             notifyListeners(e);
 	}
 	
@@ -1011,7 +1157,7 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 				double value = Tools.parseDouble(tf.getText());
 				if (!Double.isNaN(value)) {
 					Scrollbar sb = (Scrollbar)slider.elementAt(i);
-					sb.setValue((int)value);
+					sb.setValue((int)(value*sliderScales[i]));
 				}	
 				//IJ.log(i+" "+tf.getText());
 			}
@@ -1046,8 +1192,12 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 			wasCanceled = true; 
 			dispose(); 
 			IJ.resetEscape();
+		} else if (keyCode==KeyEvent.VK_W && (e.getModifiers()&Toolkit.getDefaultToolkit().getMenuShortcutKeyMask())!=0) { 
+			wasCanceled = true; 
+			dispose(); 
 		} 
 	} 
+		
 
 	void accessTextFields() {
 		if (stringField!=null) {
@@ -1084,7 +1234,8 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 			if (source==slider.elementAt(i)) {
 				Scrollbar sb = (Scrollbar)source;
 				TextField tf = (TextField)numberField.elementAt(sliderIndexes[i]);
-				tf.setText(""+sb.getValue());
+				int digits = sliderScales[i]==1.0?0:2;
+				tf.setText(""+IJ.d2s(sb.getValue()/sliderScales[i],digits));
 			}
 		}
 	}
@@ -1126,6 +1277,8 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 				tf.setEditable(false);
 				tf.setEditable(true);
 			}
+			if (numberField==null && stringField==null)
+				okay.requestFocus();
 			firstPaint = false;
 		}
 	}
@@ -1138,7 +1291,7 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
     public void addHelp(String url) {
     	helpURL = url;
     }
-    
+
     void showHelp() {
 		String macro = "run('URL...', 'url="+helpURL+"');";
 		new MacroRunner(macro);
@@ -1151,4 +1304,4 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
     public void windowDeiconified(WindowEvent e) {}
     public void windowDeactivated(WindowEvent e) {}
 
-}
\ No newline at end of file
+}
