@@ -12,7 +12,9 @@ import ij.plugin.filter.Analyzer;
 import ij.text.TextWindow;
 
 /** This class is an extended ImageWindow that displays histograms. */
-public class HistogramWindow extends ImageWindow implements Measurements, ActionListener, ClipboardOwner {
+public class HistogramWindow extends ImageWindow implements Measurements, ActionListener, ClipboardOwner,
+	MouseListener, MouseMotionListener, ImageListener, KeyListener, Runnable {
+	
 	static final int WIN_WIDTH = 300;
 	static final int WIN_HEIGHT = 240;
 	static final int HIST_WIDTH = 256;
@@ -20,12 +22,13 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 	static final int BAR_HEIGHT = 12;
 	static final int XMARGIN = 20;
 	static final int YMARGIN = 10;
+	static final int INTENSITY=0, RED=1, GREEN=2, BLUE=3;
 	
 	protected ImageStatistics stats;
 	protected int[] histogram;
 	protected LookUpTable lut;
 	protected Rectangle frame = null;
-	protected Button list, save, copy,log;
+	protected Button list, save, copy, log, live, rgb;
 	protected Label value, count;
 	protected static String defaultDirectory = null;
 	protected int decimalPlaces;
@@ -36,17 +39,24 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 	protected Calibration cal;
 	protected int yMax;
 	public static int nBins = 256;
-    
+		
+	private int srcImageID;			// ID of source image
+	private ImagePlus srcImp;		// source image for live histograms
+	private Thread bgThread;		// thread background drawing
+	private boolean doUpdate;	// tells background thread to update
+	private int channel;				// RGB channel
+	private String blankLabel;
+	    
 	/** Displays a histogram using the title "Histogram of ImageName". */
 	public HistogramWindow(ImagePlus imp) {
-		super(NewImage.createByteImage("Histogram of "+imp.getShortTitle(), WIN_WIDTH, WIN_HEIGHT, 1, NewImage.FILL_WHITE));
+		super(NewImage.createRGBImage("Histogram of "+imp.getShortTitle(), WIN_WIDTH, WIN_HEIGHT, 1, NewImage.FILL_WHITE));
 		showHistogram(imp, 256, 0.0, 0.0);
 	}
 
 	/** Displays a histogram using the specified title and number of bins. 
 		Currently, the number of bins must be 256 expect for 32 bit images. */
 	public HistogramWindow(String title, ImagePlus imp, int bins) {
-		super(NewImage.createByteImage(title, WIN_WIDTH, WIN_HEIGHT, 1, NewImage.FILL_WHITE));
+		super(NewImage.createRGBImage(title, WIN_WIDTH, WIN_HEIGHT, 1, NewImage.FILL_WHITE));
 		showHistogram(imp, bins, 0.0, 0.0);
 	}
 
@@ -54,20 +64,20 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 		Currently, the number of bins must be 256 and the histogram range range must be the 
 		same as the image range expect for 32 bit images. */
 	public HistogramWindow(String title, ImagePlus imp, int bins, double histMin, double histMax) {
-		super(NewImage.createByteImage(title, WIN_WIDTH, WIN_HEIGHT, 1, NewImage.FILL_WHITE));
+		super(NewImage.createRGBImage(title, WIN_WIDTH, WIN_HEIGHT, 1, NewImage.FILL_WHITE));
 		showHistogram(imp, bins, histMin, histMax);
 	}
 
 	/** Displays a histogram using the specified title, number of bins, histogram range and yMax. */
 	public HistogramWindow(String title, ImagePlus imp, int bins, double histMin, double histMax, int yMax) {
-		super(NewImage.createByteImage(title, WIN_WIDTH, WIN_HEIGHT, 1, NewImage.FILL_WHITE));
+		super(NewImage.createRGBImage(title, WIN_WIDTH, WIN_HEIGHT, 1, NewImage.FILL_WHITE));
 		this.yMax = yMax;
 		showHistogram(imp, bins, histMin, histMax);
 	}
 
 	/** Displays a histogram using the specified title and ImageStatistics. */
 	public HistogramWindow(String title, ImagePlus imp, ImageStatistics stats) {
-		super(NewImage.createByteImage(title, WIN_WIDTH, WIN_HEIGHT, 1, NewImage.FILL_WHITE));
+		super(NewImage.createRGBImage(title, WIN_WIDTH, WIN_HEIGHT, 1, NewImage.FILL_WHITE));
 		//IJ.log("HistogramWindow: "+stats.histMin+"  "+stats.histMax+"  "+stats.nBins);
 		this.yMax = stats.histYMax;
 		showHistogram(imp, stats);
@@ -84,13 +94,22 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 		the same as the image range expect for 32 bit images. */
 	public void showHistogram(ImagePlus imp, int bins, double histMin, double histMax) {
 		boolean limitToThreshold = (Analyzer.getMeasurements()&LIMIT)!=0;
-		stats = imp.getStatistics(AREA+MEAN+MODE+MIN_MAX+(limitToThreshold?LIMIT:0), bins, histMin, histMax);
+		if (channel!=INTENSITY && imp.getType()==ImagePlus.COLOR_RGB) {
+			ColorProcessor cp = (ColorProcessor)imp.getProcessor();
+			byte[] bytes = cp.getChannel(channel);
+			ImageProcessor ip = new ByteProcessor(imp.getWidth(), imp.getHeight(), bytes, null);
+			ImagePlus imp2 = new ImagePlus("", ip);
+			imp2.setRoi(imp.getRoi());
+			stats = imp2.getStatistics(AREA+MEAN+MODE+MIN_MAX, bins, histMin, histMax);
+		} else
+			stats = imp.getStatistics(AREA+MEAN+MODE+MIN_MAX+(limitToThreshold?LIMIT:0), bins, histMin, histMax);
 		showHistogram(imp, stats);
 	}
 
 	/** Draws the histogram using the specified title and ImageStatistics. */
 	public void showHistogram(ImagePlus imp, ImageStatistics stats) {
-		setup();
+		if (list==null)
+			setup(imp);
 		this.stats = stats;
 		cal = imp.getCalibration();
 		boolean limitToThreshold = (Analyzer.getMeasurements()&LIMIT)!=0;
@@ -111,19 +130,19 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 		int type = imp.getType();
 		boolean fixedRange = type==ImagePlus.GRAY8 || type==ImagePlus.COLOR_256 || type==ImagePlus.COLOR_RGB;
 		ImageProcessor ip = this.imp.getProcessor();
-		boolean color = !(imp.getProcessor() instanceof ColorProcessor) && !lut.isGrayscale();
-		if (color)
-			ip = ip.convertToRGB();
-		drawHistogram(ip, fixedRange);
-		if (color)
-			this.imp.setProcessor(null, ip);
-		else
-			this.imp.updateAndDraw();
+		ip.setColor(Color.white);
+		ip.resetRoi();
+		ip.fill();
+		ImageProcessor srcIP = imp.getProcessor();
+		drawHistogram(imp, ip, fixedRange, stats.histMin, stats.histMax);
+		this.imp.updateAndDraw();
 	}
 
-	public void setup() {
+	private void setup(ImagePlus imp) {
+		boolean isRGB = imp.getType()==ImagePlus.COLOR_RGB;
  		Panel buttons = new Panel();
-		buttons.setLayout(new FlowLayout(FlowLayout.RIGHT));
+ 		int hgap = IJ.isMacOSX()||isRGB?1:5;
+		buttons.setLayout(new FlowLayout(FlowLayout.RIGHT,hgap,0));
 		list = new Button("List");
 		list.addActionListener(this);
 		buttons.add(list);
@@ -133,18 +152,32 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 		log = new Button("Log");
 		log.addActionListener(this);
 		buttons.add(log);
-		Panel valueAndCount = new Panel();
-		valueAndCount.setLayout(new GridLayout(2, 1));
-		value = new Label("                  "); //21
-		value.setFont(new Font("Monospaced", Font.PLAIN, 12));
-		valueAndCount.add(value);
-		count = new Label("                  ");
-		count.setFont(new Font("Monospaced", Font.PLAIN, 12));
-		valueAndCount.add(count);
-		buttons.add(valueAndCount);
+		live = new Button("Live");
+		live.addActionListener(this);
+		buttons.add(live);
+		if (imp!=null && isRGB) {
+			rgb = new Button("RGB");
+			rgb.addActionListener(this);
+			buttons.add(rgb);
+		}
+		if (!(IJ.isMacOSX()&&isRGB)) {
+			Panel valueAndCount = new Panel();
+			valueAndCount.setLayout(new GridLayout(2,1,0,0));
+			blankLabel = IJ.isMacOSX()?"         ":"                ";
+			value = new Label(blankLabel);
+			Font font = new Font("Monospaced", Font.PLAIN, 12);
+			value.setFont(font);
+			valueAndCount.add(value);
+			count = new Label(blankLabel);
+			count.setFont(font);
+			valueAndCount.add(count);
+			buttons.add(valueAndCount);
+		}
 		add(buttons);
 		pack();
     }
+    
+	public void setup() {setup(null);}
 
 	public void mouseMoved(int x, int y) {
 		if (value==null || count==null)
@@ -153,15 +186,27 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 			x = x - frame.x;
 			if (x>255) x = 255;
 			int index = (int)(x*((double)histogram.length)/HIST_WIDTH);
-			value.setText("  Value: " + IJ.d2s(cal.getCValue(stats.histMin+index*stats.binSize), digits));
-			count.setText("  Count: " + histogram[index]);
+			String vlabel=null, clabel=null;
+			if (blankLabel.length()==9)
+				{vlabel=" "; clabel=" ";}
+			else
+				{vlabel=" value="; clabel=" count=";}
+			String v = vlabel+ResultsTable.d2s(cal.getCValue(stats.histMin+index*stats.binSize), digits)+blankLabel;
+			String c = clabel+histogram[index]+blankLabel;
+			int len = vlabel.length() + blankLabel.length();
+			value.setText(v.substring(0,len));
+			count.setText(c.substring(0,len));
 		} else {
-			value.setText("");
-			count.setText("");
+			value.setText(blankLabel);
+			count.setText(blankLabel);
 		}
 	}
     
 	protected void drawHistogram(ImageProcessor ip, boolean fixedRange) {
+		drawHistogram(null, ip, fixedRange, 0.0, 0.0);
+	}
+
+	void drawHistogram(ImagePlus imp, ImageProcessor ip, boolean fixedRange, double xMin, double xMax) {
 		int x, y;
 		int maxCount2 = 0;
 		int mode2 = 0;
@@ -172,30 +217,74 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 		decimalPlaces = Analyzer.getPrecision();
 		digits = cal.calibrated()||stats.binSize!=1.0?decimalPlaces:0;
 		saveModalCount = histogram[stats.mode];
-		for (int i = 0; i<histogram.length; i++)
- 		if ((histogram[i] > maxCount2) && (i != stats.mode)) {
-			maxCount2 = histogram[i];
-			mode2 = i;
+		for (int i = 0; i<histogram.length; i++) {
+ 			if ((histogram[i] > maxCount2) && (i != stats.mode)) {
+				maxCount2 = histogram[i];
+				mode2 = i;
+  			}
   		}
 		newMaxCount = stats.maxCount;
 		if ((newMaxCount>(maxCount2 * 2)) && (maxCount2 != 0)) {
 			newMaxCount = (int)(maxCount2 * 1.5);
   			//histogram[stats.mode] = newMaxCount;
 		}
-		if (IJ.shiftKeyDown()) {
-			logScale = true;
+		if (logScale || IJ.shiftKeyDown() && !liveMode())
 			drawLogPlot(yMax>0?yMax:newMaxCount, ip);
-		}
 		drawPlot(yMax>0?yMax:newMaxCount, ip);
 		histogram[stats.mode] = saveModalCount;
  		x = XMARGIN + 1;
 		y = YMARGIN + HIST_HEIGHT + 2;
-		lut.drawUnscaledColorBar(ip, x-1, y, 256, BAR_HEIGHT);
+		if (imp==null)
+			lut.drawUnscaledColorBar(ip, x-1, y, 256, BAR_HEIGHT);
+		else
+			drawAlignedColorBar(imp, xMin, xMax, ip, x-1, y, 256, BAR_HEIGHT);
 		y += BAR_HEIGHT+15;
   		drawText(ip, x, y, fixedRange);
+  		srcImageID = imp.getID();
+	}
+       
+	void drawAlignedColorBar(ImagePlus imp, double xMin, double xMax, ImageProcessor ip, int x, int y, int width, int height) {
+		ImageProcessor ipSource = imp.getProcessor();
+		float[] pixels = null;
+		ImageProcessor ipRamp = null;
+		if (ipSource instanceof ColorProcessor) {
+			ipRamp = new FloatProcessor(width, height);
+			if (channel==RED)
+				ipRamp.setColorModel(LUT.createLutFromColor(Color.red));
+			else if (channel==GREEN)
+				ipRamp.setColorModel(LUT.createLutFromColor(Color.green));
+			else if (channel==BLUE)
+				ipRamp.setColorModel(LUT.createLutFromColor(Color.blue));
+			pixels = (float[])ipRamp.getPixels();
+		} else
+			pixels = new float[width*height];
+		for (int j=0; j<height; j++) {
+			for(int i=0; i<width; i++)
+				pixels[i+width*j] = (float)(xMin+i*(xMax-xMin)/(width - 1));
+		}
+		if (!(ipSource instanceof ColorProcessor)) {
+			ColorModel cm = null;
+			if (imp.isComposite())
+				cm = ((CompositeImage)imp).getChannelLut();
+			else if (ipSource.getMinThreshold()==ImageProcessor.NO_THRESHOLD)
+				cm = ipSource.getColorModel();
+			else
+				cm = ipSource.getCurrentColorModel();
+			ipRamp = new FloatProcessor(width, height, pixels, cm);
+		}
+		double min = ipSource.getMin();
+		double max = ipSource.getMax();
+		ipRamp.setMinAndMax(min,max);
+		ImageProcessor bar = null;
+		if (ip instanceof ColorProcessor)
+			bar = ipRamp.convertToRGB();
+		else
+			bar = ipRamp.convertToByte(true);
+		ip.insert(bar, x,y);
+		ip.setColor(Color.black);
+		ip.drawRect(x-1, y, width+2, height);
 	}
 
-       
 	/** Scales a threshold level to the range 0-255. */
 	int scaleDown(ImageProcessor ip, double threshold) {
 		double min = ip.getMin();
@@ -213,9 +302,8 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 		int index, y;
 		for (int i = 0; i<HIST_WIDTH; i++) {
 			index = (int)(i*(double)histogram.length/HIST_WIDTH); 
-			y = (int)((double)HIST_HEIGHT*histogram[index])/maxCount;
-			if (y>HIST_HEIGHT)
-				y = HIST_HEIGHT;
+			y = (int)(((double)HIST_HEIGHT*(double)histogram[index])/maxCount);
+			if (y>HIST_HEIGHT) y = HIST_HEIGHT;
 			ip.drawLine(i+XMARGIN, YMARGIN+HIST_HEIGHT, i+XMARGIN, YMARGIN+HIST_HEIGHT-y);
 		}
 	}
@@ -257,7 +345,8 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 		int row2 = row1 + 15;
 		int row3 = row2 + 15;
 		int row4 = row3 + 15;
-		ip.drawString("Count: " + stats.pixelCount, col1, row1);
+		long count = stats.longPixelCount>0?stats.longPixelCount:stats.pixelCount;
+		ip.drawString("Count: " + count, col1, row1);
 		ip.drawString("Mean: " + d2s(stats.mean), col1, row2);
 		ip.drawString("StdDev: " + d2s(stats.stdDev), col1, row3);
 		ip.drawString("Mode: " + d2s(stats.dmode) + " (" + stats.maxCount + ")", col2, row3);
@@ -278,9 +367,9 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 		else if (Double.isInfinite(d))
 			return("Infinity");
 		else if ((int)d==d)
-			return IJ.d2s(d,0);
+			return ResultsTable.d2s(d,0);
 		else
-			return IJ.d2s(d,decimalPlaces);
+			return ResultsTable.d2s(d,decimalPlaces);
 	}
 	
 	int getWidth(double d, ImageProcessor ip) {
@@ -292,11 +381,11 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
         String vheading = stats.binSize==1.0?"value":"bin start";
 		if (cal.calibrated() && !cal.isSigned16Bit()) {
 			for (int i=0; i<stats.nBins; i++)
-				sb.append(i+"\t"+IJ.d2s(cal.getCValue(stats.histMin+i*stats.binSize), digits)+"\t"+histogram[i]+"\n");
+				sb.append(i+"\t"+ResultsTable.d2s(cal.getCValue(stats.histMin+i*stats.binSize), digits)+"\t"+histogram[i]+"\n");
 			TextWindow tw = new TextWindow(getTitle(), "level\t"+vheading+"\tcount", sb.toString(), 200, 400);
 		} else {
 			for (int i=0; i<stats.nBins; i++)
-				sb.append(IJ.d2s(cal.getCValue(stats.histMin+i*stats.binSize), digits)+"\t"+histogram[i]+"\n");
+				sb.append(ResultsTable.d2s(cal.getCValue(stats.histMin+i*stats.binSize), digits)+"\t"+histogram[i]+"\n");
 			TextWindow tw = new TextWindow(getTitle(), vheading+"\tcount", sb.toString(), 200, 400);
 		}
 	}
@@ -311,7 +400,7 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 		CharArrayWriter aw = new CharArrayWriter(stats.nBins*4);
 		PrintWriter pw = new PrintWriter(aw);
 		for (int i=0; i<stats.nBins; i++)
-			pw.print(IJ.d2s(cal.getCValue(stats.histMin+i*stats.binSize), digits)+"\t"+histogram[i]+"\n");
+			pw.print(ResultsTable.d2s(cal.getCValue(stats.histMin+i*stats.binSize), digits)+"\t"+histogram[i]+"\n");
 		String text = aw.toString();
 		pw.close();
 		StringSelection contents = new StringSelection(text);
@@ -320,7 +409,6 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 	}
 	
 	void replot() {
-		logScale = !logScale;
 		ImageProcessor ip = this.imp.getProcessor();
 		frame = new Rectangle(XMARGIN, YMARGIN, HIST_WIDTH, HIST_HEIGHT);
 		ip.setColor(Color.white);
@@ -356,12 +444,18 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 	
 	public void actionPerformed(ActionEvent e) {
 		Object b = e.getSource();
-		if (b==list)
+		if (b==live)
+			toggleLiveMode();
+		else if (b==rgb)
+			changeChannel();
+		else if (b==list)
 			showList();
 		else if (b==copy)
 			copyToClipboard();
-		else if (b==log)
+		else if (b==log) {
+			logScale = !logScale;
 			replot();
+		}
 	}
 	
 	public void lostOwnership(Clipboard clipboard, Transferable contents) {}
@@ -377,5 +471,141 @@ public class HistogramWindow extends ImageWindow implements Measurements, Action
 		return values;
 	}
 
+	private void toggleLiveMode() {
+		if (liveMode())
+			removeListeners();
+		else
+			enableLiveMode();
+	}
+	
+	private void changeChannel() {
+		ImagePlus imp = WindowManager.getImage(srcImageID);
+		if (imp==null || imp.getType()!=ImagePlus.COLOR_RGB) {
+			channel = INTENSITY;
+			return;
+		} else {
+			channel++;
+			if (channel>BLUE) channel=INTENSITY;
+			showHistogram(imp, 256);
+			String name = this.imp.getTitle();
+			if (name.startsWith("Red ")) name=name.substring(4);
+			else if (name.startsWith("Green ")) name=name.substring(6);
+			else if (name.startsWith("Blue ")) name=name.substring(5);
+			switch (channel) {
+				case INTENSITY: this.imp.setTitle(name); break;
+				case RED: this.imp.setTitle("Red "+name); break;
+				case GREEN: this.imp.setTitle("Green "+name); break;
+				case BLUE: this.imp.setTitle("Blue "+name); break;
+			}
+		}
+	}
+
+	private boolean liveMode() {
+		return live!=null && live.getForeground()==Color.red;
+	}
+
+	private void enableLiveMode() {
+		if (bgThread==null) {
+			srcImp = WindowManager.getImage(srcImageID);
+			if (srcImp==null) return;
+			bgThread = new Thread(this, "Live Profiler");
+			bgThread.setPriority(Math.max(bgThread.getPriority()-3, Thread.MIN_PRIORITY));
+			bgThread.start();
+			imageUpdated(srcImp);
+		}
+		createListeners();
+		if (srcImp!=null)
+			imageUpdated(srcImp);
+	}
+
+	// these listeners are activated if there are in the source ImagePlus
+	public synchronized void mousePressed(MouseEvent e) { doUpdate=true; notify(); }   
+	public synchronized void mouseDragged(MouseEvent e) { doUpdate=true; notify(); }
+	public synchronized void mouseClicked(MouseEvent e) { doUpdate=true; notify(); }
+	
+	public synchronized void keyPressed(KeyEvent e) {
+		ImagePlus imp = WindowManager.getImage(srcImageID);
+		if (imp==null || imp.getRoi()!=null) {
+			doUpdate = true;
+			notify();
+		}
+	}
+	
+	// unused listeners
+	public void mouseReleased(MouseEvent e) {}
+	public void mouseExited(MouseEvent e) {}
+	public void mouseEntered(MouseEvent e) {}
+	public void mouseMoved(MouseEvent e) {}
+	public void imageOpened(ImagePlus imp) {}
+	public void keyTyped(KeyEvent e) {}
+	public void keyReleased(KeyEvent e) {}
+	
+	// This listener is called if the source image content is changed
+	public synchronized void imageUpdated(ImagePlus imp) {
+		if (imp==srcImp) { 
+			doUpdate = true;
+			notify();
+		}
+	}
+	
+	// If either the source image or this image are closed, exit
+	public void imageClosed(ImagePlus imp) {
+		if (imp==srcImp || imp==this.imp) {
+			if (bgThread!=null)
+				bgThread.interrupt();
+			bgThread = null;
+			removeListeners();
+			srcImp = null;
+		}
+	}
+	
+	// the background thread for live plotting.
+	public void run() {
+		while (true) {
+			if (doUpdate && srcImp!=null) {
+				if (srcImp.getRoi()!=null)
+					IJ.wait(50);	//delay to make sure the roi has been updated
+				showHistogram(srcImp, 256);
+			}
+			synchronized(this) {
+				if (doUpdate) {
+					doUpdate = false;		//and loop again
+				} else {
+					try {wait();}	//notify wakes up the thread
+					catch(InterruptedException e) { //interrupted tells the thread to exit
+						return;
+					}
+				}
+			}
+		}
+	}
+	
+	private void createListeners() {
+		//IJ.log("createListeners");
+		if (srcImp==null) return;
+		ImageCanvas ic = srcImp.getCanvas();
+		if (ic==null) return;
+		ic.addMouseListener(this);
+		ic.addMouseMotionListener(this);
+		ic.addKeyListener(this);
+		srcImp.addImageListener(this);
+		Font font = live.getFont();
+		live.setFont(new Font(font.getName(), Font.BOLD, font.getSize()));
+		live.setForeground(Color.red);
+	}
+	
+	private void removeListeners() {
+		//IJ.log("removeListeners");
+		if (srcImp==null) return;
+		ImageCanvas ic = srcImp.getCanvas();
+		ic.removeMouseListener(this);
+		ic.removeMouseMotionListener(this);
+		ic.removeKeyListener(this);
+		srcImp.removeImageListener(this);
+		Font font = live.getFont();
+		live.setFont(new Font(font.getName(), Font.PLAIN, font.getSize()));
+		live.setForeground(Color.black);
+	}
+
 }
 
