@@ -3,7 +3,7 @@ package ij.gui;
 import java.awt.*;
 import java.util.Properties;
 import java.awt.image.*;
-import ij.process.ImageProcessor;
+import ij.process.*;
 import ij.measure.*;
 import ij.plugin.WandToolOptions;
 import ij.plugin.frame.Recorder;
@@ -13,6 +13,8 @@ import ij.*;
 import ij.util.*;
 import java.awt.event.*;
 import java.util.*;
+import java.awt.geom.*;
+
 
 /** This is a Canvas used to display images in a Window. */
 public class ImageCanvas extends Canvas implements MouseListener, MouseMotionListener, Cloneable {
@@ -37,16 +39,16 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 	private boolean showAllROIs;
 	private static Color zoomIndicatorColor;
 	private static Font smallFont, largeFont;
+	private Font font;
 	private Rectangle[] labelRects;
     private boolean maxBoundsReset;
-    private Vector displayList;
-    private boolean labelListItems;
-    private Color listColor;
-    private BasicStroke listStroke;
+    private Overlay overlay, showAllList;
     private static final int LIST_OFFSET = 100000;
-    private static Color showAllColor = Prefs.getColor(Prefs.SHOW_ALL_COLOR, new Color(128, 255, 255));
-    private static Color labelColor;
+    private static Color showAllColor = Prefs.getColor(Prefs.SHOW_ALL_COLOR, new Color(0, 255, 255));
+    private Color defaultColor = showAllColor;
+    private static Color labelColor, bgColor;
     private int resetMaxBoundsCount;
+    private Roi currentRoi;
 		
 	protected ImageJ ij;
 	protected double magnification;
@@ -63,6 +65,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 	private int offScreenHeight = 0;
 	private boolean mouseExited = true;
 	private boolean customRoi;
+	private boolean drawNames;
 	
 	
 	public ImageCanvas(ImagePlus imp) {
@@ -107,14 +110,22 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		srcRect = r;
 	}
 
+	void setSrcRect(Rectangle srcRect) {
+		this.srcRect = srcRect;
+	}
+		
+	public Rectangle getSrcRect() {
+		return srcRect;
+	}
+	
 	public void setDrawingSize(int width, int height) {
 	    dstWidth = width;
 	    dstHeight = height;
 		setSize(dstWidth, dstHeight);
 	}
 		
-	/** ImagePlus.updateAndDraw calls this method to get paint 
-		to update the image from the ImageProcessor. */
+	/** ImagePlus.updateAndDraw calls this method to force the paint()
+		method to update the image from the ImageProcessor. */
 	public void setImageUpdated() {
 		imageUpdated = true;
 	}
@@ -125,10 +136,12 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 
     public void paint(Graphics g) {
 		Roi roi = imp.getRoi();
-		if (roi != null) {
-			roi.updatePaste();
-			if (Prefs.doubleBuffer && !IJ.isMacOSX())
-				{paintDoubleBuffered(g); return;}
+		if (roi!=null || showAllROIs || overlay!=null) {
+			if (roi!=null) roi.updatePaste();
+			if (!IJ.isMacOSX() && imageWidth!=0) {
+				paintDoubleBuffered(g);
+				return;
+			}
 		}
 		try {
 			if (imageUpdated) {
@@ -140,9 +153,9 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			if (img!=null)
  				g.drawImage(img, 0, 0, (int)(srcRect.width*magnification), (int)(srcRect.height*magnification),
 				srcRect.x, srcRect.y, srcRect.x+srcRect.width, srcRect.y+srcRect.height, null);
-			if (showAllROIs) showAllROIs(g);
-			if (displayList!=null) drawDisplayList(g);
-			if (roi != null) roi.draw(g);
+			if (overlay!=null) drawOverlay(g);
+			if (showAllROIs) drawAllROIs(g);
+			if (roi!=null) drawRoi(roi, g);
 			if (srcRect.width<imageWidth || srcRect.height<imageHeight)
 				drawZoomIndicator(g);
 			if (IJ.debugMode) showFrameRate(g);
@@ -150,30 +163,96 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		catch(OutOfMemoryError e) {IJ.outOfMemory("Paint");}
     }
     
-    void showAllROIs(Graphics g) {
+    private void drawRoi(Roi roi, Graphics g) {
+		if (roi==currentRoi) {
+			Color lineColor = roi.getStrokeColor();
+			Color fillColor = roi.getFillColor();
+			float lineWidth = roi.getStrokeWidth();
+			roi.setStrokeColor(null);
+			roi.setFillColor(null);
+			boolean strokeSet = roi.getStroke()!=null;
+			if (strokeSet)
+				roi.setStrokeWidth(1);
+			roi.draw(g);
+			roi.setStrokeColor(lineColor);
+			if (strokeSet)
+				roi.setStrokeWidth(lineWidth);
+			roi.setFillColor(fillColor);
+			currentRoi = null;
+		} else
+			roi.draw(g);
+    }
+    
+    void drawAllROIs(Graphics g) {
 		RoiManager rm=RoiManager.getInstance();
-		if (rm==null) return;
-		initGraphics(g, null);
+		if (rm==null) {
+			rm = Interpreter.getBatchModeRoiManager();
+			if (rm!=null && rm.getList().getItemCount()==0)
+				rm = null;
+		}
+		if (rm==null) {
+			//if (showAllList!=null)
+			//	overlay = showAllList;
+			showAllROIs = false;
+			repaint();
+			return;
+		}
+		initGraphics(g, null, showAllColor);
 		Hashtable rois = rm.getROIs();
 		java.awt.List list = rm.getList();
+		boolean drawLabels = rm.getDrawLabels();
+		currentRoi = null;
 		int n = list.getItemCount();
+    	if (IJ.debugMode) IJ.log("paint: drawing "+n+" \"Show All\" ROIs");
 		if (labelRects==null || labelRects.length!=n)
 			labelRects = new Rectangle[n];
+		if (!drawLabels)
+			showAllList = new Overlay();
+		else
+			showAllList = null;
+    	if (imp==null)
+    		return;
+    	int currentImage = imp.getCurrentSlice();
+		int channel=0, slice=0, frame=0;
+		boolean hyperstack = imp.isHyperStack();
+		if (hyperstack) {
+			channel = imp.getChannel();
+			slice = imp.getSlice();
+			frame = imp.getFrame();
+		}
+		drawNames = Prefs.useNamesAsLabels;
 		for (int i=0; i<n; i++) {
 			String label = list.getItem(i);
 			Roi roi = (Roi)rois.get(label);
 			if (roi==null) continue;
+			if (showAllList!=null)
+				showAllList.add(roi);
+			if (i<200 && drawLabels && roi==imp.getRoi())
+				currentRoi = roi;
 			if (Prefs.showAllSliceOnly && imp.getStackSize()>1) {
-				int slice = getSliceNumber(roi.getName());
-				if (slice==-1 || slice==imp.getCurrentSlice())
-					drawRoi(g, roi, i);
+				if (hyperstack && roi.getPosition()==0) {
+					int c = roi.getCPosition();
+					int z = roi.getZPosition();
+					int t = roi.getTPosition();
+					if ((c==0||c==channel) && (z==0||z==slice) && (t==0||t==frame))
+						drawRoi(g, roi, drawLabels?i:-1);
+				} else {
+					int position = roi.getPosition();
+					if (position==0)
+						position = getSliceNumber(roi.getName());
+					if (position==0 || position==currentImage)
+						drawRoi(g, roi, drawLabels?i:-1);
+				}
 			} else
-				drawRoi(g, roi, i);
+				drawRoi(g, roi, drawLabels?i:-1);
 		}
+		((Graphics2D)g).setStroke(Roi.onePixelWide);
+		drawNames = false;
     }
-    
+       
 	public int getSliceNumber(String label) {
-		int slice = -1;
+		if (label==null) return 0;
+		int slice = 0;
 		if (label.length()>=14 && label.charAt(4)=='-' && label.charAt(9)=='-')
 			slice = (int)Tools.parseDouble(label.substring(0,4),-1);
 		else if (label.length()>=17 && label.charAt(5)=='-' && label.charAt(11)=='-')
@@ -183,96 +262,131 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		return slice;
 	}
 
-    void drawDisplayList(Graphics g) {
-		initGraphics(g, listColor);
-    	int n = displayList.size();
-    	for (int i=0; i<n; i++)
-    		drawRoi(g, (Roi)displayList.elementAt(i), labelListItems?i+LIST_OFFSET:-1);
-    	if (listStroke!=null) ((Graphics2D)g).setStroke(new BasicStroke());
-    }
-    
-    void initGraphics(Graphics g, Color c) {
+	void drawOverlay(Graphics g) {
+		if (imp!=null && imp.getHideOverlay())
+			return;
+		Color labelColor = overlay.getLabelColor();
+		if (labelColor==null) labelColor = Color.white;
+		initGraphics(g, labelColor, Roi.getColor());
+		int n = overlay.size();
+		if (IJ.debugMode) IJ.log("paint: drawing "+n+" ROI display list");
+		int currentImage = imp!=null?imp.getCurrentSlice():-1;
+		if (imp.getStackSize()==1)
+			currentImage = -1;
+		int channel=0, slice=0, frame=0;
+		boolean hyperstack = imp.isHyperStack();
+		if (hyperstack) {
+			channel = imp.getChannel();
+			slice = imp.getSlice();
+			frame = imp.getFrame();
+		}
+		drawNames = overlay.getDrawNames();
+		boolean drawLabels = drawNames || overlay.getDrawLabels();
+		font = overlay.getLabelFont();
+		for (int i=0; i<n; i++) {
+			if (overlay==null) break;
+			Roi roi = overlay.get(i);
+			if (hyperstack && roi.getPosition()==0) {
+				int c = roi.getCPosition();
+				int z = roi.getZPosition();
+				int t = roi.getTPosition();
+				if ((c==0||c==channel) && (z==0||z==slice) && (t==0||t==frame))
+					drawRoi(g, roi, drawLabels?i+LIST_OFFSET:-1);
+			} else {
+				int position = roi.getPosition();
+				if (position==0 || position==currentImage)
+					drawRoi(g, roi, drawLabels?i+LIST_OFFSET:-1);
+			}
+		}
+		((Graphics2D)g).setStroke(Roi.onePixelWide);
+		drawNames = false;
+		font = null;
+	}
+    	
+    void initGraphics(Graphics g, Color textColor, Color defaultColor) {
 		if (smallFont==null) {
 			smallFont = new Font("SansSerif", Font.PLAIN, 9);
 			largeFont = new Font("SansSerif", Font.PLAIN, 12);
 		}
-		if (labelColor==null) {
-			int red = showAllColor.getRed();
-			int green = showAllColor.getGreen();
-			int blue = showAllColor.getBlue();
+		if (textColor!=null) {
+			labelColor = textColor;
+			if (overlay!=null && overlay.getDrawBackgrounds())
+				bgColor = new Color(255-labelColor.getRed(), 255-labelColor.getGreen(), 255-labelColor.getBlue());
+			else
+				bgColor = null;
+		} else {
+			int red = defaultColor.getRed();
+			int green = defaultColor.getGreen();
+			int blue = defaultColor.getBlue();
 			if ((red+green+blue)/3<128)
 				labelColor = Color.white;
 			else
 				labelColor = Color.black;
+			bgColor = defaultColor;
 		}
-		if (c!=null) {
-			g.setColor(c);
-			if (listStroke!=null) ((Graphics2D)g).setStroke(listStroke);
-		} else
-			g.setColor(showAllColor);
+		this.defaultColor = defaultColor;
+		g.setColor(defaultColor);
     }
     
     void drawRoi(Graphics g, Roi roi, int index) {
-		if (roi.getType()==Roi.COMPOSITE) {
-			roi.setImage(imp);
-			Color c = roi.getColor();
-			if (index==-1 && listColor!=null)
-				roi.setColor(listColor);
+    	int type = roi.getType();
+		ImagePlus imp2 = roi.getImage();
+		roi.setImage(imp);
+		Color saveColor = roi.getStrokeColor();
+		if (saveColor==null)
+			roi.setStrokeColor(defaultColor);
+		if (roi instanceof TextRoi)
+			((TextRoi)roi).drawText(g);
+		else
+			roi.drawOverlay(g);
+		roi.setStrokeColor(saveColor);
+		if (index>=0) {
+			if (roi==currentRoi)
+				g.setColor(Roi.getColor());
 			else
-				roi.setColor(showAllColor);
-			roi.draw(g);
-			roi.setColor(c);
-			if (index>=0) drawRoiLabel(g, index, roi.getBounds());
-		} else {
-			Color c = index==-1?roi.getInstanceColor():null;
-			Color saveg = null;
-			if (c!=null) {
-				saveg = g.getColor();
-				g.setColor(c);
-			}
-			Polygon p = roi.getPolygon();
-			int x1=0, y1=0, x2=0, y2=0;
-			for (int j=0; j<p.npoints; j++) {
-				x2 = screenX(p.xpoints[j]);
-				y2 = screenY(p.ypoints[j]);
-				if (j>0) g.drawLine(x1, y1, x2, y2);
-				x1=x2; y1=y2;
-			}
-			if (roi.isArea()&&p.npoints>0) {
-				int x0 = screenX(p.xpoints[0]);
-				int y0 = screenY(p.ypoints[0]);
-				g.drawLine(x1, y1, x0, y0);
-			}
-			if (index>=0) drawRoiLabel(g, index, roi.getBounds());
-			if (saveg!=null) g.setColor(saveg);
+				g.setColor(defaultColor);
+			drawRoiLabel(g, index, roi);
 		}
+		if (imp2!=null)
+			roi.setImage(imp2);
+		else
+			roi.setImage(null);
     }
     
-	void drawRoiLabel(Graphics g, int index, Rectangle r) {
+	void drawRoiLabel(Graphics g, int index, Roi roi) {
+		Rectangle r = roi.getBounds();
 		int x = screenX(r.x);
 		int y = screenY(r.y);
 		double mag = getMagnification();
 		int width = (int)(r.width*mag);
 		int height = (int)(r.height*mag);
 		int size = width>40 && height>40?12:9;
-		if (size==12)
+		if (font!=null) {
+			g.setFont(font);
+			size = font.getSize();
+		} else if (size==12)
 			g.setFont(largeFont);
 		else
 			g.setFont(smallFont);
 		boolean drawingList = index >= LIST_OFFSET;
 		if (drawingList) index -= LIST_OFFSET;
 		String label = "" + (index+1);
+		if (drawNames && roi.getName()!=null)
+			label = roi.getName();
 		FontMetrics metrics = g.getFontMetrics();
 		int w = metrics.stringWidth(label);
 		x = x + width/2 - w/2;
 		y = y + height/2 + Math.max(size/2,6);
-		int h =  metrics.getHeight();
-		g.fillRoundRect(x-1, y-h+2, w+1, h-3, 5, 5);
-		if (!drawingList)
-			labelRects[index] = new Rectangle(x-1, y-h+2, w+1, h-3);
+		int h = metrics.getAscent() + metrics.getDescent();
+		if (bgColor!=null) {
+			g.setColor(bgColor);
+			g.fillRoundRect(x-1, y-h+2, w+1, h-3, 5, 5);
+		}
+		if (!drawingList && labelRects!=null && index<labelRects.length)
+			labelRects[index] = new Rectangle(x-1, y-h+2, w+1, h);
 		g.setColor(labelColor);
 		g.drawString(label, x, y-2);
-		g.setColor(showAllColor);
+		g.setColor(defaultColor);
 	} 
 
 	void drawZoomIndicator(Graphics g) {
@@ -294,6 +408,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		if (zoomIndicatorColor==null)
 			zoomIndicatorColor = new Color(128, 128, 255);
 		g.setColor(zoomIndicatorColor);
+		((Graphics2D)g).setStroke(Roi.onePixelWide);
 		g.drawRect(x1, y1, w1, h1);
 		if (w2*h2<=200 || w2<10 || h2<10)
 			g.fillRect(x1+x2, y1+y2, w2, h2);
@@ -323,9 +438,9 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			if (img!=null)
 				offScreenGraphics.drawImage(img, 0, 0, srcRectWidthMag, srcRectHeightMag,
 					srcRect.x, srcRect.y, srcRect.x+srcRect.width, srcRect.y+srcRect.height, null);
-			if (showAllROIs) showAllROIs(offScreenGraphics);
-			if (displayList!=null) drawDisplayList(offScreenGraphics);
-			if (roi!=null) roi.draw(offScreenGraphics);
+			if (overlay!=null) drawOverlay(offScreenGraphics);
+			if (showAllROIs) drawAllROIs(offScreenGraphics);
+			if (roi!=null) drawRoi(roi, offScreenGraphics);
 			if (srcRect.width<imageWidth ||srcRect.height<imageHeight)
 				drawZoomIndicator(offScreenGraphics);
 			if (IJ.debugMode) showFrameRate(offScreenGraphics);
@@ -333,6 +448,10 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		}
 		catch(OutOfMemoryError e) {IJ.outOfMemory("Paint");}
 	}
+	
+	public void resetDoubleBuffer() {
+		offScreenImage = null;
+	}
 
     long firstFrame;
     int frames, fps;
@@ -370,11 +489,21 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 	public Point getCursorLoc() {
 		return new Point(xMouse, yMouse);
 	}
+	
+	/** Returns 'true' if the cursor is over this image. */
+	public boolean cursorOverImage() {
+		return !mouseExited;
+	}
 
 	/** Returns the mouse event modifiers. */
 	public int getModifiers() {
 		return flags;
 	}
+	
+	/** Returns the ImagePlus object that is associated with this ImageCanvas. */
+	public ImagePlus getImage() {
+		return imp;
+	}
 
 	/** Sets the cursor based on the current tool and cursor location. */
 	public void setCursor(int sx, int sy, int ox, int oy) {
@@ -465,38 +594,6 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		imp.setTitle(imp.getTitle());
 	}
 
-	/*
-	public void setMagnification(double mag, int x, int y) {
-		if (mag>32.0) magnification = 32.0;
-		if (mag<0.03125) mag = 0.03125;
-		int newWidth = (int)(imageWidth*mag);
-		int newHeight = (int)(imageHeight*mag);
-		Dimension newSize = canEnlarge(newWidth, newHeight);
-		if (newSize!=null) {
-			setDrawingSize(newSize.width, newSize.height);
-			if (newSize.width!=newWidth || newSize.height!=newHeight)
-				adjustSourceRect(mag, x, y);
-			else
-				setMagnification(mag);
-			imp.getWindow().pack();
-		} else
-			adjustSourceRect(mag, x, y);
-		repaint();
-		if (srcRect.width<imageWidth || srcRect.height<imageHeight)
-			resetMaxBounds();
-		this.magnification = magnification;
-		imp.setTitle(imp.getTitle());
-	}
-	*/
-
-	public Rectangle getSrcRect() {
-		return srcRect;
-	}
-	
-	void setSrcRect(Rectangle srcRect) {
-		this.srcRect = srcRect;
-	}
-		
 	/** Enlarge the canvas if the user enlarges the window. */
 	void resizeCanvas(int width, int height) {
 		ImageWindow win = imp.getWindow();
@@ -590,8 +687,9 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 
 	/** Zooms in by making the window bigger. If it can't
 		be made bigger, then make the source rectangle 
-		(srcRect) smaller and center it at (x,y). */
-	public void zoomIn(int x, int y) {
+		(srcRect) smaller and center it at (sx,sy). Note that
+		sx and sy are screen coordinates. */
+	public void zoomIn(int sx, int sy) {
 		if (magnification>=32) return;
 		double newMag = getHigherZoomLevel(magnification);
 		int newWidth = (int)(imageWidth*newMag);
@@ -600,12 +698,12 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		if (newSize!=null) {
 			setDrawingSize(newSize.width, newSize.height);
 			if (newSize.width!=newWidth || newSize.height!=newHeight)
-				adjustSourceRect(newMag, x, y);
+				adjustSourceRect(newMag, sx, sy);
 			else
 				setMagnification(newMag);
 			imp.getWindow().pack();
 		} else
-			adjustSourceRect(newMag, x, y);
+			adjustSourceRect(newMag, sx, sy);
 		repaint();
 		if (srcRect.width<imageWidth || srcRect.height<imageHeight)
 			resetMaxBounds();
@@ -634,7 +732,6 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		ImageWindow win = imp.getWindow();
 		if (win==null) return null;
 		Rectangle r1 = win.getBounds();
-
 		Insets insets = win.getInsets();
 		Point loc = getLocation();
 		if (loc.x>insets.left+5 || loc.y>insets.top+5) {
@@ -645,7 +742,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			r1.width = r1.width - dstWidth + newWidth+10;
 			r1.height = r1.height - dstHeight + newHeight+10;
 		}
-		Rectangle max = win.getMaxWindow();
+		Rectangle max = win.getMaxWindow(r1.x, r1.y);
 		boolean fitsHorizontally = r1.x+r1.width<max.x+max.width;
 		boolean fitsVertically = r1.y+r1.height<max.y+max.height;
 		if (fitsHorizontally && fitsVertically)
@@ -787,7 +884,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 	}
 	
 	protected void setDrawingColor(int ox, int oy, boolean setBackground) {
-		//IJ.write("setDrawingColor: "+setBackground+this);
+		//IJ.log("setDrawingColor: "+setBackground+this);
 		int type = imp.getType();
 		int[] v = imp.getPixel(ox, oy);
 		switch (type) {
@@ -843,12 +940,15 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 	}
 
 	public void mousePressed(MouseEvent e) {
-		if (ij==null) return;
+		//if (ij==null) return;
 		showCursorStatus = true;
 		int toolID = Toolbar.getToolId();
 		ImageWindow win = imp.getWindow();
 		if (win!=null && win.running2 && toolID!=Toolbar.MAGNIFIER) {
-			win.running2 = false;
+			if (win instanceof StackWindow)
+				((StackWindow)win).setAnimate(false);
+			else
+				win.running2 = false;
 			return;
 		}
 		
@@ -875,17 +975,24 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			if (!(roi!=null && (roi.contains(ox, oy)||roi.isHandle(x, y)>=0)) && roiManagerSelect(x, y))
  				return;
 		}
-		if (customRoi && displayList!=null)
+		if (customRoi && overlay!=null)
 			return;
 
 		switch (toolID) {
 			case Toolbar.MAGNIFIER:
 				if (IJ.shiftKeyDown())
 					zoomToSelection(ox, oy);
-				else if ((flags & (Event.ALT_MASK|Event.META_MASK|Event.CTRL_MASK))!=0)
-					IJ.run("Out");
-				else
-					IJ.run("In");
+				else if ((flags & (Event.ALT_MASK|Event.META_MASK|Event.CTRL_MASK))!=0) {
+					//IJ.run("Out");
+					zoomOut(x, y);
+					if (getMagnification()<1.0)
+						imp.repaintWindow();
+				} else {
+					//IJ.run("In");
+	 				zoomIn(x, y);
+					if (getMagnification()<=1.0)
+						imp.repaintWindow();
+				}
 				break;
 			case Toolbar.HAND:
 				setupScroll(ox, oy);
@@ -945,11 +1052,18 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		java.awt.List list = rm.getList();
 		int n = list.getItemCount();
 		if (labelRects==null || labelRects.length!=n) return false;
+		boolean stackMode = imp!=null && imp.getStackSize()>1 && Prefs.showAllSliceOnly;
 		for (int i=0; i<n; i++) {
 			if (labelRects[i]!=null && labelRects[i].contains(x,y)) {
+				if (stackMode) {
+					int slice = getSliceNumber(list.getItem(i));
+					if (slice!=imp.getCurrentSlice() && slice!=0) continue;
+				}
 				//rm.select(i);
 				// this needs to run on a separate thread, at least on OS X
-				new ij.macro.MacroRunner("roiManager('select', "+i+"); roiManager('update');");
+				// "update2" does not clone the ROI so the "Show All"
+				// outline moves as the user moves the RO.
+				new ij.macro.MacroRunner("roiManager('select', "+i+"); roiManager('update2');");
 				return true;
 			}
 		}
@@ -1059,7 +1173,13 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		int ox = offScreenX(sx);
 		int oy = offScreenY(sy);
 		Roi roi = imp.getRoi();
-		int handle = roi!=null?roi.isHandle(sx, sy):-1;		
+		int handle = roi!=null?roi.isHandle(sx, sy):-1;
+		boolean multiPointMode = roi!=null && (roi instanceof PointRoi) && handle==-1
+			&& Toolbar.getToolId()==Toolbar.POINT && Toolbar.getMultiPointMode();
+		if (multiPointMode) {
+			imp.setRoi(((PointRoi)roi).addPoint(offScreenXD(sx), offScreenYD(sy)));
+			return;
+		}
 		setRoiModState(e, roi, handle);
 		if (roi!=null) {
 			if (handle>=0) {
@@ -1069,7 +1189,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			Rectangle r = roi.getBounds();
 			int type = roi.getType();
 			if (type==Roi.RECTANGLE && r.width==imp.getWidth() && r.height==imp.getHeight()
-			&& roi.getPasteMode()==Roi.NOT_PASTING) {
+			&& roi.getPasteMode()==Roi.NOT_PASTING && !(roi instanceof ImageRoi)) {
 				imp.killRoi();
 				return;
 			}
@@ -1116,20 +1236,38 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		disablePopupMenu = status;
 	}
 
+	/** Enables/disables the ROI Manager "Show All" mode. */
 	public void setShowAllROIs(boolean showAllROIs) {
 		this.showAllROIs = showAllROIs;
 	}
 
+	/** Returns the state of the ROI Manager "Show All" flag. */
 	public boolean getShowAllROIs() {
 		return showAllROIs;
 	}
 	
-	/** Returns the color used for "Show All" mode. */
-	public static Color getShowAllColor() {
-			return showAllColor;
+	/** Return the ROI Manager "Show All" list as an overlay. */
+	public Overlay getShowAllList() {
+		if (!showAllROIs) return null;
+		if (showAllList!=null) return showAllList;
+		RoiManager rm=RoiManager.getInstance();
+		if (rm==null) return null;
+		Roi[] rois = rm.getRoisAsArray();
+		if (rois.length==0) return null;
+		Overlay overlay = new Overlay();
+		for (int i=0; i<rois.length; i++)
+			overlay.add((Roi)rois[i].clone());
+		return overlay;
 	}
 
-	/** Sets the color used used for "Show All" mode. */
+	/** Returns the color used for "Show All" mode. */
+	public static Color getShowAllColor() {
+		if (showAllColor!=null && showAllColor.getRGB()==0xff80ffff)
+			showAllColor = Color.cyan;
+		return showAllColor;
+	}
+
+	/** Sets the color used used for the ROI Manager "Show All" mode. */
 	public static void setShowAllColor(Color c) {
 		if (c==null) return;
 		showAllColor = c;
@@ -1140,40 +1278,83 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			if (ic!=null && ic.getShowAllROIs()) img.draw();
 		}
 	}
-
-	public void setDisplayList(Vector list) {
-		displayList = list;
-		listColor = null;
-		if (list==null) customRoi = false;
-		if (list!=null&&list.size()>0&&((Roi)list.elementAt(0)).getInstanceColor()!=null)
-			labelListItems = false;
-		else
-			labelListItems = true;
-		repaint();
-	}
-
-	public void setDisplayList(Shape shape, Color color, BasicStroke stroke) {
-		if (shape==null)
-			{setDisplayList(null); return;}
-		Roi roi = new ShapeRoi(shape);
-		roi.setInstanceColor(color);
-		Vector list = new Vector();
-		list.addElement(roi);
-		displayList = list;
-		labelListItems = false;
-		listColor = color;
-		listStroke = stroke;
+	
+	/** Use ImagePlus.setOverlay(ij.gui.Overlay). */
+	public void setOverlay(Overlay overlay) {
+		this.overlay = overlay;
 		repaint();
 	}
 	
+	/** Use ImagePlus.getOverlay(). */
+	public Overlay getOverlay() {
+		return overlay;
+	}
+
+	/**
+	* @deprecated
+	* replaced by ImagePlus.setOverlay(ij.gui.Overlay)
+	*/
+	public void setDisplayList(Vector list) {
+		if (list!=null) {
+			Overlay list2 = new Overlay();
+			list2.setVector(list);
+			setOverlay(list2);
+		} else
+			setOverlay(null);
+		if (overlay!=null)
+			overlay.drawLabels(overlay.size()>0&&overlay.get(0).getStrokeColor()==null);
+		else
+			customRoi = false;
+		repaint();
+	}
+
+	/**
+	* @deprecated
+	* replaced by ImagePlus.setOverlay(Shape, Color, BasicStroke)
+	*/
+	public void setDisplayList(Shape shape, Color color, BasicStroke stroke) {
+		if (shape==null)
+			{setOverlay(null); return;}
+		Roi roi = new ShapeRoi(shape);
+		roi.setStrokeColor(color);
+		roi.setStroke(stroke);
+		Overlay list = new Overlay();
+		list.add(roi);
+		setOverlay(list);
+	}
+	
+	/**
+	* @deprecated
+	* replaced by ImagePlus.setOverlay(Roi, Color, int, Color)
+	*/
+	public void setDisplayList(Roi roi, Color color) {
+		roi.setStrokeColor(color);
+		Overlay list = new Overlay();
+		list.add(roi);
+		setOverlay(list);
+	}
+	
+	/**
+	* @deprecated
+	* replaced by ImagePlus.getOverlay()
+	*/
 	public Vector getDisplayList() {
+		if (overlay==null) return null;
+		Vector displayList = new Vector();
+		for (int i=0; i<overlay.size(); i++)
+			displayList.add(overlay.get(i));
 		return displayList;
 	}
 	
+	/** Allows plugins (e.g., Orthogonal_Views) to create a custom ROI using a display list. */
 	public void setCustomRoi(boolean customRoi) {
 		this.customRoi = customRoi;
 	}
 
+	public boolean getCustomRoi() {
+		return customRoi;
+	}
+	
 	/** Called by IJ.showStatus() to prevent status bar text from
 		being overwritten until the cursor moves at least 12 pixels. */
 	public void setShowCursorStatus(boolean status) {
@@ -1208,9 +1389,9 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			}
 		}
 	}
-
+	
 	public void mouseMoved(MouseEvent e) {
-		if (ij==null) return;
+		//if (ij==null) return;
 		int sx = e.getX();
 		int sy = e.getY();
 		int ox = offScreenX(sx);
@@ -1222,7 +1403,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		if (roi!=null && (roi.getType()==Roi.POLYGON || roi.getType()==Roi.POLYLINE || roi.getType()==Roi.ANGLE) 
 		&& roi.getState()==roi.CONSTRUCTING) {
 			PolygonRoi pRoi = (PolygonRoi)roi;
-			pRoi.handleMouseMove(ox, oy);
+			pRoi.handleMouseMove(sx, sy);
 		} else {
 			if (ox<imageWidth && oy<imageHeight) {
 				ImageWindow win = imp.getWindow();
@@ -1240,4 +1421,4 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 	public void mouseClicked(MouseEvent e) {}
 	public void mouseEntered(MouseEvent e) {}
 
-}
\ No newline at end of file
+}
