@@ -38,6 +38,7 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 	private static int yloc;
 	private static int count;
 	private static boolean centerOnScreen;
+	private static Point nextLocation;
 	
     private int textGap = centerOnScreen?0:TEXT_GAP;
 	
@@ -68,6 +69,7 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
         	else
         		setBackground(Color.white);
         }
+		boolean hyperstack = imp.isHyperStack();
 		ij = IJ.getInstance();
 		this.imp = imp;
 		if (ic==null)
@@ -84,8 +86,10 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 		if (!(this instanceof StackWindow))
 			addMouseWheelListener(this);
 		setResizable(true);
-		WindowManager.addWindow(this);
-		imp.setWindow(this);
+		if (!(this instanceof HistogramWindow&&IJ.isMacro()&&Interpreter.isBatchMode())) {
+			WindowManager.addWindow(this);
+			imp.setWindow(this);
+		}
 		if (previousWindow!=null) {
 			if (newCanvas)
 				setLocationAndSize(false);
@@ -97,6 +101,8 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 				pack();
 				show();
 			}
+			if (ic.getMagnification()!=0.0)
+				imp.setTitle(imp.getTitle());
 			boolean unlocked = imp.lockSilently();
 			boolean changes = imp.changes;
 			imp.changes = false;
@@ -104,6 +110,8 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 			imp.changes = changes;
 			if (unlocked)
 				imp.unlock();
+			if (hyperstack && this.imp!=null)
+				this.imp.setOpenAsHyperStack(true);
 			WindowManager.setCurrentWindow(this);
 		} else {
 			setLocationAndSize(false);
@@ -115,8 +123,11 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 			if (centerOnScreen) {
 				GUI.center(this);
 				centerOnScreen = false;
+			} else if (nextLocation!=null) {
+				setLocation(nextLocation);
+				nextLocation = null;
 			}
-			if (Interpreter.isBatchMode()) {
+			if (Interpreter.isBatchMode() || (IJ.getInstance()==null&&this instanceof HistogramWindow)) {
 				WindowManager.setTempCurrentImage(imp);
 				Interpreter.addBatchModeImage(imp);
 			} else
@@ -127,9 +138,9 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 	private void setLocationAndSize(boolean updating) {
 		int width = imp.getWidth();
 		int height = imp.getHeight();
-		Rectangle maxWindow = getMaxWindow();
-		if (maxWindow.x==maxWindow.width)  // work around for Linux bug
-			maxWindow = new Rectangle(0, maxWindow.y, maxWindow.width, maxWindow.height);
+		Rectangle maxWindow = getMaxWindow(0,0);
+		//if (maxWindow.x==maxWindow.width)  // work around for Linux bug
+		//	maxWindow = new Rectangle(0, maxWindow.y, maxWindow.width, maxWindow.height);
 		if (WindowManager.getWindowCount()<=1)
 			xbase = -1;
 		if (width>maxWindow.width/2 && xbase>maxWindow.x+5+XINC*6)
@@ -154,9 +165,10 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 		}
 
 		int sliderHeight = (this instanceof StackWindow)?20:0;
-		int screenHeight = maxWindow.height-sliderHeight;
+		int screenHeight = maxWindow.y+maxWindow.height-sliderHeight;
 		double mag = 1;
-		while (xbase+XINC*4+width*mag>maxWindow.width || ybase+height*mag>screenHeight) {
+		while (xbase+XINC*4+width*mag>maxWindow.x+maxWindow.width || ybase+height*mag>=screenHeight) {
+			//IJ.log(mag+"  "+xbase+"  "+width*mag+"  "+maxWindow.width);
 			double mag2 = ImageCanvas.getLowerZoomLevel(mag);
 			if (mag2==mag) break;
 			mag = mag2;
@@ -179,17 +191,39 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 			pack();
 	}
 				
-	Rectangle getMaxWindow() {
+	Rectangle getMaxWindow(int xloc, int yloc) {
 		GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
-		Rectangle maxWindow = ge.getMaximumWindowBounds();
-		Dimension ijSize = ij!=null?ij.getSize():new Dimension(0,0);
-		if (maxWindow.height>600) {
-			maxWindow.y += ijSize.height;
-			maxWindow.height -= ijSize.height;
+		Rectangle bounds = ge.getMaximumWindowBounds();
+		//bounds.x=960; bounds.y=0; bounds.width=960; bounds.height=1200;
+		if (IJ.debugMode) IJ.log("getMaxWindow: "+bounds+"  "+xloc+","+yloc);
+		if (xloc>bounds.x+bounds.width || yloc>bounds.y+bounds.height) {
+			Rectangle bounds2 = getSecondaryMonitorBounds(ge, xloc, yloc);
+			if (bounds2!=null) return bounds2;
 		}
-		return maxWindow;
+		Dimension ijSize = ij!=null?ij.getSize():new Dimension(0,0);
+		if (bounds.height>600) {
+			bounds.y += ijSize.height;
+			bounds.height -= ijSize.height;
+		}
+		return bounds;
 	}
-
+	
+	private Rectangle getSecondaryMonitorBounds(GraphicsEnvironment ge, int xloc, int yloc) {
+		//IJ.log("getSecondaryMonitorBounds "+wb);
+		GraphicsDevice[] gs = ge.getScreenDevices();
+		for (int j=0; j<gs.length; j++) {
+			GraphicsDevice gd = gs[j];
+			GraphicsConfiguration[] gc = gd.getConfigurations();
+			for (int i=0; i<gc.length; i++) {
+				Rectangle bounds = gc[i].getBounds();
+				//IJ.log(j+" "+i+" "+bounds+"  "+bounds.contains(wb.x, wb.y));
+				if (bounds!=null && bounds.contains(xloc, yloc))
+					return bounds;
+			}
+		}		
+		return null;
+	}
+	
 	public double getInitialMagnification() {
 		return initialMagnification;
 	}
@@ -306,8 +340,9 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 	public boolean close() {
 		boolean isRunning = running || running2;
 		running = running2 = false;
+		boolean virtual = imp.getStackSize()>1 && imp.getStack().isVirtual();
 		if (isRunning) IJ.wait(500);
-		if (ij==null || IJ.getApplet()!=null || Interpreter.isBatchMode() || IJ.macroRunning())
+		if (ij==null || IJ.getApplet()!=null || Interpreter.isBatchMode() || IJ.macroRunning() || virtual)
 			imp.changes = false;
 		if (imp.changes) {
 			String msg;
@@ -328,11 +363,13 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 		if (WindowManager.getWindowCount()==0)
 			{xloc = 0; yloc = 0;}
 		WindowManager.removeWindow(this);
-		setVisible(false);
+		//setVisible(false);
 		if (ij!=null && ij.quitting())  // this may help avoid thread deadlocks
 			return true;
 		dispose();
-		imp.flush();
+		if (imp!=null)
+			imp.flush();
+		imp = null;
 		return true;
 	}
 	
@@ -340,9 +377,15 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 		return imp;
 	}
 
-
-	void setImagePlus(ImagePlus imp) {
-		this.imp = imp;
+	public void setImage(ImagePlus imp2) {
+		ImageCanvas ic = getCanvas();
+		if (ic==null || imp2==null)
+			return;
+		imp = imp2;
+		imp.setWindow(this);
+		ic.updateImage(imp);
+		ic.setImageUpdated();
+		ic.repaint();
 		repaint();
 	}
 	
@@ -352,13 +395,20 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 		this.imp = imp;
         ic.updateImage(imp);
         setLocationAndSize(true);
+        if (this instanceof StackWindow) {
+        	StackWindow sw = (StackWindow)this;
+        	int stackSize = imp.getStackSize();
+        	int nScrollbars = sw.getNScrollbars();
+        	if (stackSize==1 && nScrollbars>0)
+        		sw.removeScrollbars();
+        	else if (stackSize>1 && nScrollbars==0)
+        		sw.addScrollbars(imp);
+        }
         pack();
 		repaint();
 		maxBounds = getMaximumBounds();
-		//if (!IJ.isLinux()) {
-			setMaximizedBounds(maxBounds);
-			setMaxBoundsTime = System.currentTimeMillis();
-		//}
+		setMaximizedBounds(maxBounds);
+		setMaxBoundsTime = System.currentTimeMillis();
 	}
 
 	public ImageCanvas getCanvas() {
@@ -407,7 +457,7 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 		int extraHeight = insets.top+insets.bottom + 10;
 		if (extraHeight==20) extraHeight = 42;
 		int members = getComponentCount();
-		if (IJ.debugMode) IJ.log("getExtraSize: "+members+" "+insets);
+		//if (IJ.debugMode) IJ.log("getExtraSize: "+members+" "+insets);
 		for (int i=1; i<members; i++) {
 		    Component m = getComponent(i);
 		    Dimension d = m.getPreferredSize();
@@ -419,7 +469,6 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 
 	public Component add(Component comp) {
 		comp = super.add(comp);
-		if (IJ.debugMode) IJ.log("add: "+comp);
 		maxBounds = getMaximumBounds();
 		//if (!IJ.isLinux()) {
 			setMaximizedBounds(maxBounds);
@@ -497,7 +546,7 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 			WindowManager.setCurrentWindow(this);
 			IJ.doCommand("Close");
 		} else {
-			setVisible(false);
+			//setVisible(false);
 			dispose();
 			WindowManager.removeWindow(this);
 		}
@@ -559,7 +608,7 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
     }
     
     public String toString() {
-    	return imp.getTitle();
+    	return imp!=null?imp.getTitle():"";
     }
     
     /** Causes the next image to be opened to be centered on the screen
@@ -568,6 +617,11 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
     	centerOnScreen = true;
     }
     
+    /** Causes the next image to be displayed at the specified location. */
+    public static void setNextLocation(Point loc) {
+    	nextLocation = loc;
+    }
+
     /** Moves and resizes this window. Changes the 
     	 magnification so the image fills the window. */
     public void setLocationAndSize(int x, int y, int width, int height) {
@@ -575,7 +629,7 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 		getCanvas().fitToWindow();
 		pack();
 	}
-
+	
 	/** Overrides the setBounds() method in Component so
 		we can find out when the window is resized. */
 	//public void setBounds(int x, int y, int width, int height)	{
