@@ -12,6 +12,7 @@ public class PolygonRoi extends Roi {
 
 	protected int maxPoints = 1000; // will be increased if necessary
 	protected int[] xp, yp; 	// image coordinates relative to origin of roi bounding box
+	protected float[] xpf, ypf; 	// or alternative sub-pixel coordinates
 	protected int[] xp2, yp2;	// absolute screen coordinates
 	protected int nPoints;
 	protected float[] xSpline,ySpline; // relative image coordinates
@@ -21,6 +22,8 @@ public class PolygonRoi extends Roi {
 	private double angle1, degrees=Double.NaN;
 	private int xClipMin, yClipMin, xClipMax, yClipMax;
 	private boolean userCreated;
+	private boolean subPixel;
+	private double startXD, startYD;
 
 	long mouseUpTime = 0;
 
@@ -28,6 +31,38 @@ public class PolygonRoi extends Roi {
 		Type must be Roi.POLYGON, Roi.FREEROI, Roi.TRACED_ROI, Roi.POLYLINE, Roi.FREELINE or Roi.ANGLE.*/
 	public PolygonRoi(int[] xPoints, int[] yPoints, int nPoints, int type) {
 		super(0, 0, null);
+		init1(nPoints, type);
+		xp = xPoints;
+		yp = yPoints;
+		if (type!=TRACED_ROI) {
+			xp = new int[nPoints];
+			yp = new int[nPoints];
+			for (int i=0; i<nPoints; i++) {
+				xp[i] = xPoints[i];
+				yp[i] = yPoints[i];
+			}
+		}
+		xp2 = new int[nPoints];
+		yp2 = new int[nPoints];
+		init2(type);
+	}
+	
+	/** Creates a new polygon or polyline ROI from x and y float arrays.
+		Type must be Roi.POLYGON, Roi.FREEROI, Roi.POLYLINE, Roi.FREELINE or Roi.ANGLE.*/
+	public PolygonRoi(float[] xPoints, float[] yPoints, int nPoints, int type) {
+		super(0, 0, null);
+		init1(nPoints, type);
+		xpf = xPoints;
+		ypf = yPoints;
+		subPixel = true;
+		xp2 = new int[nPoints];
+		yp2 = new int[nPoints];
+		init2(type);
+	}
+
+	private void init1(int nPoints, int type) throws IllegalArgumentException{
+		maxPoints = nPoints;
+		this.nPoints = nPoints;
 		if (type==POLYGON)
 			this.type = POLYGON;
 		else if (type==FREEROI)
@@ -44,22 +79,19 @@ public class PolygonRoi extends Roi {
 			this.type = POINT;
 		else
 			throw new IllegalArgumentException("Invalid type");
-		maxPoints = nPoints;
-		this.nPoints = nPoints;
-		xp = xPoints;
-		yp = yPoints;
-		if (type!=TRACED_ROI) {
-			xp = new int[nPoints];
-			yp = new int[nPoints];
-			for (int i=0; i<nPoints; i++) {
-				xp[i] = xPoints[i];
-				yp[i] = yPoints[i];
-			}
-		}
-		xp2 = new int[nPoints];
-		yp2 = new int[nPoints];
+	}
+
+	private void init2(int type) {
 		if (type==ANGLE && nPoints==3)
 			getAngleAsString();
+		if (type==POINT && Toolbar.getMultiPointMode()) {
+			Prefs.pointAutoMeasure = false;
+			Prefs.pointAutoNextSlice = false;
+			Prefs.pointAddToManager = false;
+			userCreated = true;
+		}
+		if (lineWidth>1 && isLine())
+			updateWideLine(lineWidth);
 		finishPolygon();
 	}
 	
@@ -69,7 +101,13 @@ public class PolygonRoi extends Roi {
 		this(p.xpoints, p.ypoints, p.npoints, type);
 	}
 
-	/** Obsolete */
+	/** Creates a new polygon or polyline ROI from a FloatPolygon. Type must be Roi.POLYGON, 
+		Roi.FREEROI, Roi.TRACED_ROI, Roi.POLYLINE, Roi.FREELINE or Roi.ANGLE.*/
+	public PolygonRoi(FloatPolygon p, int type) {
+		this(p.xpoints, p.ypoints, p.npoints, type);
+	}
+
+	/** @deprecated */
 	public PolygonRoi(int[] xPoints, int[] yPoints, int nPoints, ImagePlus imp, int type) {
 		this(xPoints, yPoints, nPoints, type);
 		setImage(imp);
@@ -79,19 +117,45 @@ public class PolygonRoi extends Roi {
 	public PolygonRoi(int sx, int sy, ImagePlus imp) {
 		super(sx, sy, imp);
 		int tool = Toolbar.getToolId();
-		if (tool==Toolbar.POLYGON)
-			type = POLYGON;
-		else if (tool==Toolbar.ANGLE)
-			type = ANGLE;
-		else
-			type = POLYLINE;
-		xp = new int[maxPoints];
-		yp = new int[maxPoints];
+		switch (tool) {
+			case Toolbar.POLYGON:
+				type = POLYGON;
+				break;
+			case Toolbar.FREEROI:
+				type = FREEROI;
+				break;
+			case Toolbar.FREELINE:
+				type = FREELINE;
+				subPixel = true;
+				break;
+			case Toolbar.ANGLE:
+				type = ANGLE;
+				break;
+			default:
+				type = POLYLINE;
+				subPixel = true;
+				break;
+		}
+		if (this instanceof EllipseRoi)
+			subPixel = true;
+		x = ic.offScreenX(sx);
+		y = ic.offScreenY(sy);
+		startXD = subPixelResolution()?ic.offScreenXD(sx):x;
+		startYD = subPixelResolution()?ic.offScreenYD(sy):y;
+		if (subPixelResolution()) {
+			xpf = new float[maxPoints];
+			ypf = new float[maxPoints];
+			xpf[0] = (float)(startXD-x);
+			ypf[0] = (float)(startYD-y);
+			xpf[1] = xpf[0];
+			ypf[1] = ypf[0];
+		} else {
+			xp = new int[maxPoints];
+			yp = new int[maxPoints];
+		}
 		xp2 = new int[maxPoints];
 		yp2 = new int[maxPoints];
 		nPoints = 2;
-		x = ic.offScreenX(sx);
-		y = ic.offScreenY(sy);
 		width=1;
 		height=1;
 		clipX = x;
@@ -100,44 +164,65 @@ public class PolygonRoi extends Roi {
 		clipHeight = 1;
 		state = CONSTRUCTING;
 		userCreated = true;
+		if (lineWidth>1 && isLine())
+			updateWideLine(lineWidth);
 	}
 
 	private void drawStartBox(Graphics g) {
-		if (type!=ANGLE)
-			g.drawRect(ic.screenX(startX)-4, ic.screenY(startY)-4, 8, 8);
+		if (type!=ANGLE) {
+			double offset = subPixelResolution()?0.5:0.0;
+			g.drawRect(ic.screenXD(startXD+offset)-4, ic.screenYD(startYD+offset)-4, 8, 8);
+		}
 	}
 	
 	public void draw(Graphics g) {
         updatePolygon();
-        g.setColor(instanceColor!=null?instanceColor:ROIColor);
+		Color color =  strokeColor!=null? strokeColor:ROIColor;
+		boolean fill = false;
+        mag = getMagnification();
+		if (fillColor!=null && !isLine() && state!=CONSTRUCTING) {
+			color = fillColor;
+			fill = true;
+		}
+		g.setColor(color);
+		Graphics2D g2d = (Graphics2D)g;
+		if (stroke!=null)
+			g2d.setStroke(getScaledStroke());
         if (xSpline!=null) {
             if (type==POLYLINE || type==FREELINE) {
-                if (lineWidth>1)
-                	drawWideLine(g, xSpline, ySpline, splinePoints);
-                else
-                	drawSpline(g, xSpline, ySpline, splinePoints, false);
+                drawSpline(g, xSpline, ySpline, splinePoints, false, fill);
+                if (wideLine && !overlay) {
+                	g2d.setStroke(onePixelWide);
+                	g.setColor(getColor());
+                	drawSpline(g, xSpline, ySpline, splinePoints, false, fill);
+                }
             } else
-                	drawSpline(g, xSpline, ySpline, splinePoints, true);
+                drawSpline(g, xSpline, ySpline, splinePoints, true, fill);
         } else {
             if (type==POLYLINE || type==FREELINE || type==ANGLE || state==CONSTRUCTING) {
-                if (lineWidth>1 && isLine())
-                	drawWideLine(g, toFloat(xp), toFloat(yp), nPoints);
                 g.drawPolyline(xp2, yp2, nPoints);
-            } else
-                g.drawPolygon(xp2, yp2, nPoints);
+                if (wideLine && !overlay) {
+                	g2d.setStroke(onePixelWide);
+                	g.setColor(getColor());
+                	g.drawPolyline(xp2, yp2, nPoints);
+                }
+            } else {
+            	if (fill)
+                	g.fillPolygon(xp2, yp2, nPoints);
+                else
+                	g.drawPolygon(xp2, yp2, nPoints);
+             }
             if (state==CONSTRUCTING && type!=FREEROI && type!=FREELINE)
                 drawStartBox(g);
         }
-        //g.drawRect(clip.x, clip.y, clip.width, clip.height);
         if ((xSpline!=null||type==POLYGON||type==POLYLINE||type==ANGLE)
-        && state!=CONSTRUCTING && clipboard==null) {
-            if (ic!=null) mag = ic.getMagnification();
+        && state!=CONSTRUCTING && clipboard==null && !overlay) {
             int size2 = HANDLE_SIZE/2;
             if (activeHandle>0)
                 drawHandle(g, xp2[activeHandle-1]-size2, yp2[activeHandle-1]-size2);
             if (activeHandle<nPoints-1)
                 drawHandle(g, xp2[activeHandle+1]-size2, yp2[activeHandle+1]-size2);
-            handleColor=instanceColor!=null?instanceColor:ROIColor; drawHandle(g, xp2[0]-size2, yp2[0]-size2); handleColor=Color.white;
+            handleColor= strokeColor!=null? strokeColor:ROIColor; drawHandle(g, xp2[0]-size2, yp2[0]-size2); handleColor=Color.white;
             for (int i=1; i<nPoints; i++)
                 drawHandle(g, xp2[i]-size2, yp2[i]-size2);
         }
@@ -148,64 +233,51 @@ public class PolygonRoi extends Roi {
             {updateFullWindow = false; imp.draw();}
 	}
 	
- 	private void drawSpline(Graphics g, float[] xpoints, float[] ypoints, int npoints, boolean closed) {
- 		if (ic==null) return;
-  		Rectangle srcRect = ic.getSrcRect();
- 		float srcx=srcRect.x, srcy=srcRect.y;
- 		float mag = (float)ic.getMagnification();
- 		float xf=x, yf=y;
+ 	private void drawSpline(Graphics g, float[] xpoints, float[] ypoints, int npoints, boolean closed, boolean fill) {
+ 		double srcx=0.0, srcy=0.9, mag=1.0;
+ 		if (ic!=null) {
+			Rectangle srcRect = ic.getSrcRect();
+			srcx=srcRect.x; srcy=srcRect.y;
+			mag = ic.getMagnification();
+ 		}
+ 		double xd=x, yd=y;
 		Graphics2D g2d = (Graphics2D)g;
 		GeneralPath path = new GeneralPath();
-		if (mag==1f && srcx==0f && srcy==0f) {
-			path.moveTo(xpoints[0]+xf, ypoints[0]+yf);
-			for (int i=0; i<npoints; i++)
-				path.lineTo(xpoints[i]+xf, ypoints[i]+yf);
+		double offset = subPixelResolution()&&mag>1.0&&type==POLYLINE?0.5:0.0;
+		if (mag==1.0 && srcx==0.0 && srcy==0.0) {
+			path.moveTo(xpoints[0]+xd, ypoints[0]+yd);
+			for (int i=1; i<npoints; i++)
+				path.lineTo(xpoints[i]+xd, ypoints[i]+yd);
 		} else {
-			path.moveTo((xpoints[0]-srcx+xf)*mag, (ypoints[0]-srcy+yf)*mag);
-			for (int i=0; i<npoints; i++)
-				path.lineTo((xpoints[i]-srcx+xf)*mag, (ypoints[i]-srcy+yf)*mag);
+			path.moveTo((xpoints[0]-srcx+xd)*mag+offset, (ypoints[0]-srcy+yd+offset)*mag);
+			for (int i=1; i<npoints; i++)
+				path.lineTo((xpoints[i]-srcx+xd+offset)*mag, (ypoints[i]-srcy+yd+offset)*mag);
 		}
 		if (closed)
-			path.lineTo((xpoints[0]-srcx+xf)*mag, (ypoints[0]-srcy+yf)*mag);
-		g2d.draw(path);
-	}
-
- 	private void drawWideLine(Graphics g, float[] xpoints, float[] ypoints, int npoints) {
- 		if (ic==null) return;
- 		Rectangle srcRect = ic.getSrcRect();
- 		float srcx=srcRect.x, srcy=srcRect.y;
- 		float mag = (float)ic.getMagnification();
- 		float xf=x, yf=y;
-		Graphics2D g2d = (Graphics2D)g;
-		GeneralPath path = new GeneralPath();
-		if (mag==1f && srcx==0f && srcy==0f) {
-			path.moveTo(xpoints[0]+xf, ypoints[0]+yf);
-			for (int i=0; i<npoints; i++)
-				path.lineTo(xpoints[i]+xf, ypoints[i]+yf);
-		} else {
-			path.moveTo((xpoints[0]-srcx+xf)*mag, (ypoints[0]-srcy+yf)*mag);
-			for (int i=0; i<npoints; i++)
-				path.lineTo((xpoints[i]-srcx+xf)*mag, (ypoints[i]-srcy+yf)*mag);
-		}
-		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
-		AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,0.25f);
-		g2d.setComposite(ac);
-		g2d.setStroke(new BasicStroke((float)(lineWidth*ic.getMagnification()),BasicStroke.CAP_BUTT,BasicStroke.JOIN_ROUND));
-		g2d.draw(path);
-		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
-		ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,1f);
-		g2d.setStroke(new BasicStroke(1));
-		g2d.setComposite(ac);
-		g2d.draw(path);
+			path.lineTo((xpoints[0]-srcx+xd+offset)*mag, (ypoints[0]-srcy+yd+offset)*mag);
+		if (fill)
+			g2d.fill(path);
+		else
+			g2d.draw(path);
 	}
 
 	public void drawPixels(ImageProcessor ip) {
+		int saveWidth = ip.getLineWidth();
+		if (getStrokeWidth()>1f)
+			ip.setLineWidth((int)Math.round(getStrokeWidth()));
+		double offset = subPixelResolution()&&getMagnification()>1.0&&(type==POLYLINE||type==FREELINE)?0.5:0.0;
 		if (xSpline!=null) {
-			ip.moveTo(x+(int)(Math.floor(xSpline[0])+0.5), y+(int)Math.floor(ySpline[0]+0.5));
+			ip.moveTo(x+(int)(Math.floor(xSpline[0])+offset), y+(int)Math.floor(ySpline[0]+offset));
 			for (int i=1; i<splinePoints; i++)
-				ip.lineTo(x+(int)(Math.floor(xSpline[i])+0.5), y+(int)Math.floor(ySpline[i]+0.5));
+				ip.lineTo(x+(int)(Math.floor(xSpline[i])+offset), y+(int)Math.floor(ySpline[i]+offset));
 			if (type==POLYGON || type==FREEROI || type==TRACED_ROI)
-				ip.lineTo(x+(int)(Math.floor(xSpline[0])+0.5), y+(int)Math.floor(ySpline[0]+0.5));
+				ip.lineTo(x+(int)(Math.floor(xSpline[0])+offset), y+(int)Math.floor(ySpline[0]+offset));
+		} else if (xpf!=null) {
+			ip.moveTo(x+(int)(Math.floor(xpf[0])+offset), y+(int)Math.floor(ypf[0]+offset));
+			for (int i=1; i<nPoints; i++)
+				ip.lineTo(x+(int)(Math.floor(xpf[i])+offset), y+(int)Math.floor(ypf[i]+offset));
+			if (type==POLYGON || type==FREEROI || type==TRACED_ROI)
+				ip.lineTo(x+(int)(Math.floor(xpf[0])+offset), y+(int)Math.floor(ypf[0]+offset));
 		} else {
 			ip.moveTo(x+xp[0], y+yp[0]);
 			for (int i=1; i<nPoints; i++)
@@ -213,8 +285,8 @@ public class PolygonRoi extends Roi {
 			if (type==POLYGON || type==FREEROI || type==TRACED_ROI)
 				ip.lineTo(x+xp[0], y+yp[0]);
 		}
-		if (xSpline!=null || (lineWidth>1&&isLine()))
-			updateFullWindow = true;
+		ip.setLineWidth(saveWidth);
+		updateFullWindow = true;
 	}
 
 	protected void grow(int sx, int sy) {
@@ -223,38 +295,65 @@ public class PolygonRoi extends Roi {
 
 
 	protected void updatePolygon() {
-		if (ic==null) return;
-		Rectangle srcRect = ic.getSrcRect();
-		if (ic.getMagnification()==1.0 && srcRect.x==0 && srcRect.y==0) {
-			for (int i=0; i<nPoints; i++) {
-				xp2[i] = xp[i]+x;
-				yp2[i] = yp[i]+y;
+		int basex=0, basey=0;
+		if (ic!=null) {
+			Rectangle srcRect = ic.getSrcRect();
+			basex=srcRect.x; basey=srcRect.y;
+		}
+		double mag = getMagnification();
+		if (mag==1.0 && basex==0 && basey==0) {
+			if (xpf!=null) {
+				for (int i=0; i<nPoints; i++) {
+					xp2[i] = (int)(xpf[i]+x);
+					yp2[i] = (int)(ypf[i]+y);
+				}
+			} else {
+				for (int i=0; i<nPoints; i++) {
+					xp2[i] = xp[i]+x;
+					yp2[i] = yp[i]+y;
+				}
 			}
 		} else {
-			for (int i=0; i<nPoints; i++) {
-				xp2[i] = ic.screenX(xp[i]+x);
-				yp2[i] = ic.screenY(yp[i]+y);
+			if (xpf!=null) {
+				double offset = subPixelResolution()&&mag>1.0&&(type==POLYLINE||type==FREELINE)?0.5:0.0;
+				for (int i=0; i<nPoints; i++) {
+					xp2[i] = ic.screenXD(xpf[i]+x+offset);
+					yp2[i] = ic.screenYD(ypf[i]+y+offset);
+				}
+			} else {
+				for (int i=0; i<nPoints; i++) {
+					xp2[i] = ic.screenX(xp[i]+x);
+					yp2[i] = ic.screenY(yp[i]+y);
+				}
 			}
 		}
 	}
 
-	void handleMouseMove(int ox, int oy) {
-	// Do rubber banding
+	void handleMouseMove(int sx, int sy) {
+		// Do rubber banding
 		int tool = Toolbar.getToolId();
 		if (!(tool==Toolbar.POLYGON || tool==Toolbar.POLYLINE || tool==Toolbar.ANGLE)) {
 			imp.killRoi();
 			imp.draw();
 			return;
 		}
-        drawRubberBand(ox, oy);
+        drawRubberBand(sx, sy);
 		degrees = Double.NaN;
 		double len = -1;
 		if (nPoints>1) {
-			int x1 = xp[nPoints-2];
-			int y1 = yp[nPoints-2];
-			int x2 = xp[nPoints-1];
-			int y2 = yp[nPoints-1];
-			degrees = getAngle(x1, y1, x2, y2);
+			double x1, y1, x2, y2;
+			if (xpf!=null) {
+				x1 = xpf[nPoints-2];
+				y1 = ypf[nPoints-2];
+				x2 = xpf[nPoints-1];
+				y2 = ypf[nPoints-1];
+			} else {
+				x1 = xp[nPoints-2];
+				y1 = yp[nPoints-2];
+				x2 = xp[nPoints-1];
+				y2 = yp[nPoints-1];
+			}
+			degrees = getAngle((int)Math.round(x1), (int)Math.round(y1), (int)Math.round(x2), (int)Math.round(y2));
 			if (tool!=Toolbar.ANGLE) {
 				Calibration cal = imp.getCalibration();
 				double pw=cal.pixelWidth, ph=cal.pixelHeight;
@@ -273,15 +372,30 @@ public class PolygonRoi extends Roi {
 			}
 		}
 		String length = len!=-1?", length=" + IJ.d2s(len):"";
-		String angle = !Double.isNaN(degrees)?", angle=" + IJ.d2s(degrees):"";
+		double degrees2 = tool==Toolbar.ANGLE&&nPoints==3&&Prefs.reflexAngle?360.0-degrees:degrees;
+		String angle = !Double.isNaN(degrees)?", angle=" + IJ.d2s(degrees2):"";
+		int ox = ic!=null?ic.offScreenX(sx):sx;
+		int oy = ic!=null?ic.offScreenY(sy):sy;
 		IJ.showStatus(imp.getLocationAsString(ox,oy) + length + angle);
 	}
 
-	void drawRubberBand(int ox, int oy) {
-		int x1 = xp[nPoints-2]+x;
-		int y1 = yp[nPoints-2]+y;
-		int x2 = xp[nPoints-1]+x;
-		int y2 = yp[nPoints-1]+y;
+	void drawRubberBand(int sx, int sy) {
+		double oxd = ic!=null?ic.offScreenXD(sx):sx;
+		double oyd = ic!=null?ic.offScreenYD(sy):sy;
+		int ox = (int)oxd;
+		int oy = (int)oyd;
+		int x1, y1, x2, y2;
+		if (xpf!=null) {
+			x1 = (int)xpf[nPoints-2]+x;
+			y1 = (int)ypf[nPoints-2]+y;
+			x2 = (int)xpf[nPoints-1]+x;
+			y2 = (int)ypf[nPoints-1]+y;
+		} else {
+			x1 = xp[nPoints-2]+x;
+			y1 = yp[nPoints-2]+y;
+			x2 = xp[nPoints-1]+x;
+			y2 = yp[nPoints-1]+y;
+		}
 		int xmin=9999, ymin=9999, xmax=0, ymax=0;
 		if (x1<xmin) xmin=x1;
 		if (x2<xmin) xmin=x2;
@@ -297,55 +411,83 @@ public class PolygonRoi extends Roi {
 		if (oy>ymax) ymax=oy;
 		//clip = new Rectangle(xmin, ymin, xmax-xmin, ymax-ymin);
 		int margin = 4;
-		if (lineWidth>margin && isLine())
-			margin = lineWidth;
 		if (ic!=null) {
 			double mag = ic.getMagnification();
 			if (mag<1.0) margin = (int)(margin/mag);
 		}
-		xp[nPoints-1] = ox-x;
-		yp[nPoints-1] = oy-y;
+		margin = (int)(margin+getStrokeWidth());
+		if (xpf!=null) {
+			xpf[nPoints-1] = (float)(oxd-x);
+			ypf[nPoints-1] = (float)(oyd-y);
+		} else {
+			xp[nPoints-1] = ox-x;
+			yp[nPoints-1] = oy-y;
+		}
 		imp.draw(xmin-margin, ymin-margin, (xmax-xmin)+margin*2, (ymax-ymin)+margin*2);
 	}
 
     void finishPolygon() {
-		Polygon poly = new Polygon(xp, yp, nPoints);
-		Rectangle r = poly.getBounds();
+    	Rectangle r;
+    	if (xpf!=null) {
+			FloatPolygon poly = new FloatPolygon(xpf, ypf, nPoints);
+			r = poly.getBounds();
+		} else {
+			Polygon poly = new Polygon(xp, yp, nPoints);
+			r = poly.getBounds();
+		}
 		x = r.x;
 		y = r.y;
 		width = r.width;
 		height = r.height;
+		if (xpf!=null) {
+			for (int i=0; i<nPoints; i++) {
+				xpf[i] = (float)(xpf[i]-x);
+				ypf[i] = (float)(ypf[i]-y);
+			}
+        } else {
+			for (int i=0; i<nPoints; i++) {
+				xp[i] = xp[i]-x;
+				yp[i] = yp[i]-y;
+			}
+        }
 		if (nPoints<2 || (!(type==FREELINE||type==POLYLINE||type==ANGLE) && (nPoints<3||width==0||height==0))) {
 			if (imp!=null) imp.killRoi();
 			if (type!=POINT) return;
 		}
-        for (int i=0; i<nPoints; i++) {
-            xp[i] = xp[i]-x;
-            yp[i] = yp[i]-y;
-        }
 		state = NORMAL;
 		if (imp!=null && !(type==TRACED_ROI))
 			imp.draw(x-5, y-5, width+10, height+10);
 		oldX=x; oldY=y; oldWidth=width; oldHeight=height;
-		if (Recorder.record && userCreated && (type==POLYGON||type==POLYLINE||type==ANGLE))
+		if (Recorder.record && userCreated && (type==POLYGON||type==POLYLINE||type==ANGLE
+		||(type==POINT&&Recorder.scriptMode()&&nPoints==3)))
 			Recorder.recordRoi(getPolygon(), type);
 		if (type!=POINT) modifyRoi();
 		LineWidthAdjuster.update();
 	}
 	
+	public void exitConstructingMode() {
+		if (type==POLYLINE && state==CONSTRUCTING) {
+            addOffset();
+			finishPolygon();
+		}
+	}
+	
     protected void moveHandle(int sx, int sy) {
 		if (clipboard!=null) return;
 		int ox = ic.offScreenX(sx);
 		int oy = ic.offScreenY(sy);
-		xp[activeHandle] = ox-x;
-		yp[activeHandle] = oy-y;
+		if (xpf!=null) {
+			double offset = subPixelResolution()&&getMagnification()>1.0&&type==POLYLINE?-0.5:0.0;
+			xpf[activeHandle] = (float)(ic.offScreenXD(sx)-x+offset);
+			ypf[activeHandle] = (float)(ic.offScreenYD(sy)-y+offset);
+		} else {
+			xp[activeHandle] = ox-x;
+			yp[activeHandle] = oy-y;
+		}
 		if (xSpline!=null) {
 			fitSpline(splinePoints);
 			updateClipRect();
-			if (Line.getWidth()>1 && isLine())
-				imp.draw();
-			else
-				imp.draw(clipX, clipY, clipWidth, clipHeight);
+			imp.draw(clipX, clipY, clipWidth, clipHeight);
 			oldX = x; oldY = y;
 			oldWidth = width; oldHeight = height;
 		} else {
@@ -360,6 +502,10 @@ public class PolygonRoi extends Roi {
 
    /** After handle is moved, find clip rect and repaint. */
    void updateClipRectAndDraw() {
+   		if (xpf!=null) {
+   			xp = toInt(xpf, xp, nPoints);
+   			yp = toInt(ypf, yp, nPoints);
+   		}
 		int xmin=Integer.MAX_VALUE, ymin=Integer.MAX_VALUE, xmax=0, ymax=0;
 		int x2, y2;
 		if (activeHandle>0)
@@ -390,16 +536,21 @@ public class PolygonRoi extends Roi {
 		if (yClipMax>ymax2) ymax2 = yClipMax;
 		xClipMin=xmin; yClipMin=ymin; xClipMax=xmax; yClipMax=ymax;
 		double mag = ic.getMagnification();
-		int handleSize = type==POINT?HANDLE_SIZE+8:HANDLE_SIZE;
-		if (handleSize<lineWidth && isLine()) handleSize= lineWidth;
+		int handleSize = type==POINT?HANDLE_SIZE+12:HANDLE_SIZE;
+		if (handleSize<getStrokeWidth() && isLine()) handleSize = (int)getStrokeWidth() ;
 		int m = mag<1.0?(int)(handleSize/mag):handleSize;
+		m = (int)(m*getStrokeWidth());
 		imp.draw(xmin2-m, ymin2-m, xmax2-xmin2+m*2, ymax2-ymin2+m*2);
 	}
 
 	void resetBoundingRect() {
+   		if (xpf!=null) {
+   			xp = toInt(xpf, xp, nPoints);
+   			yp = toInt(ypf, yp, nPoints);
+   		}
 		int xmin=Integer.MAX_VALUE, xmax=-xmin, ymin=xmin, ymax=xmax;
 		int xx, yy;
-		for(int i=0; i<nPoints; i++) {
+		for (int i=0; i<nPoints; i++) {
 			xx = xp[i];
 			if (xx<xmin) xmin=xx;
 			if (xx>xmax) xmax=xx;
@@ -407,12 +558,22 @@ public class PolygonRoi extends Roi {
 			if (yy<ymin) ymin=yy;
 			if (yy>ymax) ymax=yy;
 		}
-		if (xmin!=0)
-		   for (int i=0; i<nPoints; i++)
-			   xp[i] -= xmin;
-		if (ymin!=0)
-		   for (int i=0; i<nPoints; i++)
-			   yp[i] -= ymin;
+		if (xmin!=0) {
+			for (int i=0; i<nPoints; i++)
+				xp[i] -= xmin;
+			if (xpf!=null) {
+				for (int i=0; i<nPoints; i++)
+					xpf[i] -= xmin;
+			}
+		}
+		if (ymin!=0) {
+			for (int i=0; i<nPoints; i++)
+				yp[i] -= ymin;
+			if (ypf!=null) {
+				for (int i=0; i<nPoints; i++)
+					ypf[i] -= ymin;
+			}
+		}
 		//IJ.log("reset: "+ymin+" "+before+" "+yp[0]);
 		x+=xmin; y+=ymin;
 		width=xmax-xmin; height=ymax-ymin;
@@ -424,18 +585,20 @@ public class PolygonRoi extends Roi {
 		degrees = Math.abs(180-Math.abs(angle1-angle2));
 		if (degrees>180.0)
 			degrees = 360.0-degrees;
-		return ", angle=" + IJ.d2s(degrees);
+		double degrees2 = Prefs.reflexAngle&&type==ANGLE?360.0-degrees:degrees;
+		return ", angle=" + IJ.d2s(degrees2);
 	}
    
    protected void mouseDownInHandle(int handle, int sx, int sy) {
         if (state==CONSTRUCTING)
             return;
 		int ox=ic.offScreenX(sx), oy=ic.offScreenY(sy);
+		double oxd=ic.offScreenXD(sx), oyd=ic.offScreenYD(sy);
 		if (IJ.altKeyDown() && !(nPoints<=3 && type!=POINT)) {
-			deleteHandle(ox, oy); 
+			deleteHandle(oxd, oyd); 
 			return;
 		} else if (IJ.shiftKeyDown() && type!=POINT) {
-			addHandle(ox, oy); 
+			addHandle(oxd, oyd); 
 			return;
 		}
 		state = MOVING_HANDLE;
@@ -444,17 +607,17 @@ public class PolygonRoi extends Roi {
 		xClipMin=ox-m; yClipMin=oy-m; xClipMax=ox+m; yClipMax=oy+m;
 	}
 
-	void deleteHandle(int ox, int oy) {
+	public void deleteHandle(double ox, double oy) {
 		if (imp==null) return;
 		if (nPoints<=1)
 			{imp.killRoi(); return;}
 		boolean splineFit = xSpline != null;
 		xSpline = null;
-		Polygon points = getPolygon();
+		FloatPolygon points = getFloatPolygon();
 		modState = NO_MODS;
 		if (previousRoi!=null) previousRoi.modState = NO_MODS;
 		int pointToDelete = getClosestPoint(ox, oy, points);
-		Polygon points2 = new Polygon();
+		FloatPolygon points2 = new FloatPolygon();
 		for (int i=0; i<points.npoints; i++) {
 			if (i!=pointToDelete)
 				points2.addPoint(points.xpoints[i], points.ypoints[i]);
@@ -462,53 +625,59 @@ public class PolygonRoi extends Roi {
 		if (type==POINT)
 			imp.setRoi(new PointRoi(points2.xpoints, points2.ypoints, points2.npoints));
 		else {
-			imp.setRoi(new PolygonRoi(points2, type));
+			if (subPixelResolution())
+				imp.setRoi(new PolygonRoi(points2, type));
+			else
+				imp.setRoi(new PolygonRoi(toInt(points2.xpoints), toInt(points2.ypoints), points2.npoints, type));
 			if (splineFit) 
 				((PolygonRoi)imp.getRoi()).fitSpline(splinePoints);
 		}
 	}
 	
-	void addHandle(int ox, int oy) {
+	void addHandle(double ox, double oy) {
 		if (imp==null || type==ANGLE) return;
 		boolean splineFit = xSpline != null;
 		xSpline = null;
-		Polygon points = getPolygon();
+		FloatPolygon points = getFloatPolygon();
 		int n = points.npoints;
 		modState = NO_MODS;
 		if (previousRoi!=null) previousRoi.modState = NO_MODS;
 		int pointToDuplicate = getClosestPoint(ox, oy, points);
-		Polygon points2 = new Polygon();
+		FloatPolygon points2 = new FloatPolygon();
 		for (int i2=0; i2<n; i2++) {
 			if (i2==pointToDuplicate) {
 				int i1 = i2-1;
 				if (i1==-1) i1 = isLine()?i2:n-1;
 				int i3 = i2+1;
 				if (i3==n) i3 = isLine()?i2:0;
-				int x1 = points.xpoints[i1]  + 2*(points.xpoints[i2] - points.xpoints[i1])/3;
-				int y1 = points.ypoints[i1] + 2*(points.ypoints[i2] - points.ypoints[i1])/3;
-				int x2 = points.xpoints[i2] + (points.xpoints[i3] - points.xpoints[i2])/3;
-				int y2 = points.ypoints[i2] + (points.ypoints[i3] - points.ypoints[i2])/3;
+				double x1 = points.xpoints[i1]  + 2*(points.xpoints[i2] - points.xpoints[i1])/3;
+				double y1 = points.ypoints[i1] + 2*(points.ypoints[i2] - points.ypoints[i1])/3;
+				double x2 = points.xpoints[i2] + (points.xpoints[i3] - points.xpoints[i2])/3;
+				double y2 = points.ypoints[i2] + (points.ypoints[i3] - points.ypoints[i2])/3;
 				points2.addPoint(x1, y1);
 				points2.addPoint(x2, y2);
 			} else
 				points2.addPoint(points.xpoints[i2], points.ypoints[i2]);
 		}
 		if (type==POINT)
-			imp.setRoi(new PointRoi(points2.xpoints, points2.ypoints, points2.npoints));
+			imp.setRoi(new PointRoi(points2));
 		else {
-			imp.setRoi(new PolygonRoi(points2, type));
+			if (subPixelResolution())
+				imp.setRoi(new PolygonRoi(points2, type));
+			else
+				imp.setRoi(new PolygonRoi(toInt(points2.xpoints), toInt(points2.ypoints), points2.npoints, type));
 			if (splineFit) 
 				((PolygonRoi)imp.getRoi()).fitSpline(splinePoints);
 		}
 	}
 
-	int getClosestPoint(int x, int y, Polygon points) {
+	int getClosestPoint(double x, double y, FloatPolygon points) {
 		int index = 0;
 		double distance = Double.MAX_VALUE;
 		for (int i=0; i<points.npoints; i++) {
 			double dx = points.xpoints[i] - x;
 			double dy = points.ypoints[i] - y;
-			double distance2 = Math.sqrt(dx*dx+dy*dy);
+			double distance2 = dx*dx+dy*dy;
 			if (distance2<distance) {
 				distance = distance2;
 				index = i;
@@ -516,8 +685,13 @@ public class PolygonRoi extends Roi {
 		}
 		return index;
 	}
-
+	
 	public void fitSpline(int evaluationPoints) {
+		if (xpf==null) {
+			xpf = toFloat(xp);
+			ypf = toFloat(yp);
+			subPixel = true;
+		}
 		if (xSpline==null || splinePoints!=evaluationPoints) {
 			splinePoints = evaluationPoints;
 			xSpline = new float[splinePoints];
@@ -526,16 +700,16 @@ public class PolygonRoi extends Roi {
 		int nNodes = nPoints;
 		if (type==POLYGON) {
 			nNodes++;
-			if (nNodes>=xp.length)
+			if (nNodes>=xpf.length)
 				enlargeArrays();
-			xp[nNodes-1] = xp[0];
-			yp[nNodes-1] = yp[0];
+			xpf[nNodes-1] = xpf[0];
+			ypf[nNodes-1] = ypf[0];
 		}
-		int[] xindex = new int[nNodes];
-		for(int i=0; i<nNodes; i++)
+		float[] xindex = new float[nNodes];
+		for (int i=0; i<nNodes; i++)
 			xindex[i] = i;
-		SplineFitter sfx = new SplineFitter(xindex, xp, nNodes);
-		SplineFitter sfy = new SplineFitter(xindex, yp, nNodes);
+		SplineFitter sfx = new SplineFitter(xindex, xpf, nNodes);
+		SplineFitter sfy = new SplineFitter(xindex, ypf, nNodes);
 	   
 		// Evaluate the splines at all points
 		double scale = (double)(nNodes-1)/(splinePoints-1);
@@ -543,11 +717,11 @@ public class PolygonRoi extends Roi {
 		float xmin=Float.MAX_VALUE, xmax=-xmin, ymin=xmin, ymax=xmax;
 		for(int i=0; i<splinePoints; i++) {
 			double xvalue = i*scale;
-			xs = (float)sfx.evalSpline(xindex, xp, nNodes, xvalue);
+			xs = (float)sfx.evalSpline(xindex, xpf, nNodes, xvalue);
 			if (xs<xmin) xmin=xs;
 			if (xs>xmax) xmax=xs;
 			xSpline[i] = xs;
-			ys = (float)sfy.evalSpline(xindex, yp, nNodes, xvalue);
+			ys = (float)sfy.evalSpline(xindex, ypf, nNodes, xvalue);
 			if (ys<ymin) ymin=ys;
 			if (ys>ymax) ymax=ys;
 			ySpline[i] = ys;
@@ -558,13 +732,13 @@ public class PolygonRoi extends Roi {
 		int iymax = (int)Math.floor(ymax+0.5f);
 		if (ixmin!=0) {
 		   for (int i=0; i<nPoints; i++)
-			   xp[i] -= ixmin;
+			   xpf[i] -= ixmin;
 		   for (int i=0; i<splinePoints; i++)
 			   xSpline[i] -= ixmin;
 		}
 		if (iymin!=0) {
 		   for (int i=0; i<nPoints; i++)
-			   yp[i] -= iymin;
+			   ypf[i] -= iymin;
 		   for (int i=0; i<splinePoints; i++)
 			   ySpline[i] -= iymin;
 		}
@@ -596,22 +770,11 @@ public class PolygonRoi extends Roi {
 		return xSpline!=null;
 	}
 
-		/*
-		xSpline = new float[nPoints];
-		ySpline = new float[nPoints];
-		for (int i=1; i<nPoints; i++) {
-			xSpline[i] = xp[i];
-			ySpline[i] = yp[i];
-		}
-		splinePoints = nPoints;
-		float[] xpoints = new float[splinePoints*10];
-		float[] ypoints = new float[splinePoints*10];
-		*/
-
 	/* Creates a spline fitted polygon with one pixel segment lengths 
 		that can be retrieved using the getFloatPolygon() method. */
 	public void fitSplineForStraightening() {
 		fitSpline((int)getUncalibratedLength()*2);
+		if (splinePoints==0) return;
 		float[] xpoints = new float[splinePoints*2];
 		float[] ypoints = new float[splinePoints*2];
 		xpoints[0] = xSpline[0];
@@ -655,15 +818,15 @@ public class PolygonRoi extends Roi {
 	}
 
 	public double getUncalibratedLength() {
-		if (imp==null) return nPoints/2;
-		Calibration cal = imp.getCalibration();
-		double spw=cal.pixelWidth, sph=cal.pixelHeight;
-		cal.pixelWidth=1.0; cal.pixelHeight=1.0;
+		ImagePlus saveImp = imp;
+		imp = null;
 		double length = getLength();
-		cal.pixelWidth=spw; cal.pixelHeight=sph;
+		imp = saveImp;
 		return length;
 	}
 	
+	/** With segmented selections, ignore first mouse up and finalize
+	    when user double-clicks, control-clicks or clicks in start box. */
 	protected void handleMouseUp(int sx, int sy) {
 		if (state==MOVING)
 			{state = NORMAL; return;}				
@@ -679,10 +842,14 @@ public class PolygonRoi extends Roi {
 			return;
 		if (IJ.spaceBarDown()) // is user scrolling image?
 			return;
-		boolean samePoint = (xp[nPoints-2]==xp[nPoints-1] && yp[nPoints-2]==yp[nPoints-1]);
-		Rectangle biggerStartBox = new Rectangle(ic.screenX(startX)-5, ic.screenY(startY)-5, 10, 10);
+		boolean samePoint = false;
+		if (xpf!=null) 
+			samePoint = (xpf[nPoints-2]==xpf[nPoints-1] && ypf[nPoints-2]==ypf[nPoints-1]);
+		else
+			samePoint = (xp[nPoints-2]==xp[nPoints-1] && yp[nPoints-2]==yp[nPoints-1]);
+		Rectangle biggerStartBox = new Rectangle(ic.screenXD(startXD)-5, ic.screenYD(startYD)-5, 10, 10);
 		if (nPoints>2 && (biggerStartBox.contains(sx, sy)
-		|| (ic.offScreenX(sx)==startX && ic.offScreenY(sy)==startY)
+		|| (ic.offScreenXD(sx)==startXD && ic.offScreenYD(sy)==startYD)
 		|| (samePoint && (System.currentTimeMillis()-mouseUpTime)<=500))) {
             nPoints--;
             addOffset();
@@ -696,19 +863,34 @@ public class PolygonRoi extends Roi {
                 return;
             }
 			//add point to polygon
-			xp[nPoints] = xp[nPoints-1];
-			yp[nPoints] = yp[nPoints-1];
-			nPoints++;
-			if (nPoints==xp.length)
-				enlargeArrays();
+			if (xpf!=null) {
+				xpf[nPoints] = xpf[nPoints-1];
+				ypf[nPoints] = ypf[nPoints-1];
+				nPoints++;
+				if (nPoints==xpf.length)
+					enlargeArrays();
+			} else {
+				xp[nPoints] = xp[nPoints-1];
+				yp[nPoints] = yp[nPoints-1];
+				nPoints++;
+				if (nPoints==xp.length)
+					enlargeArrays();
+			}
 			//if (lineWidth>1) fitSpline();
 		}
 	}
 
     protected void addOffset() {
-        for (int i=0; i<nPoints; i++) {
-            xp[i] = xp[i]+x;
-            yp[i] = yp[i]+y;
+    	if (xpf!=null) {
+			for (int i=0; i<nPoints; i++) {
+				xpf[i] = (float)(xpf[i]+x);
+				ypf[i] = (float)(ypf[i]+y);
+			}
+        } else {
+			for (int i=0; i<nPoints; i++) {
+				xp[i] = xp[i]+x;
+				yp[i] = yp[i]+y;
+			}
         }
     }
     
@@ -718,6 +900,9 @@ public class PolygonRoi extends Roi {
 		if (xSpline!=null) {
 			FloatPolygon poly = new FloatPolygon(xSpline, ySpline, splinePoints);
 			return poly.contains(x-this.x, y-this.y);
+		} else if (xpf!=null) {
+			FloatPolygon poly = new FloatPolygon(xpf, ypf, nPoints);
+			return poly.contains(x-this.x, y-this.y);
 		} else {
 			Polygon poly = new Polygon(xp, yp, nPoints);
 			return poly.contains(x-this.x, y-this.y);
@@ -758,7 +943,9 @@ public class PolygonRoi extends Roi {
 			return cachedMask;
 		PolygonFiller pf = new PolygonFiller();
 		if (xSpline!=null)
-			pf.setPolygon(toInt(xSpline), toInt(ySpline), splinePoints);
+			pf.setPolygon(toIntR(xSpline), toIntR(ySpline), splinePoints);
+		else if (xpf!=null)
+			pf.setPolygon(toIntR(xpf), toIntR(ypf), nPoints);
 		else
 			pf.setPolygon(xp, yp, nPoints);
 		cachedMask = pf.getMask(width, height);
@@ -768,6 +955,8 @@ public class PolygonRoi extends Roi {
 	/** Returns the length of this line selection after
 		smoothing using a 3-point running average.*/
 	double getSmoothedLineLength() {
+		if (subPixelResolution())
+			return getFloatSmoothedLineLength();
 		double length = 0.0;
 		double w2 = 1.0;
 		double h2 = 1.0;
@@ -791,6 +980,30 @@ public class PolygonRoi extends Roi {
 		return length;
 	}
 
+	double getFloatSmoothedLineLength() {
+		double length = 0.0;
+		double w2 = 1.0;
+		double h2 = 1.0;
+		double dx, dy;
+		if (imp!=null) {
+			Calibration cal = imp.getCalibration();
+			w2 = cal.pixelWidth*cal.pixelWidth;
+			h2 = cal.pixelHeight*cal.pixelHeight;
+		}
+		dx = (xpf[0]+xpf[1]+xpf[2])/3.0-xpf[0];
+		dy = (ypf[0]+ypf[1]+ypf[2])/3.0-ypf[0];
+		length += Math.sqrt(dx*dx*w2+dy*dy*h2);
+		for (int i=1; i<nPoints-2; i++) {
+			dx = (xpf[i+2]-xpf[i-1])/3.0;
+			dy = (ypf[i+2]-ypf[i-1])/3.0;
+			length += Math.sqrt(dx*dx*w2+dy*dy*h2);
+		}
+		dx = xpf[nPoints-1]-(xpf[nPoints-3]+xpf[nPoints-2]+xpf[nPoints-1])/3.0;
+		dy = ypf[nPoints-1]-(ypf[nPoints-3]+ypf[nPoints-2]+ypf[nPoints-1])/3.0;
+		length += Math.sqrt(dx*dx*w2+dy*dy*h2);
+		return length;
+	}
+
 	/** Returns the perimeter of this ROI after
 		smoothing using a 3-point running average.*/
 	double getSmoothedPerimeter() {
@@ -884,6 +1097,18 @@ public class PolygonRoi extends Roi {
 				fdy = ySpline[0]-ySpline[splinePoints-1];
 				length += Math.sqrt(fdx*fdx*w2+fdy*fdy*h2);
 			}
+		} else if (xpf!=null) {
+			double fdx, fdy;
+			for (int i=0; i<(nPoints-1); i++) {
+				fdx = xpf[i+1]-xpf[i];
+				fdy = ypf[i+1]-ypf[i];
+				length += Math.sqrt(fdx*fdx*w2+fdy*fdy*h2);
+			}
+			if (type==POLYGON) {
+				fdx = xpf[0]-xpf[nPoints-1];
+				fdy = ypf[0]-ypf[nPoints-1];
+				length += Math.sqrt(fdx*fdx*w2+fdy*fdy*h2);
+			}
 		} else {
 			for (int i=0; i<(nPoints-1); i++) {
 				dx = xp[i+1]-xp[i];
@@ -912,28 +1137,40 @@ public class PolygonRoi extends Roi {
 			return nPoints;
 	}
 	
-	/** Returns this ROI's X-coordinates, which are relative
-		to origin of the bounding box. */
+	/** Obsolete; replaced by either getPolygon() or getFloatPolygon(). */
 	public int[] getXCoordinates() {
 		if (xSpline!=null)
 			return toInt(xSpline);
+		else if (xpf!=null)
+			return toInt(xpf);
 		else
 			return xp;
 	}
 
-	/** Returns this ROI's Y-coordinates, which are relative
-		to origin of the bounding box. */
+	/** Obsolete; replaced by either getPolygon() or getFloatPolygon(). */
 	public int[] getYCoordinates() {
 		if (xSpline!=null)
 			return toInt(ySpline);
+		else if (ypf!=null)
+			return toInt(ypf);
 		else
 			return yp;
 	}
 	
 	public Polygon getNonSplineCoordinates() {
-		return new Polygon(xp, yp, nPoints);
+		if (xpf!=null)
+			return new Polygon(toInt(xpf), toInt(ypf), nPoints);
+		else
+			return new Polygon(xp, yp, nPoints);
 	}
 		
+	public FloatPolygon getNonSplineFloatCoordinates() {
+		if (xpf!=null)
+			return (new FloatPolygon(xpf, ypf, nPoints)).duplicate();
+		else
+			return new FloatPolygon(toFloat(xp), toFloat(yp), nPoints);
+	}
+
 	/** Returns this PolygonRoi as a Polygon. 
 		@see ij.process.ImageProcessor#setRoi
 		@see ij.process.ImageProcessor#drawPolygon
@@ -946,6 +1183,10 @@ public class PolygonRoi extends Roi {
 			n = splinePoints;
 			xpoints1 = toInt(xSpline);
 			ypoints1 = toInt(ySpline);
+		} else if (xpf!=null) {
+			n = nPoints;
+			xpoints1 = toInt(xpf);
+			ypoints1 = toInt(ypf);
 		} else {
 			n = nPoints;
 			xpoints1 = xp;
@@ -970,6 +1211,11 @@ public class PolygonRoi extends Roi {
 				xpoints2[i] = xSpline[i] + x;
 				ypoints2[i] = ySpline[i] + y;
 			}
+		} else if (xpf!=null) {
+			for (int i=0; i<n; i++) {
+				xpoints2[i] = xpf[i] + x;
+				ypoints2[i] = ypf[i] + y;
+			}
 		} else {
 			for (int i=0; i<n; i++) {
 				xpoints2[i] = xp[i] + x;
@@ -979,6 +1225,10 @@ public class PolygonRoi extends Roi {
 		return new FloatPolygon(xpoints2, ypoints2, n);
 	}
 
+	public boolean subPixelResolution() {
+		return subPixel;
+	}
+
 	/** Uses the gift wrap algorithm to find the 
 		convex hull and returns it as a Polygon. */
 	public Polygon getConvexHull() {
@@ -1041,55 +1291,67 @@ public class PolygonRoi extends Roi {
 		return new Polygon(xx, yy, n2);
 	}
 
+	protected int clipRectMargin() {
+		return type==POINT?4:0;
+	}
+
 	/** Returns a copy of this PolygonRoi. */
 	public synchronized Object clone() {
 		PolygonRoi r = (PolygonRoi)super.clone();
-		r.xp = new int[maxPoints];
-		r.yp = new int[maxPoints];
+		if (xpf!=null) {
+			r.xpf = new float[maxPoints];
+			r.ypf = new float[maxPoints];
+		} else {
+			r.xp = new int[maxPoints];
+			r.yp = new int[maxPoints];
+		}
 		r.xp2 = new int[maxPoints];
 		r.yp2 = new int[maxPoints];
 		for (int i=0; i<nPoints; i++) {
-			r.xp[i] = xp[i];
-			r.yp[i] = yp[i];
+			if (xpf!=null) {
+				r.xpf[i] = xpf[i];
+				r.ypf[i] = ypf[i];
+			} else {
+				r.xp[i] = xp[i];
+				r.yp[i] = yp[i];
+			}
 			r.xp2[i] = xp2[i];
 			r.yp2[i] = yp2[i];
 		}
 		if (xSpline!=null) {
-			r.xSpline = null;
-			r.fitSpline(splinePoints);
+			r.xSpline = new float[splinePoints];
+			r.ySpline = new float[splinePoints];
+			r.splinePoints = splinePoints;
+			for (int i=0; i<splinePoints; i++) {
+				r.xSpline[i] = xSpline[i];
+				r.ySpline[i] = ySpline[i];
+			}
 		}
 		return r;
 	}
 
 	void enlargeArrays() {
-		int[] xptemp = new int[maxPoints*2];
-		int[] yptemp = new int[maxPoints*2];
+		if (xp!=null) {
+			int[] xptemp = new int[maxPoints*2];
+			int[] yptemp = new int[maxPoints*2];
+			System.arraycopy(xp, 0, xptemp, 0, maxPoints);
+			System.arraycopy(yp, 0, yptemp, 0, maxPoints);
+			xp=xptemp; yp=yptemp;
+		}
+		if (xpf!=null) {
+			float[] xpftemp = new float[maxPoints*2];
+			float[] ypftemp = new float[maxPoints*2];
+			System.arraycopy(xpf, 0, xpftemp, 0, maxPoints);
+			System.arraycopy(ypf, 0, ypftemp, 0, maxPoints);
+			xpf=xpftemp; ypf=ypftemp;
+		}
 		int[] xp2temp = new int[maxPoints*2];
 		int[] yp2temp = new int[maxPoints*2];
-		System.arraycopy(xp, 0, xptemp, 0, maxPoints);
-		System.arraycopy(yp, 0, yptemp, 0, maxPoints);
 		System.arraycopy(xp2, 0, xp2temp, 0, maxPoints);
 		System.arraycopy(yp2, 0, yp2temp, 0, maxPoints);
-		xp=xptemp; yp=yptemp;
 		xp2=xp2temp; yp2=yp2temp;
 		if (IJ.debugMode) IJ.log("PolygonRoi: "+maxPoints+" points");
 		maxPoints *= 2;
 	}
 	
-	private int[] toInt(float[] arr) {
-		int n = arr.length;
-		int[] temp = new int[n];
-		for (int i=0; i<n; i++)
-			temp[i] = (int)Math.floor(arr[i]+0.5);
-		return temp;
-	}
-
-	private float[] toFloat(int[] arr) {
-		int n = arr.length;
-		float[] temp = new float[n];
-		for (int i=0; i<n; i++)
-			temp[i] = arr[i];
-		return temp;
-	}
-
 }
