@@ -10,6 +10,7 @@ import ij.measure.*;
 import ij.plugin.frame.Recorder;
 import ij.plugin.filter.Analyzer;
 import ij.macro.Interpreter;
+import ij.io.RoiDecoder;
 
 /** A rectangular region of interest and superclass for the other ROI classes. */
 public class Roi extends Object implements Cloneable, java.io.Serializable {
@@ -26,38 +27,56 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 	int activeHandle;
 	int state;
 	int modState = NO_MODS;
+	int cornerDiameter;
 	
 	public static Roi previousRoi;
+	public static final BasicStroke onePixelWide = new BasicStroke(1);
 	protected static Color ROIColor = Prefs.getColor(Prefs.ROICOLOR,Color.yellow);
 	protected static int pasteMode = Blitter.COPY;
 	protected static int lineWidth = 1;
+	protected static Color defaultFillColor;
 	
 	protected int type;
 	protected int xMax, yMax;
 	protected ImagePlus imp;
+	private int imageID;
 	protected ImageCanvas ic;
 	protected int oldX, oldY, oldWidth, oldHeight;
 	protected int clipX, clipY, clipWidth, clipHeight;
 	protected ImagePlus clipboard;
 	protected boolean constrain; // to be square
-    protected boolean center;
+	protected boolean center;
+	protected boolean aspect;
 	protected boolean updateFullWindow;
 	protected double mag = 1.0;
-	protected String name;
+	protected double asp_bk; //saves aspect ratio if resizing takes roi very small
 	protected ImageProcessor cachedMask;
 	protected Color handleColor = Color.white;
-	protected Color instanceColor;
+	protected Color  strokeColor;
+	protected Color instanceColor; //obsolete; replaced by  strokeColor
+	protected Color fillColor;
 	protected BasicStroke stroke;
-
-
+	protected boolean nonScalable;
+	protected boolean overlay;
+	protected boolean wideLine;
+	private String name;
+	private int position;
+	private int channel, slice, frame;
+	private Overlay prototypeOverlay;
 
 	/** Creates a new rectangular Roi. */
 	public Roi(int x, int y, int width, int height) {
+		this(x, y, width, height, 0);
+	}
+
+	/** Creates a new rounded rectangular Roi. */
+	public Roi(int x, int y, int width, int height, int cornerDiameter) {
 		setImage(null);
 		if (width<1) width = 1;
 		if (height<1) height = 1;
 		if (width>xMax) width = xMax;
 		if (height>yMax) height = yMax;
+		this.cornerDiameter = cornerDiameter;
 		//setLocation(x, y);
 		this.x = x;
 		this.y = y;
@@ -78,6 +97,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 			draw(g);
 			g.dispose();
 		}
+		fillColor = defaultFillColor;
 	}
 	
 	/** Creates a new rectangular Roi. */
@@ -88,6 +108,12 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 	/** Starts the process of creating a user-defined rectangular Roi,
 		where sx and sy are the starting screen coordinates. */
 	public Roi(int sx, int sy, ImagePlus imp) {
+		this(sx, sy, imp, 0);
+	}
+	
+	/** Starts the process of creating a user-defined rectangular Roi,
+		where sx and sy are the starting screen coordinates. */
+	public Roi(int sx, int sy, ImagePlus imp, int cornerDiameter) {
 		setImage(imp);
 		int ox=sx, oy=sy;
 		if (ic!=null) {
@@ -95,24 +121,34 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 			oy = ic.offScreenY(sy);
 		}
 		setLocation(ox, oy);
+		this.cornerDiameter = cornerDiameter;
 		width = 0;
 		height = 0;
 		state = CONSTRUCTING;
 		type = RECTANGLE;
+		if (isDrawingTool()) {
+			setStrokeColor(Toolbar.getForegroundColor());
+			if (!(this instanceof TextRoi)) {
+				double mag = imp!=null&&imp.getCanvas()!=null?imp.getCanvas().getMagnification():1.0; 
+				if (mag>1.0) mag = 1.0;
+				if (Line.getWidth()==1 && !Line.widthChanged)
+					Line.setWidth((int)(2.0/mag));
+				if (mag<1.0 && Line.getWidth()*mag<1.0)
+						Line.setWidth((int)(1.0/mag));
+				setStrokeWidth(Line.getWidth());
+			}
+		}
+		fillColor = defaultFillColor;
 	}
-	
-	/** Obsolete */
+
+	/** @deprecated */
 	public Roi(int x, int y, int width, int height, ImagePlus imp) {
 		this(x, y, width, height);
 		setImage(imp);
 	}
 
+	/** Set the location of the ROI in image coordinates. */
 	public void setLocation(int x, int y) {
-		//if (x<0) x = 0;
-		//if (y<0) y = 0;
-		//if ((x+width)>xMax) x = xMax-width;
-		//if ((y+height)>yMax) y = yMax-height;
-		//IJ.write(imp.getTitle() + ": Roi.setlocation(" + x + "," + y + ")");
 		this.x = x;
 		this.y = y;
 		startX = x; startY = y;
@@ -134,6 +170,16 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		}
 	}
 	
+	/** Returns the ImagePlus associated with this ROI, or null. */
+	public ImagePlus getImage() {
+		return imp;
+	}
+	
+	/** Returns the ID of the image associated with this ROI. */
+	public int getImageID() {
+		return imp!=null?imp.getID():imageID;
+	}
+
 	public int getType() {
 		return type;
 	}
@@ -193,11 +239,11 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		double[] x = new double[n];
 		double[] y = new double[n];
 		for (int i=0; i<n; i++) {
-			x[i] = poly.xpoints[i] - cx;
-			y[i] = poly.ypoints[i] - cy;
+			x[i] = (poly.xpoints[i]-cx)*pw;
+			y[i] = (poly.ypoints[i]-cy)*ph;
 		}
 		double xr, yr;
-		for (int a=0; a<=180; a++) {
+		for (double a=0; a<=90; a+=0.5) { // rotate calipers in 0.5 degree increments
 			double cos = Math.cos(a*Math.PI/180.0);
 			double sin = Math.sin(a*Math.PI/180.0);
 			double xmin=Double.MAX_VALUE, ymin=Double.MAX_VALUE;
@@ -227,8 +273,6 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		angle = (180.0/Math.PI)*Math.atan2(dy*ph, dx*pw);
 		if (angle<0) angle = 180.0 + angle;
 		//breadth = getFeretBreadth(poly, angle, x1, y1, x2, y2);
-		if (pw==ph)
-			min *= pw;
 		double[] a = new double[5];
 		a[0] = diameter;
 		a[1] = angle;
@@ -268,7 +312,10 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		return new Rectangle(x, y, width, height);
 	}
 	
-	/** This obsolete method has been replaced by getBounds(). */
+	/**
+	* @deprecated
+	* replaced by getBounds()
+	*/
 	public Rectangle getBoundingRect() {
 		return getBounds();
 	}
@@ -294,7 +341,11 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 	}
 
 	public FloatPolygon getFloatPolygon() {
-		return null;
+		Polygon p = getPolygon();
+		if (p!=null)
+			return new FloatPolygon(toFloat(p.xpoints), toFloat(p.ypoints), p.npoints);
+		else
+			return null;
 	}
 	
 	/** Returns a copy of this roi. See Thinking is Java by Bruce Eckel
@@ -303,6 +354,9 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		try { 
 			Roi r = (Roi)super.clone();
 			r.setImage(null);
+			r.setStroke(getStroke());
+			r.setFillColor(getFillColor());
+			r.imageID = getImageID();
 			return r;
 		}
 		catch (CloneNotSupportedException e) {return null;}
@@ -318,6 +372,8 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		}
 		if (constrain) {
 			// constrain selection to be square
+			if (!center)
+				{growConstrained(xNew, yNew); return;}
 			int dx, dy, d;
 			dx = xNew - x;
 			dy = yNew - y;
@@ -328,7 +384,6 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 			xNew = x + d;
 			yNew = y + d;
 		}
-		
 		if (center) {
 			width = Math.abs(xNew - startX)*2;
 			height = Math.abs(yNew - startY)*2;
@@ -352,7 +407,31 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		oldHeight = height;
 	}
 
-	protected void moveHandle(int sx, int sy) {	
+	private void growConstrained(int xNew, int yNew) {
+		int dx = xNew - startX;
+		int dy = yNew - startY;
+		width = height = (int)Math.round(Math.sqrt(dx*dx + dy*dy));
+		if (type==RECTANGLE) {
+			x = (xNew>=startX)?startX:startX - width;
+			y = (yNew>=startY)?startY:startY - height;
+			if (x<0) x = 0;
+			if (y<0) y = 0;
+			if ((x+width) > xMax) width = xMax-x;
+			if ((y+height) > yMax) height = yMax-y;
+		} else {
+			x = startX + dx/2 - width/2;
+			y = startY + dy/2 - height/2;
+		}
+		updateClipRect();
+		imp.draw(clipX, clipY, clipWidth, clipHeight);
+		oldX = x;
+		oldY = y;
+		oldWidth = width;
+		oldHeight = height;
+	}
+
+	protected void moveHandle(int sx, int sy) {
+		double asp;
 		if (clipboard!=null) return;
 		int ox = ic.offScreenX(sx);
 		int oy = ic.offScreenY(sy);
@@ -360,15 +439,38 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		if (ox>xMax) ox=xMax; if (oy>yMax) oy=yMax;
 		//IJ.log("moveHandle: "+activeHandle+" "+ox+" "+oy);
 		int x1=x, y1=y, x2=x1+width, y2=y+height, xc=x+width/2, yc=y+height/2;
+		if (width > 7 && height > 7) {
+			asp = (double)width/(double)height;
+			asp_bk = asp;
+		} else {
+			asp = asp_bk;
+		}
+		
 		switch (activeHandle) {
-			case 0: x=ox; y=oy; break;
-			case 1: y=oy; break;
-			case 2: x2=ox; y=oy; break;
-			case 3: x2=ox; break;
-			case 4: x2=ox; y2=oy; break;
-			case 5: y2=oy; break;
-			case 6: x=ox; y2=oy; break;
-			case 7: x=ox; break;
+			case 0:
+				x=ox; y=oy;
+				break;
+			case 1:
+				y=oy;
+				break;
+			case 2:
+				x2=ox; y=oy;
+				break;
+			case 3:
+				x2=ox;
+				break;
+			case 4:
+				x2=ox; y2=oy;
+				break;
+			case 5:
+				y2=oy;
+				break;
+			case 6:
+				x=ox; y2=oy;
+				break;
+			case 7:
+				x=ox;
+				break;
 		}
 		if (x<x2)
 		   width=x2-x;
@@ -380,7 +482,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		   {height=1; y=y2;}
 		
 		if (center) {
-			switch(activeHandle) {
+			switch(activeHandle){
 				case 0:
 					width=(xc-x)*2;
 					height=(yc-y)*2;
@@ -424,6 +526,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 				height=1;
 				y=y2=yc;
 			}
+
 		}
 		
 		if (constrain) {
@@ -431,11 +534,93 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 				width=height;
 			else
 				height=width;
-			if(center){
+
+			if(x>=x2) {
+				width=1;
+				x=x2=xc;
+			}
+			if (y>=y2) {
+				height=1;
+				y=y2=yc;
+			}
+			switch(activeHandle){
+				case 0:
+					x=x2-width;
+					y=y2-height;
+					break;
+				case 1:
+					x=xc-width/2;
+					y=y2-height;
+					break;
+				case 2:
+					y=y2-height;
+					break;
+				case 3:
+					y=yc-height/2;
+					break;
+				case 5:
+					x=xc-width/2;
+					break;
+				case 6:
+					x=x2-width;
+					break;
+				case 7:
+					y=yc-height/2;
+					x=x2-width;
+					break;
+			}
+			if (center){
 				x=xc-width/2;
 				y=yc-height/2;
 			}
 		}
+
+		if (aspect && !constrain) {
+			if (activeHandle==1 || activeHandle==5) width=(int)Math.rint((double)height*asp);
+			else height=(int)Math.rint((double)width/asp);
+			
+			switch(activeHandle){
+				case 0:
+					x=x2-width;
+					y=y2-height;
+					break;
+				case 1:
+					x=xc-width/2;
+					y=y2-height;
+					break;
+				case 2:
+					y=y2-height;
+					break;
+				case 3:
+					y=yc-height/2;
+					break;
+				case 5:
+					x=xc-width/2;
+					break;
+				case 6:
+					x=x2-width;
+					break;
+				case 7:
+					y=yc-height/2;
+					x=x2-width;
+					break;
+			}
+			if (center){
+				x=xc-width/2;
+				y=yc-height/2;
+			}
+			
+			// Attempt to preserve aspect ratio when roi very small:
+			if (width<8) {
+				if(width<1) width = 1;
+				height=(int)Math.rint((double)width/asp_bk);
+			}
+			if (height<8) {
+				if(height<1) height =1;
+				width=(int)Math.rint((double)height*asp_bk);
+			}
+			
+		}
 		
 		updateClipRect();
 		imp.draw(clipX, clipY, clipWidth, clipHeight);
@@ -448,7 +633,8 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		int yNew = ic.offScreenY(sy);
 		x += xNew - startX;
 		y += yNew - startY;
-		if (clipboard==null && type==RECTANGLE) {
+		boolean isImageRoi = this instanceof ImageRoi;
+		if (clipboard==null && type==RECTANGLE && !isImageRoi) {
 			if (x<0) x=0; if (y<0) y=0;
 			if ((x+width)>xMax) x = xMax-width;
 			if ((y+height)>yMax) y = yMax-height;
@@ -464,6 +650,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		oldY = y;
 		oldWidth = width;
 		oldHeight=height;
+		if (isImageRoi) showStatus();
 	}
 
 	/** Nudge ROI one pixel on arrow key press. */
@@ -533,20 +720,27 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		clipWidth = ((x+width>=oldX+oldWidth)?x+width:oldX+oldWidth) - clipX + 1;
 		clipHeight = ((y+height>=oldY+oldHeight)?y+height:oldY+oldHeight) - clipY + 1;
 		int m = 3;
-		if (type==POINT) m += 4;
 		if (ic!=null) {
 			double mag = ic.getMagnification();
 			if (mag<1.0)
-				m = (int)(3/mag);
+				m = (int)(4.0/mag);
 		}
+		m += clipRectMargin();
+		m = (int)(m+getStrokeWidth()*2);
 		clipX-=m; clipY-=m;
 		clipWidth+=m*2; clipHeight+=m*2;
+		//if (IJ.debugMode) IJ.log("updateClipRect: "+m+"  "+clipX+" "+clipY+" "+clipWidth+" "+clipHeight);
 	 }
+	 
+	protected int clipRectMargin() {
+		return 0;
+	}
 		
 	protected void handleMouseDrag(int sx, int sy, int flags) {
 		if (ic==null) return;
 		constrain = (flags&Event.SHIFT_MASK)!=0;
 		center = (flags&Event.CTRL_MASK)!=0 || (IJ.isMacintosh()&&(flags&Event.META_MASK)!=0);
+		aspect = (flags&Event.ALT_MASK)!=0;
 		switch(state) {
 			case CONSTRUCTING:
 				grow(sx, sy);
@@ -569,21 +763,37 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 	}
 	
 	public void draw(Graphics g) {
-		if (ic==null) return;
-		g.setColor(instanceColor!=null?instanceColor:ROIColor);
-		mag = ic.getMagnification();
+		Color color =  strokeColor!=null? strokeColor:ROIColor;
+		if (fillColor!=null) color = fillColor;
+		if (Interpreter.isBatchMode() && ic!=null && ic.getDisplayList()!=null && strokeColor==null && fillColor==null)
+			return;
+		g.setColor(color);
+		mag = getMagnification();
 		int sw = (int)(width*mag);
 		int sh = (int)(height*mag);
-		//if (x+width==imp.getWidth()) sw -= 1;
-		//if (y+height==imp.getHeight()) sh -= 1;
-		int sx1 = ic.screenX(x);
-		int sy1 = ic.screenY(y);
+		int sx1 = screenX(x);
+		int sy1 = screenY(y);
 		int sx2 = sx1+sw/2;
 		int sy2 = sy1+sh/2;
 		int sx3 = sx1+sw;
 		int sy3 = sy1+sh;
-		g.drawRect(sx1, sy1, sw, sh);
-		if (state!=CONSTRUCTING && clipboard==null) {
+		Graphics2D g2d = (Graphics2D)g;
+		if (stroke!=null)
+			g2d.setStroke(getScaledStroke());
+		if (cornerDiameter>0) {
+			int sArcSize = (int)Math.round(cornerDiameter*mag);
+			g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
+			if (fillColor!=null)
+				g.fillRoundRect(sx1, sy1, sw, sh, sArcSize, sArcSize);
+			else
+				g.drawRoundRect(sx1, sy1, sw, sh, sArcSize, sArcSize);
+		} else {
+			if (fillColor!=null)
+				g.fillRect(sx1, sy1, sw, sh);
+			else
+				g.drawRect(sx1, sy1, sw, sh);
+		}
+		if (state!=CONSTRUCTING && clipboard==null && !overlay) {
 			int size2 = HANDLE_SIZE/2;
 			drawHandle(g, sx1-size2, sy1-size2);
 			drawHandle(g, sx2-size2, sy1-size2);
@@ -600,6 +810,12 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 			{updateFullWindow = false; imp.draw();}
 	}
 	
+	public void drawOverlay(Graphics g) {
+		overlay = true;
+		draw(g);
+		overlay = false;
+	}
+	
 	void drawPreviousRoi(Graphics g) {
 		if (previousRoi!=null && previousRoi!=this && previousRoi.modState!=NO_MODS) {
 			if (type!=POINT && previousRoi.getType()==POINT && previousRoi.modState!=SUBTRACT_FROM_ROI)
@@ -610,17 +826,17 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 	}
 	
 	void drawHandle(Graphics g, int x, int y) {
-		double size = (width*height)*mag;
+		double size = (width*height)*mag*mag;
 		if (type==LINE) {
 			size = Math.sqrt(width*width+height*height);
-			size *= size*mag;
+			size *= size*mag*mag;
 		}
-		if (size>6000.0) {
+		if (size>4000.0) {
 			g.setColor(Color.black);
 			g.fillRect(x,y,5,5);
 			g.setColor(handleColor);
 			g.fillRect(x+1,y+1,3,3);
-		} else if (size>1500.0) {
+		} else if (size>1000.0) {
 			g.setColor(Color.black);
 			g.fillRect(x+1,y+1,4,4);
 			g.setColor(handleColor);
@@ -633,7 +849,10 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		}
 	}
 
-	/** Obsolete, use drawPixels(ImageProcessor) */
+	/**
+	* @deprecated
+	* replaced by drawPixels(ImageProcessor)
+	*/
 	public void drawPixels() {
 		if (imp!=null)
 			drawPixels(imp.getProcessor());	
@@ -645,14 +864,29 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 	*/
 	public void drawPixels(ImageProcessor ip) {
 		endPaste();
-		ip.drawRect(x, y, width, height);
-		if (Line.getWidth()>1)
+		int saveWidth = ip.getLineWidth();
+		if (getStrokeWidth()>1f)
+			ip.setLineWidth((int)Math.round(getStrokeWidth()));
+		if (cornerDiameter>0)
+			(new ShapeRoi(new RoundRectangle2D.Float(x, y, width, height, cornerDiameter, cornerDiameter))).drawPixels(ip);
+		else {
+			if (ip.getLineWidth()==1)
+				ip.drawRect(x, y, width+1, height+1);
+			else
+				ip.drawRect(x, y, width, height);
+		}
+		ip.setLineWidth(saveWidth);
+		if (Line.getWidth()>1 || getStrokeWidth()>1)
 			updateFullWindow = true;
 	}
-
+	
 	public boolean contains(int x, int y) {
 		Rectangle r = new Rectangle(this.x, this.y, width, height);
-		return r.contains(x, y);
+		boolean contains = r.contains(x, y);
+		if (cornerDiameter==0 || contains==false)
+			return contains;
+		RoundRectangle2D rr = new RoundRectangle2D.Float(this.x, this.y, width, height, cornerDiameter, cornerDiameter);
+		return rr.contains(x, y);
 	}
 		
 	/** Returns a handle number if the specified screen coordinates are  
@@ -695,17 +929,25 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		
 	protected void handleMouseUp(int screenX, int screenY) {
 		state = NORMAL;
+		if (imp==null) return;
 		imp.draw(clipX-5, clipY-5, clipWidth+10, clipHeight+10);
 		if (Recorder.record) {
 			String method;
 			if (type==LINE) {
-				if (imp==null) return;
 				Line line = (Line)imp.getRoi();
 				Recorder.record("makeLine", line.x1, line.y1, line.x2, line.y2);
 			} else if (type==OVAL)
 				Recorder.record("makeOval", x, y, width, height);
-			else if (!(this instanceof TextRoi))
-				Recorder.record("makeRectangle", x, y, width, height);
+			else if (!(this instanceof TextRoi)) {
+				if (cornerDiameter==0)
+					Recorder.record("makeRectangle", x, y, width, height);
+				else {
+					if (Recorder.scriptMode())
+						Recorder.recordCall("imp.setRoi(new Roi("+x+", "+y+", "+width+", "+height+", "+cornerDiameter+"));");
+					else
+						Recorder.record("makeRectangle", x, y, width, height, cornerDiameter);
+				}
+			}
 		}
 		if (Toolbar.getToolId()==Toolbar.OVAL&&Toolbar.getBrushSize()>0)  {
 			int flags = ic!=null?ic.getModifiers():16;
@@ -753,7 +995,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		else
 			roi2 = s1;
 		if (roi2!=null)
-			roi2.setName(previousRoi.getName());
+			roi2.copyAttributes(previousRoi);
 		imp.setRoi(roi2);
 		previousRoi = previous;
     }
@@ -767,7 +1009,8 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		previousRoi.modState = NO_MODS;
 		PointRoi p1 = (PointRoi)previousRoi;
 		Rectangle r = getBounds();
-		imp.setRoi(p1.addPoint(r.x, r.y));
+		FloatPolygon poly = getFloatPolygon();
+		imp.setRoi(p1.addPoint(poly.xpoints[0], poly.ypoints[0]));
     }
     
     void subtractPoints() {
@@ -796,6 +1039,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
      }
 
 	protected void showStatus() {
+		if (imp==null) return;
 		String value;
 		if (state!=CONSTRUCTING && (type==RECTANGLE||type==POINT) && width<=25 && height<=25) {
 			ImageProcessor ip = imp.getProcessor();
@@ -810,12 +1054,16 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 			size = ", w="+IJ.d2s(width*cal.pixelWidth)+", h="+IJ.d2s(height*cal.pixelHeight);
 		else
 			size = ", w="+width+", h="+height;
+		size += ", ar="+IJ.d2s((double)width/height,2);
 		IJ.showStatus(imp.getLocationAsString(x,y)+size+value);
 	}
 		
 	/** Always returns null for rectangular Roi's */
 	public ImageProcessor getMask() {
-		return null;
+		if (cornerDiameter>0)
+			return (new ShapeRoi(new RoundRectangle2D.Float(x, y, width, height, cornerDiameter, cornerDiameter))).getMask();
+		else
+			return null;
 	}
 	
 	public void startPaste(ImagePlus clipboard) {
@@ -834,21 +1082,15 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 			ip.copyBits(clipboard.getProcessor(), x, y, pasteMode);
 			if (type!=RECTANGLE)
 				ip.reset(ip.getMask());
-			ic.setImageUpdated();
+			if (ic!=null)
+				ic.setImageUpdated();
 		}
 	}
 
 	public void endPaste() {
 		if (clipboard!=null) {
-			imp.getMask();
-			ImageProcessor ip = imp.getProcessor();
-			if (pasteMode!=Blitter.COPY) ip.reset();
-			ip.copyBits(clipboard.getProcessor(), x, y, pasteMode);
-			if (type!=RECTANGLE)
-				ip.reset(ip.getMask());
-			ip.snapshot();
+			updatePaste();
 			clipboard = null;
-			imp.updateAndDraw();
 			Undo.setup(Undo.FILTER, imp);
 		}
 	}
@@ -871,36 +1113,147 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		return (180.0/Math.PI)*Math.atan2(dy, dx);
 	}
 	
-	/** Returns the color used for drawing ROI outlines. */
-	public static Color getColor() {
-		return ROIColor;
-	}
-
-	/** Sets the color used for ROI outlines to the specified value. */
+	/** Sets the default (global) color used for ROI outlines.
+	 * @see #getColor()
+	 * @see #setStrokeColor(Color)
+	 */
 	public static void setColor(Color c) {
 		ROIColor = c;
 	}
 	
+	/** Returns the default (global) color used for drawing ROI outlines.
+	 * @see #setColor(Color)
+	 * @see #getStrokeColor()
+	 */
+	public static Color getColor() {
+		return ROIColor;
+	}
+
 	/** Sets the color used by this ROI to draw its outline. This color, if not null, 
-		overrides the global color set by the static setColor() method. */
+	 * overrides the global color set by the static setColor() method.
+	 * @see #getStrokeColor()
+	 * @see #setStrokeWidth(int)
+	 * @see ij.ImagePlus#setOverlay(ij.gui.Overlay)
+	 */
+	public void setStrokeColor(Color c) {
+		 strokeColor = c;
+	}
+
+	/** Returns the the color used to draw the ROI outline or null if the default color is being used.
+	 * @see #setStrokeColor(Color)
+	 */
+	public Color getStrokeColor() {
+		return  strokeColor;
+	}
+
+	/** Sets the color used to fill ROIs when they are in an overlay.
+	 * @see ij.ImagePlus#setOverlay(ij.gui.Overlay)
+	 */
+	public void setFillColor(Color color) {
+		fillColor = color;
+	}
+
+	/** Returns the color used to fill this ROI, or null if it is not filled.
+	 * @see #getStrokeColor()
+	 */
+	public Color getFillColor() {
+		return fillColor;
+	}
+	
+	public static void setDefaultFillColor(Color color) {
+		defaultFillColor = color;
+	}
+	
+	public static Color getDefaultFillColor() {
+		return defaultFillColor;
+	}
+	
+	/** Copy the attributes (outline color, fill color, outline width) 
+		of  'roi2' to the this selection. */
+	public void copyAttributes(Roi roi2) {
+		this. strokeColor = roi2. strokeColor;
+		this.fillColor = roi2.fillColor;
+		this.stroke = roi2.stroke;
+	}
+
+	/**
+	* @deprecated
+	* replaced by setStrokeColor()
+	*/
 	public void setInstanceColor(Color c) {
-		instanceColor = c;
+		 strokeColor = c;
 	}
 
-	/** Returns the the color used to draw the ROI outline or null if the default color is being used. */
-	public Color getInstanceColor() {
-		return instanceColor;
-	}
-
-	/** Sets the width of lines used to draw composite ROIs. */
+	/**
+	* @deprecated
+	* replaced by setStrokeWidth(int)
+	*/
 	public void setLineWidth(int width) {
-		this.stroke = new BasicStroke(width);
+		setStrokeWidth(width) ;
+	}
+        
+	public void updateWideLine(float width) {
+		//IJ.log("updateWideLine "+isLine()+"  "+isDrawingTool()+"  "+getType());
+		if (isLine()) {
+			wideLine = true;
+			setStrokeWidth(width);
+			if (getStrokeColor()==null) {
+				Color c = getColor();
+				setStrokeColor(new Color(c.getRed(),c.getGreen(),c.getBlue(), 77));
+			}
+		}
 	}
 
-	/** Sets the Stroke used to draw composite ROIs. */
+    /** Set 'nonScalable' true to have TextRois in a display 
+		list drawn at a fixed location and size. */
+	public void setNonScalable(boolean nonScalable) {
+		this.nonScalable = nonScalable;
+	}
+
+	/** Sets the width of the line used to draw this ROI.
+	 * @see #setStrokeColor(Color)
+	 * @see ij.ImagePlus#setOverlay(ij.gui.Overlay)
+	 */
+	public void setStrokeWidth(float width) {
+		//this.stroke = new BasicStroke(width);
+		if (width==0)
+			stroke = null;
+		else if (wideLine)
+			this.stroke = new BasicStroke(width, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
+		else
+			this.stroke = new BasicStroke(width);
+		if (width>1f) fillColor = null;
+	}
+
+	/** This is a version of setStrokeWidth() that accepts a double argument. */
+	public void setStrokeWidth(double width) {
+		setStrokeWidth((float)width);
+	}
+
+	/** Returns the lineWidth. */
+	public float getStrokeWidth() {
+		return stroke!=null?stroke.getLineWidth():1;
+	}
+
+	/** Sets the Stroke used to draw this ROI. */
 	public void setStroke(BasicStroke stroke) {
 		this.stroke = stroke;
 	}
+	
+	/** Returns the Stroke used to draw this ROI, or null if no Stroke is used. */
+	public BasicStroke getStroke() {
+		return stroke;
+	}
+	
+	protected BasicStroke getScaledStroke() {
+		if (ic==null) return stroke;
+		double mag = ic.getMagnification();
+		if (mag!=1.0) {
+			float width = stroke.getLineWidth();
+			return new BasicStroke((float)(width*mag), BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
+		} else
+			return stroke;
+	}
 
 	/** Returns the name of this ROI, or null. */
 	public String getName() {
@@ -923,6 +1276,102 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 			imp.updateAndDraw();
 	}
 	
+	/** Sets the rounded rectangle corner diameter (pixels). */
+	public void setCornerDiameter(int cornerDiameter) {
+		if (cornerDiameter<0) cornerDiameter = 0;
+		this.cornerDiameter = cornerDiameter;
+		ImagePlus imp = WindowManager.getCurrentImage();
+		if (imp!=null && this==imp.getRoi())
+			imp.updateAndDraw();
+	}
+
+	/** Returns the rounded rectangle corner diameter (pixels). */
+	public int getCornerDiameter() {
+		return cornerDiameter;
+	}
+	
+	/** Obsolete; replaced by setCornerDiameter(). */
+	public void setRoundRectArcSize(int cornerDiameter) {
+		setCornerDiameter(cornerDiameter);
+	}
+
+	/** Obsolete; replaced by getCornerDiameter(). */
+	public int getRoundRectArcSize() {
+		return cornerDiameter;
+	}
+
+	/** Sets the stack position (image number) of this ROI. In an overlay, this
+	* ROI is only displayed when the stack is at the specified position.
+	* Set to zero to have the ROI displayed on all images in the stack.
+	* @see ij.gui.Overlay
+	*/
+	public void setPosition(int n) {
+		if (n<0) n=0;
+		position = n;
+		channel = slice = frame = 0;
+	} 
+
+	/** Returns the stack position (image number) of this ROI, or
+	*  zero if the ROI is not associated with a particular stack image.
+	* @see ij.gui.Overlay
+	*/
+	public int getPosition() {
+		return position;
+	}
+	
+	/** Sets the hyperstack position of this ROI. In an overlay, this
+	* ROI is only displayed when the hyperstack is at the specified position.
+	* @see ij.gui.Overlay
+	*/
+	public void setPosition(int channel, int slice, int frame) {
+		if (channel<0) channel=0;
+		this.channel = channel;
+		if (slice<0) slice=0;
+		this.slice = slice;
+		if (frame<0) frame=0;
+		this.frame = frame;
+		position = 0;
+	}
+	
+	/** Returns the channel position of this ROI, or zero
+	*  if this ROI is not associated with a particular channel.
+	*/
+	public final int getCPosition() {
+		return channel;
+	}
+
+	/** Returns the slice position of this ROI, or zero
+	*  if this ROI is not associated with a particular slice.
+	*/
+	public final int getZPosition() {
+		return slice;
+	}
+	
+	/** Returns the frame position of this ROI, or zero
+	*  if this ROI is not associated with a particular frame.
+	*/
+	public final int getTPosition() {
+		return frame;
+	}
+
+	// Used by the FileSaver and RoiEncoder to save overlay settings. */
+	public void setPrototypeOverlay(Overlay overlay) {
+		prototypeOverlay = new Overlay();
+		prototypeOverlay.drawLabels(overlay.getDrawLabels());
+		prototypeOverlay.drawNames(overlay.getDrawNames());
+		prototypeOverlay.drawBackgrounds(overlay.getDrawBackgrounds());
+		prototypeOverlay.setLabelColor(overlay.getLabelColor());
+		prototypeOverlay.setLabelFont(overlay.getLabelFont());
+	} 
+
+	// Used by the FileOpener and RoiDecoder to restore overlay settings. */
+	public Overlay getPrototypeOverlay() {
+		if (prototypeOverlay!=null)
+			return prototypeOverlay;
+		else
+			return new Overlay();
+	}
+
 	/** Returns the current paste transfer mode, or NOT_PASTING (-1)
 		if no paste operation is in progress.
 		@see ij.process.Blitter
@@ -939,16 +1388,26 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		return pasteMode;
 	}
 	
-	/** Returns true if this is an area selection. */
+	/** Returns 'true' if this is an area selection. */
 	public boolean isArea() {
 		return (type>=RECTANGLE && type<=TRACED_ROI) || type==COMPOSITE;
 	}
 
-	/** Returns true if this is a line selection. */
+	/** Returns 'true' if this is a line selection. */
     public boolean isLine() {
         return type>=LINE && type<=FREELINE;
     }
-
+    
+	/** Returns 'true' if this is an ROI primarily used from drawing
+		(e.g., Rounded Rectangle, TextRoi or Arrow). */
+    public boolean isDrawingTool() {
+        return cornerDiameter>0;
+    }
+    
+    protected double getMagnification() {
+    	return ic!=null?ic.getMagnification():1.0;
+    }
+    
 	/** Convenience method that converts Roi type to a human-readable form. */
 	public String getTypeAsString() {
 		String s="";
@@ -973,6 +1432,10 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		return ic!=null;
 	}
 
+	/** Returns true if this is a PolygonRoi that supports sub-pixel resolution. */
+	public boolean subPixelResolution() {
+		return false;
+	}
 
     /** Checks whether two rectangles are equal. */
     public boolean equals(Object obj) {
@@ -986,8 +1449,49 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 			return false;
     }
 
+	protected int screenX(int ox) {return ic!=null?ic.screenX(ox):ox;}
+	protected int screenY(int oy) {return ic!=null?ic.screenY(oy):oy;}
+	protected int screenXD(double ox) {return ic!=null?ic.screenXD(ox):(int)ox;}
+	protected int screenYD(double oy) {return ic!=null?ic.screenYD(oy):(int)oy;}
+
+	protected int[] toInt(float[] arr) {
+		return toInt(arr, null, arr.length);
+	}
+
+	protected int[] toInt(float[] arr, int[] arr2, int size) {
+		int n = arr.length;
+		if (size>n) size=n;
+		int[] temp = arr2;
+		if (temp==null || temp.length<n)
+			temp = new int[n];
+		for (int i=0; i<size; i++)
+			temp[i] = (int)arr[i];
+			//temp[i] = (int)Math.floor(arr[i]+0.5);
+		return temp;
+	}
+
+	protected int[] toIntR(float[] arr) {
+		int n = arr.length;
+		int[] temp = new int[n];
+		for (int i=0; i<n; i++)
+			temp[i] = (int)Math.floor(arr[i]+0.5);
+		return temp;
+	}
+
+	protected float[] toFloat(int[] arr) {
+		int n = arr.length;
+		float[] temp = new float[n];
+		for (int i=0; i<n; i++)
+			temp[i] = arr[i];
+		return temp;
+	}
+
 	public String toString() {
 		return ("Roi["+getTypeAsString()+", x="+x+", y="+y+", width="+width+", height="+height+"]");
 	}
 
+	/** Deprecated */
+	public void temporarilyHide() {
+	}
+
 }
