@@ -75,6 +75,7 @@ public class ShapeRoi extends Roi {
 	private boolean forceAngle = false;
 	
 	private Vector savedRois;
+	private static Stroke defaultStroke = new BasicStroke();
 
 
 	/** Constructs a ShapeRoi from an Roi. */
@@ -120,15 +121,7 @@ public class ShapeRoi extends Roi {
 		this.forceTrace = forceTrace;
 		this.maxPoly= maxPoly;
 		this.flatten = flatten;
-		state = r.getState();
-		setImage(r.imp);
 		shape = roiToShape((Roi)r.clone());
-        //IJ.log("ShapeRoi:"+x+"  "+y);
-		if(ic!=null) {
-			Graphics g = ic.getGraphics();
-			draw(g);
-			g.dispose();
-		}
 	}
 
 	/** Constructs a ShapeRoi from an array of variable length path segments. Each
@@ -216,12 +209,14 @@ public class ShapeRoi extends Roi {
 		at = new AffineTransform();
 		at.translate(sr.x, sr.y);
 		Area a2 = new Area(at.createTransformedShape(sr.getShape()));
-		switch (op) {
-			case OR: a1.add(a2); break;
-			case AND: a1.intersect(a2); break;
-			case XOR: a1.exclusiveOr(a2); break;
-			case NOT: a1.subtract(a2); break;
-		}
+		try {
+			switch (op) {
+				case OR: a1.add(a2); break;
+				case AND: a1.intersect(a2); break;
+				case XOR: a1.exclusiveOr(a2); break;
+				case NOT: a1.subtract(a2); break;
+			}
+		} catch(Exception e) {}
 		Rectangle r = a1.getBounds();
 		at = new AffineTransform();
 		at.translate(-r.x, -r.y);
@@ -271,7 +266,11 @@ public class ShapeRoi extends Roi {
 				shape = new Line2D.Double ((double)(line.x1-r.x), (double)(line.y1-r.y), (double)(line.x2-r.x), (double)(line.y2-r.y) );
 				break;
 			case Roi.RECTANGLE:
-				shape = new Rectangle2D.Double(0.0, 0.0, (double)r.width, (double)r.height);
+				int arcSize = roi.getCornerDiameter();
+				if (arcSize>0)
+					shape = new RoundRectangle2D.Float(0, 0, r.width, r.height, arcSize, arcSize);
+				else
+					shape = new Rectangle2D.Double(0.0, 0.0, (double)r.width, (double)r.height);
 				break;
 			case Roi.OVAL:
 				Polygon p = roi.getPolygon();
@@ -280,7 +279,6 @@ public class ShapeRoi extends Roi {
 					p.ypoints[i] -= r.y;
 				}
 				shape = new Polygon(p.xpoints, p.ypoints, p.npoints);
-				//shape = new Ellipse2D.Double(0.0, 0.0, (double)r.width, (double)r.height);
 				break;
 			case Roi.POLYGON:
 				nCoords =((PolygonRoi)roi).getNCoordinates();
@@ -593,6 +591,9 @@ public class ShapeRoi extends Roi {
 
 	/** Caculates "Feret" (maximum caliper width) and "MinFeret" (minimum caliper width). */	
 	public double[] getFeretValues() {
+		Roi[] rois = getRois();
+		if (rois!=null && rois.length==1)
+			return rois[0].getFeretValues();
 		double min=Double.MAX_VALUE, diameter=0.0, angle=0.0;
 		int p1=0, p2=0;
 		double pw=1.0, ph=1.0;
@@ -639,29 +640,22 @@ public class ShapeRoi extends Roi {
 		return a;
 	}
 
-	/**Returns the length of this shape (perimeter, if shape is closed). */
+	/**Returns the perimeter if this ShapeRoi can be decomposed 
+		into simple ROIs, otherwise returns zero. */
 	public double getLength() {
-		/*
-		if(shape==null) return 0.0;
-		Rectangle2D r2d = shape.getBounds2D();
-		double w = r2d.getWidth();
-		double h = r2d.getHeight();
-		if(w==0 && h==0) return 0.0;
-		PathIterator pIter;
-		flatten = true;
-		if(flatten) pIter = getFlatteningPathIterator(shape, flatness);
-		else pIter = shape.getPathIterator(new AffineTransform());
-		double[] par = new double[1];
-		parsePath(pIter, par, null, null, null);
-		flatten = false;
-		return par[0];
-		*/
-		return 0.0;
+		double length = 0.0;
+		Roi[] rois = getRois();
+		if (rois!=null) {
+			for (int i=0; i<rois.length; i++)
+				length += rois[i].getLength();
+		}
+		return length;
 	}
 
 	/**Returns a flattened version of the path iterator for this ROi's shape*/
-	FlatteningPathIterator getFlatteningPathIterator(Shape s, double fl)
-	{ return (FlatteningPathIterator)s.getPathIterator(new AffineTransform(),fl); }
+	FlatteningPathIterator getFlatteningPathIterator(Shape s, double fl) {
+		return (FlatteningPathIterator)s.getPathIterator(new AffineTransform(),fl);
+	}
 
 	/**Length of the control polygon of the cubic B&eacute;zier curve argument, in double precision.*/
 	double cplength(CubicCurve2D.Double c) {
@@ -1043,22 +1037,35 @@ public class ShapeRoi extends Roi {
 
 	/** Non-destructively draws the shape of this object on the associated ImagePlus. */
 	public void draw(Graphics g) {
-		if (ic==null) return;
+		Color color =  strokeColor!=null? strokeColor:ROIColor;
+		if (fillColor!=null) color = fillColor;
+		g.setColor(color);
 		AffineTransform aTx = (((Graphics2D)g).getDeviceConfiguration()).getDefaultTransform();
-		g.setColor(instanceColor!=null?instanceColor:ROIColor);
-		if (stroke!=null) ((Graphics2D)g).setStroke(stroke);
-		mag = ic.getMagnification();
-		Rectangle r = ic.getSrcRect();
-		aTx.setTransform(mag, 0.0, 0.0, mag, -r.x*mag, -r.y*mag);
-        aTx.translate(x, y);
-		((Graphics2D)g).draw(aTx.createTransformedShape(shape));
-		if (Toolbar.getToolId()==Toolbar.OVAL) drawRoiBrush(g);
+		Graphics2D g2d = (Graphics2D)g;
+		if (stroke!=null)
+			g2d.setStroke(ic!=null&&ic.getCustomRoi()?stroke:getScaledStroke());
+		mag = getMagnification();
+		int basex=0, basey=0;
+		if (ic!=null) {
+			Rectangle r = ic.getSrcRect();
+			basex=r.x; basey=r.y;
+		}
+		aTx.setTransform(mag, 0.0, 0.0, mag, -basex*mag, -basey*mag);
+		aTx.translate(x, y);
+		if (fillColor!=null)
+			g2d.fill(aTx.createTransformedShape(shape));
+		else
+			g2d.draw(aTx.createTransformedShape(shape));
+		if (stroke!=null) g2d.setStroke(defaultStroke);
+		if (Toolbar.getToolId()==Toolbar.OVAL)
+			drawRoiBrush(g);
 		if (imp!=null&&imp.getRoi()!=null) showStatus();
 		if (updateFullWindow) 
 			{updateFullWindow = false; imp.draw();}
 	}
 
 	public void drawRoiBrush(Graphics g) {
+		g.setColor(ROIColor);
 		int size = Toolbar.getBrushSize();
 		if (size==0) return;
 		int flags = ic.getModifiers();
@@ -1075,7 +1082,7 @@ public class ShapeRoi extends Roi {
 	 * (i.e., all curve segments will be approximated by line segments).
 	 */
 	public void drawPixels(ImageProcessor ip) {
-		PathIterator pIter = getFlatteningPathIterator(shape,flatness);
+		PathIterator pIter = shape.getPathIterator(new AffineTransform(), flatness);
 		float[] coords = new float[6];
 		float sx=0f, sy=0f;
 		while (!pIter.isDone()) {
@@ -1104,12 +1111,16 @@ public class ShapeRoi extends Roi {
 		if(shape==null) return null;
 		if (cachedMask!=null && cachedMask.getPixels()!=null)
 			return cachedMask;
+		//Rectangle r = getBounds();
+		//if (r.x<0 || r.y<0) {
+		//	if (r.x<0) r.x = 0;
+		//	if (r.y<0) r.y = 0;
+		//	ShapeRoi clipRect = new ShapeRoi(new Roi(r.x,r.y,r.width,r.height));
+		//	setShape(getShape(this.or(clipRect)));
+		//}
 		BufferedImage bi = new BufferedImage(width, height, BufferedImage.TYPE_BYTE_GRAY);
 		Graphics2D g2d = bi.createGraphics();
 		g2d.setColor(Color.white);
-		//AffineTransform at = new AffineTransform();
-		//at.translate(-x, -y);
-		//g2d.fill(at.createTransformedShape(shape));
 		g2d.fill(shape);
 		Raster raster = bi.getRaster();
 		DataBufferByte buffer = (DataBufferByte)raster.getDataBuffer();		
@@ -1118,33 +1129,10 @@ public class ShapeRoi extends Roi {
         return cachedMask;
 	}
 
-	/**********************************************************************************/
-	/***                               Field accessors                             ****/
-	/**********************************************************************************/
-
-	/**Retrieves the value of {@link #forceTrace} flag.*/
-	//public boolean getForceTrace() { return forceTrace; }
-
-	/**Retrieves the value of {@link #forceAngle} flag.*/
-	//public boolean getForceAngle() { return forceAngle; }
-
-	/**Retrieves the value of {@link #flatten} flag.*/
-	//public boolean getFlatten() { return flatten; }
-
-	/**Retrieves the value of {@link #flatness}*/
-	//public double getFlatness() { return flatness; }
-
-	/**Retrieves the value of {@link #maxerror}*/
-	//public double getMaxError() { return maxerror;}
-
-	/**Retrieves the value of {@link #maxPoly}*/
-	//public int getMaxPoly() { return maxPoly; }
-
-	/**@return <strong><code>false</code></strong> if type equals {@link #NO_TYPE}. */
-	//public boolean isValid() { return type!=NO_TYPE; }
-
-	/**Retrieves a reference to the shape of <strong><code>this</code></strong>. */
-	Shape getShape(){ return shape; }
+	/**Returns a reference to the Shape object encapsulated by this ShapeRoi. */
+	public Shape getShape() {
+		return shape;
+	}
 
 	/**Sets the <code>java.awt.Shape</code> object encapsulated by <strong><code>this</code></strong>
 	 * to the argument.
@@ -1183,6 +1171,7 @@ public class ShapeRoi extends Roi {
 		return val;
 	}
 	
+	/**
 	public static void addCircle(String sx, String sy, String swidth) {
 		int x = Integer.parseInt(sx);
 		int y = Integer.parseInt(sy);
@@ -1213,6 +1202,7 @@ public class ShapeRoi extends Roi {
 			img.setRoi(roi);
 		}
 	}
+	*/
 
 	static ShapeRoi getCircularRoi(int x, int y, int width) {
 		return new ShapeRoi(new OvalRoi(x - width / 2, y - width / 2, width, width));
@@ -1223,9 +1213,12 @@ public class ShapeRoi extends Roi {
 		   return -1;
 	}
 	
-	/** Always returns null. */
 	public Polygon getConvexHull() {
-		return null;
+		Roi[] rois = getRois();
+		if (rois!=null && rois.length==1)
+			return rois[0].getConvexHull();
+		else
+			return null;
 	}
 
     /*
