@@ -8,6 +8,7 @@ import ij.*;
 import ij.plugin.frame.Recorder; 
 import ij.plugin.frame.Editor; 
 import ij.plugin.MacroInstaller;
+import ij.plugin.RectToolOptions;
 import ij.macro.Program;
 
 /** The ImageJ toolbar. */
@@ -39,11 +40,14 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 	
 	public static final int DOUBLE_CLICK_THRESHOLD = 650;
 
+	public static final int OVAL_ROI=0, ELLIPSE_ROI=1, BRUSH_ROI=2;
+
 	private static final int NUM_TOOLS = 23;
 	private static final int NUM_BUTTONS = 21;
-	private static final int SIZE = 24;
-	private static final int OFFSET = 4;
+	private static final int SIZE = 26;
+	private static final int OFFSET = 5;
 	private static final String BRUSH_SIZE = "toolbar.brush.size";
+	public static final String CORNER_DIAMETER = "toolbar.arc.size";
 		
 	private Dimension ps = new Dimension(SIZE*NUM_BUTTONS, SIZE);
 	private boolean[] down;
@@ -62,21 +66,25 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 	private String icon;
 	private MacroInstaller macroInstaller;
 	private int startupTime;
-	private PopupMenu ovalPopup, linePopup, switchPopup;
-	private CheckboxMenuItem ovalItem, brushItem;
-	private CheckboxMenuItem straightLineItem, polyLineItem, freeLineItem;
+	private PopupMenu rectPopup, ovalPopup, pointPopup, linePopup, switchPopup;
+	private CheckboxMenuItem rectItem, roundRectItem;
+	private CheckboxMenuItem ovalItem, ellipseItem, brushItem;
+	private CheckboxMenuItem pointItem, multiPointItem;
+	private CheckboxMenuItem straightLineItem, polyLineItem, freeLineItem, arrowItem;
 	private String currentSet = "Startup Macros";
 
 	private static Color foregroundColor = Prefs.getColor(Prefs.FCOLOR,Color.black);
 	private static Color backgroundColor = Prefs.getColor(Prefs.BCOLOR,Color.white);
-	private static boolean brushEnabled;
+	private static int ovalType = OVAL_ROI;
+	private static boolean multiPointMode = Prefs.multiPointMode;
+	private static boolean roundRectMode;
+	private static boolean arrowMode;
 	private static int brushSize = (int)Prefs.get(BRUSH_SIZE, 15);
+	private static int arcSize = (int)Prefs.get(CORNER_DIAMETER, 20);
 	private int lineType = LINE;
 	
 	private Color gray = ImageJ.backgroundColor;
 	private Color brighter = gray.brighter();
-	//private Color darker = gray.darker();
-	//private Color evenDarker = darker.darker();
 	private Color darker = new Color(175, 175, 175);
 	private Color evenDarker = new Color(110, 110, 110);
 	private Color triangleColor = new Color(150, 0, 0);
@@ -99,29 +107,57 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 	}
 
 	void addPopupMenus() {
+		rectPopup = new PopupMenu();
+		if (Menus.getFontSize()!=0)
+			rectPopup.setFont(Menus.getFont());
+		rectItem = new CheckboxMenuItem("Rectangle Tool", !roundRectMode);
+		rectItem.addItemListener(this);
+		rectPopup.add(rectItem);
+		roundRectItem = new CheckboxMenuItem("Rounded Rectangle Tool", roundRectMode);
+		roundRectItem.addItemListener(this);
+		rectPopup.add(roundRectItem);
+		add(rectPopup);
+
 		ovalPopup = new PopupMenu();
 		if (Menus.getFontSize()!=0)
 			ovalPopup.setFont(Menus.getFont());
-		ovalItem = new CheckboxMenuItem("Elliptical Selection Tool", !brushEnabled);
+		ovalItem = new CheckboxMenuItem("Oval selections", ovalType==OVAL_ROI);
 		ovalItem.addItemListener(this);
 		ovalPopup.add(ovalItem);
-		brushItem = new CheckboxMenuItem("Selection Brush Tool", brushEnabled);
+		ellipseItem = new CheckboxMenuItem("Elliptical selections", ovalType==ELLIPSE_ROI);
+		ellipseItem.addItemListener(this);
+		ovalPopup.add(ellipseItem);
+		brushItem = new CheckboxMenuItem("Selection Brush Tool", ovalType==BRUSH_ROI);
 		brushItem.addItemListener(this);
 		ovalPopup.add(brushItem);
 		add(ovalPopup);
 
+		pointPopup = new PopupMenu();
+		if (Menus.getFontSize()!=0)
+			pointPopup.setFont(Menus.getFont());
+		pointItem = new CheckboxMenuItem("Point Tool", !multiPointMode);
+		pointItem.addItemListener(this);
+		pointPopup.add(pointItem);
+		multiPointItem = new CheckboxMenuItem("Multi-point Tool", multiPointMode);
+		multiPointItem.addItemListener(this);
+		pointPopup.add(multiPointItem);
+		add(pointPopup);
+
 		linePopup = new PopupMenu();
 		if (Menus.getFontSize()!=0)
 			linePopup.setFont(Menus.getFont());
-		straightLineItem = new CheckboxMenuItem("Straight Lines", lineType==LINE);
+		straightLineItem = new CheckboxMenuItem("Straight Line", lineType==LINE&&!arrowMode);
 		straightLineItem.addItemListener(this);
 		linePopup.add(straightLineItem);
-		polyLineItem = new CheckboxMenuItem("Segmented Lines", lineType==POLYLINE);
+		polyLineItem = new CheckboxMenuItem("Segmented Line", lineType==POLYLINE);
 		polyLineItem.addItemListener(this);
 		linePopup.add(polyLineItem);
-		freeLineItem = new CheckboxMenuItem("Freehand Lines", lineType==FREELINE);
+		freeLineItem = new CheckboxMenuItem("Freehand Line", lineType==FREELINE);
 		freeLineItem.addItemListener(this);
 		linePopup.add(freeLineItem);
+		arrowItem = new CheckboxMenuItem("Arrow tool", lineType==LINE&&!arrowMode);
+		arrowItem.addItemListener(this);
+		linePopup.add(arrowItem);
 		add(linePopup);
 
 		switchPopup = new PopupMenu();
@@ -197,19 +233,28 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 		}
 		switch (tool) {
 			case RECTANGLE:
-				g.drawRect(x+1, y+2, 14, 11);
+				xOffset = x; yOffset = y;
+				if (roundRectMode)
+					g.drawRoundRect(x+1, y+2, 15, 12, 8, 8);
+				else
+					g.drawRect(x+1, y+2, 15, 12);
+				drawTriangle(15,14);
 				return;
 			case OVAL:
 				xOffset = x; yOffset = y;
-				if (brushEnabled) {
+				if (ovalType==BRUSH_ROI) {
 					m(9,2); d(13,2); d(13,2); d(15,5); d(15,8);
 					d(13,10); d(10,10); d(8,13); d(4,13); 
 					d(2,11);  d(2,7); d(4,5); d(7,5); d(9,2);
-					drawTriangle(13,14);
-				} else {
-					g.drawOval(x+1, y+2, 14, 11);
-					drawTriangle(13,14);
-				}
+				} else if (ovalType==ELLIPSE_ROI) {
+					yOffset = y + 1;
+					m(11,0); d(13,0); d(14,1); d(15,1); d(16,2); d(17,3); d(17,7);
+					d(12,12); d(11,12); d(10,13); d(8,13); d(7,14); d(4,14); d(3,13);
+					d(2,13); d(1,12); d(1,11); d(0,10); d(0,9); d(1,8); d(1,7);
+					d(6,2); d(7,2); d(8,1); d(10,1); d(11,0);
+				} else
+					g.drawOval(x+1, y+2, 15, 12);
+				drawTriangle(15,14);
 				return;
 			case POLYGON:
 				xOffset = x+1; yOffset = y+3;
@@ -224,26 +269,36 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 				return;
 			case LINE:
 				xOffset = x; yOffset = y;
-				m(0,10); d(16,4);
-				drawTriangle(11,13);
+				if (arrowMode) {
+					m(1,14); d(14,1); m(6,5); d(14,1); m(10,9); d(14,1); m(6,5); d(10,9);
+				} else {
+					m(0,12); d(17,3);
+				}
+				drawTriangle(12,14);
 				return;
 			case POLYLINE:
 				xOffset = x; yOffset = y;
 				m(14,6); d(11,3); d(1,3); d(1,4); d(6,9); d(2,13);
-				drawTriangle(11,13);
+				drawTriangle(12,14);
 				return;
 			case FREELINE:
 				xOffset = x; yOffset = y;
 				m(16,4); d(14,6); d(12,6); d(9,3); d(8,3); d(6,7); d(2,11); d(1,11);
-				drawTriangle(11,13);
+				drawTriangle(12,14);
 				return;
 			case POINT:
 				xOffset = x; yOffset = y;
-				m(1,8); d(6,8); d(6,6); d(10,6); d(10,10); d(6,10); d(6,9);
-				m(8,1); d(8,5); m(11,8); d(15,8); m(8,11); d(8,15);
-				m(8,8); d(8,8);
-				g.setColor(Roi.getColor());
-				g.fillRect(x+7, y+7, 3, 3);
+				if (multiPointMode) {
+					drawPoint(1,3); drawPoint(9,1); drawPoint(15,5);
+					drawPoint(10,11); drawPoint(2,12);
+				} else {
+					m(1,8); d(6,8); d(6,6); d(10,6); d(10,10); d(6,10); d(6,9);
+					m(8,1); d(8,5); m(11,8); d(15,8); m(8,11); d(8,15);
+					m(8,8); d(8,8);
+					g.setColor(Roi.getColor());
+					g.fillRect(x+7, y+7, 3, 3);
+				}
+				drawTriangle(14,14);
 				return;
 			case WAND:
 				xOffset = x+2; yOffset = y+2;
@@ -300,9 +355,18 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 		m(0,0); d(4,0); m(1,1); d(3,1); dot(2,2);
 	}
 	
+	void drawPoint(int x, int y) {
+		g.setColor(toolColor);
+		m(x-2,y); d(x+2,y);
+		m(x,y-2); d(x,y+2);
+		g.setColor(Roi.getColor());
+		dot(x,y);
+	}
+	
 	void drawIcon(Graphics g, int tool, int x, int y) {
 		if (null==g) return;
 		icon = icons[tool];
+		if (icon==null) return;
 		this.icon = icon;
 		int length = icon.length();
 		int x1, y1, x2, y2;
@@ -344,7 +408,7 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 		}
 		if (menus[tool]!=null && menus[tool].getItemCount()>0) { 
 			xOffset = x; yOffset = y;
-			drawTriangle(13, 14);
+			drawTriangle(14, 14);
 		}
 	}
 	
@@ -387,15 +451,21 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 			IJ.showStatus(name);
 			return;
 		}
+		String hint = " (right click to switch)";
 		switch (tool) {
 			case RECTANGLE:
-				IJ.showStatus("Rectangular selections");
+				if (roundRectMode)
+					IJ.showStatus("Rectangular or *rounded rectangular* selections"+hint);
+				else
+					IJ.showStatus("*Rectangular* or rounded rectangular selections"+hint);
 				return;
 			case OVAL:
-				if (brushEnabled)
-					IJ.showStatus("Elliptical or *brush* selections");
+				if (ovalType==BRUSH_ROI)
+					IJ.showStatus("Oval, elliptical or *brush* selections"+hint);
+				else if (ovalType==ELLIPSE_ROI)
+					IJ.showStatus("Oval, *elliptical* or brush selections"+hint);
 				else
-					IJ.showStatus("*Elliptical* or brush selections");
+					IJ.showStatus("*Oval*, elliptical or brush selections"+hint);
 				return;
 			case POLYGON:
 				IJ.showStatus("Polygon selections");
@@ -404,16 +474,22 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 				IJ.showStatus("Freehand selections");
 				return;
 			case LINE:
-				IJ.showStatus("Straight line selections (right click for other types)");
+				if (arrowMode)
+					IJ.showStatus("Straight, segmented or freehand lines, or *arrows*"+hint);
+				else
+					IJ.showStatus("*Straight*, segmented or freehand lines, or arrows"+hint);
 				return;
 			case POLYLINE:
-				IJ.showStatus("Segmented line selections");
+				IJ.showStatus("Straight, *segmented* or freehand lines, or arrows"+hint);
 				return;
 			case FREELINE:
-				IJ.showStatus("Freehand line selections");
+				IJ.showStatus("Straight, segmented or *freehand* lines, or arrows"+hint);
 				return;
 			case POINT:
-				IJ.showStatus("Point selections (shift click for multiple points)");
+				if (multiPointMode)
+					IJ.showStatus("Point or *multi-point* selections"+hint);
+				else
+					IJ.showStatus("*Point* or multi-point selections"+hint);
 				return;
 			case WAND:
 				IJ.showStatus("Wand (tracing) tool");
@@ -468,16 +544,32 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 		drawButtons(g);
 	}
 
-	public boolean setTool(String name) { 
+	public boolean setTool(String name) {
+		if (name==null) return false;
+		if (name.indexOf(" Tool")!=-1) { // macro tool?
+			for (int i=SPARE1; i<=SPARE9; i++) {
+				if (name.equals(names[i])) {
+					setTool(i);
+					return true;
+				}
+			}
+		}
 		name = name.toLowerCase(Locale.US);
 		boolean ok = true;
-		if (name.indexOf("rect")!=-1)
+		if (name.indexOf("round")!=-1) {
+			roundRectMode = true;
 			setTool(RECTANGLE);
-		else if (name.indexOf("ellip")!=-1 || name.indexOf("oval")!=-1) {
-			brushEnabled = false;
+		} else if (name.indexOf("rect")!=-1) {
+			roundRectMode = false;
+			setTool(RECTANGLE);
+		} else if (name.indexOf("oval")!=-1) {
+			ovalType = OVAL_ROI;
+			setTool(OVAL);
+		} else if (name.indexOf("ellip")!=-1) {
+			ovalType = ELLIPSE_ROI;
 			setTool(OVAL);
 		} else if (name.indexOf("brush")!=-1) {
-			brushEnabled = true;
+			ovalType = BRUSH_ROI;
 			setTool(OVAL);
 		} else if (name.indexOf("polygon")!=-1)
 			setTool(POLYGON);
@@ -485,13 +577,23 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 			setTool(POLYLINE);
 		else if (name.indexOf("freeline")!=-1)
 			setTool(FREELINE);
-		else if (name.indexOf("line")!=-1)
+		else if (name.indexOf("line")!=-1) {
+			arrowMode = false;
 			setTool(LINE);
-		else if (name.indexOf("free")!=-1)
+		} else if (name.indexOf("arrow")!=-1) {
+			arrowMode = true;
+			setTool(LINE);
+		} else if (name.indexOf("free")!=-1)
 			setTool(FREEROI);
-		else if (name.indexOf("point")!=-1)
+		else if (name.indexOf("multi")!=-1) {
+			multiPointMode = true;
+			Prefs.multiPointMode = true;
 			setTool(POINT);
-		else if (name.indexOf("wand")!=-1)
+		} else if (name.indexOf("point")!=-1) {
+			multiPointMode = false;
+			Prefs.multiPointMode = false;
+			setTool(POINT);
+		} else if (name.indexOf("wand")!=-1)
 			setTool(WAND);
 		else if (name.indexOf("text")!=-1)
 			setTool(TEXT);
@@ -508,8 +610,43 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 		return ok;
 	}
 	
+	/** Returns the name of the current tool. */
+	public static String getToolName() {
+		String name = instance.getName(current);
+		if (current>=SPARE1 && current<=SPARE9 && instance.names[current]!=null)
+			name = instance.names[current];
+		return name!=null?name:"";
+	}
+
+	/** Returns the name of the specified tool. */
+	String getName(int id) {
+		switch (id) {
+			case RECTANGLE: return roundRectMode?"roundrect":"rectangle";
+			case OVAL:
+				switch (ovalType) {
+					case OVAL_ROI: return "oval";
+					case ELLIPSE_ROI: return "ellipse";
+					case BRUSH_ROI: return "brush";
+				}
+			case POLYGON: return "polygon";
+			case FREEROI: return "freehand";
+			case LINE: return arrowMode?"arrow":"line";
+			case POLYLINE: return "polyline";
+			case FREELINE: return "freeline";
+			case ANGLE: return "angle";
+			case POINT: return Prefs.multiPointMode?"multipoint":"point";
+			case WAND: return "wand";
+			case TEXT: return "text";
+			case HAND: return "hand";
+			case MAGNIFIER: return "zoom";
+			case DROPPER: return "dropper";
+			default: return null;
+		}
+		
+	}
+	
 	public void setTool(int tool) {
-		if ((tool==current&&tool!=OVAL) || tool<0 || tool>=NUM_TOOLS-1)
+		if ((tool==current&&!(tool==RECTANGLE||tool==OVAL||tool==POINT)) || tool<0 || tool>=NUM_TOOLS-1)
 			return;
 		if (tool==SPARE1||(tool>=SPARE2&&tool<=SPARE8)) {
 			if (names[tool]==null)
@@ -522,11 +659,15 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 	}
 		
 	private void setTool2(int tool) {
-		if ((tool==current&&tool!=OVAL) || !isValidTool(tool)) return;
+		if (!isValidTool(tool)) return;
+		String previousName = getToolName();
 		current = tool;
 		down[current] = true;
-		down[previous] = false;
+		if (current!=previous)
+			down[previous] = false;
 		Graphics g = this.getGraphics();
+		if (g==null)
+			return;
 		if (Prefs.antialiasedTools) {
 			Graphics2D g2d = (Graphics2D)g;
 			g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
@@ -537,10 +678,14 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 		g.dispose();
 		showMessage(current);
 		previous = current;
-		if (Recorder.record)
-			Recorder.record("setTool", current);
+		if (Recorder.record) {
+			String name = getName(current);
+			if (name!=null) Recorder.record("setTool", name);
+		}
 		if (IJ.isMacOSX())
 			repaint();
+		if (!previousName.equals(getToolName()))
+			IJ.notifyEventListeners(IJEventListener.TOOL_CHANGED);
 	}
 	
 	boolean isValidTool(int tool) {
@@ -551,19 +696,25 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 		return true;
 	}
 
-	/** Obsolete. Use getForegroundColor(). */
+	/**
+	* @deprecated
+	* replaced by getForegroundColor()
+	*/
 	public Color getColor() {
 		return foregroundColor;
 	}
 
-	/** Obsolete. Use setForegroundColor(). */
+	/**
+	* @deprecated
+	* replaced by setForegroundColor()
+	*/
 	public void setColor(Color c) {
 		if (c!=null) {
 			foregroundColor = c;
 			drawButton(this.getGraphics(), DROPPER);
 		}
 	}
-
+	
 	public static Color getForegroundColor() {
 		return foregroundColor;
 	}
@@ -572,6 +723,8 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 		if (c!=null) {
 			foregroundColor = c;
 			repaintTool(DROPPER);
+			if (!IJ.isMacro()) setRoiColor(c);
+			IJ.notifyEventListeners(IJEventListener.FOREGROUND_COLOR_CHANGED);
 		}
 	}
 
@@ -583,28 +736,77 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 		if (c!=null) {
 			backgroundColor = c;
 			repaintTool(DROPPER);
+			IJ.notifyEventListeners(IJEventListener.BACKGROUND_COLOR_CHANGED);
 		}
 	}
 	
-	/** Returns the size of the brush tool or 0 if the brush tool is not enabled. */
+	private static void setRoiColor(Color c) {
+		ImagePlus imp = WindowManager.getCurrentImage();
+		if (imp==null) return;
+		Roi roi = imp.getRoi();
+		if (roi!=null && (roi.isDrawingTool())) {
+			roi.setStrokeColor(c);
+			imp.draw();
+		}
+	}
+
+	/** Returns the size of the brush tool, or 0 if the brush tool is not enabled. */
 	public static int getBrushSize() {
-		if (brushEnabled)
+		if (ovalType==BRUSH_ROI)
 			return brushSize;
 		else
 			return 0;
 	}
 
-	/** Set the size of the brush tool, which must be greater than 4. */
+	/** Set the size of the brush tool, in pixels. */
 	public static void setBrushSize(int size) {
 		brushSize = size;
-		if (brushSize<5) brushSize = 5;
+		if (brushSize<1) brushSize = 1;
 		Prefs.set(BRUSH_SIZE, brushSize);
 	}
+		
+	/** Returns the rounded rectangle arc size, or 0 if the rounded rectangle tool is not enabled. */
+	public static int getRoundRectArcSize() {
+		if (!roundRectMode)
+			return 0;
+		else
+			return arcSize;
+	}
+
+	/** Sets the rounded rectangle corner diameter (pixels). */
+	public static void setRoundRectArcSize(int size) {
+		if (size<=0)
+			roundRectMode = false;
+		else {
+			arcSize = size;
+			Prefs.set(CORNER_DIAMETER, arcSize);
+		}
+		repaintTool(RECTANGLE);
+		ImagePlus imp = WindowManager.getCurrentImage();
+		Roi roi = imp!=null?imp.getRoi():null;
+		if (roi!=null && roi.getType()==Roi.RECTANGLE)
+			roi.setCornerDiameter(roundRectMode?arcSize:0);
+	}
+
+	/** Returns 'true' if the multi-point tool is enabled. */
+	public static boolean getMultiPointMode() {
+		return multiPointMode;
+	}
+
+	/** Returns the oval tool type (OVAL_ROI, ELLIPSE_ROI or BRUSH_ROI). */
+	public static int getOvalToolType() {
+		return ovalType;
+	}
+
+	public static int getButtonSize() {
+		return SIZE;
+	}
 	
 	static void repaintTool(int tool) {
 		if (IJ.getInstance()!=null) {
 			Toolbar tb = getInstance();
 			Graphics g = tb.getGraphics();
+			if (g==null) return;
 			if (Prefs.antialiasedTools)
 				((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
 			tb.drawButton(g, tool);
@@ -698,17 +900,33 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 			}
 			setTool2(newTool);
 			boolean isRightClick = e.isPopupTrigger()||e.isMetaDown();
+			if (current==RECTANGLE && isRightClick) {
+				rectItem.setState(!roundRectMode);
+				roundRectItem.setState(roundRectMode);
+				if (IJ.isMacOSX()) IJ.wait(10);
+				rectPopup.show(e.getComponent(),x,y);
+				mouseDownTime = 0L;
+			}
 			if (current==OVAL && isRightClick) {
-				ovalItem.setState(!brushEnabled);
-				brushItem.setState(brushEnabled);
+				ovalItem.setState(ovalType==OVAL_ROI);
+				ellipseItem.setState(ovalType==ELLIPSE_ROI);
+				brushItem.setState(ovalType==BRUSH_ROI);
 				if (IJ.isMacOSX()) IJ.wait(10);
 				ovalPopup.show(e.getComponent(),x,y);
 				mouseDownTime = 0L;
 			}
+			if (current==POINT && isRightClick) {
+				pointItem.setState(!multiPointMode);
+				multiPointItem.setState(multiPointMode);
+				if (IJ.isMacOSX()) IJ.wait(10);
+				pointPopup.show(e.getComponent(),x,y);
+				mouseDownTime = 0L;
+			}
 			if (isLine(current) && isRightClick) {
-				straightLineItem.setState(lineType==LINE);
+				straightLineItem.setState(lineType==LINE&&!arrowMode);
 				polyLineItem.setState(lineType==POLYLINE);
 				freeLineItem.setState(lineType==FREELINE);
+				arrowItem.setState(lineType==LINE&&arrowMode);
 				if (IJ.isMacOSX()) IJ.wait(10);
 				linePopup.show(e.getComponent(),x,y);
 				mouseDownTime = 0L;
@@ -725,6 +943,10 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 			}
 			ImagePlus imp = WindowManager.getCurrentImage();
 			switch (current) {
+				case RECTANGLE:
+					if (roundRectMode)
+						IJ.doCommand("Rounded Rect Tool...");
+					break;
 				case OVAL:
 					showBrushDialog();
 					break;
@@ -735,10 +957,20 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 					}
 					break;
 				case LINE: case POLYLINE: case FREELINE:
-					IJ.runPlugIn("ij.plugin.frame.LineWidthAdjuster", "");
+					if (current==LINE && arrowMode) {
+						IJ.doCommand("Arrow Tool...");
+					} else
+						IJ.runPlugIn("ij.plugin.frame.LineWidthAdjuster", "");
+					break;
+				case ANGLE:
+					showAngleDialog();
 					break;
 				case POINT:
-					IJ.doCommand("Point Tool...");
+					if (multiPointMode) {
+						if (imp!=null && imp.getRoi()!=null)
+							IJ.doCommand("Add Selection...");
+					} else
+						IJ.doCommand("Point Tool...");
 					break;
 				case WAND:
 					IJ.doCommand("Wand Tool...");
@@ -821,11 +1053,6 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 		return tool==LINE || tool==POLYLINE || tool==FREELINE;
 	}
 	
-	// Returns the line type (LINE, POLYLINE or FREELINE). 
-	//public int getLineType() {
-	//	return lineType;
-	//}
-	
 	public void restorePreviousTool() {
 		setTool2(mpPrevious);
 	}
@@ -842,12 +1069,38 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 	
 	public void itemStateChanged(ItemEvent e) {
 		CheckboxMenuItem item = (CheckboxMenuItem)e.getSource();
-		if (item==ovalItem || item==brushItem) {
-			brushEnabled = item==brushItem;
+		String previousName = getToolName();
+		if (item==rectItem || item==roundRectItem) {
+			roundRectMode = item==roundRectItem;
+			repaintTool(RECTANGLE);
+			showMessage(RECTANGLE);
+			ImagePlus imp = WindowManager.getCurrentImage();
+			Roi roi = imp!=null?imp.getRoi():null;
+			if (roi!=null && roi.getType()==Roi.RECTANGLE)
+				roi.setCornerDiameter(roundRectMode?arcSize:0);
+			if (!previousName.equals(getToolName()))
+				IJ.notifyEventListeners(IJEventListener.TOOL_CHANGED);
+		} else if (item==ovalItem || item==ellipseItem || item==brushItem) {
+			if (item==brushItem)
+				ovalType = BRUSH_ROI;
+			else if (item==ellipseItem)
+				ovalType = ELLIPSE_ROI;
+			else
+				ovalType = OVAL_ROI;
 			repaintTool(OVAL);
 			showMessage(OVAL);
+			if (!previousName.equals(getToolName()))
+				IJ.notifyEventListeners(IJEventListener.TOOL_CHANGED);
+		} else if (item==pointItem || item==multiPointItem) {
+			multiPointMode = item==multiPointItem;
+			Prefs.multiPointMode = multiPointMode;
+			repaintTool(POINT);
+			showMessage(POINT);
+			if (!previousName.equals(getToolName()))
+				IJ.notifyEventListeners(IJEventListener.TOOL_CHANGED);
 		} else if (item==straightLineItem) {
 			lineType = LINE;
+			arrowMode = false;
 			setTool2(LINE);
 			showMessage(LINE);
 		} else if (item==polyLineItem) {
@@ -858,6 +1111,11 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 			lineType = FREELINE;
 			setTool2(FREELINE);
 			showMessage(FREELINE);
+		} else if (item==arrowItem) {
+			lineType = LINE;
+			arrowMode = true;
+			setTool2(LINE);
+			showMessage(LINE);
 		} else {
 			String label = item.getActionCommand();
 			if (!label.equals("Help...")) currentSet = label;
@@ -942,7 +1200,7 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 
 	/** Adds a tool to the toolbar. The 'toolTip' string is displayed in the status bar
 		 when the mouse is over the tool icon. The 'toolTip' string may include icon 
-		(http://rsb.info.nih.gov/ij/developer/macro/macros.html#tools).
+		(http://imagej.nih.gov/ij/developer/macro/macros.html#tools).
 		Returns the tool ID, or -1 if all tools are in use. */
 	public int addTool(String toolTip) {
 		int index = toolTip.indexOf('-');
@@ -1024,18 +1282,28 @@ public class Toolbar extends Canvas implements MouseListener, MouseMotionListene
 	
 	void showBrushDialog() {
 		GenericDialog gd = new GenericDialog("Selection Brush");
-		gd.addCheckbox("Enable Selection Brush", brushEnabled);
+		gd.addCheckbox("Enable selection brush", ovalType==BRUSH_ROI);
 		gd.addNumericField("           Size:", brushSize, 0, 4, "pixels");
 		gd.showDialog();
 		if (gd.wasCanceled()) return;
-		brushEnabled = gd.getNextBoolean();
+		if (gd.getNextBoolean())
+			ovalType = BRUSH_ROI;
 		brushSize = (int)gd.getNextNumber();
+		if (brushSize<1) brushSize=1;
 		repaintTool(OVAL);
 		ImagePlus img = WindowManager.getCurrentImage();
 		Roi roi = img!=null?img.getRoi():null;
-		if (roi!=null && roi.getType()==Roi.OVAL && brushEnabled) img.killRoi();
+		if (roi!=null && roi.getType()==Roi.OVAL && ovalType==BRUSH_ROI)
+			img.killRoi();
 		Prefs.set(BRUSH_SIZE, brushSize);
 	}
 
+	void showAngleDialog() {
+		GenericDialog gd = new GenericDialog("Angle Tool");
+		gd.addCheckbox("Measure reflex angle", Prefs.reflexAngle);
+		gd.showDialog();
+		if (!gd.wasCanceled())
+			Prefs.reflexAngle = gd.getNextBoolean();
+	}
 
 }
