@@ -8,12 +8,15 @@ import ij.measure.Calibration;
 import ij.plugin.filter.Analyzer;
 import ij.plugin.frame.Recorder;
 import ij.plugin.JpegWriter;
+import ij.plugin.Orthogonal_Views;
+import ij.gui.*;
+import ij.measure.Measurements;
 import javax.imageio.*;
 
 /** Saves images in tiff, gif, jpeg, raw, zip and text format. */
 public class FileSaver {
 
-	public static final int DEFAULT_JPEG_QUALITY = 75;
+	public static final int DEFAULT_JPEG_QUALITY = 85;
 	private static int jpegQuality;
 	
     static {setJpegQuality(ij.Prefs.getInt(ij.Prefs.JPEG, DEFAULT_JPEG_QUALITY));}
@@ -23,6 +26,7 @@ public class FileSaver {
 	private FileInfo fi;
 	private String name;
 	private String directory;
+	private boolean saveName;
 
 	/** Constructs a FileSaver from an ImagePlus. */
 	public FileSaver(ImagePlus imp) {
@@ -45,12 +49,16 @@ public class FileSaver {
 			if (f==null || !f.exists())
 				return saveAsTiff();
 			if (!IJ.isMacro()) {
-				if (!IJ.showMessageWithCancel("Save as TIFF", "The file "+ofi.fileName+" already exists.\nDo you want to replace it?"))
+				GenericDialog gd = new GenericDialog("Save as TIFF");
+				gd.addMessage("\""+ofi.fileName+"\" already exists.\nDo you want to replace it?");
+				gd.setOKLabel("Replace");
+				gd.showDialog();
+				if (gd.wasCanceled())
 					return false;
 			}
 			IJ.showStatus("Saving "+path);
 			if (imp.getStackSize()>1) {
-				IJ.saveAs("tif", path);
+				IJ.saveAs(imp, "tif", path);
 				return true;
 			} else
 		    	return saveAsTiff(path);
@@ -94,31 +102,76 @@ public class FileSaver {
 			fi.sliceLabels[0] = (String)label;
 		}
 		fi.description = getDescriptionString();
+		fi.roi = RoiEncoder.saveAsByteArray(imp.getRoi());
+		fi.overlay = getOverlay(imp);
 		try {
 			TiffEncoder file = new TiffEncoder(fi);
 			DataOutputStream out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(path)));
 			file.write(out);
 			out.close();
-		}
-		catch (IOException e) {
+		} catch (IOException e) {
 			showErrorMessage(e);
 			return false;
 		}
-		updateImp(fi, fi.TIFF);
+		updateImp(fi, FileInfo.TIFF);
 		return true;
 	}
+	
+	byte[][] getOverlay(ImagePlus imp) {
+		if (imp.getHideOverlay())
+			return null;
+		Overlay overlay = imp.getOverlay();
+		if (overlay==null) {
+			ImageCanvas ic = imp.getCanvas();
+			if (ic==null) return null;
+			overlay = ic.getShowAllList(); // ROI Manager "Show All" list
+			if (overlay==null) return null;
+		}
+		int n = overlay.size();
+		if (n==0) return null;
+		if (Orthogonal_Views.isOrthoViewsImage(imp))
+			return null;
+		byte[][] array = new byte[n][];
+		for (int i=0; i<overlay.size(); i++) {
+			Roi roi = overlay.get(i);
+			if (i==0)
+				roi.setPrototypeOverlay(overlay);
+			array[i] = RoiEncoder.saveAsByteArray(roi);
+		}
+		return array;
+	}
 
 	/** Save the stack as a multi-image TIFF using the specified path. */
 	public boolean saveAsTiffStack(String path) {
 		if (fi.nImages==1)
 			{IJ.error("This is not a stack"); return false;}
-		if (fi.pixels==null && imp.getStack().isVirtual())
-			{IJ.error("Save As Tiff", "Virtual stacks not supported."); return false;}
+		boolean virtualStack = imp.getStack().isVirtual();
+		if (virtualStack)
+			fi.virtualStack = (VirtualStack)imp.getStack();
 		Object info = imp.getProperty("Info");
 		if (info!=null && (info instanceof String))
 			fi.info = (String)info;
 		fi.description = getDescriptionString();
-		fi.sliceLabels = imp.getStack().getSliceLabels();
+		if (virtualStack) {
+			FileInfo fi = imp.getOriginalFileInfo();
+			if (path!=null && path.equals(fi.directory+fi.fileName)) {
+				IJ.error("TIFF virtual stacks cannot be saved in place.");
+				return false;
+			}
+			String[] labels = null;
+			ImageStack vs = imp.getStack();
+			for (int i=1; i<=vs.getSize(); i++) {
+				ImageProcessor ip = vs.getProcessor(i);
+				String label = vs.getSliceLabel(i);
+				if (i==1 && (label==null||label.length()<200)) break;
+				if (labels==null) labels = new String[vs.getSize()];
+				labels[i-1] = label;
+			}
+			fi.sliceLabels = labels;
+		} else
+			fi.sliceLabels = imp.getStack().getSliceLabels();
+		fi.roi = RoiEncoder.saveAsByteArray(imp.getRoi());
+		fi.overlay = getOverlay(imp);
 		if (imp.isComposite()) saveDisplayRangesAndLuts(imp, fi);
 		try {
 			TiffEncoder file = new TiffEncoder(fi);
@@ -130,10 +183,37 @@ public class FileSaver {
 			showErrorMessage(e);
 			return false;
 		}
-		updateImp(fi, fi.TIFF);
+		updateImp(fi, FileInfo.TIFF);
 		return true;
 	}
 	
+	/** Converts this image to a TIFF encoded array of bytes, 
+		which can be decoded using Opener.deserialize(). */
+	public byte[] serialize() {
+		if (imp.getStack().isVirtual())
+			return null;
+		Object info = imp.getProperty("Info");
+		if (info!=null && (info instanceof String))
+			fi.info = (String)info;
+		saveName = true;
+		fi.description = getDescriptionString();
+		saveName = false;
+		fi.sliceLabels = imp.getStack().getSliceLabels();
+		fi.roi = RoiEncoder.saveAsByteArray(imp.getRoi());
+		fi.overlay = getOverlay(imp);
+		if (imp.isComposite()) saveDisplayRangesAndLuts(imp, fi);
+		ByteArrayOutputStream out = null;
+		try {
+			TiffEncoder encoder = new TiffEncoder(fi);
+			out = new ByteArrayOutputStream();
+			encoder.write(out);
+			out.close();
+		} catch (IOException e) {
+			return null;
+		}
+		return out.toByteArray();
+	}
+
 	void  saveDisplayRangesAndLuts(ImagePlus imp, FileInfo fi) {
 		CompositeImage ci = (CompositeImage)imp;
 		int channels = imp.getNChannels();
@@ -180,8 +260,12 @@ public class FileSaver {
 		Object info = imp.getProperty("Info");
 		if (info!=null && (info instanceof String))
 			fi.info = (String)info;
+		fi.roi = RoiEncoder.saveAsByteArray(imp.getRoi());
+		fi.overlay = getOverlay(imp);
 		fi.sliceLabels = imp.getStack().getSliceLabels();
 		if (imp.isComposite()) saveDisplayRangesAndLuts(imp, fi);
+		if (fi.nImages>1 && imp.getStack().isVirtual())
+			fi.virtualStack = (VirtualStack)imp.getStack();
 		try {
 			ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(path));
 			DataOutputStream out = new DataOutputStream(new BufferedOutputStream(zos));
@@ -194,7 +278,7 @@ public class FileSaver {
 			showErrorMessage(e);
 			return false;
 		}
-		updateImp(fi, fi.TIFF);
+		updateImp(fi, FileInfo.TIFF);
 		return true;
 	}
 
@@ -225,7 +309,7 @@ public class FileSaver {
 	public boolean saveAsGif(String path) {
 		if (!okForGif(imp)) return false;
 		IJ.runPlugIn(imp, "ij.plugin.GifWriter", path);
-		updateImp(fi, fi.GIF_OR_JPG);
+		updateImp(fi, FileInfo.GIF_OR_JPG);
 		return true;
 	}
 
@@ -255,7 +339,7 @@ public class FileSaver {
 	public boolean saveAsJpeg(String path) {
 		String err = JpegWriter.save(imp, path, jpegQuality);
 		if (err==null && !(imp.getType()==ImagePlus.GRAY16 || imp.getType()==ImagePlus.GRAY32))
-			updateImp(fi, fi.GIF_OR_JPG);
+			updateImp(fi, FileInfo.GIF_OR_JPG);
 		return true;
 	}
 
@@ -272,6 +356,7 @@ public class FileSaver {
 	/** Save the image in BMP format using the specified path. */
 	public boolean saveAsBmp(String path) {
 		IJ.runPlugIn(imp, "ij.plugin.BMP_Writer", path);
+		updateImp(fi, FileInfo.BMP);
 		return true;
 	}
 
@@ -294,12 +379,12 @@ public class FileSaver {
 		using the specified path. */
 	public boolean saveAsPgm(String path) {
 		IJ.runPlugIn(imp, "ij.plugin.PNM_Writer", path);
+		updateImp(fi, FileInfo.PGM);
 		return true;
 	}
 
 	/** Save the image in PNG format using a save file dialog. 
-		Returns false if the user selects cancel. Requires
-		java 1.4 or later. */
+		Returns false if the user selects cancel. */
 	public boolean saveAsPng() {
 		String path = getPath("PNG", ".png");
 		if (path==null)
@@ -308,10 +393,10 @@ public class FileSaver {
 			return saveAsPng(path);
 	}
 
-	/** Save the image in PNG format using the specified path. 
-		Requires Java 1,4 or later. */
+	/** Save the image in PNG format using the specified path. */
 	public boolean saveAsPng(String path) {
 		IJ.runPlugIn(imp, "ij.plugin.PNG_Writer", path);
+		updateImp(fi, FileInfo.IMAGEIO);
 		return true;
 	}
 
@@ -330,6 +415,7 @@ public class FileSaver {
 	public boolean saveAsFits(String path) {
 		if (!okForFits(imp)) return false;
 		IJ.runPlugIn(imp, "ij.plugin.FITS_Writer", path);
+		updateImp(fi, FileInfo.FITS);
 		return true;
 	}
 
@@ -394,9 +480,14 @@ public class FileSaver {
 		boolean signed16Bit = false;
 		Object[] stack = null;
 		int n = 0;
+		boolean virtualStack = imp.getStackSize()>1 && imp.getStack().isVirtual();
+		if (virtualStack) {
+			fi.virtualStack = (VirtualStack)imp.getStack();
+			if (imp.getProperty("AnalyzeFormat")!=null) fi.fileName="FlipTheseImages";
+		}
 		try {
 			signed16Bit = imp.getCalibration().isSigned16Bit();
-			if (signed16Bit) {
+			if (signed16Bit && !virtualStack) {
 				stack = (Object[])fi.pixels;
 				n = imp.getWidth()*imp.getHeight();
 				for (int slice=0; slice<fi.nImages; slice++) {
@@ -439,6 +530,10 @@ public class FileSaver {
 		try {
 			Calibration cal = imp.getCalibration();
 			int precision = Analyzer.getPrecision();
+			int measurements = Analyzer.getMeasurements();
+			boolean scientificNotation = (measurements&Measurements.SCIENTIFIC_NOTATION)!=0;
+			if (scientificNotation)
+				precision = -precision;
 			TextEncoder file = new TextEncoder(imp.getProcessor(), cal, precision);
 			DataOutputStream out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(path)));
 			file.write(out);
@@ -507,6 +602,16 @@ public class FileSaver {
 		imp.changes = false;
 		if (name!=null) {
 			fi.fileFormat = fileFormat;
+			FileInfo ofi = imp.getOriginalFileInfo();
+			if (ofi!=null) {
+				if (ofi.openNextName==null) {
+					fi.openNextName = ofi.fileName;
+					fi.openNextDir = ofi.directory;
+				} else {
+					fi.openNextName = ofi.openNextName;
+					fi.openNextDir = ofi.openNextDir ;
+				}
+			}
 			fi.fileName = name;
 			fi.directory = directory;
 			//if (fileFormat==fi.TIFF)
@@ -546,7 +651,7 @@ public class FileSaver {
 			sb.append("mode="+mode+"\n");
 		}
 		if (fi.unit!=null)
-			sb.append("unit="+fi.unit+"\n");
+			sb.append("unit="+(fi.unit.equals("\u00B5m")?"um":fi.unit)+"\n");
 		if (fi.valueUnit!=null && fi.calibrationFunction!=Calibration.CUSTOM) {
 			sb.append("cf="+fi.calibrationFunction+"\n");
 			if (fi.coefficients!=null) {
@@ -597,7 +702,9 @@ public class FileSaver {
 		if (cal.zOrigin!=0.0)
 			sb.append("zorigin="+cal.zOrigin+"\n");
 		if (cal.info!=null && cal.info.length()<=64 && cal.info.indexOf('=')==-1 && cal.info.indexOf('\n')==-1)
-			sb.append("info="+cal.info+"\n");			
+			sb.append("info="+cal.info+"\n");
+		if (saveName)
+			sb.append("name="+imp.getTitle()+"\n");
 		sb.append((char)0);
 		return new String(sb);
 	}
