@@ -5,6 +5,9 @@ import java.io.*;
 import java.net.*;
 import java.awt.image.BufferedImage;
 import javax.imageio.ImageIO;
+import java.util.zip.Inflater;
+import java.util.zip.DataFormatException;
+
 
 /** Reads raw 8-bit, 16-bit or 32-bit (float or RGB)
 	images from a stream or URL. */
@@ -16,7 +19,8 @@ public class ImageReader {
     private FileInfo fi;
     private int width, height;
     private long skipCount;
-    private int bytesPerPixel, bufferSize, byteCount, nPixels;
+    private int bytesPerPixel, bufferSize, nPixels;
+    private long byteCount;
 	private boolean showProgressBar=true;
 	private int eofErrorCount;
 	private long startTime;
@@ -38,7 +42,7 @@ public class ImageReader {
 	}
 	
 	byte[] read8bitImage(InputStream in) throws IOException {
-		if (fi.compression==FileInfo.LZW || fi.compression==FileInfo.LZW_WITH_DIFFERENCING || fi.compression==FileInfo.PACK_BITS)
+		if (fi.compression>FileInfo.COMPRESSION_NONE)
 			return readCompressed8bitImage(in);
 		byte[] pixels = new byte[nPixels];
 		// assume contiguous strips
@@ -46,7 +50,7 @@ public class ImageReader {
 		int totalRead = 0;
 	  	while (totalRead<byteCount) {
 	  		if (totalRead+bufferSize>byteCount)
-	  			count = byteCount-totalRead;
+	  			count = (int)(byteCount-totalRead);
   			else
   				count = bufferSize;
   			actuallyRead = in.read(pixels, totalRead, count);
@@ -65,8 +69,8 @@ public class ImageReader {
 			if (in instanceof RandomAccessStream)
 				((RandomAccessStream)in).seek(fi.stripOffsets[i]);
 			else if (i > 0) {
-				int skip = fi.stripOffsets[i] - fi.stripOffsets[i-1] - fi.stripLengths[i-1];
-				if (skip > 0) in.skip(skip);
+				long skip = (fi.stripOffsets[i]&0xffffffffL) - (fi.stripOffsets[i-1]&0xffffffffL) - fi.stripLengths[i-1];
+				if (skip > 0L) in.skip(skip);
 			}
 			byte[] byteArray = new byte[fi.stripLengths[i]];
 			int read = 0, left = byteArray.length;
@@ -79,7 +83,7 @@ public class ImageReader {
 			byteArray = uncompress(byteArray);
 			int length = byteArray.length;
 			length = length - (length%fi.width);
-			if (fi.compression == FileInfo.LZW_WITH_DIFFERENCING) {
+			if (fi.compression==FileInfo.LZW_WITH_DIFFERENCING) {
 				for (int b=0; b<length; b++) {
 					byteArray[b] += last;
 					last = b % fi.width == fi.width - 1 ? 0 : byteArray[b];
@@ -95,19 +99,19 @@ public class ImageReader {
 	
 	/** Reads a 16-bit image. Signed pixels are converted to unsigned by adding 32768. */
 	short[] read16bitImage(InputStream in) throws IOException {
-		if (fi.compression==FileInfo.LZW || fi.compression==FileInfo.LZW_WITH_DIFFERENCING || fi.compression==FileInfo.PACK_BITS)
+		if (fi.compression>FileInfo.COMPRESSION_NONE || (fi.stripOffsets!=null&&fi.stripOffsets.length>1))
 			return readCompressed16bitImage(in);
 		int pixelsRead;
 		byte[] buffer = new byte[bufferSize];
 		short[] pixels = new short[nPixels];
-		int totalRead = 0;
+		long totalRead = 0L;
 		int base = 0;
 		int count, value;
 		int bufferCount;
 		
 		while (totalRead<byteCount) {
 			if ((totalRead+bufferSize)>byteCount)
-				bufferSize = byteCount-totalRead;
+				bufferSize = (int)(byteCount-totalRead);
 			bufferCount = 0;
 			while (bufferCount<bufferSize) { // fill the buffer
 				count = in.read(buffer, bufferCount, bufferSize-bufferCount);
@@ -144,17 +148,16 @@ public class ImageReader {
 	}
 	
 	short[] readCompressed16bitImage(InputStream in) throws IOException {
-		in = new DataInputStream(in);
 		short[] pixels = new short[nPixels];
 		int base = 0;
 		short last = 0;
 		for (int k=0; k<fi.stripOffsets.length; k++) {
-			//IJ.log("seek: "+fi.stripOffsets[k]+" "+(in instanceof RandomAccessStream));
+			//IJ.log("seek: "+fi.stripOffsets[k]+" "+fi.stripLengths[k]+"  "+(in instanceof RandomAccessStream));
 			if (in instanceof RandomAccessStream)
 				((RandomAccessStream)in).seek(fi.stripOffsets[k]);
 			else if (k > 0) {
-				int skip = fi.stripOffsets[k] - fi.stripOffsets[k-1] - fi.stripLengths[k-1];
-				if (skip > 0) in.skip(skip);
+				long skip = (fi.stripOffsets[k]&0xffffffffL) - (fi.stripOffsets[k-1]&0xffffffffL) - fi.stripLengths[k-1];
+				if (skip > 0L) in.skip(skip);
 			}
 			byte[] byteArray = new byte[fi.stripLengths[k]];
 			int read = 0, left = byteArray.length;
@@ -170,19 +173,11 @@ public class ImageReader {
 			int pmax = base+pixelsRead;
 			if (pmax > nPixels) pmax = nPixels;
 			if (fi.intelByteOrder) {
-				if (fi.fileType==FileInfo.GRAY16_SIGNED)
-					for (int i=base,j=0; i<pmax; i++,j+=2)
-						pixels[i] = (short)((((byteArray[j+1]&0xff)<<8) | (byteArray[j]&0xff))+32768);
-				else
-					for (int i=base,j=0; i<pmax; i++,j+=2)
-						pixels[i] = (short)(((byteArray[j+1]&0xff)<<8) | (byteArray[j]&0xff));
+				for (int i=base,j=0; i<pmax; i++,j+=2)
+					pixels[i] = (short)(((byteArray[j+1]&0xff)<<8) | (byteArray[j]&0xff));
 			} else {
-				if (fi.fileType==FileInfo.GRAY16_SIGNED)
-					for (int i=base,j=0; i<pmax; i++,j+=2)
-						pixels[i] = (short)((((byteArray[j]&0xff)<<8) | (byteArray[j+1]&0xff))+32768);
-				else
-					for (int i=base,j=0; i<pmax; i++,j+=2)
-						pixels[i] = (short)(((byteArray[j]&0xff)<<8) | (byteArray[j+1]&0xff));
+				for (int i=base,j=0; i<pmax; i++,j+=2)
+					pixels[i] = (short)(((byteArray[j]&0xff)<<8) | (byteArray[j+1]&0xff));
 			}
 			if (fi.compression==FileInfo.LZW_WITH_DIFFERENCING) {
 				for (int b=base; b<pmax; b++) {
@@ -193,14 +188,21 @@ public class ImageReader {
 			base += pixelsRead;
 			showProgress(k+1, fi.stripOffsets.length);
 		}
+		if (fi.fileType==FileInfo.GRAY16_SIGNED) {
+			// convert to unsigned
+			for (int i=0; i<nPixels; i++)
+				pixels[i] = (short)(pixels[i]+32768);
+		}
 		return pixels;
 	}
 
 	float[] read32bitImage(InputStream in) throws IOException {
+		if (fi.compression>FileInfo.COMPRESSION_NONE)
+			return readCompressed32bitImage(in);
 		int pixelsRead;
 		byte[] buffer = new byte[bufferSize];
 		float[] pixels = new float[nPixels];
-		int totalRead = 0;
+		long totalRead = 0L;
 		int base = 0;
 		int count, value;
 		int bufferCount;
@@ -208,7 +210,7 @@ public class ImageReader {
 		
 		while (totalRead<byteCount) {
 			if ((totalRead+bufferSize)>byteCount)
-				bufferSize = byteCount-totalRead;
+				bufferSize = (int)(byteCount-totalRead);
 			bufferCount = 0;
 			while (bufferCount<bufferSize) { // fill the buffer
 				count = in.read(buffer, bufferCount, bufferSize-bufferCount);
@@ -224,9 +226,11 @@ public class ImageReader {
 			totalRead += bufferSize;
 			showProgress(totalRead, byteCount);
 			pixelsRead = bufferSize/bytesPerPixel;
+			int pmax = base+pixelsRead;
+			if (pmax>nPixels) pmax = nPixels;
 			int j = 0;
 			if (fi.intelByteOrder)
-				for (int i=base; i < (base+pixelsRead); i++) {
+				for (int i=base; i<pmax; i++) {
 					tmp = (int)(((buffer[j+3]&0xff)<<24) | ((buffer[j+2]&0xff)<<16) | ((buffer[j+1]&0xff)<<8) | (buffer[j]&0xff));
 					if (fi.fileType==FileInfo.GRAY32_FLOAT)
 						pixels[i] = Float.intBitsToFloat(tmp);
@@ -237,7 +241,7 @@ public class ImageReader {
 					j += 4;
 				}
 			else
-				for (int i=base; i < (base+pixelsRead); i++) {
+				for (int i=base; i<pmax; i++) {
 					tmp = (int)(((buffer[j]&0xff)<<24) | ((buffer[j+1]&0xff)<<16) | ((buffer[j+2]&0xff)<<8) | (buffer[j+3]&0xff));
 					if (fi.fileType==FileInfo.GRAY32_FLOAT)
 						pixels[i] = Float.intBitsToFloat(tmp);
@@ -252,11 +256,60 @@ public class ImageReader {
 		return pixels;
 	}
 	
+	float[] readCompressed32bitImage(InputStream in) throws IOException {
+		float[] pixels = new float[nPixels];
+		int base = 0;
+		float last = 0;
+		for (int k=0; k<fi.stripOffsets.length; k++) {
+			//IJ.log("seek: "+fi.stripOffsets[k]+" "+(in instanceof RandomAccessStream));
+			if (in instanceof RandomAccessStream)
+				((RandomAccessStream)in).seek(fi.stripOffsets[k]);
+			else if (k > 0) {
+				long skip = (fi.stripOffsets[k]&0xffffffffL) - (fi.stripOffsets[k-1]&0xffffffffL) - fi.stripLengths[k-1];
+				if (skip > 0L) in.skip(skip);
+			}
+			byte[] byteArray = new byte[fi.stripLengths[k]];
+			int read = 0, left = byteArray.length;
+			while (left > 0) {
+				int r = in.read(byteArray, read, left);
+				if (r == -1) {eofError(); break;}
+				read += r;
+				left -= r;
+			}
+			byteArray = uncompress(byteArray);
+			int pixelsRead = byteArray.length/bytesPerPixel;
+			pixelsRead = pixelsRead - (pixelsRead%fi.width);
+			int pmax = base+pixelsRead;
+			if (pmax > nPixels) pmax = nPixels;
+			int tmp;
+			if (fi.intelByteOrder) {
+				for (int i=base,j=0; i<pmax; i++,j+=4) {
+					tmp = (int)(((byteArray[j+3]&0xff)<<24) | ((byteArray[j+2]&0xff)<<16) | ((byteArray[j+1]&0xff)<<8) | (byteArray[j]&0xff));
+					pixels[i] = Float.intBitsToFloat(tmp);
+				}
+			} else {
+				for (int i=base,j=0; i<pmax; i++,j+=4) {
+					tmp = (int)(((byteArray[j]&0xff)<<24) | ((byteArray[j+1]&0xff)<<16) | ((byteArray[j+2]&0xff)<<8) | (byteArray[j+3]&0xff));
+					pixels[i] = Float.intBitsToFloat(tmp);
+				}
+			}
+			if (fi.compression==FileInfo.LZW_WITH_DIFFERENCING) {
+				for (int b=base; b<pmax; b++) {
+					pixels[b] += last;
+					last = b % fi.width == fi.width - 1 ? 0 : pixels[b];
+				}
+			}
+			base += pixelsRead;
+			showProgress(k+1, fi.stripOffsets.length);
+		}
+		return pixels;
+	}
+
 	float[] read64bitImage(InputStream in) throws IOException {
 		int pixelsRead;
 		byte[] buffer = new byte[bufferSize];
 		float[] pixels = new float[nPixels];
-		int totalRead = 0;
+		long totalRead = 0L;
 		int base = 0;
 		int count, value;
 		int bufferCount;
@@ -265,7 +318,7 @@ public class ImageReader {
 		
 		while (totalRead<byteCount) {
 			if ((totalRead+bufferSize)>byteCount)
-				bufferSize = byteCount-totalRead;
+				bufferSize = (int)(byteCount-totalRead);
 			bufferCount = 0;
 			while (bufferCount<bufferSize) { // fill the buffer
 				count = in.read(buffer, bufferCount, bufferSize-bufferCount);
@@ -298,15 +351,15 @@ public class ImageReader {
 	}
 
 	int[] readChunkyRGB(InputStream in) throws IOException {
-		if (fi.compression==FileInfo.LZW || fi.compression==FileInfo.LZW_WITH_DIFFERENCING || fi.compression==FileInfo.PACK_BITS)
-			return readCompressedChunkyRGB(in);
-		else if (fi.compression==FileInfo.JPEG)
+		if (fi.compression==FileInfo.JPEG)
 			return readJPEG(in);
+		else if (fi.compression>FileInfo.COMPRESSION_NONE)
+			return readCompressedChunkyRGB(in);
 		int pixelsRead;
 		bufferSize = 24*width;
 		byte[] buffer = new byte[bufferSize];
 		int[] pixels = new int[nPixels];
-		int totalRead = 0;
+		long totalRead = 0L;
 		int base = 0;
 		int count, value;
 		int bufferCount;
@@ -314,7 +367,7 @@ public class ImageReader {
 		
 		while (totalRead<byteCount) {
 			if ((totalRead+bufferSize)>byteCount)
-				bufferSize = byteCount-totalRead;
+				bufferSize = (int)(byteCount-totalRead);
 			bufferCount = 0;
 			while (bufferCount<bufferSize) { // fill the buffer
 				count = in.read(buffer, bufferCount, bufferSize-bufferCount);
@@ -339,16 +392,16 @@ public class ImageReader {
 						j++; // ignore alfa byte
 						r = buffer[j++]&0xff;
 						g = buffer[j++]&0xff;
-					} else if (fi.intelByteOrder) {
+					} else if (fi.fileType==FileInfo.ABGR) {
+						b = buffer[j++]&0xff;
+						g = buffer[j++]&0xff;
+						r = buffer[j++]&0xff;
+						j++; // ignore alfa byte
+					} else { // ARGB
 						r = buffer[j++]&0xff;
 						g = buffer[j++]&0xff;
 						b = buffer[j++]&0xff;
 						j++; // ignore alfa byte
-					} else {
-						j++; // ignore alfa byte
-						b = buffer[j++]&0xff;
-						g = buffer[j++]&0xff;
-						r = buffer[j++]&0xff;
 					}
 				} else {
 					r = buffer[j++]&0xff;
@@ -375,8 +428,8 @@ public class ImageReader {
 		boolean differencing = fi.compression == FileInfo.LZW_WITH_DIFFERENCING;
 		for (int i=0; i<fi.stripOffsets.length; i++) {
 			if (i > 0) {
-				int skip = fi.stripOffsets[i] - fi.stripOffsets[i-1] - fi.stripLengths[i-1];
-				if (skip > 0) in.skip(skip);
+				long skip = (fi.stripOffsets[i]&0xffffffffL) - (fi.stripOffsets[i-1]&0xffffffffL) - fi.stripLengths[i-1];
+				if (skip > 0L) in.skip(skip);
 			}
 			byte[] byteArray = new byte[fi.stripLengths[i]];
 			int read = 0, left = byteArray.length;
@@ -400,17 +453,10 @@ public class ImageReader {
 			if (pmax > nPixels) pmax = nPixels;
 			for (int j=base; j<pmax; j++) {
 				if (bytesPerPixel==4) {
-					if (fi.intelByteOrder) {
-						red = byteArray[k++]&0xff;
-						green = byteArray[k++]&0xff;
-						blue = byteArray[k++]&0xff;
-						k++; // ignore alfa byte
-					} else {
-						k++; // ignore alfa byte
-						blue = byteArray[k++]&0xff;
-						green = byteArray[k++]&0xff;
-						red = byteArray[k++]&0xff;
-					}
+					red = byteArray[k++]&0xff;
+					green = byteArray[k++]&0xff;
+					blue = byteArray[k++]&0xff;
+					k++; // ignore alfa byte
 				} else {
 					red = byteArray[k++]&0xff;
 					green = byteArray[k++]&0xff;
@@ -434,7 +480,7 @@ public class ImageReader {
 	}
 
 	int[] readPlanarRGB(InputStream in) throws IOException {
-		if (fi.compression == FileInfo.LZW || fi.compression == FileInfo.LZW_WITH_DIFFERENCING || fi.compression==FileInfo.PACK_BITS)
+		if (fi.compression>FileInfo.COMPRESSION_NONE)
 			return readCompressedPlanarRGBImage(in);
 		DataInputStream dis = new DataInputStream(in);
 		int planeSize = nPixels; // 1/3 image size
@@ -494,16 +540,26 @@ public class ImageReader {
 			IJ.showProgress(current, last);
 	}
 	
+	private void showProgress(long current, long last) {
+		showProgress((int)(current/10L), (int)(last/10L));
+	}
+	
 	Object readRGB48(InputStream in) throws IOException {
+		if (fi.compression>FileInfo.COMPRESSION_NONE)
+			return readCompressedRGB48(in);
 		int channels = 3;
 		short[][] stack = new short[channels][nPixels];
 		DataInputStream dis = new DataInputStream(in);
 		int pixel = 0;
 		int min=65535, max=0;
+		if (fi.stripLengths==null) {
+			fi.stripLengths = new int[fi.stripOffsets.length];
+			fi.stripLengths[0] = width*height*bytesPerPixel;
+		}
 		for (int i=0; i<fi.stripOffsets.length; i++) {
 			if (i>0) {
-				int skip = fi.stripOffsets[i] - fi.stripOffsets[i-1] - fi.stripLengths[i-1];
-				if (skip>0) dis.skip(skip);
+				long skip = (fi.stripOffsets[i]&0xffffffffL) - (fi.stripOffsets[i-1]&0xffffffffL) - fi.stripLengths[i-1];
+				if (skip>0L) dis.skip(skip);
 			}
 			int len = fi.stripLengths[i];
 			int bytesToGo = (nPixels-pixel)*channels*2;
@@ -533,6 +589,48 @@ public class ImageReader {
 		return stack;
 	}
 
+	Object readCompressedRGB48(InputStream in) throws IOException {
+		if (fi.compression==FileInfo.LZW_WITH_DIFFERENCING)
+			throw new IOException("ImageJ cannot open 48-bit LZW compressed TIFFs with predictor");
+		int channels = 3;
+		short[][] stack = new short[channels][nPixels];
+		DataInputStream dis = new DataInputStream(in);
+		int pixel = 0;
+		int min=65535, max=0;
+		for (int i=0; i<fi.stripOffsets.length; i++) {
+			if (i>0) {
+				long skip = (fi.stripOffsets[i]&0xffffffffL) - (fi.stripOffsets[i-1]&0xffffffffL) - fi.stripLengths[i-1];
+				if (skip>0L) dis.skip(skip);
+			}
+			int len = fi.stripLengths[i];
+			byte[] buffer = new byte[len];
+			dis.readFully(buffer);
+			buffer = uncompress(buffer);
+			len = buffer.length;
+			if (len % 2 != 0) len--;
+			int value;
+			int channel=0;
+			boolean intel = fi.intelByteOrder;
+			for (int base=0; base<len && pixel<nPixels; base+=2) {
+				if (intel)
+					value = ((buffer[base+1]&0xff)<<8) | (buffer[base]&0xff);
+				else
+					value = ((buffer[base]&0xff)<<8) | (buffer[base+1]&0xff);
+				if (value<min) min = value;
+				if (value>max) max = value;
+				stack[channel][pixel] = (short)(value);
+				channel++;
+				if (channel==channels) {
+					channel = 0;
+					pixel++;
+				}
+			}
+			showProgress(i+1, fi.stripOffsets.length);
+		}
+		this.min=min; this.max=max;
+		return stack;
+	}
+
 	Object readRGB48Planar(InputStream in) throws IOException {
 		short[] red = read16bitImage(in);
 		short[] green = read16bitImage(in);
@@ -545,18 +643,23 @@ public class ImageReader {
 	}
 
 	short[] read12bitImage(InputStream in) throws IOException {
-		int nBytes = (int)(nPixels*1.5);
-		if ((nPixels&1)==1) nBytes++; // add 1 if odd
-		byte[] buffer = new byte[nBytes];
+		int bytesPerLine = (int)(width*1.5);
+		if ((width&1)==1) bytesPerLine++; // add 1 if odd
+		byte[] buffer = new byte[bytesPerLine*height];
 		short[] pixels = new short[nPixels];
 		DataInputStream dis = new DataInputStream(in);
 		dis.readFully(buffer);
-		int i = 0;
-		int j = 0;
-		for (int index=0; index<buffer.length/3; index++) {
-			pixels[j++] = (short)(((buffer[i]&0xff)*16) + ((buffer[i+1]>>4)&0xf));
-			pixels[j++] = (short)(((buffer[i+1]&0xf)*256) + (buffer[i+2]&0xff));
-			i += 3;
+		for (int y=0; y<height; y++) {
+			int index1 = y*bytesPerLine;
+			int index2 = y*width;
+			int count = 0;
+			while (count<width) {
+				pixels[index2+count] = (short)(((buffer[index1]&0xff)*16) + ((buffer[index1+1]>>4)&0xf));
+				count++;
+				if (count==width) break;
+				pixels[index2+count] = (short)(((buffer[index1+1]&0xf)*256) + (buffer[index1+2]&0xff));
+				count++; index1+=3;
+			}
 		}
 		return pixels;
 	}
@@ -581,6 +684,8 @@ public class ImageReader {
 	}
 
 	byte[] read1bitImage(InputStream in) throws IOException {
+		if (fi.compression==FileInfo.LZW)
+			throw new IOException("ImageJ cannot open 1-bit LZW compressed TIFFs");
  		int scan=(int)Math.ceil(width/8.0);
 		int len = scan*height;
 		byte[] buffer = new byte[len];
@@ -616,14 +721,14 @@ public class ImageReader {
 				//IJ.log("skip: "+skipCount+" "+count+" "+bytesRead+" "+skipAttempts);
 			}
 		}
-		byteCount = width*height*bytesPerPixel;
+		byteCount = ((long)width)*height*bytesPerPixel;
 		if (fi.fileType==FileInfo.BITMAP) {
  			int scan=width/8, pad = width%8;
 			if (pad>0) scan++;
 			byteCount = scan*height;
 		}
 		nPixels = width*height;
-		bufferSize = byteCount/25;
+		bufferSize = (int)(byteCount/25L);
 		if (bufferSize<8192)
 			bufferSize = 8192;
 		else
@@ -667,6 +772,7 @@ public class ImageReader {
 				case FileInfo.RGB:
 				case FileInfo.BGR:
 				case FileInfo.ARGB:
+				case FileInfo.ABGR:
 				case FileInfo.BARG:
 					bytesPerPixel = fi.getBytesPerPixel();
 					skip(in);
@@ -743,10 +849,32 @@ public class ImageReader {
 	}
 	
 	byte[] uncompress(byte[] input) {
-			if (fi.compression==FileInfo.PACK_BITS)
-				return packBitsUncompress(input, fi.rowsPerStrip*fi.width*fi.getBytesPerPixel());
-			else
-				return lzwUncompress(input);
+		if (fi.compression==FileInfo.PACK_BITS)
+			return packBitsUncompress(input, fi.rowsPerStrip*fi.width*fi.getBytesPerPixel());
+		else if (fi.compression==FileInfo.LZW || fi.compression==FileInfo.LZW_WITH_DIFFERENCING)
+			return lzwUncompress(input);
+		else if (fi.compression==FileInfo.ZIP)
+			return zipUncompress(input);
+		else
+			return input;
+	}
+
+	/** TIFF Adobe ZIP support contributed by Jason Newton. */
+	public byte[] zipUncompress(byte[] input) {
+		ByteArrayOutputStream imageBuffer = new ByteArrayOutputStream();
+		byte[] buffer = new byte[1024];
+		Inflater decompressor = new Inflater();
+		decompressor.setInput(input);
+		try {
+			while(!decompressor.finished()) {
+				int rlen = decompressor.inflate(buffer);
+				imageBuffer.write(buffer, 0, rlen);
+			}
+		} catch(DataFormatException e){
+			IJ.log(e.toString());
+		}
+		decompressor.end();
+		return imageBuffer.toByteArray();
 	}
 
   /**
@@ -756,7 +884,7 @@ public class ImageReader {
  * @author Curtis Rueden (ctrueden at wisc.edu)
  */
 	public byte[] lzwUncompress(byte[] input) {
-		if (input == null || input.length == 0)
+		if (input==null || input.length==0)
 			return input;
 		byte[][] symbolTable = new byte[4096][1];
 		int bitsToRead = 9;
@@ -768,23 +896,23 @@ public class ImageReader {
 		byte[] byteBuffer1 = new byte[16];
 		byte[] byteBuffer2 = new byte[16];
 		
-		while (true) {
+		while (out.size()<byteCount) {
 			code = bb.getBits(bitsToRead);
-			if (code == EOI_CODE || code == -1)
+			if (code==EOI_CODE || code==-1)
 				break;
-			if (code == CLEAR_CODE) {
+			if (code==CLEAR_CODE) {
 				// initialize symbol table
 				for (int i = 0; i < 256; i++)
 					symbolTable[i][0] = (byte)i;
 				nextSymbol = 258;
 				bitsToRead = 9;
 				code = bb.getBits(bitsToRead);
-				if (code == EOI_CODE || code == -1)
+				if (code==EOI_CODE || code==-1)
 					break;
 				out.add(symbolTable[code]);
 				oldCode = code;
 			} else {
-				if (code < nextSymbol) {
+				if (code<nextSymbol) {
 					// code is in table
 					out.add(symbolTable[code]);
 					// add string to table
@@ -815,6 +943,7 @@ public class ImageReader {
 	 
 	/** Based on the Bio-Formats PackbitsCodec written by Melissa Linkert. */
 	public byte[] packBitsUncompress(byte[] input, int expected) {
+		if (expected==0) expected = Integer.MAX_VALUE;
 		ByteVector output = new ByteVector(1024);
 		int index = 0;
 		while (output.size()<expected && index<input.length) {
