@@ -5,8 +5,11 @@ import ij.process.*;
 import ij.plugin.frame.*;
 import ij.plugin.DICOM;
 import ij.plugin.AVI_Reader;
+import ij.plugin.SimpleCommands;
 import ij.text.TextWindow;
 import ij.util.Java2;
+import ij.measure.ResultsTable;
+import ij.macro.Interpreter;
 import java.awt.*;
 import java.awt.image.*;
 import java.io.*;
@@ -27,9 +30,9 @@ public class Opener {
 
 	public static final int UNKNOWN=0,TIFF=1,DICOM=2,FITS=3,PGM=4,JPEG=5,
 		GIF=6,LUT=7,BMP=8,ZIP=9,JAVA_OR_TEXT=10,ROI=11,TEXT=12,PNG=13,
-		TIFF_AND_DICOM=14,CUSTOM=15, AVI=16; // don't forget to also update 'types'
-	private static final String[] types = {"unknown","tif","dcm","fits","pgm",
-		"jpg","gif","lut","bmp","zip","java/txt","roi","txt","png","t&d","custom","avi"};
+		TIFF_AND_DICOM=14,CUSTOM=15, AVI=16, OJJ=17, TABLE=18; // don't forget to also update 'types'
+	public static final String[] types = {"unknown","tif","dcm","fits","pgm",
+		"jpg","gif","lut","bmp","zip","java/txt","roi","txt","png","t&d","custom","ojj","table"};
 	private static String defaultDirectory = null;
 	private static int fileType;
 	private boolean error;
@@ -113,15 +116,28 @@ public class Opener {
 		roi, or text file. Displays an error message if the specified file
 		is not in one of the supported formats. */
 	public void open(String path) {
-        boolean fullPath = path.startsWith("/") || path.startsWith("\\") || path.indexOf(":\\")==1 || path.startsWith("http://");
-        if (!fullPath && IJ.getInstance()!=null) {
-            String workingDir = OpenDialog.getDefaultDirectory();
-            if (workingDir!=null)
-                path = workingDir + path;
-        }
+		boolean isURL = path.indexOf("://")>0;
+		if (isURL && isText(path)) {
+			openTextURL(path);
+			return;
+		}
+		if (path.endsWith(".jar") || path.endsWith(".class")) {
+				(new PluginInstaller()).install(path);
+				return;
+		}
+		boolean fullPath = path.startsWith("/") || path.startsWith("\\") || path.indexOf(":\\")==1 || isURL;
+		if (!fullPath) {
+			String defaultDir = OpenDialog.getDefaultDirectory();
+			if (defaultDir!=null)
+				path = defaultDir + path;
+			else
+				path = (new File(path)).getAbsolutePath();
+		}
 		if (!silentMode) IJ.showStatus("Opening: " + path);
 		long start = System.currentTimeMillis();
 		ImagePlus imp = openImage(path);
+		if (imp==null && isURL)
+			return;
 		if (imp!=null) {
 			WindowManager.checkForDuplicateName = true;
 			if (isRGB48)
@@ -158,6 +174,12 @@ public class Opener {
 					} else
 						new TextWindow(path,400,450);
 					break;
+				case OJJ:  // ObjectJ project
+					IJ.runPlugIn("ObjectJ_", path);
+					break;
+				case TABLE:  // ImageJ Results table
+					openResultsTable(path);
+					break;
 				case UNKNOWN:
 					String msg =
 						"File is not in a supported format, a reader\n"+
@@ -165,8 +187,12 @@ public class Opener {
 					if (path!=null) {
 						if (path.length()>64)
 							path = (new File(path)).getName();
-						if (path.length()<=64)
-							msg += " \n  \n"+path;
+						if (path.length()<=64) {
+							if (IJ.redirectingErrorMessages())
+								msg += " \n   "+path;
+							else
+								msg += " \n	 \n"+path;
+						}
 					}
 					if (openUsingPlugins)
 						msg += "\n \nNOTE: The \"OpenUsingPlugins\" option is set.";
@@ -178,9 +204,23 @@ public class Opener {
 		}
 	}
 	
+	private boolean isText(String path) {
+		if (path.endsWith(".txt") || path.endsWith(".ijm") || path.endsWith(".java")
+		|| path.endsWith(".js") || path.endsWith(".html") || path.endsWith(".htm")
+		|| path.endsWith("/"))
+			return true;
+		int lastSlash = path.lastIndexOf("/");
+		if (lastSlash==-1) lastSlash = 0;
+		int lastDot = path.lastIndexOf(".");
+		if (lastDot==-1 || lastDot<lastSlash || (path.length()-lastDot)>6)
+			return true;  // no extension
+		else
+			return false;
+	}
+	
 	/** Opens the specified file and adds it to the File/Open Recent menu.
 		Returns true if the file was opened successfully.  */
-	public  boolean openAndAddToRecent(String path) {
+	public	boolean openAndAddToRecent(String path) {
 		open(path);
 		if (!error)
 			Menus.addOpenRecentItem(path);
@@ -194,7 +234,7 @@ public class Opener {
 	public ImagePlus openImage(String directory, String name) {
 		ImagePlus imp;
 		FileOpener.setSilentMode(silentMode);
-		if (directory.length()>0 && !directory.endsWith(Prefs.separator))
+		if (directory.length()>0 && !(directory.endsWith("/")||directory.endsWith("\\")))
 			directory += Prefs.separator;
 		String path = directory+name;
 		fileType = getFileType(path);
@@ -211,8 +251,11 @@ public class Opener {
 				// "hybrid" files created by GE-Senographe 2000 D */
 				imp = openTiff(directory,name);
 				ImagePlus imp2 = (ImagePlus)IJ.runPlugIn("ij.plugin.DICOM", path);
-				if (imp!=null)				
+				if (imp!=null && imp2!=null)	 {		
 					imp.setProperty("Info",imp2.getProperty("Info"));
+					imp.setCalibration(imp2.getCalibration());
+				}
+				if (imp==null) imp=imp2;
 				return imp;
 			case FITS:
 				imp = (ImagePlus)IJ.runPlugIn("ij.plugin.FITS_Reader", path);
@@ -254,13 +297,9 @@ public class Opener {
 		pgm, gif or jpeg. Displays a file open dialog if 'path' is null or
 		an empty string. Returns an ImagePlus object if successful. */
 	public ImagePlus openImage(String path) {
-		if (path==null || path.equals("")) {
-			OpenDialog od = new OpenDialog("Open", "");
-			String dir = od.getDirectory();
-			String name = od.getFileName();
-			if (name==null) return null;
-			path = dir+name;
-		}
+		if (path==null || path.equals(""))
+			path = getPath();
+		if (path==null) return null;
 		ImagePlus img = null;
 		if (path.indexOf("://")>0)
 			img = openURL(path);
@@ -268,13 +307,34 @@ public class Opener {
 			img = openImage(getDir(path), getName(path));
 		return img;
 	}
+	
+	/** Open the nth image of the specified tiff stack. */
+	public ImagePlus openImage(String path, int n) {
+		if (path==null || path.equals(""))
+			path = getPath();
+		if (path==null) return null;
+		int type = getFileType(path);
+		if (type!=TIFF)
+			throw new IllegalArgumentException("TIFF file require");
+		return openTiff(path, n);
+	}
+
+	String getPath() {
+		OpenDialog od = new OpenDialog("Open", "");
+		String dir = od.getDirectory();
+		String name = od.getFileName();
+		if (name==null)
+			return null;
+		else
+			return dir+name;
+	}
 
 	/** Attempts to open the specified url as a tiff, zip compressed tiff, 
 		dicom, gif or jpeg. Tiff file names must end in ".tif", ZIP file names 
 		must end in ".zip" and dicom file names must end in ".dcm". Returns an 
 		ImagePlus object if successful. */
 	public ImagePlus openURL(String url) {
-	   	try {
+		try {
 			String name = "";
 			int index = url.lastIndexOf('/');
 			if (index==-1)
@@ -289,16 +349,16 @@ public class Opener {
 			IJ.showStatus(""+url);
 			String lurl = url.toLowerCase(Locale.US);
 			ImagePlus imp = null;
-		    if (lurl.endsWith(".tif"))
+			if (lurl.endsWith(".tif"))
 				imp = openTiff(u.openStream(), name);
-	 	    else if (lurl.endsWith(".zip"))
+			else if (lurl.endsWith(".zip"))
 				imp = openZipUsingUrl(u);
-	 	    else if (lurl.endsWith(".dcm")) {
+			else if (lurl.endsWith(".jpg") || lurl.endsWith(".gif"))
+				imp = openJpegOrGifUsingURL(name, u);
+			else if (lurl.endsWith(".dcm") || lurl.endsWith(".ima")) {
 				imp = (ImagePlus)IJ.runPlugIn("ij.plugin.DICOM", url);
 				if (imp!=null && imp.getWidth()==0) imp = null;
-			} else if (lurl.endsWith(".jpg") || lurl.endsWith(".gif"))
-				imp = openJpegOrGifUsingURL(name, u);
-			else if (lurl.endsWith(".png"))
+			} else if (lurl.endsWith(".png"))
 				imp = openPngUsingURL(name, u);
 			else {
 				URLConnection uc = u.openConnection();
@@ -312,20 +372,34 @@ public class Opener {
 			}
 			IJ.showStatus("");
 			return imp;
-    	} catch (Exception e) {
-    		String msg = e.getMessage();
-    		if (msg==null || msg.equals(""))
-    			msg = "" + e;	
+		} catch (Exception e) {
+			String msg = e.getMessage();
+			if (msg==null || msg.equals(""))
+				msg = "" + e;	
 			IJ.error("Open URL",msg + "\n \n" + url);
 			return null;
-	   	} 
+		} 
 	}
 	
-		//for (int i=1;; i++) {
-		//	String header = uc.getHeaderField(i);
-		//	if (header==null) break;
-		//	IJ.log(uc.getHeaderFieldKey(i) + " " + header);
-		//}
+	/** Used by open() and IJ.open() to open text URLs. */
+	void openTextURL(String url) {
+		if (url.endsWith(".pdf")||url.endsWith(".zip"))
+			return;
+		String text = IJ.openUrlAsString(url);
+		String name = url.substring(7);
+		int index = name.lastIndexOf("/");
+		int len = name.length();
+		if (index==len-1)
+			name = name.substring(0, len-1);
+		else if (index!=-1 && index<len-1)
+			name = name.substring(index+1);
+		name = name.replaceAll("%20", " ");
+		Editor ed = new Editor();
+		ed.setSize(600, 300);
+		ed.create(name, text);
+		IJ.showStatus("");
+	}
+
 	
 	public ImagePlus openWithHandleExtraFileTypes(String path, int[] fileType) {
 		ImagePlus imp = null;
@@ -358,25 +432,65 @@ public class Opener {
 	/** Opens the ZIP compressed TIFF or DICOM at the specified URL. */
 	ImagePlus openZipUsingUrl(URL url) throws IOException {
 		URLConnection uc = url.openConnection();
-		int fileSize = uc.getContentLength(); // compressed size
-		fileSize *=2; // estimate uncompressed size
-      	InputStream in = uc.getInputStream();
-		ZipInputStream zin = new ZipInputStream(in);
-		ByteArrayOutputStream out = new ByteArrayOutputStream();
-		byte[] buf = new byte[4096];
-		ZipEntry entry = zin.getNextEntry();
-		if (entry==null)
-			return null;
+		InputStream in = uc.getInputStream();
+		ZipInputStream zis = new ZipInputStream(in);
+		ZipEntry entry = zis.getNextEntry();
+		if (entry==null) return null;
 		String name = entry.getName();
-		//double fileSize = entry.getSize(); //returns -1
 		if (!(name.endsWith(".tif")||name.endsWith(".dcm")))
-			throw new IOException("This ZIP archive does not appear to contain a .tif or .dcm file");
-		if (name.endsWith(".dcm")) {
-			DICOM dcm = new DICOM(zin);
-			dcm.run(name);
-			return dcm;
-		} else
-			return openTiff(zin, name);
+			throw new IOException("This ZIP archive does not appear to contain a .tif or .dcm file\n"+name);
+		if (name.endsWith(".dcm"))
+			return openDicomStack(zis, entry);
+		else
+			return openTiff(zis, name);
+	}
+	
+	ImagePlus openDicomStack(ZipInputStream zis, ZipEntry entry) throws IOException {
+		ImagePlus imp = null;
+		int count = 0;
+		ImageStack stack = null;
+		while (true) {
+			if (count>0) entry = zis.getNextEntry();
+			if (entry==null) break;
+			String name = entry.getName();
+			ImagePlus imp2 = null;
+			if (name.endsWith(".dcm")) {
+				ByteArrayOutputStream out = new ByteArrayOutputStream();
+				byte[] buf = new byte[4096];
+				int len, byteCount=0, progress=0;
+				while (true) {
+					len = zis.read(buf);
+					if (len<0) break;
+					out.write(buf, 0, len);
+					byteCount += len;
+					//IJ.showProgress((double)(byteCount%fileSize)/fileSize);
+				}
+				byte[] bytes = out.toByteArray();
+				out.close();
+				DICOM dcm = new DICOM(new ByteArrayInputStream(bytes));
+				dcm.run(name);
+				imp2 = dcm;
+			}
+			zis.closeEntry();
+			if (imp2==null) continue;
+			count++;
+			String label = imp2.getTitle();
+			String info = (String)imp2.getProperty("Info");
+			if (info!=null) label += "\n" + info;
+			if (count==1) {
+				imp = imp2;
+				imp.getStack().setSliceLabel(label, 1);
+			} else {
+				stack = imp.getStack();
+				stack.addSlice(label, imp2.getProcessor());
+				imp.setStack(stack);
+			}
+		}
+		zis.close();
+		IJ.showProgress(1.0);
+		if (count==0)
+			throw new IOException("This ZIP archive does not appear to contain any .dcm files");
+		return imp;
 	}
 
 	ImagePlus openJpegOrGifUsingURL(String title, URL url) {
@@ -405,23 +519,23 @@ public class Opener {
 	}
 
 	ImagePlus openJpegOrGif(String dir, String name) {
-	   	ImagePlus imp = null;
+		ImagePlus imp = null;
 		Image img = Toolkit.getDefaultToolkit().createImage(dir+name);
- 		if (img!=null) {
- 			try {
- 				imp = new ImagePlus(name, img);
- 			} catch (IllegalStateException e) {
+		if (img!=null) {
+			try {
+				imp = new ImagePlus(name, img);
+			} catch (IllegalStateException e) {
 				return null; // error loading image
 			}				
-	    	if (imp.getType()==ImagePlus.COLOR_RGB)
-	    		convertGrayJpegTo8Bits(imp);
-	    	FileInfo fi = new FileInfo();
-	    	fi.fileFormat = fi.GIF_OR_JPG;
-	    	fi.fileName = name;
-	    	fi.directory = dir;
-	    	imp.setFileInfo(fi);
-	    }
-	    return imp;
+			if (imp.getType()==ImagePlus.COLOR_RGB)
+				convertGrayJpegTo8Bits(imp);
+			FileInfo fi = new FileInfo();
+			fi.fileFormat = fi.GIF_OR_JPG;
+			fi.fileName = name;
+			fi.directory = dir;
+			imp.setFileInfo(fi);
+		}
+		return imp;
 	}
 	
 	ImagePlus openUsingImageIO(String path) {
@@ -440,31 +554,27 @@ public class Opener {
 		fi.fileName = f.getName();
 		fi.directory = f.getParent()+File.separator;
 		imp.setFileInfo(fi);
-	    return imp;
+		return imp;
 	}
 
-	/** If this image is grayscale, convert it to 8-bits. */
+	/** If the specified image is grayscale, convert it to 8-bits. */
 	public static void convertGrayJpegTo8Bits(ImagePlus imp) {
 		ImageProcessor ip = imp.getProcessor();
 		int width = ip.getWidth();
 		int height = ip.getHeight();
 		int[] pixels = (int[])ip.getPixels();
 		int c,r,g,b,offset;
-		for (int y=0; y<(height-8); y++) {
+		for (int y=0; y<height; y++) {
 			offset = y*width;
-			for (int x=0; x<(width-8); x++) {
+			for (int x=0; x<width; x++) {
 				c = pixels[offset+x];
 				r = (c&0xff0000)>>16;
 				g = (c&0xff00)>>8;
 				b = c&0xff;
-				if (!((r==g)&&(g==b))) {
-					//IJ.write("count: "+count+" "+r+" "+g+" "+b);
-					return;
-				}
+				if (!((r==g)&&(g==b))) return;
 			}
-			//count++;
 		}
-		IJ.showStatus("Converting to 8-bits");
+		IJ.showStatus("Converting to 8-bit grayscale");
 		new ImageConverter(imp).convertToGray8();
 	}
 
@@ -472,20 +582,20 @@ public class Opener {
 	boolean allSameSizeAndType(FileInfo[] info) {
 		boolean sameSizeAndType = true;
 		boolean contiguous = true;
-		int startingOffset = info[0].offset;
+		long startingOffset = info[0].getOffset();
 		int size = info[0].width*info[0].height*info[0].getBytesPerPixel();
 		for (int i=1; i<info.length; i++) {
 			sameSizeAndType &= info[i].fileType==info[0].fileType
 				&& info[i].width==info[0].width
 				&& info[i].height==info[0].height;
-			contiguous &= info[i].offset==startingOffset+i*size;
+			contiguous &= info[i].getOffset()==startingOffset+i*size;
 		}
 		if (contiguous &&  info[0].fileType!=FileInfo.RGB48)
 			info[0].nImages = info.length;
-		if (IJ.debugMode) {
-			IJ.log("  sameSizeAndType: " + sameSizeAndType);
-			IJ.log("  contiguous: " + contiguous);
-		}
+		//if (IJ.debugMode) {
+		//	IJ.log("sameSizeAndType: " + sameSizeAndType);
+		//	IJ.log("contiguous: " + contiguous);
+		//}
 		return sameSizeAndType;
 	}
 	
@@ -501,7 +611,7 @@ public class Opener {
 			ColorModel cm = createColorModel(fi);
 			ImageStack stack = new ImageStack(fi.width, fi.height, cm);
 			Object pixels = null;
-			int skip = fi.offset;
+			long skip = fi.getOffset();
 			int imageSize = fi.width*fi.height*fi.getBytesPerPixel();
 			if (info[0].fileType==FileInfo.GRAY12_UNSIGNED) {
 				imageSize = (int)(fi.width*fi.height*1.5);
@@ -510,7 +620,7 @@ public class Opener {
 				int scan=(int)Math.ceil(fi.width/8.0);
 				imageSize = scan*fi.height;
 			}
-			int loc = 0;
+			long loc = 0L;
 			int nChannels = 1;
 			try {
 				InputStream is = createInputStream(fi);
@@ -534,7 +644,7 @@ public class Opener {
 						nChannels = fi.samplesPerPixel;
 						channels = new Object[nChannels];
 						for (int c=0; c<nChannels; c++) {
-							pixels = reader.readPixels(is, c==0?skip:0);
+							pixels = reader.readPixels(is, c==0?skip:0L);
 							channels[c] = pixels;
 						}
 					} else 
@@ -542,10 +652,12 @@ public class Opener {
 					if (pixels==null && channels==null) break;
 					loc += imageSize*nChannels+skip;
 					if (i<(info.length-1)) {
-						skip = info[i+1].offset-loc;
+						skip = info[i+1].getOffset()-loc;
 						if (info[i+1].compression>=FileInfo.LZW) skip = 0;
-						if (skip<0)
-							throw new IOException("Images are not in order");
+						if (skip<0L) {
+							IJ.error("Opener", "Unexpected image offset");
+							break;
+						}
 					}
 					if (fi.fileType==FileInfo.RGB48) {
 						Object[] pixels2 = (Object[])pixels;
@@ -559,14 +671,13 @@ public class Opener {
 								stack.addSlice(null, channels[c]);
 						}
 					} else
-						stack.addSlice(null, pixels);					
+						stack.addSlice(null, pixels);
 					IJ.showProgress(i, info.length);
 				}
 				is.close();
 			}
 			catch (Exception e) {
-				IJ.log("TiffDecoder: " + e);
-				e.printStackTrace();
+				IJ.handleException(e);
 			}
 			catch(OutOfMemoryError e) {
 				IJ.outOfMemory(fi.fileName);
@@ -616,6 +727,39 @@ public class Opener {
 		return openTiff2(info);
 	}
 	
+	/** Opens the nth image of the specified TIFF stack. */
+	public ImagePlus openTiff(String path, int n) {
+		TiffDecoder td = new TiffDecoder(getDir(path), getName(path));
+		if (IJ.debugMode) td.enableDebugging();
+		FileInfo[] info=null;
+		try {info = td.getTiffInfo();}
+		catch (IOException e) {
+			String msg = e.getMessage();
+			if (msg==null||msg.equals("")) msg = ""+e;
+			IJ.error("TiffDecoder", msg);
+			return null;
+		}
+		if (info==null) return null;
+		FileInfo fi = info[0];
+		if (info.length==1 && fi.nImages>1) {
+			if (n<1 || n>fi.nImages)
+				throw new IllegalArgumentException("N out of 1-"+fi.nImages+" range");
+			long size = fi.width*fi.height*fi.getBytesPerPixel();
+			fi.longOffset = fi.getOffset() + (n-1)*(size+fi.gapBetweenImages);
+			fi.offset = 0;
+			fi.nImages = 1;
+		} else {
+			if (n<1 || n>info.length)
+				throw new IllegalArgumentException("N out of 1-"+info.length+" range");
+			fi.longOffset = info[n-1].getOffset();
+			fi.offset = 0;
+			fi.stripOffsets = info[n-1].stripOffsets; 
+			fi.stripLengths = info[n-1].stripLengths; 
+		}
+		FileOpener fo = new FileOpener(fi);
+		return fo.open(false);
+	}
+
 	/** Attempts to open the specified inputStream as a
 		TIFF, returning an ImagePlus object if successful. */
 	public ImagePlus openTiff(InputStream in, String name) {
@@ -634,33 +778,39 @@ public class Opener {
 		return openTiff2(info);
 	}
 
-		/** Opens a single TIFF or DICOM contained in a ZIP archive,
-			or a ZIPed collection of ".roi" files created by the ROI manager. */	
+	/** Opens a single TIFF or DICOM contained in a ZIP archive,
+		or a ZIPed collection of ".roi" files created by the ROI manager. */	
 	public ImagePlus openZip(String path) {
 		ImagePlus imp = null;
 		try {
-			ZipInputStream in = new ZipInputStream(new FileInputStream(path));
-			ZipEntry entry = in.getNextEntry();
+			ZipInputStream zis = new ZipInputStream(new FileInputStream(path));
+			if (zis==null) return null;
+			ZipEntry entry = zis.getNextEntry();
 			if (entry==null) return null;
 			String name = entry.getName();
 			if (name.endsWith(".roi")) {
-				in.close();
-				IJ.runMacro("roiManager(\"Open\", getArgument());", path);
+				zis.close();
+				if (!silentMode)
+					if (IJ.isMacro() && Interpreter.isBatchMode() && RoiManager.getInstance()==null)
+						IJ.log("Use roiManager(\"Open\", path) instead of open(path)\nto open ROI sets in batch mode macros.");
+					else
+						IJ.runMacro("roiManager(\"Open\", getArgument());", path);
 				return null;
 			}
 			if (name.endsWith(".tif")) {
-				imp = openTiff(in, name);
+				imp = openTiff(zis, name);
 			} else if (name.endsWith(".dcm")) {
-				DICOM dcm = new DICOM(in);
+				DICOM dcm = new DICOM(zis);
 				dcm.run(name);
 				imp = dcm;
 			} else {
-				in.close();
+				zis.close();
 				IJ.error("This ZIP archive does not appear to contain a \nTIFF (\".tif\") or DICOM (\".dcm\") file, or ROIs (\".roi\").");
 				return null;
 			}
 		} catch (Exception e) {
 			IJ.error("ZipDecoder", ""+e);
+			return null;
 		}
 		File f = new File(path);
 		FileInfo fi = imp.getOriginalFileInfo();
@@ -670,6 +820,44 @@ public class Opener {
 		return imp;
 	}
 	
+	/** Deserialize a byte array that was serialized using the FileSaver.serialize(). */
+	public ImagePlus deserialize(byte[] bytes) {
+		ByteArrayInputStream stream = new ByteArrayInputStream(bytes);
+		TiffDecoder decoder = new TiffDecoder(stream, "Untitled");
+		if (IJ.debugMode)
+			decoder.enableDebugging();
+		FileInfo[] info = null;
+		try {
+			info = decoder.getTiffInfo();
+		} catch (IOException e) {
+			return null;
+		}
+		FileOpener opener = new FileOpener(info[0]);
+		ImagePlus imp = opener.open(false);
+		if (imp==null)
+			return null;
+		imp.setTitle(info[0].fileName);
+		imp = makeComposite(imp, info[0]);
+		return imp;
+   }
+   
+	private ImagePlus makeComposite(ImagePlus imp, FileInfo fi) {
+		int c = imp.getNChannels();
+		boolean composite = c>1 && fi.description!=null && fi.description.indexOf("mode=")!=-1;
+		if (c>1 && (imp.getOpenAsHyperStack()||composite) && !imp.isComposite() && imp.getType()!=ImagePlus.COLOR_RGB) {
+			int mode = CompositeImage.COLOR;
+			if (fi.description!=null) {
+				if (fi.description.indexOf("mode=composite")!=-1)
+					mode = CompositeImage.COMPOSITE;
+				else if (fi.description.indexOf("mode=gray")!=-1)
+					mode = CompositeImage.GRAYSCALE;
+			}
+			imp = new CompositeImage(imp, mode);
+		}
+		return imp;
+	}
+
+		
 	public String getName(String path) {
 		int i = path.lastIndexOf('/');
 		if (i==-1)
@@ -704,18 +892,10 @@ public class Opener {
 		FileOpener fo = new FileOpener(info[0]);
 		imp = fo.open(false);
 		if (imp==null) return null;
-		int c = imp.getNChannels();
-		boolean composite = c>1 && info[0].description!=null && info[0].description.indexOf("mode=")!=-1;
-		if (c>1 && (imp.getOpenAsHyperStack()||composite) && !imp.isComposite() && imp.getType()!=ImagePlus.COLOR_RGB) {
-			int mode = CompositeImage.COLOR;
-			if (info[0].description!=null) {
-				if (info[0].description.indexOf("mode=composite")!=-1)
-					mode = CompositeImage.COMPOSITE;
-				else if (info[0].description.indexOf("mode=gray")!=-1)
-					mode = CompositeImage.GRAYSCALE;
-			}
-			imp = new CompositeImage(imp, mode);
-		}
+		int[] offsets = info[0].stripOffsets;
+		if (offsets!=null&&offsets.length>1 && offsets[offsets.length-1]<offsets[0])
+			ij.IJ.run(imp, "Flip Vertically", "stack");
+		imp = makeComposite(imp, info[0]);
 		return imp;
 	}
 	
@@ -730,7 +910,24 @@ public class Opener {
 		}
 		return roi;
 	}
-
+	
+	/** Opens a tab or comma delimited text file in the Results window. */
+	public static void openResultsTable(String path) {
+		try {
+			ResultsTable rt = ResultsTable.open(path);
+			if (rt!=null) rt.show("Results");
+		} catch(IOException e) {
+			IJ.error("Open Results", e.getMessage());
+		}
+	}
+	
+	public static String getFileFormat(String path) {
+		if (!((new File(path)).exists()))
+			return("not found");
+		else
+			return Opener.types[(new Opener()).getFileType(path)];
+	}
+	
 	/**
 	Attempts to determine the image file type by looking for
 	'magic numbers' and the file name extension.
@@ -759,8 +956,8 @@ public class Opener {
 			return TIFF_AND_DICOM;
 
 		 // Big-endian TIFF ("MM")
-        if (name.endsWith(".lsm"))
-        		return UNKNOWN; // The LSM  Reader plugin opens these files
+		if (name.endsWith(".lsm"))
+				return UNKNOWN; // The LSM	Reader plugin opens these files
 		if (b0==73 && b1==73 && b2==42 && b3==0 && !(bioformats&&name.endsWith(".flex")))
 			return TIFF;
 
@@ -783,9 +980,9 @@ public class Opener {
 			return DICOM;
 		}
 
- 		// ACR/NEMA with first tag = 00002,00xx or 00008,00xx
- 		if ((b0==8||b0==2) && b1==0 && b3==0 && !name.endsWith(".spe") && !name.equals("fid")) 	
-  			 	return DICOM;
+		// ACR/NEMA with first tag = 00002,00xx or 00008,00xx
+		if ((b0==8||b0==2) && b1==0 && b3==0 && !name.endsWith(".spe") && !name.equals("fid"))	
+				return DICOM;
 
 		// PGM ("P1", "P4", "P2", "P5", "P3" or "P6")
 		if (b0==80&&(b1==49||b1==52||b1==50||b1==53||b1==51||b1==54)&&(b2==10||b2==13||b2==32||b2==9))
@@ -808,24 +1005,32 @@ public class Opener {
 			return FITS;
 			
 		// Java source file, text file or macro
-		if (name.endsWith(".java") || name.endsWith(".txt") || name.endsWith(".ijm") || name.endsWith(".js"))
+		if (name.endsWith(".java") || name.endsWith(".txt") || name.endsWith(".ijm") || name.endsWith(".js") || name.endsWith(".html"))
 			return JAVA_OR_TEXT;
 
 		// ImageJ, NIH Image, Scion Image for Windows ROI
 		if (b0==73 && b1==111) // "Iout"
 			return ROI;
 			
-        // Text file
-        boolean isText = true;
-        for (int i=0; i<10; i++) {
-          int c = buf[i]&255;
-          if ((c<32&&c!=9&&c!=10&&c!=13) || c>126) {
-              isText = false;
-              break;
-          }
-        }
-        if (isText)
-           return TEXT;
+		// ObjectJ project
+		if ((b0=='o' && b1=='j' && b2=='j' && b3==0) || name.endsWith(".ojj") )
+			return OJJ;
+
+		// Results table (tab-delimited or comma-separated tabular text)
+		if (name.endsWith(".xls") || name.endsWith(".csv")) 
+			return TABLE;
+
+		// Text file
+		boolean isText = true;
+		for (int i=0; i<10; i++) {
+		  int c = buf[i]&255;
+		  if ((c<32&&c!=9&&c!=10&&c!=13) || c>126) {
+			  isText = false;
+			  break;
+		  }
+		}
+		if (isText)
+		   return TEXT;
 
 		// BMP ("BM")
 		if ((b0==66 && b1==77)||name.endsWith(".dib"))
@@ -853,13 +1058,13 @@ public class Opener {
 		else if (fi.url!=null && !fi.url.equals(""))
 			return new URL(fi.url+fi.fileName).openStream();
 		else {
-		    File f = new File(fi.directory + fi.fileName);
-		    if (f==null || f.isDirectory())
-		    	return null;
-		    else {
-		    	InputStream is = new FileInputStream(f);
-		    	if (fi.compression>=FileInfo.LZW)
-		    		is = new RandomAccessStream(is);
+			File f = new File(fi.directory + fi.fileName);
+			if (f==null || f.isDirectory())
+				return null;
+			else {
+				InputStream is = new FileInputStream(f);
+				if (fi.compression>=FileInfo.LZW)
+					is = new RandomAccessStream(is);
 				return is;
 			}
 		}
@@ -888,5 +1093,5 @@ public class Opener {
 	public static boolean getOpenUsingPlugins() {
 		return openUsingPlugins;
 	}
-
+	
 }
