@@ -29,8 +29,11 @@ public class TiffDecoder {
 	public static final int RESOLUTION_UNIT = 296;
 	public static final int SOFTWARE = 305;
 	public static final int DATE_TIME = 306;
+	public static final int ARTEST = 315;
+	public static final int HOST_COMPUTER = 316;
 	public static final int PREDICTOR = 317;
 	public static final int COLOR_MAP = 320;
+	public static final int TILE_WIDTH = 322;
 	public static final int SAMPLE_FORMAT = 339;
 	public static final int JPEG_TABLES = 347;
 	public static final int METAMORPH1 = 33628;
@@ -55,7 +58,9 @@ public class TiffDecoder {
 	static final int LABELS = 0x6c61626c;  // "labl" (slice labels)
 	static final int RANGES = 0x72616e67;  // "rang" (display ranges)
 	static final int LUTS = 0x6c757473;  // "luts" (channel LUTs)
-
+	static final int ROI = 0x726f6920;  // "roi " (ROI)
+	static final int OVERLAY = 0x6f766572;  // "over" (overlay)
+	
 	private String directory;
 	private String name;
 	private String url;
@@ -65,6 +70,7 @@ public class TiffDecoder {
 	private String dInfo;
 	private int ifdCount;
 	private int[] metaDataCounts;
+	private String tiffMetadata;
 		
 	public TiffDecoder(String directory, String name) {
 		this.directory = directory;
@@ -150,20 +156,24 @@ public class TiffDecoder {
 		fi.greens = new byte[256];
 		fi.blues = new byte[256];
 		int j = 0;
-		if (littleEndian)
-			j++;
+		if (littleEndian) j++;
+		int sum = 0;
 		for (int i=0; i<256; i++) {
 			fi.reds[i] = colorTable16[j];
+			sum += fi.reds[i];
 			fi.greens[i] = colorTable16[512+j];
+			sum += fi.greens[i];
 			fi.blues[i] = colorTable16[1024+j];
+			sum += fi.blues[i];
 			j += 2;
 		}
-		fi.fileType = FileInfo.COLOR8;
+		if (sum!=0)
+			fi.fileType = FileInfo.COLOR8;
 	}
 	
 	byte[] getString(int count, long offset) throws IOException {
 		count--; // skip null byte at end of string
-		if (count<=0)
+		if (count<=3)
 			return null;
 		byte[] bytes = new byte[count];
 		long saveLoc = in.getLongFilePointer();
@@ -178,9 +188,10 @@ public class TiffDecoder {
 		stacks, it also saves the number of images to avoid having to
 		decode an IFD for each image. */
 	public void saveImageDescription(byte[] description, FileInfo fi) {
-		if (description.length<7)
-			return;
         String id = new String(description);
+        if (!id.startsWith("ImageJ"))
+			saveMetadata(getName(IMAGE_DESCRIPTION), id);
+		if (id.length()<7) return;
 		fi.description = id;
         int index1 = id.indexOf("images=");
         if (index1>0) {
@@ -193,6 +204,15 @@ public class TiffDecoder {
         }
 	}
 
+	public void saveMetadata(String name, String data) {
+		if (data==null) return;
+        String str = name+": "+data+"\n";
+        if (tiffMetadata==null)
+        	tiffMetadata = str;
+        else
+        	tiffMetadata += str;
+	}
+
 	void decodeNIHImageHeader(int offset, FileInfo fi) throws IOException {
 		long saveLoc = in.getLongFilePointer();
 		
@@ -275,6 +295,14 @@ public class TiffDecoder {
 	}
 	
 	void dumpTag(int tag, int count, int value, FileInfo fi) {
+		long lvalue = ((long)value)&0xffffffffL;
+		String name = getName(tag);
+		String cs = (count==1)?"":", count=" + count;
+		dInfo += "    " + tag + ", \"" + name + "\", value=" + lvalue + cs + "\n";
+		//ij.IJ.log(tag + ", \"" + name + "\", value=" + value + cs + "\n");
+	}
+
+	String getName(int tag) {
 		String name;
 		switch (tag) {
 			case NEW_SUBFILE_TYPE: name="NewSubfileType"; break;
@@ -293,6 +321,8 @@ public class TiffDecoder {
 			case RESOLUTION_UNIT: name="ResolutionUnit"; break;
 			case SOFTWARE: name="Software"; break;
 			case DATE_TIME: name="DateTime"; break;
+			case ARTEST: name="Artest"; break;
+			case HOST_COMPUTER: name="HostComputer"; break;
 			case PLANAR_CONFIGURATION: name="PlanarConfiguration"; break;
 			case COMPRESSION: name="Compression"; break; 
 			case PREDICTOR: name="Predictor"; break; 
@@ -304,12 +334,10 @@ public class TiffDecoder {
 			case META_DATA: name="MetaData"; break; 
 			default: name="???"; break;
 		}
-		String cs = (count==1)?"":", count=" + count;
-		dInfo += "    " + tag + ", \"" + name + "\", value=" + value + cs + "\n";
-		//ij.IJ.log(tag + ", \"" + name + "\", value=" + value + cs + "\n");
+		return name;
 	}
 
-	double getRational(int loc) throws IOException {
+	double getRational(long loc) throws IOException {
 		long saveLoc = in.getLongFilePointer();
 		in.seek(loc);
 		int numerator = getInt();
@@ -330,6 +358,8 @@ public class TiffDecoder {
 		if (nEntries<1 || nEntries>1000)
 			return null;
 		ifdCount++;
+		if ((ifdCount%100)==0 && ifdCount>0)
+			ij.IJ.showStatus(""+ifdCount);
 		FileInfo fi = new FileInfo();
 		for (int i=0; i<nEntries; i++) {
 			tag = getShort();
@@ -355,14 +385,13 @@ public class TiffDecoder {
 						long saveLoc = in.getLongFilePointer();
 						in.seek(lvalue);
 						fi.stripOffsets = new int[count];
-						for (int c=0; c<count; c++) {
+						for (int c=0; c<count; c++)
 							fi.stripOffsets[c] = getInt();
-							if (c > 0 && fi.stripOffsets[c]<fi.stripOffsets[c-1]&&fi.stripOffsets[c]>0)
-								error("Strip offsets are not in order");
-						}
 						in.seek(saveLoc);
 					}
 					fi.offset = count>0?fi.stripOffsets[0]:value;
+					if (count>1 && fi.stripOffsets[count-1]<fi.stripOffsets[0])
+						fi.offset = fi.stripOffsets[count-1];
 					break;
 				case STRIP_BYTE_COUNT:
 					if (count==1)
@@ -371,8 +400,12 @@ public class TiffDecoder {
 						long saveLoc = in.getLongFilePointer();
 						in.seek(lvalue);
 						fi.stripLengths = new int[count];
-						for (int c=0; c<count; c++)
-							fi.stripLengths[c] = getInt();
+						for (int c=0; c<count; c++) {
+							if (fieldType==SHORT)
+								fi.stripLengths[c] = getShort();
+							else
+								fi.stripLengths[c] = getInt();
+						}
 						in.seek(saveLoc);
 					}
 					break;
@@ -408,16 +441,18 @@ public class TiffDecoder {
 					fi.samplesPerPixel = value;
 					if (value==3 && fi.fileType!=FileInfo.RGB48)
 						fi.fileType = fi.fileType==FileInfo.GRAY16_UNSIGNED?FileInfo.RGB48:FileInfo.RGB;
+					else if (value==4 && fi.fileType==FileInfo.GRAY8)
+						fi.fileType = FileInfo.ARGB;
 					break;
 				case ROWS_PER_STRIP:
 					fi.rowsPerStrip = value;
 					break;
 				case X_RESOLUTION:
-					double xScale = getRational(value); 
+					double xScale = getRational(lvalue); 
 					if (xScale!=0.0) fi.pixelWidth = 1.0/xScale; 
 					break;
 				case Y_RESOLUTION:
-					double yScale = getRational(value); 
+					double yScale = getRational(lvalue); 
 					if (yScale!=0.0) fi.pixelHeight = 1.0/yScale; 
 					break;
 				case RESOLUTION_UNIT:
@@ -440,19 +475,18 @@ public class TiffDecoder {
 					else if (value==1 && fi.samplesPerPixel==4)
 						fi.fileType = FileInfo.ARGB;
 					else if (value!=2 && !((fi.samplesPerPixel==1)||(fi.samplesPerPixel==3))) {
-						String msg = "Unsupported interleaved SamplesPerPixel: " + fi.samplesPerPixel;
+						String msg = "Unsupported SamplesPerPixel: " + fi.samplesPerPixel;
 						error(msg);
 					}
 					break;
 				case COMPRESSION:
-					if (value==5) { // LZW compression
-						int bpp = fi.getBytesPerPixel();
-						if (bpp==6)
-							error("ImageJ cannot open 48-bit LZW compressed TIFFs");
+					if (value==5)  // LZW compression
 						fi.compression = FileInfo.LZW;
-					} else if (value==32773) { // PackBits compression
+					else if (value==32773)  // PackBits compression
 						fi.compression = FileInfo.PACK_BITS;
-					} else if (value!=1 && !(value==7&&fi.width<500)) {
+					else if (value==32946 || value==8)
+						fi.compression = FileInfo.ZIP;
+					else if (value!=1 && value!=0 && !(value==7&&fi.width<500)) {
 						// don't abort with Spot camera compressed (7) thumbnails
 						// otherwise, this is an unknown compression type
 						fi.compression = FileInfo.COMPRESSION_UNKNOWN;
@@ -460,6 +494,13 @@ public class TiffDecoder {
 							"compressed in this fashion ("+value+")");
 					}
 					break;
+				case SOFTWARE: case DATE_TIME: case HOST_COMPUTER: case ARTEST:
+					if (ifdCount==1) {
+						byte[] bytes = getString(count, lvalue);
+						String s = bytes!=null?new String(bytes):null;
+						saveMetadata(getName(tag), s);
+					}
+					break;
 				case PREDICTOR:
 					if (value==2 && fi.compression==FileInfo.LZW)
 						fi.compression = FileInfo.LZW_WITH_DIFFERENCING;
@@ -468,6 +509,9 @@ public class TiffDecoder {
 					if (count==768 && fi.fileType==fi.GRAY8)
 						getColorMap(lvalue, fi);
 					break;
+				case TILE_WIDTH:
+					error("ImageJ cannot open tiled TIFFs");
+					break;
 				case SAMPLE_FORMAT:
 					if (fi.fileType==FileInfo.GRAY32_INT && value==FLOATING_POINT)
 						fi.fileType = FileInfo.GRAY32_FLOAT;
@@ -475,7 +519,7 @@ public class TiffDecoder {
 						if (value==SIGNED)
 							fi.fileType = FileInfo.GRAY16_SIGNED;
 						if (value==FLOATING_POINT)
-							error("ImageJ cannot open16-bit float TIFFs");
+							error("ImageJ cannot open 16-bit float TIFFs");
 					}
 					break;
 				case JPEG_TABLES:
@@ -560,6 +604,8 @@ public class TiffDecoder {
 				if (types[i]==LABELS) id = " (slice labels)";
 				if (types[i]==RANGES) id = " (display ranges)";
 				if (types[i]==LUTS) id = " (luts)";
+				if (types[i]==ROI) id = " (roi)";
+				if (types[i]==OVERLAY) id = " (overlay)";
 				dInfo += "   "+i+" "+Integer.toHexString(types[i])+" "+counts[i]+id+"\n";
 			}
 		}
@@ -576,6 +622,10 @@ public class TiffDecoder {
 				getDisplayRanges(start, fi);
 			else if (types[i]==LUTS)
 				getLuts(start, start+counts[i]-1, fi);
+			else if (types[i]==ROI)
+				getRoi(start, fi);
+			else if (types[i]==OVERLAY)
+				getOverlay(start, start+counts[i]-1, fi);
 			else if (types[i]<0xffffff) {
 				for (int j=start; j<start+counts[i]; j++) { 
 					int len = metaDataCounts[j]; 
@@ -651,6 +701,23 @@ public class TiffDecoder {
 		}
 	}
 
+	void getRoi(int first, FileInfo fi) throws IOException {
+		int len = metaDataCounts[first];
+		fi.roi = new byte[len]; 
+		in.readFully(fi.roi, len); 
+	}
+
+	void getOverlay(int first, int last, FileInfo fi) throws IOException {
+		fi.overlay = new byte[last-first+1][];
+	    int index = 0;
+		for (int i=first; i<=last; i++) {
+			int len = metaDataCounts[i];
+			fi.overlay[index] = new byte[len];
+            in.readFully(fi.overlay[index], len);
+            index++;
+		}
+	}
+
 	void error(String message) throws IOException {
 		if (in!=null) in.close();
 		throw new IOException(message);
@@ -669,14 +736,13 @@ public class TiffDecoder {
 	public void enableDebugging() {
 		debugMode = true;
 	}
-	
-	
+		
 	public FileInfo[] getTiffInfo() throws IOException {
 		long ifdOffset;
 		Vector info;
 				
 		if (in==null)
-			in = new RandomAccessStream(new RandomAccessFile(directory + name, "r"));
+			in = new RandomAccessStream(new RandomAccessFile(new File(directory, name), "r"));
 		info = new Vector();
 		ifdOffset = OpenImageFileHeader();
 		if (ifdOffset<0L) {
@@ -710,8 +776,35 @@ public class TiffDecoder {
 				fi[0].inputStream = in;
 			} else
 				in.close();
+			if (fi[0].info==null)
+				fi[0].info = tiffMetadata;
+			if (debugMode) {
+				int n = fi.length;
+				fi[0].debugInfo += "number of images: "+ n + "\n";
+				fi[0].debugInfo += "offset to first image: "+fi[0].getOffset()+ "\n";
+				fi[0].debugInfo += "gap between images: "+getGapInfo(fi) + "\n";
+				fi[0].debugInfo += "little-endian byte order: "+fi[0].intelByteOrder + "\n";
+			}
 			return fi;
 		}
 	}
+	
+	String getGapInfo(FileInfo[] fi) {
+		if (fi.length<2) return "0";
+		long minGap = Long.MAX_VALUE;
+		long maxGap = -Long.MAX_VALUE;
+		for (int i=1; i<fi.length; i++) {
+			long gap = fi[i].getOffset()-fi[i-1].getOffset();
+			if (gap<minGap) minGap = gap;
+			if (gap>maxGap) maxGap = gap;
+		}
+		long imageSize = fi[0].width*fi[0].height*fi[0].getBytesPerPixel();
+		minGap -= imageSize;
+		maxGap -= imageSize;
+		if (minGap==maxGap)
+			return ""+minGap;
+		else 
+			return "varies ("+minGap+" to "+maxGap+")";
+	}
 
 }
