@@ -6,6 +6,7 @@ import ij.plugin.Macro_Runner;
 import ij.plugin.frame.*;
 import ij.util.Tools;
 import ij.text.*;
+import ij.measure.ResultsTable;
 import java.awt.*;
 import java.util.*;
 import java.awt.event.KeyEvent;
@@ -78,7 +79,9 @@ public class Interpreter implements MacroConstants {
 		calledMacro = true;
 		if (IJ.getInstance()==null)
 			setBatchMode(true);
+		Interpreter saveInstance = instance;
 		run(macro);
+		instance = saveInstance;
 		return returnValue;
 	}
 	
@@ -244,7 +247,8 @@ public class Interpreter implements MacroConstants {
 				inPrint = true;
 				String s = getString();
 				inPrint = false;
-				if (s!=null && !s.equals("NaN")) IJ.log(s);
+				if (s!=null && s.length()>0 && !s.equals("NaN") && !s.equals("[aborted]"))
+					IJ.log(s);
 				return;
 			case ARRAY_FUNCTION: func.getArrayFunction(pgm.table[tokenAddress].type); break;
 			case EOF: break;
@@ -274,6 +278,8 @@ public class Interpreter implements MacroConstants {
 			doBlock();
 		} catch (ReturnException e) {
 			value = new Variable(0, e.value, e.str, e.array);
+			if (value.getArray()!=null && e.arraySize!=0)
+				value.setArraySize(e.arraySize);
 		}
 		inFunction = saveInFunction;
 		pc = savePC;
@@ -311,6 +317,7 @@ public class Interpreter implements MacroConstants {
 				} else if (next==WORD && (nextPlus==',' || nextPlus==')')) {
 					value = 0.0;
 					Variable[] array = null;
+					int arraySize = 0;
 					String str = null;
 					getToken();
 					Variable v = lookupVariable();
@@ -318,20 +325,25 @@ public class Interpreter implements MacroConstants {
 						int type = v.getType();
 						if (type==Variable.VALUE)
 							value = v.getValue();
-						else if (type==Variable.ARRAY)
+						else if (type==Variable.ARRAY) {
 							array = v.getArray();
-						else
+							arraySize = v.getArraySize();
+						} else
 							str = v.getString();
 					}
 					args[count] = new Variable(0, value, str, array);
+					if (array!=null) args[count].setArraySize(arraySize);
 				} else if (next==WORD && nextPlus=='[' ) {
+					int savePC = pc;
 					getToken();
 					Variable v = lookupVariable();
 					v = getArrayElement(v);
 					if (v.getString()!=null)
 						args[count] = new Variable(0, 0.0, v.getString(), null);
-					else
-						args[count] = new Variable(0, v.getValue(), null);
+					else {
+						pc = savePC;
+						args[count] = new Variable(0, getExpression(), null);
+					}
 				} else
 					args[count] = new Variable(0, getExpression(), null);
 				count++;
@@ -374,6 +386,7 @@ public class Interpreter implements MacroConstants {
 		double value = 0.0;
 		String str = null;
 		Variable[] array = null;
+		int arraySize = 0;
 		getToken();
 		if (token!=';') {
 			boolean isString = token==STRING_CONSTANT || token==STRING_FUNCTION;
@@ -382,6 +395,7 @@ public class Interpreter implements MacroConstants {
 				Variable v = lookupLocalVariable(tokenAddress);
 				if (v!=null && nextToken()==';') {
 					array = v.getArray();
+					if (array!=null) arraySize=v.getArraySize();
 					isString = v.getString()!=null;
 					//IJ.log("token==WORD: "+isString+" "+pgm.decodeToken(token, tokenAddress));
 				}
@@ -401,6 +415,7 @@ public class Interpreter implements MacroConstants {
 			returnException.value = value;
 			returnException.str = str;
 			returnException.array = array;
+			returnException.arraySize = arraySize;
 			//throw new ReturnException(value, str, array);
 			throw returnException;
 		} else {
@@ -516,7 +531,6 @@ public class Interpreter implements MacroConstants {
 
 	final void skipStatement() {
 		getToken();
-		//IJ.write("skipStatement: " +pgm.decodeToken(token, tokenAddress));
 		switch (token) {
 			case PREDEFINED_FUNCTION: case USER_FUNCTION: case VAR:
 			case WORD: case '(': case PLUS_PLUS: case RETURN:
@@ -592,7 +606,7 @@ public class Interpreter implements MacroConstants {
 		while (!finished && !done) {
 			if (token==';')
 				finished = true;
-			else if (token==ELSE||token==PREDEFINED_FUNCTION)
+			else if (token==ELSE||(token==PREDEFINED_FUNCTION&&pgm.code[pc-1]!='.'))
 				error("';' expected");
 			else
 				getToken();
@@ -722,8 +736,30 @@ public class Interpreter implements MacroConstants {
 		Variable[] array = v.getArray();
 		if (array==null)
 			error("Array expected");
-		if (index<0 || index>=array.length)
-			error("Index ("+index+") out of 0-"+(array.length-1)+" range");
+		if (index<0)
+			error("Negative index");
+		if (index>=array.length) {  // expand array
+			if (!func.expandableArrays)
+				error("Index ("+index+") out of range");
+			Variable[] array2 = new Variable[index+array.length/2+1];
+			//IJ.log(array.length+" "+array2.length);
+			boolean strings = array.length>0 && array[0].getString()!=null;
+			for (int i=0; i<array2.length; i++) {
+				if (i<array.length)
+					array2[i] = array[i];
+				else {
+					array2[i] = new Variable(Double.NaN);
+					if (strings)
+						array2[i].setString("undefined");
+				}
+			}
+			v.setArray(array2);
+			v.setArraySize(index+1);
+			array = v.getArray();
+		}
+		int size = v.getArraySize();
+		if (index+1>size)
+			v.setArraySize(index+1);
 		int next = nextToken();
 		switch (expressionType) {
 			case Variable.STRING:
@@ -773,9 +809,10 @@ public class Interpreter implements MacroConstants {
 			int type = v2.getType();
 			if (type==Variable.VALUE)
 				v1.setValue(v2.getValue());
-			else if (type==Variable.ARRAY)
+			else if (type==Variable.ARRAY) {
 				v1.setArray(v2.getArray());
-			else
+				v1.setArraySize(v2.getArraySize());
+			} else
 				v1.setString(v2.getString());
 		}	
 	}
@@ -808,11 +845,12 @@ public class Interpreter implements MacroConstants {
 				error("Undefined identifier");
 		}
 		getToken();
-		if (token!='=') {
-			error("'=' expected");
-			return;
-		}
-		v.setString(getString());
+		if (token=='=')
+			v.setString(getString());
+		else if (token==PLUS_EQUAL)
+			v.setString(v.getString()+getString());
+		else
+			error("'=' or '+=' expected");
 	}
 
 	final void doArrayAssignment() {
@@ -1056,14 +1094,16 @@ public class Interpreter implements MacroConstants {
 		batchMode = false;
 		imageTable = null;
 		WindowManager.setTempCurrentImage(null);
+		wasError = true;
+		instance = null;
 		if (showMessage) {
 			String line = getErrorLine();
+			done = true;
 			if (line.length()>120)
 				line = line.substring(0,119)+"...";
 			showError("Macro Error", message+" in line "+lineNumber+".\n \n"+line, variables);
 			throw new RuntimeException(Macro.MACRO_CANCELED);
 		}
-		wasError = true;
 		done = true;
 	}
 	
@@ -1242,7 +1282,9 @@ public class Interpreter implements MacroConstants {
 			case STRING_FUNCTION:
 				String str = func.getStringFunction(pgm.table[tokenAddress].type);
 				value = Tools.parseDouble(str);
-				if (Double.isNaN(value))
+				if ("NaN".equals(str))
+					value = Double.NaN;
+				else if (Double.isNaN(value))
 					error("Numeric value expected");
 				break;
 			case USER_FUNCTION:
@@ -1336,10 +1378,9 @@ public class Interpreter implements MacroConstants {
 		getToken();
 		if (!(token==WORD && tokenString.equals("length")))
 			error("'length' expected");
-		Variable[] array = v.getArray();
-		if (array==null)
+		if (v.getArray()==null)
 			error("Array expected");
-		return array.length;
+		return v.getArraySize();
 	}
 	
 	final double getStringExpression() {
@@ -1550,7 +1591,6 @@ public class Interpreter implements MacroConstants {
 	
 	void finishUp() {
 		func.updateDisplay();
-		if (func.plot!=null) func.plot.show();
 		instance = null;
 		if (!calledMacro) {
 			if (batchMode) showingProgress = true;
@@ -1558,6 +1598,10 @@ public class Interpreter implements MacroConstants {
 			imageTable = null;
 			WindowManager.setTempCurrentImage(null);
 		}
+		if (func.plot!=null) {
+			func.plot.show();
+			func.plot = null;
+		}
 		if (showingProgress)
 			IJ.showProgress(0, 0);
 		if (keysSet) {
@@ -1568,6 +1612,11 @@ public class Interpreter implements MacroConstants {
 		if (rgbWeights!=null)
 			ColorProcessor.setWeightingFactors(rgbWeights[0], rgbWeights[1], rgbWeights[2]);
 		if (func.writer!=null) func.writer.close();
+		func.roiManager = null;
+		if (func.resultsPending) {
+			ResultsTable rt = ResultsTable.getResultsTable();
+			if (rt!=null && rt.getCounter()>0) rt.show("Results");
+		}
 	}
 	
 	/** Aborts currently running macro. */
@@ -1598,6 +1647,8 @@ public class Interpreter implements MacroConstants {
 			imageTable = null;
 		}
 		done = true;
+		if (func!=null && !(macroName!=null&&macroName.indexOf(" Tool")!=-1))
+			func.abortDialog();
 		IJ.showStatus("Macro aborted");
 	}
 
@@ -1605,6 +1656,10 @@ public class Interpreter implements MacroConstants {
 		return instance;
 	}
 	
+	//public boolean  inLoop() {
+	//	return !looseSyntax;
+	//}
+
 	static void setBatchMode(boolean b) {
 		batchMode = b;
 		if (b==false) imageTable = null;
@@ -1625,7 +1680,6 @@ public class Interpreter implements MacroConstants {
 	public static void removeBatchModeImage(ImagePlus imp) {
 		if (imageTable!=null && imp!=null) {
 			int index = imageTable.indexOf(imp);
-			//IJ.log("remove: "+imp+"  "+imageTable.size());
 			if (index!=-1)
 				imageTable.removeElementAt(index);
 		}
@@ -1695,6 +1749,8 @@ public class Interpreter implements MacroConstants {
 	}
 	
 	public void setEditor(Editor ed) {
+		if (ed!=null&&editor==null)
+			ed.fixLineEndings();
 		editor = ed;
 		if (ed!=null)
 			debugMode = STEP;
@@ -1773,6 +1829,36 @@ public class Interpreter implements MacroConstants {
 		return Double.NaN;
 	}
 
+	public double getVariable2(String name) {
+		int index;
+		for (int i=topOfStack; i>=0; i--) {
+			index = stack[i].symTabIndex;
+			if (pgm.table[index].str.equals(name))
+				return stack[i].getValue();
+		}
+		return Double.NaN;
+	}
+
+	public String getStringVariable(String name) {
+		int index;
+		for (int i=topOfStack; i>=0; i--) {
+			index = stack[i].symTabIndex;
+			if (pgm.table[index].str.equals(name))
+				return stack[i].getString();
+		}
+		return null;
+	}
+	
+	public String getVariableAsString(String name) {
+		String s = getStringVariable(name);
+		if (s==null) {
+			double value = getVariable2(name);
+			if (!Double.isNaN(value)) s=""+value;
+		}
+		return s;
+	}
+
+
 } // class Interpreter
 
 
