@@ -4,7 +4,13 @@ import ij.plugin.filter.Analyzer;
 import ij.text.*;
 import ij.process.*;
 import ij.gui.Roi;
+import ij.util.Tools;
+import ij.io.SaveDialog;
 import java.awt.*;
+import java.text.*;
+import java.util.Locale;
+import java.io.*;
+
 
 /** This is a table for storing measurement results as columns of numeric values. 
 	Call the static ResultsTable.getResultsTable() method to get a reference to the 
@@ -25,12 +31,12 @@ public class ResultsTable implements Cloneable {
 		PERIMETER=10, ROI_X=11, ROI_Y=12, ROI_WIDTH=13, ROI_HEIGHT=14,
 		MAJOR=15, MINOR=16, ANGLE=17, CIRCULARITY=18, FERET=19, 
 		INTEGRATED_DENSITY=20, MEDIAN=21, SKEWNESS=22, KURTOSIS=23, 
-		AREA_FRACTION=24, CHANNEL=25, SLICE=26, FRAME=27, FERET_X=28,
-		FERET_Y=29, FERET_ANGLE=30, MIN_FERET=31, ASPECT_RATIO=32,
-		ROUNDNESS=33, SOLIDITY=34, LAST_HEADING=34;
+		AREA_FRACTION=24, RAW_INTEGRATED_DENSITY=25, CHANNEL=26, SLICE=27, FRAME=28, 
+		FERET_X=29, FERET_Y=30, FERET_ANGLE=31, MIN_FERET=32, ASPECT_RATIO=33,
+		ROUNDNESS=34, SOLIDITY=35, LAST_HEADING=35;
 	private static final String[] defaultHeadings = {"Area","Mean","StdDev","Mode","Min","Max",
 		"X","Y","XM","YM","Perim.","BX","BY","Width","Height","Major","Minor","Angle",
-		"Circ.", "Feret", "IntDen", "Median","Skew","Kurt", "%Area", "Ch", "Slice", "Frame", 
+		"Circ.", "Feret", "IntDen", "Median","Skew","Kurt", "%Area", "RawIntDen", "Ch", "Slice", "Frame", 
 		 "FeretX", "FeretY", "FeretAngle", "MinFeret", "AR", "Round", "Solidity"};
 
 	private int maxRows = 100; // will be increased as needed
@@ -44,6 +50,10 @@ public class ResultsTable implements Cloneable {
 	private	StringBuffer sb;
 	private int precision = 3;
 	private String rowLabelHeading = "";
+	private char delimiter = '\t';
+	private boolean headingSet; 
+	private boolean showRowNumbers = true;
+	private boolean autoFormat = true;
 
 	/** Constructs an empty ResultsTable with the counter=0 and no columns. */
 	public ResultsTable() {
@@ -54,8 +64,16 @@ public class ResultsTable implements Cloneable {
 	public static ResultsTable getResultsTable() {
 		return Analyzer.getResultsTable();
 	}
-	
-	
+		
+	/** Returns the "Results" TextWindow. */
+	public static TextWindow getResultsWindow() {
+		Frame f = WindowManager.getFrame("Results");
+		if (f==null || !(f instanceof TextWindow))
+			return null;
+		else
+			return (TextWindow)f;
+	}
+
 	/** Increments the measurement counter by one. */
 	public synchronized void incrementCounter() {
 		counter++;
@@ -76,18 +94,21 @@ public class ResultsTable implements Cloneable {
 		}
 	}
 	
+	/** Obsolete; the addValue() method automatically adds columns as needed.
+	* @see #addValue(String, double)
+	*/
 	public synchronized void addColumns() {
-			String[] tmp1 = new String[maxColumns*2];
-			System.arraycopy(headings, 0, tmp1, 0, maxColumns);
-			headings = tmp1;
-			double[][] tmp2 = new double[maxColumns*2][];
-			for (int i=0; i<maxColumns; i++)
-				tmp2[i] = columns[i];
-			columns = tmp2;
-			boolean[] tmp3 = new boolean[maxColumns*2];
-			System.arraycopy(keep, 0, tmp3, 0, maxColumns);
-			keep = tmp3;
-			maxColumns *= 2;
+		String[] tmp1 = new String[maxColumns*2];
+		System.arraycopy(headings, 0, tmp1, 0, maxColumns);
+		headings = tmp1;
+		double[][] tmp2 = new double[maxColumns*2][];
+		for (int i=0; i<maxColumns; i++)
+			tmp2[i] = columns[i];
+		columns = tmp2;
+		boolean[] tmp3 = new boolean[maxColumns*2];
+		System.arraycopy(keep, 0, tmp3, 0, maxColumns);
+		keep = tmp3;
+		maxColumns *= 2;
 	}
 	
 	/** Returns the current value of the measurement counter. */
@@ -113,8 +134,13 @@ public class ResultsTable implements Cloneable {
 	}
 	
 	/** Adds a value to the end of the given column. If the column
-		does not exist, it is created.  Counter must be >0. */
+		does not exist, it is created.  Counter must be >0.
+		There is an example at:<br>
+		http://imagej.nih.gov/ij/plugins/sine-cosine.html
+		*/
 	public void addValue(String column, double value) {
+		if (column==null)
+			throw new IllegalArgumentException("Column is null");
 		int index = getColumnIndex(column);
 		if (index==COLUMN_NOT_FOUND)
 			index = getFreeColumn(column);
@@ -154,9 +180,10 @@ public class ResultsTable implements Cloneable {
 		rowLabels[row] = label;
 	}
 	
-	/** Set the row label column to null. */
+	/** Set the row label column to null if the column label is "Label". */
 	public void disableRowLabels() {
-		rowLabels = null;
+		if (rowLabelHeading.equals("Label"))
+			rowLabels = null;
 	}
 	
 	/** Returns a copy of the given column as a float array,
@@ -242,7 +269,10 @@ public class ResultsTable implements Cloneable {
 		return columns[column][row];
 	}
 	
-	/**	Obsolete, replaced by getValueAsDouble. */
+	/**
+	* @deprecated
+	* replaced by getValueAsDouble
+	*/
 	public float getValue(int column, int row) {
 		return (float)getValueAsDouble(column, row);
 	}
@@ -278,6 +308,8 @@ public class ResultsTable implements Cloneable {
 		<code>show()</code> must be called to update the 
 		window that displays the table.*/
 	public void setValue(String column, int row, double value) {
+		if (column==null)
+			throw new IllegalArgumentException("Column is null");
 		int col = getColumnIndex(column);
 		if (col==COLUMN_NOT_FOUND) {
 			col = getFreeColumn(column);
@@ -301,18 +333,27 @@ public class ResultsTable implements Cloneable {
 		columns[column][row] = value;
 	}
 
-	/** Returns a tab-delimited string containing the column headings. */
+	/** Returns a tab or comma delimited string containing the column headings. */
 	public String getColumnHeadings() {
+		if (headingSet && !rowLabelHeading.equals("")) { // workaround setHeading() bug
+			for (int i=0; i<=lastColumn; i++) {
+				if (columns[i]!=null && rowLabelHeading.equals(headings[i]))
+					{headings[i]=null; columns[i]=null;}
+			}
+			headingSet = false;
+		}
 		StringBuffer sb = new StringBuffer(200);
-		sb.append(" \t");
+		if (showRowNumbers)
+			sb.append(" "+delimiter);
 		if (rowLabels!=null)
-			sb.append(rowLabelHeading + "\t");
+			sb.append(rowLabelHeading + delimiter);
 		String heading;
 		for (int i=0; i<=lastColumn; i++) {
 			if (columns[i]!=null) {
 				heading = headings[i];
 				if (heading==null) heading ="---"; 
-				sb.append(heading + "\t");
+				sb.append(heading);
+				if (i!=lastColumn) sb.append(delimiter);
 			}
 		}
 		return new String(sb);
@@ -325,7 +366,7 @@ public class ResultsTable implements Cloneable {
 		return headings[column];
 	}
 
-	/** Returns a tab-delimited string representing the
+	/** Returns a tab or comma delimited string representing the
 		given row, where 0<=row<=counter-1. */
 	public String getRowAsString(int row) {
 		if ((row<0) || (row>=counter))
@@ -334,25 +375,39 @@ public class ResultsTable implements Cloneable {
 			sb = new StringBuffer(200);
 		else
 			sb.setLength(0);
-		sb.append(Integer.toString(row+1));
-		sb.append("\t");
+		if (showRowNumbers) {
+			sb.append(Integer.toString(row+1));
+			sb.append(delimiter);
+		}
 		if (rowLabels!=null) {
-			if (rowLabels[row]!=null)
-				sb.append(rowLabels[row]);
-			sb.append("\t");
+			if (rowLabels[row]!=null) {
+				String label = rowLabels[row];
+				if (delimiter==',') label = label.replaceAll(",", ";");
+				sb.append(label);
+			}
+			sb.append(delimiter);
 		}
 		for (int i=0; i<=lastColumn; i++) {
-			if (columns[i]!=null)
+			if (columns[i]!=null) {
 				sb.append(n(columns[i][row]));
+				if (i!=lastColumn) sb.append(delimiter);
+			}
 		}
 		return new String(sb);
 	}
 	
-	/** Changes the heading of the given column. */
+	/**
+	* @deprecated
+	* replaced by addValue(String,double) and setValue(String,int,double)
+	*/
 	public void setHeading(int column, String heading) {
 		if ((column<0) || (column>=headings.length))
 			throw new IllegalArgumentException("Column out of range: "+column);
 		headings[column] = heading;
+		if (columns[column]==null)
+			columns[column] = new double[maxRows];
+		if (column>lastColumn) lastColumn = column;
+		headingSet = true;
 	}
 	
 	/** Sets the headings used by the Measure command ("Area", "Mean", etc.). */
@@ -366,20 +421,74 @@ public class ResultsTable implements Cloneable {
 		this.precision = precision;
 	}
 	
+	public void showRowNumbers(boolean showNumbers) {
+		showRowNumbers = showNumbers;
+	}
+
 	String n(double n) {
 		String s;
-		if (Math.round(n)==n && precision>=0)
-			s = IJ.d2s(n,0);
+		if (autoFormat && Math.round(n)==n && precision>=0)
+			s = d2s(n, 0);
 		else
-			s = IJ.d2s(n,precision);
-		return s+"\t";
+			s = d2s(n, precision);
+		return s;
 	}
 		
+	private static DecimalFormat[] df;
+	private static DecimalFormat[] sf;
+	private static DecimalFormatSymbols dfs;
+
+	/** This is a version of IJ.d2s() that uses scientific notation for
+		small numbes that would otherwise display as zero. */
+	public static String d2s(double n, int decimalPlaces) {
+		if (Double.isNaN(n)||Double.isInfinite(n))
+			return ""+n;
+		if (n==Float.MAX_VALUE) // divide by 0 in FloatProcessor
+			return "3.4e38";
+		double np = n;
+		if (n<0.0) np = -n;
+		if (df==null) {
+			dfs = new DecimalFormatSymbols(Locale.US);
+			df = new DecimalFormat[10];
+			df[0] = new DecimalFormat("0", dfs);
+			df[1] = new DecimalFormat("0.0", dfs);
+			df[2] = new DecimalFormat("0.00", dfs);
+			df[3] = new DecimalFormat("0.000", dfs);
+			df[4] = new DecimalFormat("0.0000", dfs);
+			df[5] = new DecimalFormat("0.00000", dfs);
+			df[6] = new DecimalFormat("0.000000", dfs);
+			df[7] = new DecimalFormat("0.0000000", dfs);
+			df[8] = new DecimalFormat("0.00000000", dfs);
+			df[9] = new DecimalFormat("0.000000000", dfs);
+		}
+		if ((np<0.001 && np!=0.0 && np<1.0/Math.pow(10,decimalPlaces)) || np>999999999999d || decimalPlaces<0) {
+			if (decimalPlaces<0) {
+				decimalPlaces = -decimalPlaces;
+				if (decimalPlaces>9) decimalPlaces=9;
+			} else
+				decimalPlaces = 3;
+			if (sf==null) {
+				sf = new DecimalFormat[10];
+				sf[1] = new DecimalFormat("0.0E0",dfs);
+				sf[2] = new DecimalFormat("0.00E0",dfs);
+				sf[3] = new DecimalFormat("0.000E0",dfs);
+				sf[4] = new DecimalFormat("0.0000E0",dfs);
+				sf[5] = new DecimalFormat("0.00000E0",dfs);
+				sf[6] = new DecimalFormat("0.000000E0",dfs);
+				sf[7] = new DecimalFormat("0.0000000E0",dfs);
+				sf[8] = new DecimalFormat("0.00000000E0",dfs);
+				sf[9] = new DecimalFormat("0.000000000E0",dfs);
+			}
+			return sf[decimalPlaces].format(n); // use scientific notation
+		}
+		if (decimalPlaces<0) decimalPlaces = 0;
+		if (decimalPlaces>9) decimalPlaces = 9;
+		return df[decimalPlaces].format(n);
+	}
+
 	/** Deletes the specified row. */
 	public synchronized void deleteRow(int row) {
-		if (counter==0 || row>counter-1) return;
-		//if (counter==1)
-		//	{reset(); return;}
+		if (counter==0 || row<0 || row>counter-1) return;
 		if (rowLabels!=null) {
 			for (int i=row; i<counter-1; i++)
 				rowLabels[i] = rowLabels[i+1];
@@ -441,9 +550,11 @@ public class ResultsTable implements Cloneable {
 			IJ.log("ResultsTable.show(): the system ResultTable should only be displayed in the \"Results\" window.");
 		String tableHeadings = getColumnHeadings();		
 		TextPanel tp;
+		boolean newWindow = false;
 		if (windowTitle.equals("Results")) {
 			tp = IJ.getTextPanel();
 			if (tp==null) return;
+			newWindow = tp.getLineCount()==0;
 			IJ.setColumnHeadings(tableHeadings);
 			if (this!=Analyzer.getResultsTable())
 				Analyzer.setResultsTable(this);
@@ -458,6 +569,8 @@ public class ResultsTable implements Cloneable {
 				win = new TextWindow(windowTitle, "", 400, 300);
 			tp = win.getTextPanel();
 			tp.setColumnHeadings(tableHeadings);
+			newWindow = tp.getLineCount()==0;
+			autoFormat = false;
 		}
 		tp.setResultsTable(this);
 		int n = getCounter();
@@ -468,6 +581,7 @@ public class ResultsTable implements Cloneable {
 				sb.append(getRowAsString(i)+"\n");
 			tp.append(new String(sb));
 		}
+		if (newWindow) tp.scrollToTop();
 	}
 	
 	public void update(int measurements, ImagePlus imp, Roi roi) {
@@ -515,6 +629,128 @@ public class ResultsTable implements Cloneable {
 		return rowLabels;
 	}
 	
+	/** Opens a tab or comma delimited text file as a ResultsTable.
+	     Displays a file open dialog if 'path' is empty or null.
+	     Displays non-numeric tables in a TextWindow and returns null. */
+	public static ResultsTable open(String path) throws IOException {
+		final String lineSeparator =  "\n";
+		final String cellSeparator =  ",\t";
+		String text =IJ.openAsString(path);
+		if (text.startsWith("Error:"))
+			throw new IOException("text.substring(7)");
+		String[] lines = Tools.split(text, lineSeparator);
+		if (lines.length==0)
+			throw new IOException("Table is empty or invalid");
+		String[] headings = Tools.split(lines[0], cellSeparator);
+		if (headings.length==1)
+			throw new IOException("This is not a tab or comma delimited text file.");
+		int numbersInHeadings = 0;
+		for (int i=0; i<headings.length; i++) {
+			if (headings[i].equals("NaN") || !Double.isNaN(Tools.parseDouble(headings[i])))
+				numbersInHeadings++;
+		}
+		boolean allNumericHeadings = numbersInHeadings==headings.length;
+		if (allNumericHeadings) {
+			for (int i=0; i<headings.length; i++)
+				headings[i] = "C"+(i+1);
+		}
+		int firstColumn = headings[0].equals(" ")?1:0;
+		for (int i=0; i<headings.length; i++)
+			headings[i] = headings[i].trim();
+		int firstRow = allNumericHeadings?0:1;
+		boolean labels = firstColumn==1 && headings[1].equals("Label");
+		int rtn = 0;
+		if (!labels && (rtn=openNonNumericTable(path, lines, firstRow, cellSeparator))==2)
+			return null;
+		if (!labels && rtn==1) labels = true;
+		if (lines[0].startsWith("\t")) {
+			String[] headings2 = new String[headings.length+1];
+			headings2[0] = " ";
+			for (int i=0; i<headings.length; i++)
+				headings2[i+1] = headings[i];
+			headings = headings2;
+			firstColumn = 1;
+		}
+		ResultsTable rt = new ResultsTable();
+		for (int i=firstRow; i<lines.length; i++) {
+			rt.incrementCounter();
+			String[] items=Tools.split(lines[i], cellSeparator);
+			for (int j=firstColumn; j<items.length; j++) {
+				if (j==1&&labels)
+					rt.addLabel(headings[1], items[1]);
+				else if (j<headings.length)
+					rt.addValue(headings[j], Tools.parseDouble(items[j]));
+			}
+		}
+		return rt;
+	}
+	
+	static int openNonNumericTable(String path, String[] lines, int firstRow, String cellSeparator) {
+		if (lines.length<2) return 0;
+		String[] items=Tools.split(lines[1], cellSeparator);
+		int nonNumericCount = 0;
+		int nonNumericIndex = 0;
+		for (int i=0; i<items.length; i++) {
+			if (!items[i].equals("NaN") && Double.isNaN(Tools.parseDouble(items[i]))) {
+				nonNumericCount++;
+				nonNumericIndex = i;
+			}
+		}
+		boolean csv = path.endsWith(".csv");
+		if (nonNumericCount==0)
+			return 0; // assume this is all-numeric table
+		if (nonNumericCount==1 && nonNumericIndex==1)
+			return 1; // assume this is ImageJ Results table with row labels
+		if (csv) lines[0] = lines[0].replaceAll(",", "\t");
+		StringBuffer sb = new StringBuffer();
+		for (int i=1; i<lines.length; i++) {
+			sb.append(lines[i]);
+			sb.append("\n");
+		}
+		String str = sb.toString();
+		if (csv) str = str.replaceAll(",", "\t");
+		new TextWindow(new File(path).getName(), lines[0], str, 500, 400);
+		return 2;
+	}
+	
+	/** Saves this ResultsTable as a tab or comma delimited text file. The table
+	     is saved as a CSV (comma-separated values) file if 'path' ends with ".csv".
+	     Displays a file save dialog if 'path' is empty or null. Does nothing if the
+	     table is empty. */
+	public void saveAs(String path) throws IOException {
+		if (getCounter()==0 && lastColumn<0) return;
+		if (path==null || path.equals("")) {
+			SaveDialog sd = new SaveDialog("Save Results", "Results", Prefs.get("options.ext", ".xls"));
+			String file = sd.getFileName();
+			if (file==null) return;
+			path = sd.getDirectory() + file;
+		}
+		delimiter = path.endsWith(".csv")?',':'\t';
+		PrintWriter pw = null;
+		FileOutputStream fos = new FileOutputStream(path);
+		BufferedOutputStream bos = new BufferedOutputStream(fos);
+		pw = new PrintWriter(bos);
+		boolean saveShowRowNumbers = showRowNumbers;
+		if (Prefs.dontSaveRowNumbers)	
+			showRowNumbers = false;
+		if (!Prefs.dontSaveHeaders) {
+			String headings = getColumnHeadings();
+			pw.println(headings);
+		}
+		for (int i=0; i<getCounter(); i++)
+			pw.println(getRowAsString(i));
+		showRowNumbers = saveShowRowNumbers;
+		pw.close();
+		delimiter = '\t';
+	}
+	
+	public static String getDefaultHeading(int index) {
+		if (index>=0 && index<defaultHeadings.length)
+			return defaultHeadings[index];
+		else
+			return "null";
+	}
+
 	/** Creates a copy of this ResultsTable. */
 	public synchronized Object clone() {
 		try { 
