@@ -53,7 +53,9 @@ import ij.measure.Calibration;
 
 public class DICOM extends ImagePlus implements PlugIn {
 	private boolean showErrors = true;
+	private boolean gettingInfo;
 	private BufferedInputStream inputStream;
+	private String info;
 	
 	/** Default constructor. */
 	public DICOM() {
@@ -86,8 +88,9 @@ public class DICOM extends ImagePlus implements PlugIn {
 		DicomDecoder dd = new DicomDecoder(directory, fileName);
 		dd.inputStream = inputStream;
 		FileInfo fi = null;
-		try {fi = dd.getFileInfo();}
-		catch (IOException e) {
+		try {
+			fi = dd.getFileInfo();
+		} catch (IOException e) {
 			String msg = e.getMessage();
 			IJ.showStatus("");
 			if (msg.indexOf("EOF")<0&&showErrors) {
@@ -101,32 +104,40 @@ public class DICOM extends ImagePlus implements PlugIn {
 				return;
 			}
 		}
+		if (gettingInfo) {
+			info = dd.getDicomInfo();
+			return;
+		}
 		if (fi!=null && fi.width>0 && fi.height>0 && fi.offset>0) {
 			FileOpener fo = new FileOpener(fi);
 			ImagePlus imp = fo.open(false);
 			ImageProcessor ip = imp.getProcessor();
-			if (fi.fileType==FileInfo.GRAY16_SIGNED) {
-				//if (dd.rescaleSlope!=0.0&&dd.rescaleSlope!=1.0) 
-				//	ip.multiply(dd.rescaleSlope);
+			if (Prefs.openDicomsAsFloat) {
+				ip = ip.convertToFloat();
+				if (dd.rescaleSlope!=1.0)
+					ip.multiply(dd.rescaleSlope);
+				if (dd.rescaleIntercept!=0.0)
+					ip.add(dd.rescaleIntercept);
+				imp.setProcessor(ip);
+			} else if (fi.fileType==FileInfo.GRAY16_SIGNED) {
 				if (dd.rescaleIntercept!=0.0 && dd.rescaleSlope==1.0)
 					ip.add(dd.rescaleIntercept);
-			} else {
-				if (dd.rescaleIntercept!=0.0 && dd.rescaleSlope==1.0) {
-					double[] coeff = new double[2];
-					coeff[0] = dd.rescaleIntercept;
-					coeff[1] = dd.rescaleSlope;
- 					imp.getCalibration().setFunction(Calibration.STRAIGHT_LINE, coeff, "gray value");
-  				}
+			} else if (dd.rescaleIntercept!=0.0 && (dd.rescaleSlope==1.0||fi.fileType==FileInfo.GRAY8)) {
+				double[] coeff = new double[2];
+				coeff[0] = dd.rescaleIntercept;
+				coeff[1] = dd.rescaleSlope;
+				imp.getCalibration().setFunction(Calibration.STRAIGHT_LINE, coeff, "Gray Value");
 			}
-			//if (fi.fileType==FileInfo.GRAY16_SIGNED && imp.getStackSize()==1)
-			//	convertToUnsigned(imp, fi);
 			if (dd.windowWidth>0.0) {
 				double min = dd.windowCenter-dd.windowWidth/2;
 				double max = dd.windowCenter+dd.windowWidth/2;
-				Calibration cal = imp.getCalibration();
-				if (cal.getCValue(0)!=0) {
-					min -= cal.getCValue(0);
-					max -= cal.getCValue(0);
+				if (Prefs.openDicomsAsFloat) {
+					min -= dd.rescaleIntercept;
+					max -= dd.rescaleIntercept;
+				} else {
+					Calibration cal = imp.getCalibration();
+					min = cal.getRawValue(min);
+					max = cal.getRawValue(max);
 				}
 				ip.setMinAndMax(min, max);
 				if (IJ.debugMode) IJ.log("window: "+min+"-"+max);
@@ -161,6 +172,14 @@ public class DICOM extends ImagePlus implements PlugIn {
 		run(path);
 	}
 	
+	/** Returns the DICOM tags of the specified file as a string. */ 
+	public String getInfo(String path) {
+		showErrors = false;
+		gettingInfo = true;
+		run(path);
+		return info;
+	}
+
 	/** Convert 16-bit signed to unsigned if all pixels>=0. */
 	void convertToUnsigned(ImagePlus imp, FileInfo fi) {
 		ImageProcessor ip = imp.getProcessor();
@@ -190,8 +209,10 @@ class DicomDecoder {
 
 	private static final int PIXEL_REPRESENTATION = 0x00280103;
 	private static final int TRANSFER_SYNTAX_UID = 0x00020010;
+	private static final int MODALITY = 0x00080060;
 	private static final int SLICE_THICKNESS = 0x00180050;
 	private static final int SLICE_SPACING = 0x00180088;
+	private static final int IMAGER_PIXEL_SPACING = 0x00181164;
 	private static final int SAMPLES_PER_PIXEL = 0x00280002;
 	private static final int PHOTOMETRIC_INTERPRETATION = 0x00280004;
 	private static final int PLANAR_CONFIGURATION = 0x00280006;
@@ -239,9 +260,10 @@ class DicomDecoder {
  	private boolean oddLocations;  // one or more tags at odd locations
  	private boolean bigEndianTransferSyntax = false;
 	double windowCenter, windowWidth;
-	double rescaleIntercept, rescaleSlope;
+	double rescaleIntercept, rescaleSlope=1.0;
 	boolean inSequence;
  	BufferedInputStream inputStream;
+ 	String modality;
 
 	public DicomDecoder(String directory, String fileName) {
 		this.directory = directory;
@@ -514,6 +536,10 @@ class DicomDecoder {
 					if (s.indexOf("1.2.840.10008.1.2.2")>=0)
 						bigEndianTransferSyntax = true;
 					break;
+				case MODALITY:
+					modality = getString(elementLength);
+					addInfo(tag, modality);
+					break;
 				case NUMBER_OF_FRAMES:
 					s = getString(elementLength);
 					addInfo(tag, s);
@@ -541,7 +567,7 @@ class DicomDecoder {
 					fi.width = getShort();
 					addInfo(tag, fi.width);
 					break;
-				case PIXEL_SPACING:
+				case IMAGER_PIXEL_SPACING: case PIXEL_SPACING:
 					String scale = getString(elementLength);
 					getSpatialScale(fi, scale);
 					addInfo(tag, scale);
@@ -803,6 +829,9 @@ class DicomDecoder {
 	}
 	
  	double s2d(String s) {
+ 		if (s==null) return 0.0;
+ 		if (s.startsWith("\\"))
+ 			s = s.substring(1);
 		Double d;
 		try {d = new Double(s);}
 		catch (NumberFormatException e) {d = null;}
@@ -871,6 +900,7 @@ class DicomDictionary {
 		"00080033=TMImage Time",
 		"00080034=TMOverlay Time",
 		"00080035=TMCurve Time",
+		"00080040=USData Set Type",
 		"00080041=LOData Set Subtype",
 		"00080042=CSNuclear Medicine Series Type",
 		"00080050=SHAccession Number",
@@ -930,7 +960,7 @@ class DicomDictionary {
 		"00082200=CSTransducer Position",
 		"00082204=CSTransducer Orientation",
 		"00082208=CSAnatomic Structure",
-
+		
 		"00100010=PNPatient's Name",
 		"00100020=LOPatient ID",
 		"00100021=LOIssuer of Patient ID",
@@ -950,7 +980,7 @@ class DicomDictionary {
 		"001021A0=CSSmoking Status",
 		"001021B0=LTAdditional Patient History",
 		"00104000=LTPatient Comments",
-
+		
 		"00180010=LOContrast/Bolus Agent",
 		"00180015=CSBody Part Examined",
 		"00180020=CSScanning Sequence",
@@ -1214,7 +1244,7 @@ class DicomDictionary {
 		"00187062=LTExposure Control Mode Description",
 		"00187064=CSExposure Status",
 		"00187065=DSPhototimer Setting",
-
+		
 		"0020000D=UIStudy Instance UID",
 		"0020000E=UISeries Instance UID",
 		"00200010=SHStudy ID",
@@ -1252,7 +1282,7 @@ class DicomDictionary {
 		"00201206=ISNumber of Study Related Series",
 		"00201208=ISNumber of Study Related Images",
 		"00204000=LTImage Comments",
-
+		
 		"00280002=USSamples per Pixel",
 		"00280004=CSPhotometric Interpretation",
 		"00280006=USPlanar Configuration",
@@ -1301,7 +1331,7 @@ class DicomDictionary {
 		"30020011=DSImage Plane Pixel Spacing",
 		"30020022=DSRadiation Machine SAD",
 		"30020026=DSRT IMAGE SID",
-
+		
 		"0032000A=CSStudy Status ID",
 		"0032000C=CSStudy Priority ID",
 		"00320012=LOStudy ID Issuer",
@@ -1327,7 +1357,7 @@ class DicomDictionary {
 		"00321064=SQRequested Procedure Code Sequence",
 		"00321070=LORequested Contrast Agent",
 		"00324000=LTStudy Comments",
-
+		
 		"00400001=AEScheduled Station AE Title",
 		"00400002=DAScheduled Procedure Step Start Date",
 		"00400003=TMScheduled Procedure Step Start Time",
@@ -1463,88 +1493,117 @@ class DicomDictionary {
 		"0040DB0C=UITemplate Extension Organization UID",
 		"0040DB0D=UITemplate Extension Creator UID",
 		"0040DB73=ULReferenced Content Item Identifier",
+		
+		"00540011=USNumber of Energy Windows",
+		"00540012=SQEnergy Window Information Sequence",
+		"00540013=SQEnergy Window Range Sequence",
+		"00540014=DSEnergy Window Lower Limit",
+		"00540015=DSEnergy Window Upper Limit",
+		"00540016=SQRadiopharmaceutical Information Sequence",
+		"00540017=ISResidual Syringe Counts",
+		"00540018=SHEnergy Window Name",
+		"00540020=USDetector Vector",
+		"00540021=USNumber of Detectors",
+		"00540022=SQDetector Information Sequence",
+		"00540030=USPhase Vector",
+		"00540031=USNumber of Phases",
+		"00540032=SQPhase Information Sequence",
+		"00540033=USNumber of Frames in Phase",
+		"00540036=ISPhase Delay",
+		"00540038=ISPause Between Frames",
+		"00540039=CSPhase Description",
+		"00540050=USRotation Vector",
+		"00540051=USNumber of Rotations",
+		"00540052=SQRotation Information Sequence",
+		"00540053=USNumber of Frames in Rotation",
+		"00540060=USR-R Interval Vector",
+		"00540061=USNumber of R-R Intervals",
+		"00540062=SQGated Information Sequence",
+		"00540063=SQData Information Sequence",
+		"00540070=USTime Slot Vector",
+		"00540071=USNumber of Time Slots",
+		"00540072=SQTime Slot Information Sequence",
+		"00540073=DSTime Slot Time",
+		"00540080=USSlice Vector",
+		"00540081=USNumber of Slices",
+		"00540090=USAngular View Vector",
+		"00540100=USTime Slice Vector",
+		"00540101=USNumber of Time Slices",
+		"00540200=DSStart Angle",
+		"00540202=CSType of Detector Motion",
+		"00540210=ISTrigger Vector",
+		"00540211=USNumber of Triggers in Phase",
+		"00540220=SQView Code Sequence",
+		"00540222=SQView Modifier Code Sequence",
+		"00540300=SQRadionuclide Code Sequence",
+		"00540302=SQAdministration Route Code Sequence",
+		"00540304=SQRadiopharmaceutical Code Sequence",
+		"00540306=SQCalibration Data Sequence",
+		"00540308=USEnergy Window Number",
+		"00540400=SHImage ID",
+		"00540410=SQPatient Orientation Code Sequence",
+		"00540412=SQPatient Orientation Modifier Code Sequence",
+		"00540414=SQPatient Gantry Relationship Code Sequence",
+		"00540500=CSSlice Progression Direction",
+		"00541000=CSSeries Type",
+		"00541001=CSUnits",
+		"00541002=CSCounts Source",
+		"00541004=CSReprojection Method",
+		"00541100=CSRandoms Correction Method",
+		"00541101=LOAttenuation Correction Method",
+		"00541102=CSDecay Correction",
+		"00541103=LOReconstruction Method",
+		"00541104=LODetector Lines of Response Used",
+		"00541105=LOScatter Correction Method",
+		"00541200=DSAxial Acceptance",
+		"00541201=ISAxial Mash",
+		"00541202=ISTransverse Mash",
+		"00541203=DSDetector Element Size",
+		"00541210=DSCoincidence Window Width",
+		"00541220=CSSecondary Counts Type",
+		"00541300=DSFrame Reference Time",
+		"00541310=ISPrimary (Prompts) Counts Accumulated",
+		"00541311=ISSecondary Counts Accumulated",
+		"00541320=DSSlice Sensitivity Factor",
+		"00541321=DSDecay Factor",
+		"00541322=DSDose Calibration Factor",
+		"00541323=DSScatter Fraction Factor",
+		"00541324=DSDead Time Factor",
+		"00541330=USImage Index",
+		"00541400=CSCounts Included",
+		"00541401=CSDead Time Correction Flag",
+		
+		"30020002=SHRT Image Label",
+		"30020003=LORT Image Name",
+		"30020004=STRT Image Description",
+		"3002000A=CSReported Values Origin",
+		"3002000C=CSRT Image Plane",
+		"3002000D=DSX-Ray Image Receptor Translation",
+		"3002000E=DSX-Ray Image Receptor Angle",
+		"30020011=DSImage Plane Pixel Spacing",
+		"30020012=DSRT Image Position",
+		"30020020=SHRadiation Machine Name",
+		"30020022=DSRadiation Machine SAD",
+		"30020026=DSRT Image SID",
+		"30020029=ISFraction Number",
+		"30020030=SQExposure Sequence",
+		"30020032=DSMeterset Exposure",
+		
+		"300A011E=DSGantry Angle",
+		"300A0120=DSBeam Limiting Device Angle",
+		"300A0122=DSPatient Support Angle",
+		"300A0128=DSTable Top Vertical Position",
+		"300A0129=DSTable Top Longitudinal Position",
+		"300A012A=DSTable Top Lateral Position",
+		"300A00B3=CSPrimary Dosimeter Unit",
+		"300A00F0=ISNumber of Blocks",
+		
+		"300C0006=ISReferenced Beam Number",
+		"300C0008=DSStart Cumulative Meterset Weight",
+		"300C0022=ISReferenced Fraction Group Number",
 
-        "00540011=USNumber of Energy Windows",
-        "00540012=SQEnergy Window Information Sequence",
-        "00540013=SQEnergy Window Range Sequence",
-        "00540014=DSEnergy Window Lower Limit",
-        "00540015=DSEnergy Window Upper Limit",
-        "00540016=SQRadiopharmaceutical Information Sequence",
-        "00540017=ISResidual Syringe Counts",
-        "00540018=SHEnergy Window Name",
-        "00540020=USDetector Vector",
-        "00540021=USNumber of Detectors",
-        "00540022=SQDetector Information Sequence",
-        "00540030=USPhase Vector",
-        "00540031=USNumber of Phases",
-        "00540032=SQPhase Information Sequence",
-        "00540033=USNumber of Frames in Phase",
-        "00540036=ISPhase Delay",
-        "00540038=ISPause Between Frames",
-        "00540039=CSPhase Description",
-        "00540050=USRotation Vector",
-        "00540051=USNumber of Rotations",
-        "00540052=SQRotation Information Sequence",
-        "00540053=USNumber of Frames in Rotation",
-        "00540060=USR-R Interval Vector",
-        "00540061=USNumber of R-R Intervals",
-        "00540062=SQGated Information Sequence",
-        "00540063=SQData Information Sequence",
-        "00540070=USTime Slot Vector",
-        "00540071=USNumber of Time Slots",
-        "00540072=SQTime Slot Information Sequence",
-        "00540073=DSTime Slot Time",
-        "00540080=USSlice Vector",
-        "00540081=USNumber of Slices",
-        "00540090=USAngular View Vector",
-        "00540100=USTime Slice Vector",
-        "00540101=USNumber of Time Slices",
-        "00540200=DSStart Angle",
-        "00540202=CSType of Detector Motion",
-        "00540210=ISTrigger Vector",
-        "00540211=USNumber of Triggers in Phase",
-        "00540220=SQView Code Sequence",
-        "00540222=SQView Modifier Code Sequence",
-        "00540300=SQRadionuclide Code Sequence",
-        "00540302=SQAdministration Route Code Sequence",
-        "00540304=SQRadiopharmaceutical Code Sequence",
-        "00540306=SQCalibration Data Sequence",
-        "00540308=USEnergy Window Number",
-        "00540400=SHImage ID",
-        "00540410=SQPatient Orientation Code Sequence",
-        "00540412=SQPatient Orientation Modifier Code Sequence",
-        "00540414=SQPatient Gantry Relationship Code Sequence",
-        "00540500=CSSlice Progression Direction",
-        "00541000=CSSeries Type",
-        "00541001=CSUnits",
-        "00541002=CSCounts Source",
-        "00541004=CSReprojection Method",
-        "00541100=CSRandoms Correction Method",
-        "00541101=LOAttenuation Correction Method",
-        "00541102=CSDecay Correction",
-        "00541103=LOReconstruction Method",
-        "00541104=LODetector Lines of Response Used",
-        "00541105=LOScatter Correction Method",
-        "00541200=DSAxial Acceptance",
-        "00541201=ISAxial Mash",
-        "00541202=ISTransverse Mash",
-        "00541203=DSDetector Element Size",
-        "00541210=DSCoincidence Window Width",
-        "00541220=CSSecondary Counts Type",
-        "00541300=DSFrame Reference Time",
-        "00541310=ISPrimary (Prompts) Counts Accumulated",
-        "00541311=ISSecondary Counts Accumulated",
-        "00541320=DSSlice Sensitivity Factor",
-        "00541321=DSDecay Factor",
-        "00541322=DSDose Calibration Factor",
-        "00541323=DSScatter Fraction Factor",
-        "00541324=DSDead Time Factor",
-        "00541330=USImage Index",
-        "00541400=CSCounts Included",
-        "00541401=CSDead Time Correction Flag",
-        
 		"7FE00010=OXPixel Data",
-
+		
 		"FFFEE000=DLItem",
 		"FFFEE00D=DLItem Delimitation Item",
 		"FFFEE0DD=DLSequence Delimitation Item"
