@@ -3,7 +3,6 @@ import ij.*;
 import ij.process.*;
 import ij.gui.*;
 import ij.measure.*;
-import ij.plugin.filter.RGBStackSplitter;
 import ij.util.Tools;
 import java.awt.*;
 import java.awt.event.*;
@@ -21,7 +20,8 @@ public class Slicer implements PlugIn, TextListener, ItemListener {
 	private static String startAt = starts[0];
 	private static boolean rotate;
 	private static boolean flip;
-	private static boolean nointerpolate;
+	private static int sliceCount = 1;
+	private boolean nointerpolate = Prefs.avoidResliceInterpolation;
 	private double inputZSpacing = 1.0;
 	private double outputZSpacing = 1.0;
 	private int outputSlices = 1;
@@ -97,7 +97,9 @@ public class Slicer implements PlugIn, TextListener, ItemListener {
 				tmpCal.pixelHeight = 1.0;
 				tmpCal.pixelDepth = 1.0;
 				imp.setCalibration(tmpCal);
-				inputZSpacing = outputZSpacing = 1.0;
+				inputZSpacing = 1.0;
+				if (roiType!=Roi.LINE)
+					outputZSpacing = 1.0;
 		 }
 		double zSpacing = inputZSpacing/imp.getCalibration().pixelWidth;
 		 if (roi==null || roiType==Roi.RECTANGLE || roiType==Roi.LINE) {
@@ -125,6 +127,7 @@ public class Slicer implements PlugIn, TextListener, ItemListener {
 			horizontal  = (l.y2-l.y1)==0;
 			vertical = (l.x2-l.x1)==0;
 		}
+		if (imp2==null) return null;
 		imp2.setCalibration(imp.getCalibration());
 		Calibration cal = imp2.getCalibration();
 		if (horizontal) {
@@ -139,7 +142,8 @@ public class Slicer implements PlugIn, TextListener, ItemListener {
 			cal.pixelDepth = origCal.pixelWidth*outputZSpacing;;
 		} else { // oblique line, polyLine or freeline
 				if (origCal.pixelHeight==origCal.pixelWidth) {
-					cal.pixelWidth = cal.pixelHeight=origCal.pixelDepth/zSpacing;
+					cal.pixelWidth = origCal.pixelWidth;
+					cal.pixelHeight=origCal.pixelDepth/zSpacing;
 					cal.pixelDepth = origCal.pixelWidth*outputZSpacing;
 				} else {
 					cal.pixelWidth = cal.pixelHeight=cal.pixelDepth=1.0;
@@ -204,57 +208,79 @@ public class Slicer implements PlugIn, TextListener, ItemListener {
 	}
 
 	boolean showDialog(ImagePlus imp) {
-		 Calibration cal = imp.getCalibration();
-		 if (cal.pixelDepth<0.0)
-		 	cal.pixelDepth = -cal.pixelDepth;
-		 String units = cal.getUnits();
-		 if (cal.pixelWidth==0.0)
-				cal.pixelWidth = 1.0;
-		 inputZSpacing = cal.pixelDepth;
-		 double outputSpacing = cal.pixelDepth;
-		 Roi roi = imp.getRoi();
-		 boolean line = roi!=null && roi.getType()==Roi.LINE;
-		 if (line) saveLineInfo(roi);
+		Calibration cal = imp.getCalibration();
+		if (cal.pixelDepth<0.0)
+			cal.pixelDepth = -cal.pixelDepth;
+		String units = cal.getUnits();
+		if (cal.pixelWidth==0.0)
+			cal.pixelWidth = 1.0;
+		inputZSpacing = cal.pixelDepth;
+		double outputSpacing = cal.pixelDepth;
+		Roi roi = imp.getRoi();
+		boolean line = roi!=null && roi.getType()==Roi.LINE;
+		if (line) saveLineInfo(roi);
 		String macroOptions = Macro.getOptions();
-		if (macroOptions!=null && macroOptions.indexOf("output=")!=-1) {
-			Macro.setOptions(macroOptions.replaceAll("output=", "slice="));
-			Macro.setOptions(macroOptions.replaceAll("slice=", "slice_count="));
+		if (macroOptions!=null) {
+			if (macroOptions.indexOf("input=")!=-1)
+				macroOptions = macroOptions.replaceAll("slice=", "slice_count=");
+			macroOptions = macroOptions.replaceAll("slice=", "output=");
+			Macro.setOptions(macroOptions);
+			nointerpolate = false;
 		}
 		GenericDialog gd = new GenericDialog("Reslice");
-		 //gd.addNumericField("Input Z Spacing ("+units+"):", cal.pixelDepth, 3);
-		 gd.addNumericField("Slice Spacing ("+units+"):", outputSpacing, 3);
-		 if (line)
-				gd.addNumericField("Slice_Count:", outputSlices, 0);
-		 else
-				gd.addChoice("Start At:", starts, startAt);
-		 gd.addCheckbox("Flip Vertically", flip);
-		 gd.addCheckbox("Rotate 90 Degrees", rotate);
-		 gd.addCheckbox("Avoid Interpolation", nointerpolate);
-		 gd.setInsets(0, 32, 10);
-		 gd.addMessage("(use 1.0 for spacings)");
-		 gd.addMessage(getSize(cal.pixelDepth,outputSpacing,outputSlices)+"				");
-		 fields = gd.getNumericFields();
-		 for (int i=0; i<fields.size(); i++)
-				((TextField)fields.elementAt(i)).addTextListener(this);
+		gd.addNumericField("Output spacing ("+units+"):", outputSpacing, 3);
+		if (line) {
+			if (!IJ.isMacro()) outputSlices=sliceCount;
+			gd.addNumericField("Slice_count:", outputSlices, 0);
+		} else
+			gd.addChoice("Start at:", starts, startAt);
+		gd.addCheckbox("Flip vertically", flip);
+		gd.addCheckbox("Rotate 90 degrees", rotate);
+		gd.addCheckbox("Avoid interpolation", nointerpolate);
+		gd.setInsets(0, 32, 0);
+		gd.addMessage("(use 1 pixel spacing)");
+		gd.setInsets(15, 0, 0);
+		gd.addMessage("Voxel size: "+d2s(cal.pixelWidth)+"x"+d2s(cal.pixelHeight)
+			+"x"+d2s(cal.pixelDepth)+" "+cal.getUnit());
+		gd.setInsets(5, 0, 0);
+		gd.addMessage("Output size: "+getSize(cal.pixelDepth,outputSpacing,outputSlices)+"				");
+		fields = gd.getNumericFields();
+		for (int i=0; i<fields.size(); i++)
+			((TextField)fields.elementAt(i)).addTextListener(this);
 		checkboxes = gd.getCheckboxes();
-			((Checkbox)checkboxes.elementAt(2)).addItemListener(this);
-		 message = (Label)gd.getMessage();
-		 gd.showDialog();
-		 if (gd.wasCanceled())
-				return false;
-		 //inputZSpacing = gd.getNextNumber();
-		 //if (cal.pixelDepth==0.0) cal.pixelDepth = 1.0;
-		 outputZSpacing = gd.getNextNumber()/cal.pixelWidth;
-		 if (line) {
-				outputSlices = (int)gd.getNextNumber();
-				imp.setRoi(roi);
-		 } else
-				startAt = gd.getNextChoice();
-		 flip = gd.getNextBoolean();
-		 rotate = gd.getNextBoolean();
-		 nointerpolate = gd.getNextBoolean();
+		((Checkbox)checkboxes.elementAt(2)).addItemListener(this);
+		message = (Label)gd.getMessage();
+        gd.addHelp(IJ.URL+"/docs/menus/image.html#reslice");
+		gd.showDialog();
+		if (gd.wasCanceled())
+			return false;
+		//inputZSpacing = gd.getNextNumber();
+		//if (cal.pixelDepth==0.0) cal.pixelDepth = 1.0;
+		outputZSpacing = gd.getNextNumber()/cal.pixelWidth;
+		if (line) {
+			outputSlices = (int)gd.getNextNumber();
+			if (!IJ.isMacro()) sliceCount=outputSlices;
+			imp.setRoi(roi);
+		} else
+			startAt = gd.getNextChoice();
+		flip = gd.getNextBoolean();
+		rotate = gd.getNextBoolean();
+		nointerpolate = gd.getNextBoolean();
+		if (!IJ.isMacro())
+			Prefs.avoidResliceInterpolation = nointerpolate;
 		return true;
 	}
+	
+	String d2s(double n) {
+		String s;
+		if (n==(int)n)
+			s = ResultsTable.d2s(n, 0);
+		else
+			s = ResultsTable.d2s(n, 2);
+		if (s.indexOf(".")!=-1 && s.endsWith("0"))
+			s = s.substring(0, s.length()-1);
+		return s;
+	}
 
 	void saveLineInfo(Roi roi) {
 		 Line line = (Line)roi;
@@ -330,7 +356,8 @@ public class Slicer implements PlugIn, TextListener, ItemListener {
 				return null;
 
 		 if (outputSlices==0) {
-				IJ.error("Reslicer", "Output Z spacing ("+IJ.d2s(outputZSpacing,0)+" pixels) is too large.");
+				IJ.error("Reslicer", "Output Z spacing ("+IJ.d2s(outputZSpacing,0)+" pixels) is too large.\n"
+					+"Is the voxel size in Image>Properties correct?.");
 				return null;
 		 }
 		 boolean virtualStack = imp.getStack().isVirtual();
@@ -464,9 +491,6 @@ public class Slicer implements PlugIn, TextListener, ItemListener {
 						 } else
 								values[index] = (float)ip.getInterpolatedValue(rx, ry);
 					}
-					//d = Math.sqrt((rx-oldx)*(rx-oldx)+(ry-oldy)*(ry-oldy));
-					//IJ.write(IJ.d2s(rx)+"	"+IJ.d2s(ry)+"	 "+IJ.d2s(d));
-					//oldx = rx; oldy = ry;
 					rx += xinc;
 					ry += yinc;
 				}
@@ -566,8 +590,7 @@ public class Slicer implements PlugIn, TextListener, ItemListener {
 		 ImageCanvas ic = imp.getCanvas();
 		 if (ic==null) return;
 		 Graphics g = ic.getGraphics();
-		 g.setColor(Roi.getColor());
-		 g.setXORMode(Color.black);
+		 g.setColor(new Color(1f, 1f, 0f, 0.4f));
 		 g.drawLine(ic.screenX((int)(x1+0.5)), ic.screenY((int)(y1+0.5)), ic.screenX((int)(x2+0.5)), ic.screenY((int)(y2+0.5)));
 	}
 
@@ -592,7 +615,7 @@ public class Slicer implements PlugIn, TextListener, ItemListener {
 				if (count>0) makePolygon(count, outSpacing);
 		 }
 		 String size = getSize(inputZSpacing, outSpacing, count);
-		 message.setText(size);
+		 message.setText("Output Size: "+size);
 	}
 
 	String getSize(double inSpacing, double outSpacing, int count) {
