@@ -34,8 +34,8 @@ public class AVI_Writer implements PlugInFilter {
     private final static int FOURCC_00dc = 0x63643030;    //'00dc' compressed frame
 
     //compression options: dialog parameters
-    private static int      compressionIndex = 0; //0=none, 1=PNG, 2=JPEG
-    private static int      jpegQuality = 90;    //0 is worst, 100 best
+    private static int      compressionIndex = 2; //0=none, 1=PNG, 2=JPEG
+    private static int      jpegQuality = 90;    //0 is worst, 100 best (not currently used)
     private final static String[] COMPRESSION_STRINGS = new String[] {"Uncompressed", "PNG", "JPEG"};
     private final static int[] COMPRESSION_TYPES = new int[] {NO_COMPRESSION, PNG_COMPRESSION, JPEG_COMPRESSION};
 
@@ -67,7 +67,11 @@ public class AVI_Writer implements PlugInFilter {
         if (fileName == null)
             return;
         String fileDir = sd.getDirectory();
-        
+        FileInfo fi = imp.getOriginalFileInfo();
+        if (imp.getStack().isVirtual() && fileDir.equals(fi.directory)&& fileName.equals(fi.fileName)) {
+            IJ.error("AVI Writer", "Virtual stacks cannot be saved in place.");
+            return;
+        }
         try {
             writeImage(imp, fileDir + fileName, COMPRESSION_TYPES[compressionIndex], jpegQuality);
             IJ.showStatus("");
@@ -85,16 +89,16 @@ public class AVI_Writer implements PlugInFilter {
  		int decimalPlaces = (int) fps == fps?0:1;
         GenericDialog gd = new GenericDialog("Save as AVI...");
         gd.addChoice("Compression:", COMPRESSION_STRINGS, COMPRESSION_STRINGS[compressionIndex]);
-        gd.addNumericField("JPEG Quality (0-100):", jpegQuality, 0, 3, "");
+        //gd.addNumericField("JPEG Quality (0-100):", jpegQuality, 0, 3, "");
 		gd.addNumericField("Frame Rate:", fps, decimalPlaces, 3, "fps");
         gd.showDialog();                            // user input (or reading from macro) happens here
         if (gd.wasCanceled())                       // dialog cancelled?
             return false;
         compressionIndex = gd.getNextChoiceIndex();
-        jpegQuality = (int)gd.getNextNumber();
+        //jpegQuality = (int)gd.getNextNumber();
         fps = gd.getNextNumber();
         if (fps<=0.5) fps = 0.5;
-        if (fps>60.0) fps = 60.0;
+        //if (fps>60.0) fps = 60.0;
 		imp.getCalibration().fps = fps;
 		return true;
     }
@@ -116,6 +120,7 @@ public class AVI_Writer implements PlugInFilter {
         //  G e t   s t a c k   p r o p e r t i e s
         boolean isComposite = imp.isComposite();
         boolean isHyperstack = imp.isHyperStack();
+        boolean isOverlay = imp.getOverlay()!=null && !imp.getHideOverlay();
         xDim = imp.getWidth();   //image width
         yDim = imp.getHeight();   //image height
         zDim = imp.getStackSize(); //number of frames in video
@@ -140,18 +145,17 @@ public class AVI_Writer implements PlugInFilter {
 				isHyperstack = false;
 		}
 
-        if (imp.getType()==ImagePlus.COLOR_RGB || isComposite || biCompression==JPEG_COMPRESSION)
+        if (imp.getType()==ImagePlus.COLOR_RGB || isComposite || biCompression==JPEG_COMPRESSION || isOverlay)
             bytesPerPixel = 3;  //color and JPEG-compressed files
         else
             bytesPerPixel = 1;  //gray 8, 16, 32 bit and indexed color: all written as 8 bit
-        //boolean isColor = imp.getType()==ImagePlus.COLOR_RGB || isComposite || imp.getProcessor().isColorLut();
         boolean writeLUT = bytesPerPixel==1; // QuickTime reads the avi palette also for PNG
         linePad = 0;
         int minLineLength = bytesPerPixel*xDim;
         if (biCompression==NO_COMPRESSION && minLineLength%4!=0)
             linePad = 4 - minLineLength%4; //uncompressed lines written must be a multiple of 4 bytes
         frameDataSize = (bytesPerPixel*xDim+linePad)*yDim;
-        int microSecPerFrame = (int)((1.0/getFrameRate(imp))*1.0e6);
+        int microSecPerFrame = (int)Math.round((1.0/getFrameRate(imp))*1.0e6);
 
         //  W r i t e   A V I   f i l e   h e a d e r
         writeString("RIFF");    // signature
@@ -194,7 +198,7 @@ public class AVI_Writer implements PlugInFilter {
         writeInt(0);            // wPriority, wLanguage
         writeInt(0);            // dwInitialFrames
         writeInt(1);            // dwScale
-        writeInt((int)getFrameRate(imp)); //  dwRate - frame rate for video streams
+        writeInt((int)Math.round(getFrameRate(imp))); //  dwRate - frame rate for video streams
         writeInt(0);            // dwStart - this field is usually set to zero
         writeInt(zDim);         // dwLength - playing time of AVI file as defined by scale and rate
                                 // Set equal to the number of frames
@@ -266,14 +270,20 @@ public class AVI_Writer implements PlugInFilter {
             IJ.showProgress(z, zDim);
             IJ.showStatus(z+"/"+zDim);
             ImageProcessor ip = null;      // get the image to write ...
-            if (isComposite || isHyperstack) {
+            if (isComposite || isHyperstack || isOverlay) {
 				if (saveFrames)
 					imp.setPositionWithoutUpdate(channel, slice, z+1);
 				else if (saveSlices)
 					imp.setPositionWithoutUpdate(channel, z+1, frame);
 				else if (saveChannels)
 					imp.setPositionWithoutUpdate(z+1, slice, frame);
-				ip = new ColorProcessor(imp.getImage());
+				ImagePlus imp2 = imp;
+				if (isOverlay) {
+					if (!(saveFrames||saveSlices||saveChannels))
+						imp.setSliceWithoutUpdate(z+1);
+					imp2 = imp.flatten();
+				}
+				ip = new ColorProcessor(imp2.getImage());
             } else
                 ip = zDim==1 ? imp.getProcessor() : imp.getStack().getProcessor(z+1);
             int chunkPointer = (int)raFile.getFilePointer();
@@ -388,12 +398,21 @@ public class AVI_Writer implements PlugInFilter {
 
     /** Write a frame as jpeg- or png-compressed image */
 	private void writeCompressedFrame(ImageProcessor ip) throws IOException {
-		BufferedImage bufferedImage = ip.getBufferedImage();
 		//IJ.log("BufferdImage Type="+bufferedImage.getType()); // 1=RGB, 13=indexed
-		if (biCompression == JPEG_COMPRESSION)
-			ImageIO.write(bufferedImage, "jpeg", raOutputStream);
-		else //if (biCompression == PNG_COMPRESSION)
-			ImageIO.write(bufferedImage, "png", raOutputStream);
+		if (biCompression==JPEG_COMPRESSION) {
+			BufferedImage bi = getBufferedImage(ip);
+			ImageIO.write(bi, "jpeg", raOutputStream);
+		} else { //if (biCompression==PNG_COMPRESSION) {
+			BufferedImage bi = ip.getBufferedImage();
+			ImageIO.write(bi, "png", raOutputStream);
+		}
+	}
+
+	private BufferedImage getBufferedImage(ImageProcessor ip) {
+		BufferedImage bi = new BufferedImage(ip.getWidth(), ip.getHeight(), BufferedImage.TYPE_INT_RGB);
+		Graphics2D g = (Graphics2D)bi.getGraphics();
+		g.drawImage(ip.createImage(), 0, 0, null);
+		return bi;
 	}
 
     /** Write the color table entries (for 8 bit grayscale or indexed color).
@@ -418,7 +437,7 @@ public class AVI_Writer implements PlugInFilter {
         if (rate==0.0)
             rate = Animator.getFrameRate();
         if (rate<=0.5) rate = 0.5;
-        if (rate>60.0) rate = 60.0;
+        //if (rate>60.0) rate = 60.0;
         return rate;
     }
 
