@@ -25,7 +25,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 	private static final int[] list = {AREA,MEAN,STD_DEV,MODE,MIN_MAX,
 		CENTROID,CENTER_OF_MASS,PERIMETER,RECT,ELLIPSE,SHAPE_DESCRIPTORS, FERET,
 		INTEGRATED_DENSITY,MEDIAN,SKEWNESS,KURTOSIS,AREA_FRACTION,STACK_POSITION,
-		LIMIT,LABELS,INVERT_Y,SCIENTIFIC_NOTATION};
+		LIMIT,LABELS,INVERT_Y,SCIENTIFIC_NOTATION,ADD_TO_OVERLAY};
 
 	private static final String MEASUREMENTS = "measurements";
 	private static final String MARK_WIDTH = "mark.width";
@@ -40,6 +40,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 	private static ResultsTable systemRT = new ResultsTable();
 	private static int redirectTarget;
 	private static String redirectTitle = "";
+	private static ImagePlus redirectImage; // non-displayed images
 	static int firstParticle, lastParticle;
 	private static boolean summarized;
 	private static boolean switchingModes;
@@ -47,6 +48,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 	
 	public Analyzer() {
 		rt = systemRT;
+		rt.showRowNumbers(true);
 		rt.setPrecision((systemMeasurements&SCIENTIFIC_NOTATION)!=0?-precision:precision);
 		measurements = systemMeasurements;
 	}
@@ -85,6 +87,23 @@ public class Analyzer implements PlugInFilter, Measurements {
 	public void run(ImageProcessor ip) {
 		measure();
 		displayResults();
+		if ((measurements&ADD_TO_OVERLAY)!=0)
+			addToOverlay();
+	}
+	
+	void addToOverlay() {
+		Roi roi = imp.getRoi();
+		if (roi==null)
+			return;
+		if (imp.getStackSize()>1) {
+			if (imp.isHyperStack()||imp.isComposite())
+				roi.setPosition(0, imp.getSlice(), imp.getFrame());
+			else
+				roi.setPosition(imp.getCurrentSlice());
+		}
+		//if (rt.getCounter()==1)
+		//	RoiProperties.setShowLabels(true);
+		IJ.run(imp, "Add Selection...", "");
 	}
 
 	void doSetDialog() {
@@ -114,55 +133,59 @@ public class Analyzer implements PlugInFilter, Measurements {
 		String[] labels = new String[18];
 		boolean[] states = new boolean[18];
 		labels[0]="Area"; states[0]=(systemMeasurements&AREA)!=0;
-		labels[1]="Mean Gray Value"; states[1]=(systemMeasurements&MEAN)!=0;
-		labels[2]="Standard Deviation"; states[2]=(systemMeasurements&STD_DEV)!=0;
-		labels[3]="Modal Gray Value"; states[3]=(systemMeasurements&MODE)!=0;
-		labels[4]="Min & Max Gray Value"; states[4]=(systemMeasurements&MIN_MAX)!=0;
+		labels[1]="Mean gray value"; states[1]=(systemMeasurements&MEAN)!=0;
+		labels[2]="Standard deviation"; states[2]=(systemMeasurements&STD_DEV)!=0;
+		labels[3]="Modal gray value"; states[3]=(systemMeasurements&MODE)!=0;
+		labels[4]="Min & max gray value"; states[4]=(systemMeasurements&MIN_MAX)!=0;
 		labels[5]="Centroid"; states[5]=(systemMeasurements&CENTROID)!=0;
-		labels[6]="Center of Mass"; states[6]=(systemMeasurements&CENTER_OF_MASS)!=0;
+		labels[6]="Center of mass"; states[6]=(systemMeasurements&CENTER_OF_MASS)!=0;
 		labels[7]="Perimeter"; states[7]=(systemMeasurements&PERIMETER)!=0;
-		labels[8]="Bounding Rectangle"; states[8]=(systemMeasurements&RECT)!=0;
-		labels[9]="Fit Ellipse"; states[9]=(systemMeasurements&ELLIPSE)!=0;
-		labels[10]="Shape Descriptors"; states[10]=(systemMeasurements&SHAPE_DESCRIPTORS)!=0;
-		labels[11]="Feret's Diameter"; states[11]=(systemMeasurements&FERET)!=0;
-		labels[12]="Integrated Density"; states[12]=(systemMeasurements&INTEGRATED_DENSITY)!=0;
+		labels[8]="Bounding rectangle"; states[8]=(systemMeasurements&RECT)!=0;
+		labels[9]="Fit ellipse"; states[9]=(systemMeasurements&ELLIPSE)!=0;
+		labels[10]="Shape descriptors"; states[10]=(systemMeasurements&SHAPE_DESCRIPTORS)!=0;
+		labels[11]="Feret's diameter"; states[11]=(systemMeasurements&FERET)!=0;
+		labels[12]="Integrated density"; states[12]=(systemMeasurements&INTEGRATED_DENSITY)!=0;
 		labels[13]="Median"; states[13]=(systemMeasurements&MEDIAN)!=0;
 		labels[14]="Skewness"; states[14]=(systemMeasurements&SKEWNESS)!=0;
 		labels[15]="Kurtosis"; states[15]=(systemMeasurements&KURTOSIS)!=0;
-		labels[16]="Area_Fraction"; states[16]=(systemMeasurements&AREA_FRACTION)!=0;
-		labels[17]="Stack Position"; states[17]=(systemMeasurements&STACK_POSITION)!=0;
+		labels[16]="Area_fraction"; states[16]=(systemMeasurements&AREA_FRACTION)!=0;
+		labels[17]="Stack position"; states[17]=(systemMeasurements&STACK_POSITION)!=0;
 		gd.setInsets(0, 0, 0);
 		gd.addCheckboxGroup(10, 2, labels, states);
-		labels = new String[4];
-		states = new boolean[4];
-		labels[0]="Limit to Threshold"; states[0]=(systemMeasurements&LIMIT)!=0;
-		labels[1]="Display Label"; states[1]=(systemMeasurements&LABELS)!=0;
-		labels[2]="Invert Y Coordinates"; states[2]=(systemMeasurements&INVERT_Y)!=0;
-		labels[3]="Scientific Notation"; states[3]=(systemMeasurements&SCIENTIFIC_NOTATION)!=0;;
+		labels = new String[5];
+		states = new boolean[5];
+		labels[0]="Limit to threshold"; states[0]=(systemMeasurements&LIMIT)!=0;
+		labels[1]="Display label"; states[1]=(systemMeasurements&LABELS)!=0;
+		labels[2]="Invert Y coordinates"; states[2]=(systemMeasurements&INVERT_Y)!=0;
+		labels[3]="Scientific notation"; states[3]=(systemMeasurements&SCIENTIFIC_NOTATION)!=0;;
+		labels[4]="Add to overlay"; states[4]=(systemMeasurements&ADD_TO_OVERLAY)!=0;;
 		gd.setInsets(0, 0, 0);
-		gd.addCheckboxGroup(2, 2, labels, states);
+		gd.addCheckboxGroup(3, 2, labels, states);
 		gd.setInsets(15, 0, 0);
-        gd.addChoice("Redirect To:", titles, target);
+        gd.addChoice("Redirect to:", titles, target);
 		gd.setInsets(5, 0, 0);
-		gd.addNumericField("Decimal Places (0-9):", precision, 0, 2, "");
+		gd.addNumericField("Decimal places (0-9):", precision, 0, 2, "");
+		gd.addHelp(IJ.URL+"/docs/menus/analyze.html#set");
 		gd.showDialog();
 		if (gd.wasCanceled())
 			return;
+		int oldMeasurements = systemMeasurements;
 		setOptions(gd);
 		int index = gd.getNextChoiceIndex();
 		redirectTarget = index==0?0:wList[index-1];
 		redirectTitle = titles[index];
+		ImagePlus imp = WindowManager.getImage(redirectTarget);
+		redirectImage = imp!=null && imp.getWindow()==null?imp:null;
 
 		int prec = (int)gd.getNextNumber();
 		if (prec<0) prec = 0;
 		if (prec>9) prec = 9;
-		if (prec!=precision) {
+		boolean notationChanged = (oldMeasurements&SCIENTIFIC_NOTATION)!=(systemMeasurements&SCIENTIFIC_NOTATION);
+		if (prec!=precision || notationChanged) {
 			precision = prec;
 			rt.setPrecision((systemMeasurements&SCIENTIFIC_NOTATION)!=0?-precision:precision);
-			if (IJ.isResultsWindow()) {
-				IJ.setColumnHeadings("");
-				updateHeadings();
-			}
+			if (rt.getCounter()>0)
+				rt.show("Results");
 		}
 	}
 	
@@ -179,7 +202,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 			else
 				systemMeasurements &= ~list[i];
 		}
-		if ((oldMeasurements&(~LIMIT))!=(systemMeasurements&(~LIMIT))&&IJ.isResultsWindow()) {
+		if ((oldMeasurements&(~LIMIT)&(~SCIENTIFIC_NOTATION))!=(systemMeasurements&(~LIMIT)&(~SCIENTIFIC_NOTATION))&&IJ.isResultsWindow()) {
 				rt.setPrecision((systemMeasurements&SCIENTIFIC_NOTATION)!=0?-precision:precision);
 				rt.update(systemMeasurements, imp, null);
 		}
@@ -233,35 +256,59 @@ public class Analyzer implements PlugInFilter, Measurements {
 		return redirectTarget!=0;
 	}
 	
+	/** Set the "Redirect To" image. Pass 'null' as the 
+	    argument to disable redirected sampling. */
+	public static void setRedirectImage(ImagePlus imp) {
+		if (imp==null) {
+			redirectTarget = 0;
+			redirectTitle = null;
+			redirectImage = null;
+		} else {
+			redirectTarget = imp.getID();
+			redirectTitle = imp.getTitle();
+			if (imp.getWindow()==null)
+				redirectImage = imp;
+		}
+	}
+	
+	private ImagePlus getRedirectImageOrStack(ImagePlus cimp) {
+		ImagePlus rimp = getRedirectImage(cimp);
+		if (rimp!=null) {
+			int depth = rimp.getStackSize();
+			if (depth>1 && depth==cimp.getStackSize() && rimp.getCurrentSlice()!=cimp.getCurrentSlice())
+				rimp.setSlice(cimp.getCurrentSlice());
+		}
+		return rimp;
+	}
+
 	/** Returns the image selected in the "Redirect To:" popup
-		menu of the Analyze/Set Measurements dialog or null
+		menu of the Analyze/Set Measurements dialog, or null
 		if "None" is selected, the image was not found or the 
 		image is not the same size as <code>currentImage</code>. */
-	public static ImagePlus getRedirectImage(ImagePlus currentImage) {
-		ImagePlus rImp = WindowManager.getImage(redirectTarget);
-		if (rImp==null) {
+	public static ImagePlus getRedirectImage(ImagePlus cimp) {
+		ImagePlus rimp = WindowManager.getImage(redirectTarget);
+		if (rimp==null)
+			rimp = redirectImage;
+		if (rimp==null) {
 			IJ.error("Analyzer", "Redirect image (\""+redirectTitle+"\")\n"
 				+ "not found.");
 			redirectTarget = 0;
 			Macro.abort();
 			return null;
 		}
-		if (rImp.getWidth()!=currentImage.getWidth() || rImp.getHeight()!=currentImage.getHeight()) {
+		if (rimp.getWidth()!=cimp.getWidth() || rimp.getHeight()!=cimp.getHeight()) {
 			IJ.error("Analyzer", "Redirect image (\""+redirectTitle+"\") \n"
 				+ "is not the same size as the current image.");
 			Macro.abort();
 			return null;
 		}
-		return rImp;
+		return rimp;
 	}
 
 	ImageStatistics getRedirectStats(int measurements, Roi roi) {
-		ImagePlus redirectImp = getRedirectImage(imp);
+		ImagePlus redirectImp = getRedirectImageOrStack(imp);
 		if (redirectImp==null)
 			return null;
-		int depth = redirectImp.getStackSize();
-		if (depth>1 && depth==imp.getStackSize())
-			redirectImp.setSlice(imp.getCurrentSlice());
 		ImageProcessor ip = redirectImp.getProcessor();
 		if (imp.getTitle().equals("mask") && imp.getBitDepth()==8) {
 			ip.setMask(imp.getProcessor());
@@ -278,15 +325,16 @@ public class Analyzer implements PlugInFilter, Measurements {
 			if (index<0 || !rt.columnExists(index))
 				rt.update(measurements, imp, roi);
 		}
-		Polygon p = roi.getPolygon();
+		FloatPolygon p = roi.getFloatPolygon();
+		ImagePlus imp2 = isRedirectImage()?getRedirectImageOrStack(imp):null;
+		if (imp2==null) imp2 = imp;
 		for (int i=0; i<p.npoints; i++) {
-			ImageProcessor ip = imp.getProcessor();
-			ip.setRoi(p.xpoints[i], p.ypoints[i], 1, 1);
-			ImageStatistics stats = ImageStatistics.getStatistics(ip, measurements, imp.getCalibration());
+			ImageProcessor ip = imp2.getProcessor();
+			ip.setRoi((int)p.xpoints[i], (int)p.ypoints[i], 1, 1);
+			ImageStatistics stats = ImageStatistics.getStatistics(ip, measurements, imp2.getCalibration());
 			saveResults(stats, new PointRoi(p.xpoints[i], p.ypoints[i]));
 			if (i!=p.npoints-1) displayResults();
 		}
-		//IJ.write(rt.getCounter()+"\t"+n(cal.getX(x))+n(cal.getY(y))+n(value));
 	}
 	
 	void measureAngle(Roi roi) {
@@ -298,12 +346,16 @@ public class Analyzer implements PlugInFilter, Measurements {
 		}
 		ImageProcessor ip = imp.getProcessor();
 		ip.setRoi(roi.getPolygon());
-		ImageStatistics stats = ImageStatistics.getStatistics(ip, measurements, imp.getCalibration());
+		ImageStatistics stats = new ImageStatistics();
 		saveResults(stats, roi);
-		//IJ.write(rt.getCounter()+"\t"+n(((PolygonRoi)roi).getAngle()));
 	}
 	
 	void measureLength(Roi roi) {
+		ImagePlus imp2 = isRedirectImage()?getRedirectImageOrStack(imp):null;
+		if (imp2!=null)
+			imp2.setRoi(roi);
+		else
+			imp2 = imp;
 		if (rt.getCounter()>0) {
 			if (!IJ.isResultsWindow()) reset();
 			boolean update = false;
@@ -313,20 +365,25 @@ public class Analyzer implements PlugInFilter, Measurements {
 				index = rt.getColumnIndex("Angle");
 				if (index<0 || !rt.columnExists(index)) update=true;
 			}
-			if (update) rt.update(measurements, imp, roi);
+			if (update) rt.update(measurements, imp2, roi);
 		}
 		boolean straightLine = roi.getType()==Roi.LINE;
-		int lineWidth = Line.getWidth();
+		int lineWidth = (int)Math.round(roi.getStrokeWidth());
 		ImageProcessor ip2;
 		Rectangle saveR = null;
 		if (straightLine && lineWidth>1) {
-			ip2 = imp.getProcessor();
+			ip2 = imp2.getProcessor();
 			saveR = ip2.getRoi();
 			ip2.setRoi(roi.getPolygon());
-		} else if (lineWidth>1)
-			ip2 = (new Straightener()).straightenLine(imp, lineWidth);
-		else {
-			ProfilePlot profile = new ProfilePlot(imp);
+		} else if (lineWidth>1) {
+			if ((measurements&AREA)!=0 || (measurements&MEAN)!=0)
+				ip2 = (new Straightener()).straightenLine(imp2, lineWidth);
+			else {
+				saveResults(new ImageStatistics(), roi);
+				return;
+			}
+		} else {
+			ProfilePlot profile = new ProfilePlot(imp2);
 			double[] values = profile.getProfile();
 			if (values==null) return;
 			ip2 = new FloatProcessor(values.length, 1, values);
@@ -336,7 +393,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 					ip2.setRoi(0, 0, ip2.getWidth()-1, 1);
 			}
 		}
-		ImageStatistics stats = ImageStatistics.getStatistics(ip2, AREA+MEAN+STD_DEV+MODE+MIN_MAX, null);
+		ImageStatistics stats = ImageStatistics.getStatistics(ip2, AREA+MEAN+STD_DEV+MODE+MIN_MAX, imp2.getCalibration());
 		if (saveR!=null) ip2.setRoi(saveR);
 		saveResults(stats, roi);
 	}
@@ -376,7 +433,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 			if (roi!=null)
 				perimeter = roi.getLength();
 			else
-				perimeter = 0.0;
+				perimeter = imp!=null?imp.getWidth()*2+imp.getHeight()*2:0.0;
 			if ((measurements&PERIMETER)!=0) 
 				rt.addValue(ResultsTable.PERIMETER,perimeter);
 			if ((measurements&SHAPE_DESCRIPTORS)!=0) {
@@ -384,12 +441,11 @@ public class Analyzer implements PlugInFilter, Measurements {
 				if (circularity>1.0) circularity = 1.0;
 				rt.addValue(ResultsTable.CIRCULARITY, circularity);
 				Polygon ch = null;
-				boolean isArea = roi!=null && roi.isArea();
-				if (isArea)
-					ch = roi.getConvexHull();
+				boolean isArea = roi==null || roi.isArea();
+				double convexArea = roi!=null?getArea(roi.getConvexHull()):stats.pixelCount;
 				rt.addValue(ResultsTable.ASPECT_RATIO, isArea?stats.major/stats.minor:0.0);
 				rt.addValue(ResultsTable.ROUNDNESS, isArea?4.0*stats.area/(Math.PI*stats.major*stats.major):0.0);
-				rt.addValue(ResultsTable.SOLIDITY, ch!=null?stats.pixelCount/getArea(ch):Double.NaN);
+				rt.addValue(ResultsTable.SOLIDITY, isArea?stats.pixelCount/convexArea:Double.NaN);
 				//rt.addValue(ResultsTable.CONVEXITY, getConvexPerimeter(roi, ch)/perimeter);
 			}
 		}
@@ -416,8 +472,11 @@ public class Analyzer implements PlugInFilter, Measurements {
 			boolean extras = true;
 			double FeretDiameter=Double.NaN, feretAngle=Double.NaN, minFeret=Double.NaN,
 				feretX=Double.NaN, feretY=Double.NaN;
-			if (roi!=null) {
-				double[] a = roi.getFeretValues();
+			Roi roi2 = roi;
+			if (roi2==null && imp!=null)
+				roi2 = new Roi(0, 0, imp.getWidth(), imp.getHeight());
+			if (roi2!=null) {
+				double[] a = roi2.getFeretValues();
 				if (a!=null) {
 					FeretDiameter = a[0];
 					feretAngle = a[1];
@@ -432,8 +491,10 @@ public class Analyzer implements PlugInFilter, Measurements {
 			rt.addValue(ResultsTable.FERET_ANGLE, feretAngle);
 			rt.addValue(ResultsTable.MIN_FERET, minFeret);
 		}
-		if ((measurements&INTEGRATED_DENSITY)!=0)
+		if ((measurements&INTEGRATED_DENSITY)!=0) {
 			rt.addValue(ResultsTable.INTEGRATED_DENSITY,stats.area*stats.mean);
+			rt.addValue(ResultsTable.RAW_INTEGRATED_DENSITY,stats.pixelCount*stats.umean);
+		}
 		if ((measurements&MEDIAN)!=0) rt.addValue(ResultsTable.MEDIAN, stats.median);
 		if ((measurements&SKEWNESS)!=0) rt.addValue(ResultsTable.SKEWNESS, stats.skewness);
 		if ((measurements&KURTOSIS)!=0) rt.addValue(ResultsTable.KURTOSIS, stats.kurtosis);
@@ -441,20 +502,21 @@ public class Analyzer implements PlugInFilter, Measurements {
 		if ((measurements&STACK_POSITION)!=0) {
 			boolean update = false;
 			if (imp!=null && (imp.isHyperStack()||imp.isComposite())) {
+				int[] position = imp.convertIndexToPosition(imp.getCurrentSlice());
 				if (imp.getNChannels()>1) {
 					int index = rt.getColumnIndex("Ch");
 					if (index<0 || !rt.columnExists(index)) update=true;
-					rt.addValue("Ch", imp.getChannel());
+					rt.addValue("Ch", position[0]);
 				}
 				if (imp.getNSlices()>1) {
 					int index = rt.getColumnIndex("Slice");
 					if (index<0 || !rt.columnExists(index)) update=true;
-					rt.addValue("Slice", imp.getSlice());
+					rt.addValue("Slice", position[1]);
 				}
 				if (imp.getNFrames()>1) {
 					int index = rt.getColumnIndex("Frame");
 					if (index<0 || !rt.columnExists(index)) update=true;
-					rt.addValue("Frame", imp.getFrame());
+					rt.addValue("Frame", position[2]);
 				}
 			} else {
 				int index = rt.getColumnIndex("Slice");
@@ -472,14 +534,17 @@ public class Analyzer implements PlugInFilter, Measurements {
 					angle = roi.getAngle(l.x1, l.y1, l.x2, l.y2);
 					rt.addValue("Angle", angle);
 				}
-			} else if (roi.getType()==Roi.ANGLE)
-				rt.addValue("Angle", ((PolygonRoi)roi).getAngle());
-			else if (roi.getType()==Roi.POINT)
+			} else if (roi.getType()==Roi.ANGLE) {
+				double angle = ((PolygonRoi)roi).getAngle();
+				if (Prefs.reflexAngle) angle = 360.0-angle;
+				rt.addValue("Angle", angle);
+			} else if (roi.getType()==Roi.POINT)
 				savePoints(roi);
 		}
 	}
 		
 	final double getArea(Polygon p) {
+		if (p==null) return Double.NaN;
 		int carea = 0;
 		int iminus1;
 		for (int i=0; i<p.npoints; i++) {
@@ -520,22 +585,24 @@ public class Analyzer implements PlugInFilter, Measurements {
 		if (imp==null) {
 			rt.addValue("X", 0.0);
 			rt.addValue("Y", 0.0);
-			rt.addValue("Slice", 0.0);
+			if (imp.getStackSize()>1)
+				rt.addValue("Slice", 0.0);
 			return;
 		}
 		if ((measurements&AREA)!=0)
 			rt.addValue(ResultsTable.AREA,0);
-		Polygon p = roi.getPolygon();
+		FloatPolygon p = roi.getFloatPolygon();
 		ImageProcessor ip = imp.getProcessor();
 		Calibration cal = imp.getCalibration();
-		int x = p.xpoints[0];
-		int y = p.ypoints[0];
-		double value = ip.getPixelValue(x,y);
-		if (markWidth>0) {
+		double x = p.xpoints[0];
+		double y = p.ypoints[0];
+		int ix=(int)x, iy=(int)y;
+		double value = ip.getPixelValue(ix,iy);
+		if (markWidth>0 && !Toolbar.getMultiPointMode()) {
 			ip.setColor(Toolbar.getForegroundColor());
 			ip.setLineWidth(markWidth);
-			ip.moveTo(x,y);
-			ip.lineTo(x,y);
+			ip.moveTo(ix,iy);
+			ip.lineTo(ix,iy);
 			imp.updateAndDraw();
 			ip.setLineWidth(Line.getWidth());
 		}
@@ -548,7 +615,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 				rt.addValue("Slice", imp.getSlice());
 			if (imp.getNFrames()>1)
 				rt.addValue("Frame", imp.getFrame());
-		} else
+		} else if (imp.getStackSize()>1)
 			rt.addValue("Slice", cal.getZ(imp.getCurrentSlice()));
 		if (imp.getProperty("FHT")!=null) {
 			double center = imp.getWidth()/2.0;
@@ -561,8 +628,8 @@ public class Analyzer implements PlugInFilter, Measurements {
 			rt.addValue("R", (imp.getWidth()/r)*cal.pixelWidth);
 			rt.addValue("Theta", theta);
 		}
-		if ((measurements&MEAN)==0)
-			rt.addValue("Mean", value);
+		//if ((measurements&MEAN)==0)
+		//	rt.addValue("Mean", value);
 	}
 
 	String getFileName() {
@@ -570,6 +637,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 		if (imp!=null) {
 			if (redirectTarget!=0) {
 				ImagePlus rImp = WindowManager.getImage(redirectTarget);
+				if (rImp==null) rImp = redirectImage;
 				if (rImp!=null) s = rImp.getTitle();				
 			} else
 				s = imp.getTitle();
@@ -602,35 +670,18 @@ public class Analyzer implements PlugInFilter, Measurements {
 		IJ.write(rt.getRowAsString(counter-1));
 	}
 
-	/** Updates the displayed column headings. Does nothing if
-	    the results table headings and the displayed headings are
-        the same. Redisplays the results if the headings are
-        different and the results table is not empty. */
+	/** Redisplays the results table. */
 	public void updateHeadings() {
-		TextPanel tp = IJ.getTextPanel();
-		if (tp==null)
-			return;
-		String worksheetHeadings = tp.getColumnHeadings();		
-		String tableHeadings = rt.getColumnHeadings();		
-		if (worksheetHeadings.equals(tableHeadings))
-			return;
-		IJ.setColumnHeadings(tableHeadings);
-		int n = rt.getCounter();
-		if (n>0) {
-			StringBuffer sb = new StringBuffer(n*tableHeadings.length());
-			for (int i=0; i<n; i++)
-				sb.append(rt.getRowAsString(i)+"\n");
-			tp.append(new String(sb));
-		}
+		rt.show("Results");
 	}
 
 	/** Converts a number to a formatted string with a tab at the end. */
 	public String n(double n) {
 		String s;
 		if (Math.round(n)==n)
-			s = IJ.d2s(n,0);
+			s = ResultsTable.d2s(n,0);
 		else
-			s = IJ.d2s(n,precision);
+			s = ResultsTable.d2s(n,precision);
 		return s+"\t";
 	}
 		
@@ -663,9 +714,11 @@ public class Analyzer implements PlugInFilter, Measurements {
 			sd.append("\t");
 		}
 		summarizeAreas();
-		int index = rt.getColumnIndex("Angle");
-		if (rt.columnExists(index)) add2(index);
-		index = rt.getColumnIndex("Length");
+		if ((measurements&ELLIPSE)==0) {
+			int index = rt.getColumnIndex("Angle");
+			if (rt.columnExists(index)) add2(index);
+		}
+		int index = rt.getColumnIndex("Length");
 		if (rt.columnExists(index)) add2(index);
 		TextPanel tp = IJ.getTextPanel();
 		if (tp!=null) {
@@ -673,11 +726,23 @@ public class Analyzer implements PlugInFilter, Measurements {
 			if (worksheetHeadings.equals(""))
 				IJ.setColumnHeadings(rt.getColumnHeadings());
 		}		
-		IJ.write("");		
-		IJ.write(new String(mean));		
-		IJ.write(new String(sd));		
-		IJ.write(new String(min));		
-		IJ.write(new String(max));
+		IJ.write("");	
+		String meanS = new String(mean);		
+		String sdS = new String(sd);		
+		String minS = new String(min);		
+		String maxS =new String(max);
+		if (meanS.endsWith("\t"))
+			meanS = meanS.substring(0, meanS.length()-1);
+		if (sdS.endsWith("\t"))
+			sdS = sdS.substring(0, sdS.length()-1);
+		if (minS.endsWith("\t"))
+			minS = minS.substring(0, minS.length()-1);
+		if (maxS.endsWith("\t"))
+			maxS = maxS.substring(0, maxS.length()-1);
+		IJ.write(meanS);		
+		IJ.write(sdS);		
+		IJ.write(minS);		
+		IJ.write(maxS);
 		IJ.write("");		
 		mean = null;		
 		sd = null;		
@@ -720,8 +785,10 @@ public class Analyzer implements PlugInFilter, Measurements {
 			add2(ResultsTable.CIRCULARITY);
 		if ((measurements&FERET)!=0)
 			add2(ResultsTable.FERET);
-		if ((measurements&INTEGRATED_DENSITY)!=0)
+		if ((measurements&INTEGRATED_DENSITY)!=0) {
 			add2(ResultsTable.INTEGRATED_DENSITY);
+			add2(ResultsTable.RAW_INTEGRATED_DENSITY);
+		}
 		if ((measurements&MEDIAN)!=0)
 			add2(ResultsTable.MEDIAN);
 		if ((measurements&SKEWNESS)!=0)
@@ -731,9 +798,12 @@ public class Analyzer implements PlugInFilter, Measurements {
 		if ((measurements&AREA_FRACTION)!=0)
 			add2(ResultsTable.AREA_FRACTION);
 		if ((measurements&STACK_POSITION)!=0) {
-			add2(ResultsTable.CHANNEL);
-			add2(ResultsTable.SLICE);
-			add2(ResultsTable.FRAME);
+			int index = rt.getColumnIndex("Ch");
+			if (rt.columnExists(index)) add2(index);
+			index = rt.getColumnIndex("Slice");
+			if (rt.columnExists(index)) add2(index);
+			index = rt.getColumnIndex("Frame");
+			if (rt.columnExists(index)) add2(index);
 		}
 		if ((measurements&FERET)!=0) {
 			add2(ResultsTable.FERET_X);
@@ -880,6 +950,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 	}
 	
 	public static void setResultsTable(ResultsTable rt) {
+		if (rt==null) rt = new ResultsTable();
 		systemRT = rt;
 	}
 
