@@ -67,6 +67,18 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 	/** Display image containing binary masks of measured particles. */
 	public static final int SHOW_MASKS = 4096;
 
+	/** Use 4-connected particle tracing. */
+	public static final int FOUR_CONNECTED = 8192;
+
+	/** Replace original image with masks. */
+	public static final int IN_SITU_SHOW = 16384;
+
+	/** Display particle outlines as an overlay. */
+	public static final int SHOW_OVERLAY_OUTLINES = 32768;
+	
+	/** Display filled particle as an overlay. */
+	public static final int SHOW_OVERLAY_MASKS = 65536;
+
 	static final String OPTIONS = "ap.options";
 	
 	static final int BYTE=0, SHORT=1, FLOAT=2, RGB=3;
@@ -77,24 +89,27 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 	private static double staticMaxSize = DEFAULT_MAX_SIZE;
 	private static boolean pixelUnits;
 	private static int staticOptions = Prefs.getInt(OPTIONS,CLEAR_WORKSHEET);
-	private static String[] showStrings = {"Nothing", "Outlines", "Masks", "Ellipses", "Count Masks"};
-	private static double minCircularity=0.0, maxCircularity=1.0;
+	private static String[] showStrings = {"Nothing", "Outlines", "Bare Outlines", "Ellipses", "Masks", "Count Masks", "Overlay Outlines", "Overlay Masks"};
+	private static double staticMinCircularity=0.0, staticMaxCircularity=1.0;
 	private static String prevHdr;
 		
-	protected static final int NOTHING=0,OUTLINES=1,MASKS=2,ELLIPSES=3,ROI_MASKS=4;
-	protected static int showChoice;
+	protected static final int NOTHING=0, OUTLINES=1, BARE_OUTLINES=2, ELLIPSES=3, MASKS=4, ROI_MASKS=5,
+		OVERLAY_OUTLINES=6, OVERLAY_MASKS=7;
+	protected static int staticShowChoice;
 	protected ImagePlus imp;
 	protected ResultsTable rt;
 	protected Analyzer analyzer;
 	protected int slice;
 	protected boolean processStack;
 	protected boolean showResults,excludeEdgeParticles,showSizeDistribution,
-		resetCounter,showProgress, recordStarts, displaySummary, floodFill, addToManager;
+		resetCounter,showProgress, recordStarts, displaySummary, floodFill,
+		addToManager, inSituShow;
 		
-	private String summaryHdr = "Slice\tCount\tTotal Area\tAverage Size\tArea Fraction";
+	private String summaryHdr = "Slice\tCount\tTotal Area\tAverage Size\t%Area";
 	private double level1, level2;
-	private double minSize;
-	private double maxSize;
+	private double minSize, maxSize;
+	private double minCircularity, maxCircularity;
+	private int showChoice;
 	private int options;
 	private int measurements;
 	private Calibration calibration;
@@ -128,8 +143,16 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 	private ImagePlus outputImage;
 	private boolean hideOutputImage;
 	private int roiType;
-		
-	
+	private int wandMode = Wand.LEGACY_MODE;
+	private Overlay overlay;
+	boolean blackBackground;
+	private static int defaultFontSize = 9;
+	private static int nextFontSize = defaultFontSize;
+	private static int nextLineWidth = 1;
+	private int fontSize = nextFontSize;
+	private int lineWidth = nextLineWidth;
+
+			
 	/** Constructs a ParticleAnalyzer.
 		@param options	a flag word created by Oring SHOW_RESULTS, EXCLUDE_EDGE_PARTICLES, etc.
 		@param measurements a flag word created by ORing constants defined in the Measurements interface
@@ -152,12 +175,21 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		slice = 1;
 		if ((options&SHOW_ROI_MASKS)!=0)
 			showChoice = ROI_MASKS;
+		if ((options&SHOW_OVERLAY_OUTLINES)!=0)
+			showChoice = OVERLAY_OUTLINES;
+		if ((options&SHOW_OVERLAY_MASKS)!=0)
+			showChoice = OVERLAY_MASKS;
 		if ((options&SHOW_OUTLINES)!=0)
 			showChoice = OUTLINES;
 		if ((options&SHOW_MASKS)!=0)
 			showChoice = MASKS;
 		if ((options&SHOW_NONE)!=0)
 			showChoice = NOTHING;
+		if ((options&FOUR_CONNECTED)!=0) {
+			wandMode = Wand.FOUR_CONNECTED;
+			options |= INCLUDE_HOLES;
+		}
+		nextFontSize=defaultFontSize; nextLineWidth=1;
 	}
 	
 	/** Constructs a ParticleAnalyzer using the default min and max circularity values (0 and 1). */
@@ -176,9 +208,15 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		IJ.register(ParticleAnalyzer.class);
 		if (imp==null)
 			{IJ.noImage();return DONE;}
+		if (imp.getBitDepth()==24 && !isThresholdedRGB(imp)) {
+			IJ.error("Particle Analyzer",
+			"RGB images must be thresholded using\n"
+			+"Image>Adjust>Color Threshold.");
+			return DONE;
+		}
 		if (!showDialog())
 			return DONE;
-		int baseFlags = DOES_8G+DOES_16+DOES_32+NO_CHANGES+NO_UNDO;
+		int baseFlags = DOES_ALL+NO_CHANGES+NO_UNDO;
 		int flags = IJ.setupDialog(imp, baseFlags);
 		processStack = (flags&DOES_STACKS)!=0;
 		slice = 0;
@@ -186,6 +224,7 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		if (saveRoi!=null && saveRoi.getType()!=Roi.RECTANGLE && saveRoi.isArea())
 			polygon = saveRoi.getPolygon();
 		imp.startTiming();
+		nextFontSize=defaultFontSize; nextLineWidth=1;
 		return flags;
 	}
 
@@ -195,6 +234,10 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		slice++;
 		if (imp.getStackSize()>1 && processStack)
 			imp.setSlice(slice);
+		if (imp.getType()==ImagePlus.COLOR_RGB) {
+			ip = (ImageProcessor)imp.getProperty("Mask");
+			ip.setThreshold(255, 255, ImageProcessor.NO_LUT_UPDATE);
+		}		
 		if (!analyze(imp, ip))
 			canceled = true;
 		if (slice==imp.getStackSize()) {
@@ -212,10 +255,16 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		if (Macro.getOptions()!=null) {
 			boolean oldMacro = updateMacroOptions();
 			if (oldMacro) unitSquared = 1.0;
+			staticMinSize = 0.0; staticMaxSize = DEFAULT_MAX_SIZE;
+			staticMinCircularity=0.0; staticMaxCircularity=1.0;
+			staticShowChoice = NOTHING;
 		}
 		GenericDialog gd = new GenericDialog("Analyze Particles");
 		minSize = staticMinSize;
 		maxSize = staticMaxSize;
+		minCircularity = staticMinCircularity;
+		maxCircularity = staticMaxCircularity;
+		showChoice = staticShowChoice;
 		if (maxSize==999999) maxSize = DEFAULT_MAX_SIZE;
 		options = staticOptions;
 		String unit = cal.getUnit();
@@ -232,17 +281,17 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		if ((int)cmin!=cmin) places = 2;
 		double cmax = maxSize*unitSquared;
 		if ((int)cmax!=cmax && cmax!=DEFAULT_MAX_SIZE) places = 2;
-		String minStr = IJ.d2s(cmin,places);
+		String minStr = ResultsTable.d2s(cmin,places);
 		if (minStr.indexOf("-")!=-1) {
 			for (int i=places; i<=6; i++) {
-				minStr = IJ.d2s(cmin, i);
+				minStr = ResultsTable.d2s(cmin, i);
 				if (minStr.indexOf("-")==-1) break;
 			}
 		}
-		String maxStr = IJ.d2s(cmax, places);
+		String maxStr = ResultsTable.d2s(cmax, places);
 		if (maxStr.indexOf("-")!=-1) {
 			for (int i=places; i<=6; i++) {
-				maxStr = IJ.d2s(cmax, i);
+				maxStr = ResultsTable.d2s(cmax, i);
 				if (maxStr.indexOf("-")==-1) break;
 			}
 		}
@@ -251,21 +300,22 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		gd.addStringField("Size ("+units+"):", minStr+"-"+maxStr, 12);
 		if (scaled) {
 			gd.setInsets(0, 40, 5);
-			gd.addCheckbox("Pixel Units", pixelUnits);
+			gd.addCheckbox("Pixel units", pixelUnits);
 		}
 		gd.addStringField("Circularity:", IJ.d2s(minCircularity)+"-"+IJ.d2s(maxCircularity), 12);
 		gd.addChoice("Show:", showStrings, showStrings[showChoice]);
-		String[] labels = new String[7];
-		boolean[] states = new boolean[7];
-		labels[0]="Display Results"; states[0] = (options&SHOW_RESULTS)!=0;
-		labels[1]="Exclude on Edges"; states[1]=(options&EXCLUDE_EDGE_PARTICLES)!=0;
-		labels[2]="Clear Results"; states[2]=(options&CLEAR_WORKSHEET)!=0;
-		labels[3]="Include Holes"; states[3]=(options&INCLUDE_HOLES)!=0;
+		String[] labels = new String[8];
+		boolean[] states = new boolean[8];
+		labels[0]="Display results"; states[0] = (options&SHOW_RESULTS)!=0;
+		labels[1]="Exclude on edges"; states[1]=(options&EXCLUDE_EDGE_PARTICLES)!=0;
+		labels[2]="Clear results"; states[2]=(options&CLEAR_WORKSHEET)!=0;
+		labels[3]="Include holes"; states[3]=(options&INCLUDE_HOLES)!=0;
 		labels[4]="Summarize"; states[4]=(options&DISPLAY_SUMMARY)!=0;
-		labels[5]="Record Starts"; states[5]=(options&RECORD_STARTS)!=0;
+		labels[5]="Record starts"; states[5]=(options&RECORD_STARTS)!=0;
 		labels[6]="Add to Manager"; states[6]=(options&ADD_TO_MANAGER)!=0;
+		labels[7]="In_situ Show"; states[7]=(options&IN_SITU_SHOW)!=0;
 		gd.addCheckboxGroup(4, 2, labels, states);
-
+		gd.addHelp(IJ.URL+"/docs/menus/analyze.html#ap");
 		gd.showDialog();
 		if (gd.wasCanceled())
 			return false;
@@ -278,8 +328,8 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		else
 			unitSquared = cal.pixelWidth*cal.pixelHeight;
 		String[] minAndMax = Tools.split(size, " -");
-		double mins = Tools.parseDouble(minAndMax[0]);
-		double maxs = minAndMax.length==2?Tools.parseDouble(minAndMax[1]):Double.NaN;
+		double mins = gd.parseDouble(minAndMax[0]);
+		double maxs = minAndMax.length==2?gd.parseDouble(minAndMax[1]):Double.NaN;
 		minSize = Double.isNaN(mins)?DEFAULT_MIN_SIZE:mins/unitSquared;
 		maxSize = Double.isNaN(maxs)?DEFAULT_MAX_SIZE:maxs/unitSquared;
 		if (minSize<DEFAULT_MIN_SIZE) minSize = DEFAULT_MIN_SIZE;
@@ -288,13 +338,15 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		staticMaxSize = maxSize;
 		
 		minAndMax = Tools.split(gd.getNextString(), " -"); // min-max circularity
-		double minc = Tools.parseDouble(minAndMax[0]);
-		double maxc = minAndMax.length==2?Tools.parseDouble(minAndMax[1]):Double.NaN;
+		double minc = gd.parseDouble(minAndMax[0]);
+		double maxc = minAndMax.length==2?gd.parseDouble(minAndMax[1]):Double.NaN;
 		minCircularity = Double.isNaN(minc)?0.0:minc;
 		maxCircularity = Double.isNaN(maxc)?1.0:maxc;
 		if (minCircularity<0.0 || minCircularity>1.0) minCircularity = 0.0;
 		if (maxCircularity<minCircularity || maxCircularity>1.0) maxCircularity = 1.0;
 		if (minCircularity==1.0 && maxCircularity==1.0) minCircularity = 0.0;
+		staticMinCircularity = minCircularity;
+		staticMaxCircularity = maxCircularity;
 		
 		if (gd.invalidNumber()) {
 			IJ.error("Bins invalid.");
@@ -302,6 +354,7 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 			return false;
 		}
 		showChoice = gd.getNextChoiceIndex();
+		staticShowChoice = showChoice;
 		if (gd.getNextBoolean())
 			options |= SHOW_RESULTS; else options &= ~SHOW_RESULTS;
 		if (gd.getNextBoolean())
@@ -316,6 +369,8 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 			options |= RECORD_STARTS; else options &= ~RECORD_STARTS;
 		if (gd.getNextBoolean())
 			options |= ADD_TO_MANAGER; else options &= ~ADD_TO_MANAGER;
+		if (gd.getNextBoolean())
+			options |= IN_SITU_SHOW; else options &= ~IN_SITU_SHOW;
 		staticOptions = options;
 		options |= SHOW_PROGRESS;
 		if ((options&DISPLAY_SUMMARY)!=0)
@@ -323,6 +378,14 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		return true;
 	}
 	
+	private boolean isThresholdedRGB(ImagePlus imp) {
+		Object obj = imp.getProperty("Mask");
+		if (obj==null || !(obj instanceof ImageProcessor))
+			return false;
+		ImageProcessor mask = (ImageProcessor)obj;
+		return mask.getWidth()==imp.getWidth() && mask.getHeight()==imp.getHeight();
+	}
+
 	boolean updateMacroOptions() {
 		String options = Macro.getOptions();
 		int index = options.indexOf("maximum=");
@@ -357,6 +420,7 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		recordStarts = (options&RECORD_STARTS)!=0;
 		addToManager = (options&ADD_TO_MANAGER)!=0;
 		displaySummary = (options&DISPLAY_SUMMARY)!=0;
+		inSituShow = (options&IN_SITU_SHOW)!=0;
 		outputImage = null;
 		ip.snapshot();
 		ip.setProgressBar(null);
@@ -369,36 +433,51 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 				redirectIP = redirectStack.getProcessor(imp.getCurrentSlice());
 			} else
 				redirectIP = redirectImp.getProcessor();
+		} else if (imp.getType()==ImagePlus.COLOR_RGB) {
+			ImagePlus original = (ImagePlus)imp.getProperty("OriginalImage");
+			if (original!=null && original.getWidth()==imp.getWidth() && original.getHeight()==imp.getHeight()) {
+				redirectImp = original;
+				redirectIP = original.getProcessor();
+			}
 		}
 		if (!setThresholdLevels(imp, ip))
 			return false;
 		width = ip.getWidth();
 		height = ip.getHeight();
-		if (showChoice!=NOTHING) {
+		if (!(showChoice==NOTHING||showChoice==OVERLAY_OUTLINES||showChoice==OVERLAY_MASKS)) {
+			blackBackground = Prefs.blackBackground && inSituShow;
 			if (slice==1)
 				outlines = new ImageStack(width, height);
 			if (showChoice==ROI_MASKS)
 				drawIP = new ShortProcessor(width, height);
 			else
 				drawIP = new ByteProcessor(width, height);
+			drawIP.setLineWidth(lineWidth);
 			if (showChoice==ROI_MASKS)
 				{} // Place holder for now...
-			else if (showChoice==MASKS)
+			else if (showChoice==MASKS&&!blackBackground)
 				drawIP.invertLut();
 			else if (showChoice==OUTLINES) {
-				if (customLut==null)
-					makeCustomLut();
-				drawIP.setColorModel(customLut);
-				drawIP.setFont(new Font("SansSerif", Font.PLAIN, 9));
-			}
+				if (!inSituShow) {
+					if (customLut==null)
+						makeCustomLut();
+					drawIP.setColorModel(customLut);
+				}
+				drawIP.setFont(new Font("SansSerif", Font.PLAIN, fontSize));
+				if (fontSize>12 && inSituShow)
+					drawIP.setAntialiasedText(true);
+			} 
 			outlines.addSlice(null, drawIP);
 
-			if (showChoice==ROI_MASKS)
+			if (showChoice==ROI_MASKS || blackBackground) {
 				drawIP.setColor(Color.black);
-			else
+				drawIP.fill();
 				drawIP.setColor(Color.white);
-			drawIP.fill();
-			drawIP.setColor(Color.black);
+			} else {
+				drawIP.setColor(Color.white);
+				drawIP.fill();
+				drawIP.setColor(Color.black);
+			}
 		}
 		calibration = redirectImp!=null?redirectImp.getCalibration():imp.getCalibration();
 		
@@ -463,7 +542,7 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 				else
 					value = ip.getPixelValue(x, y);
 				if (value>=level1 && value<=level2)
-					analyzeParticle(x,y,imp,ip);
+					analyzeParticle(x, y, imp, ip);
 			}
 			if (showProgress && ((y%inc)==0))
 				IJ.showProgress((double)(y-r.y)/r.height);
@@ -483,10 +562,8 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		ip.reset();
 		if (displaySummary && IJ.getInstance()!=null)
 			updateSliceSummary();
-		if (addToManager && roiManager!=null) {
-			ImageCanvas ic = imp.getCanvas();
-			if (ic!=null) ic.setShowAllROIs(true);
-		}
+		if (addToManager && roiManager!=null)
+			roiManager.setEditMode(imp, true);
 		maxParticleCount = (particleCount > maxParticleCount) ? particleCount : maxParticleCount;
 		totalCount += particleCount;
 		if (!canceled)
@@ -497,13 +574,14 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 	void updateSliceSummary() {
 		int slices = imp.getStackSize();
 		float[] areas = rt.getColumn(ResultsTable.AREA);
+		if (areas==null)
+			areas = new float[0];
 		String label = imp.getTitle();
 		if (slices>1) {
 			label = imp.getStack().getShortSliceLabel(slice);
 			label = label!=null&&!label.equals("")?label:""+slice;
 		}
 		String aLine = null;
-		if (areas==null) return;
 		double sum = 0.0;
 		int start = areas.length-particleCount;
 		if (start<0)
@@ -512,11 +590,11 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 			sum += areas[i];
 		int places = Analyzer.getPrecision();
 		Calibration cal = imp.getCalibration();
-		String total = "\t"+IJ.d2s(sum,places);
-		String average = "\t"+IJ.d2s(sum/particleCount,places);
-		String fraction = "\t"+IJ.d2s(sum*100.0/totalArea,1);
+		String total = "\t"+ResultsTable.d2s(sum,places);
+		String average = "\t"+ResultsTable.d2s(sum/particleCount,places);
+		String fraction = "\t"+ResultsTable.d2s(sum*100.0/totalArea,places);
 		aLine = label+"\t"+particleCount+total+average+fraction;
-		aLine = addMeans(aLine, start);
+		aLine = addMeans(aLine, areas.length>0?start:-1);
 		if (slices==1) {
 			Frame frame = WindowManager.getFrame("Summary");
 			if (frame!=null && (frame instanceof TextWindow) && summaryHdr.equals(prevHdr))
@@ -563,28 +641,33 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 	}
 
 	private String addMean(int column, String line, int start) {
-		float[] c = column>=0?rt.getColumn(column):null;
-		if (c!=null) {
-			ImageProcessor ip = new FloatProcessor(c.length, 1, c, null);
-			if (ip==null) return line;
-			ip.setRoi(start, 0, ip.getWidth()-start, 1);
-			ip = ip.crop();
-			ImageStatistics stats = new FloatStatistics(ip);
-			if (stats==null)
-				return line;
-			line += n(stats.mean);
-		} else
-			line += "-\t";
-		summaryHdr += "\t"+rt.getColumnHeading(column);
+		if (start==-1) {
+			line += "\tNaN";
+			summaryHdr += "\t"+ResultsTable.getDefaultHeading(column);
+		} else {
+			float[] c = column>=0?rt.getColumn(column):null;
+			if (c!=null) {
+				ImageProcessor ip = new FloatProcessor(c.length, 1, c, null);
+				if (ip==null) return line;
+				ip.setRoi(start, 0, ip.getWidth()-start, 1);
+				ip = ip.crop();
+				ImageStatistics stats = new FloatStatistics(ip);
+				if (stats==null)
+					return line;
+				line += n(stats.mean);
+			} else
+				line += "\tNaN";
+			summaryHdr += "\t"+rt.getColumnHeading(column);
+		}
 		return line;
 	}
 
 	String n(double n) {
 		String s;
 		if (Math.round(n)==n)
-			s = IJ.d2s(n,0);
+			s = ResultsTable.d2s(n,0);
 		else
-			s = IJ.d2s(n, Analyzer.getPrecision());
+			s = ResultsTable.d2s(n, Analyzer.getPrecision());
 		return "\t"+s;
 	}
 
@@ -653,7 +736,10 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 				canceled = true;
 				return false;
 			}
-			if (invertedLut) {
+			boolean threshold255 = invertedLut;
+			if (Prefs.blackBackground)
+				threshold255 = !threshold255;
+			if (threshold255) {
 				level1 = 255;
 				level2 = 255;
 				fillColor = 64;
@@ -699,7 +785,7 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 	void analyzeParticle(int x, int y, ImagePlus imp, ImageProcessor ip) {
 		//Wand wand = new Wand(ip);
 		ImageProcessor ip2 = redirectIP!=null?redirectIP:ip;
-		wand.autoOutline(x,y, level1, level2);
+		wand.autoOutline(x, y, level1, level2, wandMode);
 		if (wand.npoints==0)
 			{IJ.log("wand error: "+x+" "+y); return;}
 		Roi roi = new PolygonRoi(wand.xpoints, wand.ypoints, wand.npoints, roiType);
@@ -745,6 +831,7 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 			particleCount++;
 			if (roiNeedsImage)
 				roi.setImage(imp);
+			stats.xstart=x; stats.ystart=y;
 			saveResults(stats, roi);
 			if (showChoice!=NOTHING)
 				drawParticle(drawIP, roi, stats, mask);
@@ -774,12 +861,8 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 	protected void saveResults(ImageStatistics stats, Roi roi) {
 		analyzer.saveResults(stats, roi);
 		if (recordStarts) {
-			int coordinates = ((PolygonRoi)roi).getNCoordinates();
-			Rectangle r = roi.getBounds();
-			int x = r.x+((PolygonRoi)roi).getXCoordinates()[coordinates-1];
-			int y = r.y+((PolygonRoi)roi).getYCoordinates()[coordinates-1];
-			rt.addValue("XStart", x);
-			rt.addValue("YStart", y);
+			rt.addValue("XStart", stats.xstart);
+			rt.addValue("YStart", stats.ystart);
 		}
 		if (addToManager) {
 			if (roiManager==null) {
@@ -797,6 +880,10 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 				if (resetCounter)
 					roiManager.runCommand("reset");
 			}
+			if (imp.getStackSize()>1)
+				roi.setPosition(imp.getCurrentSlice());
+			if (lineWidth!=1)
+				roi.setStrokeWidth(lineWidth);
 			roiManager.add(imp, roi, rt.getCounter());
 		}
 		if (showResults)
@@ -809,7 +896,8 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 	ImageStatistics stats, ImageProcessor mask) {
 		switch (showChoice) {
 			case MASKS: drawFilledParticle(drawIP, roi, mask); break;
-			case OUTLINES: drawOutline(drawIP, roi, rt.getCounter()); break;
+			case OUTLINES: case BARE_OUTLINES: case OVERLAY_OUTLINES: case OVERLAY_MASKS:
+				drawOutline(drawIP, roi, rt.getCounter()); break;
 			case ELLIPSES: drawEllipse(drawIP, stats, rt.getCounter()); break;
 			case ROI_MASKS: drawRoiFilledParticle(drawIP, roi, mask, rt.getCounter()); break;
 			default:
@@ -823,20 +911,39 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 	}
 
 	void drawOutline(ImageProcessor ip, Roi roi, int count) {
-		Rectangle r = roi.getBounds();
-		int nPoints = ((PolygonRoi)roi).getNCoordinates();
-		int[] xp = ((PolygonRoi)roi).getXCoordinates();
-		int[] yp = ((PolygonRoi)roi).getYCoordinates();
-		int x=r.x, y=r.y;
-		ip.setValue(0.0);
-		ip.moveTo(x+xp[0], y+yp[0]);
-		for (int i=1; i<nPoints; i++)
-			ip.lineTo(x+xp[i], y+yp[i]);
-		ip.lineTo(x+xp[0], y+yp[0]);
-		String s = IJ.d2s(count,0);
-		ip.moveTo(r.x+r.width/2-ip.getStringWidth(s)/2, r.y+r.height/2+4);
-		ip.setValue(1.0);
-		ip.drawString(s);
+		if (showChoice==OVERLAY_OUTLINES || showChoice==OVERLAY_MASKS) {
+			if (overlay==null) {
+				overlay = new Overlay();
+				overlay.drawLabels(true);
+				overlay.setLabelFont(new Font("SansSerif", Font.PLAIN, fontSize));
+			}
+			Roi roi2 = (Roi)roi.clone();
+			roi2.setStrokeColor(Color.cyan);
+			if (lineWidth!=1)
+				roi2.setStrokeWidth(lineWidth);
+			if (showChoice==OVERLAY_MASKS)
+				roi2.setFillColor(Color.cyan);
+			overlay.add(roi2);
+		} else {
+			Rectangle r = roi.getBounds();
+			int nPoints = ((PolygonRoi)roi).getNCoordinates();
+			int[] xp = ((PolygonRoi)roi).getXCoordinates();
+			int[] yp = ((PolygonRoi)roi).getYCoordinates();
+			int x=r.x, y=r.y;
+			if (!inSituShow)
+				ip.setValue(0.0);
+			ip.moveTo(x+xp[0], y+yp[0]);
+			for (int i=1; i<nPoints; i++)
+				ip.lineTo(x+xp[i], y+yp[i]);
+			ip.lineTo(x+xp[0], y+yp[0]);
+			if (showChoice!=BARE_OUTLINES) {
+				String s = ResultsTable.d2s(count,0);
+				ip.moveTo(r.x+r.width/2-ip.getStringWidth(s)/2, r.y+r.height/2+fontSize/2);
+				if (!inSituShow)
+					ip.setValue(1.0);
+				ip.drawString(s);
+			}
+		}
 	}
 
 	void drawEllipse(ImageProcessor ip, ImageStatistics stats, int count) {
@@ -852,10 +959,11 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 
 	void showResults() {
 		int count = rt.getCounter();
-		if (count==0)
-			return;
+		// if (count==0) return;
 		boolean lastSlice = !processStack||slice==imp.getStackSize();
-		if (outlines!=null && lastSlice) {
+		if ((showChoice==OVERLAY_OUTLINES||showChoice==OVERLAY_MASKS) && slice==1 && count>0)
+			imp.setOverlay(overlay);
+		else if (outlines!=null && lastSlice) {
 			String title = imp!=null?imp.getTitle():"Outlines";
 			String prefix;
 			if (showChoice == MASKS)
@@ -866,9 +974,17 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 				prefix = "Drawing of ";
 			outlines.update(drawIP);
 			outputImage = new ImagePlus(prefix+title, outlines);
-			if (!hideOutputImage) outputImage.show();
+			if (inSituShow) {
+				if (imp.getStackSize()==1)
+					Undo.setup(Undo.TRANSFORM, imp);
+				imp.setStack(null, outputImage.getStack());
+			} else if (!hideOutputImage)
+				outputImage.show();
 		}
 		if (showResults && !processStack) {
+			TextPanel tp = IJ.getTextPanel();
+			if (beginningCount>0 && tp!=null && tp.getLineCount()!=count)
+				rt.show("Results");
 			Analyzer.firstParticle = beginningCount;
 			Analyzer.lastParticle = Analyzer.getCounter()-1;
 		} else
@@ -886,6 +1002,16 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		this.hideOutputImage = hideOutputImage;
 	}
 
+	/** Sets the size of the font used to label outlines in the next particle analyzer instance. */
+	public static void setFontSize(int size) {
+		nextFontSize = size;
+	}
+
+	/** Sets the outline line width for the next particle analyzer instance. */
+	public static void setLineWidth(int width) {
+		nextLineWidth = width;
+	}
+	
 	int getColumnID(String name) {
 		int id = rt.getFreeColumn(name);
 		if (id==ResultsTable.COLUMN_IN_USE)
