@@ -1,5 +1,4 @@
 package ij.plugin.frame;
-//import ij.plugin.frame.*;
 import java.awt.*;
 import java.awt.event.*;
 import java.awt.image.*;
@@ -22,6 +21,7 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 	static final String[] channelLabels = {"Red", "Green", "Blue", "Cyan", "Magenta", "Yellow", "All"};
 	static final String[] altChannelLabels = {"Channel 1", "Channel 2", "Channel 3", "Channel 4", "Channel 5", "Channel 6", "All"};
 	static final int[] channelConstants = {4, 2, 1, 3, 5, 6, 7};
+	static final String[] ranges = {"Automatic", "8-bit (0-255)", "10-bit (0-1023)", "12-bit (0-4095)", "15-bit (0-32767)", "16-bit (0-65535)"};
 	
 	ContrastPlot plot = new ContrastPlot();
 	Thread thread;
@@ -55,8 +55,6 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 	Font sanFont = new Font("SansSerif", Font.PLAIN, 12);
 	int channels = 7; // RGB
 	Choice choice;
-	boolean updatingRGBStack;
-
 
 	public ContrastAdjuster() {
 		super("B&C");
@@ -78,7 +76,7 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 				Prefs.saveLocation(LOC_KEY, ca.getLocation());
 				ca.close();
 			} else {
-				instance.toFront();
+				WindowManager.toFront(instance);
 				return;
 			}
 		}
@@ -316,6 +314,7 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 
 	void setupNewImage(ImagePlus imp, ImageProcessor ip)  {
 		//IJ.write("setupNewImage");
+		Undo.reset();
 		previousMin = min;
 		previousMax = max;
 	 	if (RGBImage) {
@@ -390,10 +389,14 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 	}
 	
 	void setMinAndMax(ImagePlus imp, double min, double max) {
-		if (channels!=7 && imp.getType()==ImagePlus.COLOR_RGB)
+		boolean rgb = imp.getType()==ImagePlus.COLOR_RGB;
+		if (channels!=7 && rgb)
 			imp.setDisplayRange(min, max, channels);
 		else
 			imp.setDisplayRange(min, max);
+		if (!rgb)
+			imp.getProcessor().setSnapshotPixels(null); // disable undo
+
 	}
 
 	void updatePlot() {
@@ -570,8 +573,14 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 				pixels = b;
 			ImageProcessor ip = new ByteProcessor(w, h, pixels, null);
 			stats = ImageStatistics.getStatistics(ip, 0, imp.getCalibration());
-		} else
-			stats = imp.getStatistics();
+		} else {
+			int range = imp.getType()==ImagePlus.GRAY16?ImagePlus.getDefault16bitRange():0;
+			if (range!=0 && imp.getProcessor().getMax()==Math.pow(2,range)-1 && !(imp.getCalibration().isSigned16Bit())) {
+				ImagePlus imp2 = new ImagePlus("Temp", imp.getProcessor());
+				stats = new StackStatistics(imp2, 256, 0, Math.pow(2,range));
+			} else
+				stats = imp.getStatistics();
+		}
 		Color color = Color.gray;
 		if (imp.isComposite() && !(balance&&channels==7))
 			color = ((CompositeImage)imp).getChannelColor();
@@ -579,19 +588,26 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 	}
 
 	void apply(ImagePlus imp, ImageProcessor ip) {
+		if (balance && imp.isComposite())
+			return;
 		String option = null;
 		if (RGBImage)
 			imp.unlock();
 		if (!imp.lock())
 			return;
-		if (imp.getType()==ImagePlus.COLOR_RGB) {
+		if (RGBImage) {
 			if (imp.getStackSize()>1)
 				applyRGBStack(imp);
 			else {
 				ip.snapshot();
 				reset(imp, ip);
 				imp.changes = true;
-				if (Recorder.record) Recorder.record("run", "Apply LUT");
+				if (Recorder.record) {
+					if (Recorder.scriptMode())
+						Recorder.recordCall("IJ.run(imp, \"Apply LUT\", \"\");");
+					else
+						Recorder.record("run", "Apply LUT");
+				}
 			}
 			imp.unlock();
 			return;
@@ -652,14 +668,22 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 		imp.changes = true;
 		imp.unlock();
 		if (Recorder.record) {
-			if (option!=null)
-				Recorder.record("run", "Apply LUT", option);
-			else
-				Recorder.record("run", "Apply LUT");
+			if (Recorder.scriptMode()) {
+				if (option==null) option = "";
+				Recorder.recordCall("IJ.run(imp, \"Apply LUT\", \""+option+"\");");
+			} else {
+				if (option!=null)
+					Recorder.record("run", "Apply LUT", option);
+				else
+					Recorder.record("run", "Apply LUT");
+			}
 		}
 	}
 
 	void applyRGBStack(ImagePlus imp) {
+		double min = imp.getDisplayRangeMin();
+		double max = imp.getDisplayRangeMax();
+		if (IJ.debugMode) IJ.log("applyRGBStack: "+min+"-"+max);
 		int current = imp.getCurrentSlice();
 		int n = imp.getStackSize();
 		if (!IJ.showMessageWithCancel("Update Entire Stack?",
@@ -668,22 +692,31 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 		"NOTE: There is no Undo for this operation."))
 			return;
  		ImageProcessor mask = imp.getMask();
- 		updatingRGBStack = true;
+ 		Rectangle roi = imp.getRoi()!=null?imp.getRoi().getBounds():null;
+ 		ImageStack stack = imp.getStack();
 		for (int i=1; i<=n; i++) {
+			IJ.showProgress(i, n);
+			IJ.showStatus(i+"/"+n);
 			if (i!=current) {
-				imp.setSlice(i);
-				ImageProcessor ip = imp.getProcessor();
+				ImageProcessor ip = stack.getProcessor(i);
+				ip.setRoi(roi);
 				if (mask!=null) ip.snapshot();
-				setMinAndMax(imp, min, max);
-				ip.reset(mask);
-				IJ.showProgress((double)i/n);
+				if (channels!=7)
+					((ColorProcessor)ip).setMinAndMax(min, max, channels);
+				else
+					ip.setMinAndMax(min, max);
+				if (mask!=null) ip.reset(mask);
 			}
 		}
+		imp.setStack(null, stack);
 		imp.setSlice(current);
- 		updatingRGBStack = false;
 		imp.changes = true;
-		if (Recorder.record)
-			Recorder.record("run", "Apply LUT", "stack");
+		if (Recorder.record) {
+			if (Recorder.scriptMode())
+				Recorder.recordCall("IJ.run(imp, \"Apply LUT\", \"stack\");");
+			else
+				Recorder.record("run", "Apply LUT", "stack");
+		}
 	}
 
 	void setThreshold(ImageProcessor ip) {
@@ -746,8 +779,12 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 		//	if (mask!=null)
 		//		ip.reset(mask);
 		//}
-		if (Recorder.record)
-			Recorder.record("run", "Enhance Contrast", "saturated=0.5");
+		if (Recorder.record) {
+			if (Recorder.scriptMode())
+				Recorder.recordCall("IJ.run(imp, \"Enhance Contrast\", \"saturated=0.35\");");
+			else
+				Recorder.record("run", "Enhance Contrast", "saturated=0.35");
+		}
 	}
 	
 	void setMinAndMax(ImagePlus imp, ImageProcessor ip) {
@@ -759,11 +796,18 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 		double maxValue = cal.getCValue(max);
 		int channels = imp.getNChannels();
 		GenericDialog gd = new GenericDialog("Set Display Range");
-		gd.addNumericField("Minimum Displayed Value: ", minValue, digits);
-		gd.addNumericField("Maximum Displayed Value: ", maxValue, digits);
-		gd.addCheckbox("Propagate to all open images", false);
-		if (imp.isComposite())
-			gd.addCheckbox("Propagate to all "+channels+" channels", false);
+		gd.addNumericField("Minimum displayed value: ", minValue, digits);
+		gd.addNumericField("Maximum displayed value: ", maxValue, digits);
+		gd.addChoice("Unsigned 16-bit range:", ranges, ranges[getRangeIndex()]);
+		String label = "Propagate to all other ";
+		label = imp.isComposite()?label+channels+" channel images":label+"open images";
+		gd.addCheckbox(label, false);
+		boolean allChannels = false;
+		if (imp.isComposite() && channels>1) {	
+			label = "Propagate to the other ";
+			label = channels==2?label+"channel of this image":label+(channels-1)+" channels of this image";
+			gd.addCheckbox(label, allChannels);
+		}
 		gd.showDialog();
 		if (gd.wasCanceled())
 			return;
@@ -771,16 +815,23 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 		maxValue = gd.getNextNumber();
 		minValue = cal.getRawValue(minValue);
 		maxValue = cal.getRawValue(maxValue);
+		int rangeIndex = gd.getNextChoiceIndex();
+		int range1 = ImagePlus.getDefault16bitRange();
+		int range2 = setRange(rangeIndex);
+		if (range1!=range2 && imp.getType()==ImagePlus.GRAY16 && !cal.isSigned16Bit()) {
+			reset(imp, ip);
+			minValue = imp.getDisplayRangeMin();
+			maxValue = imp.getDisplayRangeMax();
+		}
 		boolean propagate = gd.getNextBoolean();
-		boolean allChannels = imp.isComposite()&&gd.getNextBoolean();
+		if (imp.isComposite() && channels>1)
+			allChannels = gd.getNextBoolean();
 		if (maxValue>=minValue) {
 			min = minValue;
 			max = maxValue;
 			setMinAndMax(imp, min, max);
 			updateScrollBars(null, false);
 			if (RGBImage) doMasking(imp, ip);
-			if (propagate)
-				IJ.runMacroFile("ij.jar:PropagateMinAndMax");
 			if (allChannels) {
 				int channel = imp.getChannel();
 				for (int c=1; c<=channels; c++) {
@@ -790,9 +841,11 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 				((CompositeImage)imp).reset();
 				imp.setPosition(channel, imp.getSlice(), imp.getFrame());
 			}
+			if (propagate)
+				propagate(imp);
 			if (Recorder.record) {
 				if (imp.getBitDepth()==32)
-					Recorder.record("setMinAndMax", min, max);
+					recordSetMinAndMax(min, max);
 				else {
 					int imin = (int)min;
 					int imax = (int)max;
@@ -800,10 +853,79 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 						imin = (int)cal.getCValue(imin);
 						imax = (int)cal.getCValue(imax);
 					}
-					Recorder.record("setMinAndMax", imin, imax);
+					recordSetMinAndMax(imin, imax);
+				}
+				if (Recorder.scriptMode())
+					Recorder.recordCall("ImagePlus.setDefault16bitRange("+range2+");");
+				else
+					Recorder.recordString("call(\"ij.ImagePlus.setDefault16bitRange\", "+range2+");\n");
+
+			}
+		}
+	}
+	
+	private void propagate(ImagePlus img) {
+		int[] list = WindowManager.getIDList();
+		if (list==null) return;
+		int nImages = list.length;
+		if (nImages<=1) return;
+		ImageProcessor ip = img.getProcessor();
+		double min = ip.getMin();
+		double max = ip.getMax();
+		int depth = img.getBitDepth();
+		if (depth==24) return;
+		int id = img.getID();
+		if (img.isComposite()) {
+			int nChannels = img.getNChannels();
+			for (int i=0; i<nImages; i++) {
+				ImagePlus img2 = WindowManager.getImage(list[i]);
+				if (img2==null) continue;
+				int nChannels2 = img2.getNChannels();
+				if (img2.isComposite() && img2.getBitDepth()==depth && img2.getID()!=id
+				&& img2.getNChannels()==nChannels && img2.getWindow()!=null) {
+					int channel = img2.getChannel();
+					for (int c=1; c<=nChannels; c++) {
+						LUT  lut = ((CompositeImage)img).getChannelLut(c);
+						img2.setPosition(c, img2.getSlice(), img2.getFrame());
+						img2.setDisplayRange(lut.min, lut.max);
+						img2.updateAndDraw();
+					}
+					img2.setPosition(channel, img2.getSlice(), img2.getFrame());
+				}
+			}
+		} else {
+			for (int i=0; i<nImages; i++) {
+				ImagePlus img2 = WindowManager.getImage(list[i]);
+				if (img2!=null && img2.getBitDepth()==depth && img2.getID()!=id
+				&& img2.getNChannels()==1 && img2.getWindow()!=null) {
+					ImageProcessor ip2 = img2.getProcessor();
+					ip2.setMinAndMax(min, max);
+					img2.updateAndDraw();
 				}
 			}
 		}
+    }
+	
+	int getRangeIndex() {
+		int range = ImagePlus.getDefault16bitRange();
+		int index = 0;
+		if (range==8) index = 1;
+		else if (range==10) index = 2;
+		else if (range==12) index = 3;
+		else if (range==15) index = 4;
+		else if (range==16) index = 5;
+		return index;
+	}
+
+	int setRange(int index) {
+		int range = 0;
+		if (index==1) range = 8;
+		else if (index==2) range = 10;
+		else if (index==3) range = 12;
+		else if (index==4) range = 15;
+		else if (index==5) range = 16;
+		ImagePlus.setDefault16bitRange(range);
+		return range;
 	}
 
 	void setWindowLevel(ImagePlus imp, ImageProcessor ip) {
@@ -840,7 +962,7 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 				IJ.runMacroFile("ij.jar:PropagateMinAndMax");
 			if (Recorder.record) {
 				if (imp.getBitDepth()==32)
-					Recorder.record("setMinAndMax", min, max);
+					recordSetMinAndMax(min, max);
 				else {
 					int imin = (int)min;
 					int imax = (int)max;
@@ -848,12 +970,27 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 						imin = (int)cal.getCValue(imin);
 						imax = (int)cal.getCValue(imax);
 					}
-					Recorder.record("setMinAndMax", imin, imax);
+					recordSetMinAndMax(imin, imax);
 				}
 			}
 		}
 	}
 
+	void recordSetMinAndMax(double min, double max) {
+		if ((int)min==min && (int)max==max) {
+			int imin=(int)min, imax = (int)max;
+			if (Recorder.scriptMode())
+				Recorder.recordCall("IJ.setMinAndMax(imp, "+imin+", "+imax+");");
+			else
+				Recorder.record("setMinAndMax", imin, imax);
+		} else {
+			if (Recorder.scriptMode())
+				Recorder.recordCall("IJ.setMinAndMax(imp, "+min+", "+max+");");
+			else
+				Recorder.record("setMinAndMax", min, max);
+		}
+	}
+	
 	static final int RESET=0, AUTO=1, SET=2, APPLY=3, THRESHOLD=4, MIN=5, MAX=6, 
 		BRIGHTNESS=7, CONTRAST=8, UPDATE=9;
 
@@ -900,7 +1037,12 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 		switch (action) {
 			case RESET:
 				reset(imp, ip);
-				if (Recorder.record) Recorder.record("resetMinAndMax");
+				if (Recorder.record) {
+						if (Recorder.scriptMode())
+							Recorder.recordCall("IJ.resetMinAndMax(imp);");
+						else
+							Recorder.record("resetMinAndMax");
+				}
 				break;
 			case AUTO: autoAdjust(imp, ip); break;
 			case SET: if (windowLevel) setWindowLevel(imp, ip); else setMinAndMax(imp, ip); break;
@@ -912,24 +1054,20 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 		}
 		updatePlot();
 		updateLabels(imp);
-		if ((IJ.shiftKeyDown()||(balance&&channels==7)) && imp.isComposite()) {
+		if ((IJ.shiftKeyDown()||(balance&&channels==7)) && imp.isComposite())
 			((CompositeImage)imp).updateAllChannelsAndDraw();
-		} else
+		else
 			imp.updateChannelAndDraw();
 		if (RGBImage)
 			imp.unlock();
 	}
 
-	public void windowClosing(WindowEvent e) {
-	 	close();
-		Prefs.saveLocation(LOC_KEY, getLocation());
-	}
-
     /** Overrides close() in PlugInFrame. */
     public void close() {
     	super.close();
 		instance = null;
 		done = true;
+		Prefs.saveLocation(LOC_KEY, getLocation());
 		synchronized(this) {
 			notify();
 		}
@@ -937,6 +1075,13 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
 
 	public void windowActivated(WindowEvent e) {
 		super.windowActivated(e);
+		if (IJ.isMacro()) {
+			// do nothing if macro and RGB image
+			ImagePlus imp2 = WindowManager.getCurrentImage();
+			if (imp2!=null && imp2.getBitDepth()==24) {
+				return;
+			}
+		}
 		previousImageID = 0; // user may have modified image
 		setup();
 		WindowManager.setWindow(this);
@@ -968,10 +1113,8 @@ public class ContrastAdjuster extends PlugInFrame implements Runnable,
     public static void update() {
 		if (instance!=null) {
 			ContrastAdjuster ca = ((ContrastAdjuster)instance);
-			if (!ca.updatingRGBStack) {
-				ca.previousImageID = 0;
-				ca.setup();
-			}
+			ca.previousImageID = 0;
+			ca.setup();
 		}
     }
     
