@@ -26,10 +26,15 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 		"importPackage(Packages.ij.measure);"+
 		"importPackage(Packages.ij.util);"+
 		"importPackage(Packages.ij.plugin);"+
+		"importPackage(Packages.ij.io);"+
 		"importPackage(Packages.ij.plugin.filter);"+
 		"importPackage(Packages.ij.plugin.frame);"+
 		"importPackage(java.lang);"+
 		"importPackage(java.awt);"+
+		"importPackage(java.awt.image);"+
+		"importPackage(java.awt.geom);"+
+		"importPackage(java.util);"+
+		"importPackage(java.io);"+
 		"function print(s) {IJ.log(s);};";
 
 	public static String JS_NOT_FOUND = 
@@ -40,6 +45,7 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 	static final String FONT_SIZE = "editor.font.size";
 	static final String FONT_MONO= "editor.font.mono";
 	static final String CASE_SENSITIVE= "editor.case-sensitive";
+	static final String DEFAULT_DIR= "editor.dir";
 	private TextArea ta;
 	private String path;
 	private boolean changes;
@@ -60,10 +66,10 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 	private String shortcutsInUse;
 	private int inUseCount;
 	private MacroInstaller installer;
-	private static String defaultDir;
+	private static String defaultDir = Prefs.get(DEFAULT_DIR, null);;
 	private boolean dontShowWindow;
     private int[] sizes = {9, 10, 11, 12, 13, 14, 16, 18, 20, 24, 36, 48, 60, 72};
-    private int fontSize = (int)Prefs.get(FONT_SIZE, 5);
+    private int fontSize = (int)Prefs.get(FONT_SIZE, 6); // defaults to 16-point
     private CheckboxMenuItem monospaced;
     private static boolean wholeWords;
     private boolean isMacroWindow;
@@ -187,7 +193,7 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 		ta.setCaretPosition(0);
 		setWindowTitle(name);
 		boolean macroExtension = name.endsWith(".txt") || name.endsWith(".ijm");
-		if (macroExtension || name.endsWith(".js")|| name.indexOf(".")==-1) {
+		if (macroExtension || name.endsWith(".js") || name.indexOf(".")==-1) {
 			macrosMenu = new Menu("Macros");			
 			macrosMenu.add(new MenuItem("Run Macro", new MenuShortcut(KeyEvent.VK_R)));
 			macrosMenu.add(new MenuItem("Evaluate Line", new MenuShortcut(KeyEvent.VK_Y)));
@@ -207,7 +213,7 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 				debugMenu.add(new MenuItem("Trace", new MenuShortcut(KeyEvent.VK_T)));
 				debugMenu.add(new MenuItem("Fast Trace", new MenuShortcut(KeyEvent.VK_T,true)));
 				debugMenu.add(new MenuItem("Run"));
-				debugMenu.add(new MenuItem("Run to Insertion Point"));
+				debugMenu.add(new MenuItem("Run to Insertion Point", new MenuShortcut(KeyEvent.VK_E, true)));
 				debugMenu.add(new MenuItem("Abort"));
 				debugMenu.addActionListener(this);
 				mb.add(debugMenu);
@@ -230,7 +236,6 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 
 	public void createMacro(String name, String text) {
 		create(name, text);
-		editMenu.add(new MenuItem("Convert to Plugin"));
 	}
 
 	void installMacros(String text, boolean installInPluginsMenu) {
@@ -267,7 +272,7 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 			changes = false;
 		}
 		catch (Exception e) {
-			IJ.error(e.getMessage());
+			IJ.handleException(e);
 			return;
 		}
 	}
@@ -328,6 +333,9 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 			saveAs();
 		if (path!=null) {
 			save();
+			String text = ta.getText();
+			if (text.contains("implements PlugInFilter") && text.contains("IJ.run("))
+				IJ.log("Plugins that call IJ.run() should probably implement PlugIns, not PlugInFilters.");
 			IJ.runPlugIn("ij.plugin.Compiler", path);
 		}
 	}
@@ -357,10 +365,9 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 			text = ta.getSelectedText();
 		if (text.equals("")) return;
 		text = getJSPrefix("") + text;
-		if (IJ.isJava16() && !IJ.isMacOSX()) {
+		if (IJ.isJava16() && !(IJ.isMacOSX()&&!IJ.is64Bit()))
 			IJ.runPlugIn("JavaScriptEvaluator", text);
-			return;
-		} else {
+		else {
 			Object js = IJ.runPlugIn("JavaScript", text);
 			if (js==null) IJ.error(JS_NOT_FOUND);
 		}
@@ -451,7 +458,7 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 		} catch (EOFException eof) {
 	   // Fine, ignore
 		} catch (Throwable t) { // Anything else
-			t.printStackTrace();
+			IJ.handleException(t);
 		}
 	}
 	
@@ -506,12 +513,8 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 		catch  (Exception e)  {
 			s  = e.toString( );
 		}
-		if (!fixedLineEndings && IJ.isWindows()) {
-			String text = ta.getText();
-			int len = text.length();
-			text = text.replaceAll("\r\n", "\n");
-			if (text.length()!=len) ta.setText(text);
-		}
+		if (!fixedLineEndings && IJ.isWindows())
+			fixLineEndings();
 		fixedLineEndings = true;
 		int start = ta.getSelectionStart( );
 		int end = ta.getSelectionEnd( );
@@ -596,8 +599,6 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 			gotoLine();
 		else if ("Zap Gremlins".equals(what))
 			zapGremlins();
-		else if ("Convert to Plugin".equals(what))
-			convertToPlugin();
 		else if ("Make Text Larger".equals(what))
 			changeFontSize(true);
 		else if ("Make Text Smaller".equals(what))
@@ -701,15 +702,11 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 			instance = this;
 	}
 
-	public void windowClosing(WindowEvent e) {
-		close();
-	}
-
 	/** Overrides close() in PlugInFrame. */
 	public void close() {
 		boolean okayToClose = true;
 		ImageJ ij = IJ.getInstance();
-		if (!getTitle().equals("Errors") && changes && !IJ.isMacro() && ij!=null && !ij.quitting()) {
+		if (!getTitle().equals("Errors") && changes && !IJ.isMacro() && ij!=null) {
 			String msg = "Save changes to \"" + getTitle() + "\"?";
 			YesNoCancelDialog d = new YesNoCancelDialog(this, "Editor", msg);
 			if (d.cancelPressed())
@@ -718,11 +715,12 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 				save();
 		}
 		if (okayToClose) {
-			setVisible(false);
+			//setVisible(false);
 			dispose();
 			WindowManager.removeWindow(this);
 			nWindows--;
 			instance = null;
+			changes = false;
 		}
 	}
 
@@ -745,6 +743,9 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 			save();
 			changes = false;
 			setWindowTitle(name2);
+			setDefaultDirectory(dir);
+			if (defaultDir!=null)
+				Prefs.set(DEFAULT_DIR, defaultDir);
 			if (Recorder.record)
 				Recorder.record("saveAs", "Text", path);
 		}
@@ -876,44 +877,6 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 	void selectAll() {
 		ta.selectAll();
 	}
-
-	void convertToPlugin() {
-		if (!(getTitle().endsWith(".txt")||getTitle().endsWith(".ijm"))) return;
-		String text = ta.getText();
-		if (text==null || text.equals("")) {
-			IJ.runPlugIn("ij.plugin.NewPlugin", " ");
-			return;
-		}
-		if (text.indexOf("{")>-1) {
-			IJ.showMessage("Convert to Plugin", "Conversion limited to recorder generated macro code.");
-			return;
-		}
-		StringTokenizer st = new StringTokenizer(text, "\n");
-		int n = st.countTokens();
-		String line;
-		StringBuffer sb = new StringBuffer();
-		for(int i=0; i<n; i++) {
-			line = st.nextToken();
-			if (line!=null && line.length()>3) {
-				if (line.equals("close();")) line = "run(\"Close\");";
-				sb.append("\t\tIJ.");
-				if (line.startsWith("//run"))
-					line = line.substring(2);
-				sb.append(line);
-				sb.append('\n');
-			}
-		}
-		NewPlugin np = (NewPlugin)IJ.runPlugIn("ij.plugin.NewPlugin", new String(sb));
-		Editor ed = np.getEditor();
-		String title = getTitle();
-		if (title.equals("Macro.txt")||title.equals("Macro.ijm"))
-			title = "Converted_Macro";
-		if (title.endsWith(".txt")||title.endsWith(".ijm")) title = title.substring(0, title.length()-4);
-		if (title.indexOf('_')==-1) title += "_";
-		title += ".java";
-		ed.updateClassName(ed.getTitle(), title);
-		ed.setTitle(title);
-	}
     
     void changeFontSize(boolean larger) {
         int in = fontSize;
@@ -958,6 +921,8 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 
 	public static void setDefaultDirectory(String defaultDirectory) {
 		defaultDir = defaultDirectory;
+		if (defaultDir!=null && !(defaultDir.endsWith(File.separator)||defaultDir.endsWith("/")))
+			defaultDir += File.separator;
 	}
 	
 	//public void keyReleased(KeyEvent e) {}
@@ -1036,5 +1001,16 @@ public class Editor extends PlugInFrame implements ActionListener, ItemListener,
 		return JavaScriptIncludes+"function getArgument() {return \""+arg+"\";};";
 	}
 	
+	/** Changes Mac OS 9 (CR) and Windows (CRLF) line separators to line feeds (LF). */
+	public void fixLineEndings() {
+		String text = ta.getText();
+		text = text.replaceAll("\r\n", "\n");
+		text = text.replaceAll("\r", "\n");
+		ta.setText(text);
+	}
+	
+	public boolean fileChanged() {
+		return changes;
+	}
+	
 }
-
