@@ -17,6 +17,7 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 
 	public static final String LOC_KEY = "threshold.loc";
 	public static final String MODE_KEY = "threshold.mode";
+	public static final String DARK_BACKGROUND = "threshold.dark";
 	static final int RED=0, BLACK_AND_WHITE=1, OVER_UNDER=2;
 	static final String[] modes = {"Red","B&W", "Over/Under"};
 	static final double defaultMinThreshold = 85; 
@@ -26,7 +27,7 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 	static boolean fill2 = true;
 	static boolean useBW = true;
 	static boolean backgroundToNaN = true;
-	static Frame instance; 
+	static ThresholdAdjuster instance; 
 	static int mode = RED;	
 	static String[] methodNames = AutoThresholder.getMethods();
 	static String method = methodNames[DEFAULT];
@@ -53,15 +54,21 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 	boolean invertedLut;
 	int lutColor;	
 	Choice methodChoice, modeChoice;
-	Checkbox darkBackground;
+	Checkbox darkBackground, stackHistogram;
 	boolean firstActivation;
 	boolean useExistingTheshold;
 
 
 	public ThresholdAdjuster() {
 		super("Threshold");
+		ImagePlus cimp = WindowManager.getCurrentImage();
+		if (cimp!=null && cimp.getBitDepth()==24) {
+			IJ.run(cimp, "Color Threshold...", "");
+			return;
+		}
 		if (instance!=null) {
-			instance.toFront();
+			instance.firstActivation = true;
+			WindowManager.toFront(instance);
 			return;
 		}
 		
@@ -118,7 +125,7 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 		c.gridy = y++;
 		c.gridwidth = 1;
 		c.weightx = 100;
-		c.insets = new Insets(0, 10, 0, 0);
+		c.insets = new Insets(2, 10, 0, 0);
 		add(maxSlider, c);
 		maxSlider.addAdjustmentListener(this);
 		maxSlider.addKeyListener(ij);
@@ -129,7 +136,7 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 		c.gridx = 1;
 		c.gridwidth = 1;
 		c.weightx = 0;
-		c.insets = new Insets(0, 0, 0, 10);
+		c.insets = new Insets(2, 0, 0, 10);
 		label2 = new Label("       ", Label.RIGHT);
     	label2.setFont(font);
 		add(label2, c);
@@ -153,20 +160,27 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 		c.gridx = 0;
 		c.gridy = y++;
 		c.gridwidth = 2;
-		c.insets = new Insets(5, 5, 0, 5);
+		c.insets = new Insets(8, 5, 0, 5);
 		c.anchor = GridBagConstraints.CENTER;
 		c.fill = GridBagConstraints.NONE;
 		add(panel, c);
 
-		// checkbox
+		// checkboxes
+		panel = new Panel();
+		boolean db = Prefs.get(DARK_BACKGROUND, Prefs.blackBackground?true:false);
         darkBackground = new Checkbox("Dark background");
-        darkBackground.setState(false);
+        darkBackground.setState(db);
         darkBackground.addItemListener(this);
+        panel.add(darkBackground);
+        stackHistogram = new Checkbox("Stack histogram");
+        stackHistogram.setState(false);
+        stackHistogram.addItemListener(this);
+        panel.add(stackHistogram);
         c.gridx = 0;
         c.gridy = y++;
         c.gridwidth = 2;
-        c.insets = new Insets(5, 35, 0, 5);
-        add(darkBackground, c);
+		c.insets = new Insets(5, 5, 0, 5);
+        add(panel, c);
 
 		// buttons
 		int trim = IJ.isMacOSX()?11:0;
@@ -200,7 +214,6 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 			setLocation(loc);
 		else
 			GUI.center(this);
-		firstActivation = true;
 		if (IJ.isMacOSX()) setResizable(false);
 		show();
 
@@ -286,6 +299,7 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 		int id = imp.getID();
 		if (minMaxChange || id!=previousImageID || type!=previousImageType) {
             //IJ.log(minMaxChange +"  "+ (id!=previousImageID)+"  "+(type!=previousImageType));
+            Undo.reset();
             if (not8Bits && minMaxChange && !useExistingTheshold) {
                 ip.resetMinAndMax();
                 imp.updateAndDraw();
@@ -294,7 +308,7 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 			invertedLut = imp.isInvertedLut();
 			minThreshold = ip.getMinThreshold();
 			maxThreshold = ip.getMaxThreshold();
-			ImageStatistics stats = plot.setHistogram(imp, false);
+			ImageStatistics stats = plot.setHistogram(imp, entireStack(imp));
 			if (minThreshold==ImageProcessor.NO_THRESHOLD)
 				autoSetLevels(ip, stats);
 			else {
@@ -313,6 +327,10 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 	 	return ip;
 	}
 	
+    boolean entireStack(ImagePlus imp) {
+        return stackHistogram!=null && stackHistogram.getState() && imp.getStackSize()>1;
+    }
+
 	void autoSetLevels(ImageProcessor ip, ImageStatistics stats) {
 		if (stats==null || stats.histogram==null) {
 			minThreshold = defaultMinThreshold;
@@ -330,18 +348,21 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 			if (invertedLut)
 				{minThreshold=0; maxThreshold=threshold;}
 			else
-				{minThreshold=threshold; maxThreshold=255;}
+				{minThreshold=threshold+1; maxThreshold=255;}
 		} else {
 			if (invertedLut)
-				{minThreshold=threshold; maxThreshold=255;}
+				{minThreshold=threshold+1; maxThreshold=255;}
 			else
 				{minThreshold=0; maxThreshold=threshold;}
 		}
+		if (minThreshold>255) minThreshold = 255;
 		if (Recorder.record) {
-			if (method.equals("Default"))
-				Recorder.record("setAutoThreshold");
+			boolean stack = stackHistogram!=null && stackHistogram.getState();
+			String options = method+(darkb?" dark":"")+(stack?" stack":"");
+			if (Recorder.scriptMode())
+				Recorder.recordCall("IJ.setAutoThreshold(imp, \""+options+"\");");
 			else
-				Recorder.record("setAutoThreshold", method+(darkb?" dark":""));
+				Recorder.record("setAutoThreshold", options);
 		}
 	}
 	
@@ -357,6 +378,7 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 				minThreshold = maxThreshold = min;
 		}
 		ip.setThreshold(minThreshold, maxThreshold, lutColor);
+		ip.setSnapshotPixels(null); // disable undo
 	}
 
 	/** Scales a threshold level to the range 0-255. */
@@ -457,16 +479,21 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 	}
 
 	void reset(ImagePlus imp, ImageProcessor ip) {
-		boolean useStackMinAndMax = false;
-		if (!(ip instanceof ByteProcessor)) {
-			ip.resetMinAndMax();
-			useStackMinAndMax = imp.getStackSize()>1 && IJ.altKeyDown();
-		}
 		ip.resetThreshold();
-		plot.setHistogram(imp, useStackMinAndMax);
+		ImageStatistics stats = plot.setHistogram(imp, entireStack(imp));
+		if (!(ip instanceof ByteProcessor)) {
+			if (entireStack(imp))
+				ip.setMinAndMax(stats.min, stats.max);
+			else
+				ip.resetMinAndMax();
+		}
 		updateScrollBars();
-		if (Recorder.record)
-			Recorder.record("resetThreshold");
+		if (Recorder.record) {
+			if (Recorder.scriptMode())
+				Recorder.recordCall("IJ.resetThreshold(imp);");
+			else
+				Recorder.record("resetThreshold");
+		}
 	}
 
 	void doSet(ImagePlus imp, ImageProcessor ip) {
@@ -500,24 +527,26 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 		if (level1<minValue) level1 = minValue;
 		if (level2>maxValue) level2 = maxValue;
 		IJ.wait(500);
-		ip.setThreshold(level1, level2, lutColor);	
+		ip.setThreshold(level1, level2, lutColor);
+		ip.setSnapshotPixels(null); // disable undo
 		setup(imp);
-		//boolean outOfRange = level1<minDisplay || level2>maxDisplay;
-		//if (outOfRange)
-		//	plot.setHistogram(imp, false);
-		//else
-		//	ip.setMinAndMax(minDisplay, maxDisplay);
 		if (Recorder.record) {
-			if (imp.getBitDepth()==32)
-				Recorder.record("setThreshold", ip.getMinThreshold(), ip.getMaxThreshold());
-			else {
+			if (imp.getBitDepth()==32) {
+				if (Recorder.scriptMode())
+					Recorder.recordCall("IJ.setThreshold("+ip.getMinThreshold()+", "+ip.getMaxThreshold()+");");
+				else
+					Recorder.record("setThreshold", ip.getMinThreshold(), ip.getMaxThreshold());
+			} else {
 				int min = (int)ip.getMinThreshold();
 				int max = (int)ip.getMaxThreshold();
 				if (cal.isSigned16Bit()) {
 					min = (int)cal.getCValue(level1);
 					max = (int)cal.getCValue(level2);
 				}
-				Recorder.record("setThreshold", min, max);
+				if (Recorder.scriptMode())
+					Recorder.recordCall("IJ.setThreshold(imp, "+min+", "+max+");");
+				else
+					Recorder.record("setThreshold", min, max);
 			}
 		}
 	}
@@ -629,17 +658,14 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 		imp.updateAndDraw();
 	}
 
-    public void windowClosing(WindowEvent e) {
-    	close();
-		Prefs.saveLocation(LOC_KEY, getLocation());
-		Prefs.set(MODE_KEY, mode);
-	}
-
     /** Overrides close() in PlugInFrame. */
     public void close() {
     	super.close();
 		instance = null;
 		done = true;
+		Prefs.saveLocation(LOC_KEY, getLocation());
+		Prefs.set(MODE_KEY, mode);
+		Prefs.set(DARK_BACKGROUND, darkBackground.getState());
 		synchronized(this) {
 			notify();
 		}
@@ -649,12 +675,10 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
     	super.windowActivated(e);
     	plot.requestFocus();
 		ImagePlus imp = WindowManager.getCurrentImage();
-		if (imp!=null) {
-			if (!firstActivation) {
-				previousImageID = 0;
-				useExistingTheshold = isThresholded(imp);
-				setup(imp);
-			}
+		if (imp!=null && firstActivation) {
+			previousImageID = 0;
+			useExistingTheshold = isThresholded(imp);
+			setup(imp);
 			firstActivation = false;
 		}
 	}
@@ -664,6 +688,28 @@ public class ThresholdAdjuster extends PlugInFrame implements PlugIn, Measuremen
 		return ip.getMinThreshold()!=ImageProcessor.NO_THRESHOLD && ip.isColorLut();
 	}
 
+    /** Notifies the ThresholdAdjuster that the image has changed. */
+    public static void update() {
+		if (instance!=null) {
+			ThresholdAdjuster ta = ((ThresholdAdjuster)instance);
+			ImagePlus imp = WindowManager.getCurrentImage();
+			if (imp!=null && ta.previousImageID==imp.getID()) {
+				ta.previousImageID = 0;
+				ta.setup(imp);
+			}
+		}
+    }
+
+	/** Returns the current method ("Default", "Huang", etc). */
+	public static String getMethod() {
+		return method;
+	}
+	
+	/** Returns the current mode ("Red","B&W" or"Over/Under"). */
+	public static String getMode() {
+		return modes[mode];
+	}
+
 } // ThresholdAdjuster class
 
 
@@ -671,6 +717,7 @@ class ThresholdPlot extends Canvas implements Measurements, MouseListener {
 	static final int WIDTH = 256, HEIGHT=48;
 	double minThreshold = 85;
 	double maxThreshold = 170;
+	ImageStatistics stats;
 	int[] histogram;
 	Color[] hColors;
 	int hmax;
@@ -679,6 +726,9 @@ class ThresholdPlot extends Canvas implements Measurements, MouseListener {
 	int mode;
 	int originalModeCount;
 	double stackMin, stackMax;
+	int imageID2;
+	boolean entireStack2;
+	double mean2;
 	
 	public ThresholdPlot() {
 		addMouseListener(this);
@@ -691,19 +741,31 @@ class ThresholdPlot extends Canvas implements Measurements, MouseListener {
         return new Dimension(WIDTH+1, HEIGHT+1);
     }
     
-	ImageStatistics setHistogram(ImagePlus imp, boolean useStackMinAndMax) {
+	ImageStatistics setHistogram(ImagePlus imp, boolean entireStack) {
+		double mean = entireStack?imp.getProcessor().getStatistics().mean:0.0;
+		if (entireStack && stats!=null && imp.getID()==imageID2 
+		&& entireStack==entireStack2 && mean==mean2)
+			return stats;
+		mean2 = mean;
 		ImageProcessor ip = imp.getProcessor();
-		ImageStatistics stats = null;
+		stats = null;
+		if (entireStack)
+			stats = new StackStatistics(imp);
 		if (!(ip instanceof ByteProcessor)) {
-			if (useStackMinAndMax) {
-				stats = new StackStatistics(imp);
+			if (entireStack) {
 				if (imp.getLocalCalibration().isSigned16Bit()) 
 					{stats.min += 32768; stats.max += 32768;}
 				stackMin = stats.min;
 				stackMax = stats.max;
 				ip.setMinAndMax(stackMin, stackMax);
-			} else
+				imp.updateAndDraw();
+			} else {
 				stackMin = stackMax = 0.0;
+				if (entireStack2) {
+					ip.resetMinAndMax();
+					imp.updateAndDraw();
+				}
+			}
 			Calibration cal = imp.getCalibration();
 			if (ip instanceof FloatProcessor) {
 				int digits = Math.max(Analyzer.getPrecision(), 2);
@@ -747,9 +809,11 @@ class ThresholdPlot extends Canvas implements Measurements, MouseListener {
 		hColors = new Color[256];
 		for (int i=0; i<256; i++)
 			hColors[i] = new Color(r[i]&255, g[i]&255, b[i]&255);
+		imageID2 = imp.getID();
+		entireStack2 = entireStack;
 		return stats;
 	}
-
+	
 	public void update(Graphics g) {
 		paint(g);
 	}
