@@ -5,45 +5,72 @@ import java.util.Arrays;
 /** Autothresholding methods from the Auto_Threshold plugin (http://pacific.mpi-cbg.de/wiki/index.php/Auto_Threshold)
     by G.Landini at bham dot ac dot uk. */
 public class AutoThresholder {
-	private static final String[] methods = {"Default", "Huang", "Intermodes", "IsoData", "Li", "MaxEntropy", "Mean", "MinError(I)", "Minimum",
-		"Moments", "Otsu", "Percentile", "RenyiEntropy", "Shanbhag", "Triangle", "Yen"};
-	private static final int IJ_ISODATA=0, HUANG=1, INTERMODES=2, ISODATA=3, LI=4, MAXENTROPY=5, MEAN=6, MINERROR=7, MINIMUM=8,
-		MOMENTS=9, OTSU=10, PERCENTILE=11, RENYIENTROPY=12, SHANBHAG=13, TRIANGLE=14, YEN=15;
+	private static String[] mStrings;
 			
+	public enum Method {
+		Default, 
+		Huang,
+		Intermodes,
+		IsoData, 
+		IJ_IsoData,
+		Li, 
+		MaxEntropy, 
+		Mean, 
+		MinError, 
+		Minimum,
+		Moments, 
+		Otsu, 
+		Percentile, 
+		RenyiEntropy, 
+		Shanbhag, 
+		Triangle, 
+		Yen
+	};
+
 	public static String[] getMethods() {
-		return methods;
-	}
-	
-	public int getThreshold(String method, int[] histogram) {
-		int index = Arrays.binarySearch(methods, method);
-		if (index<0) throw new IllegalArgumentException("Threshold method not found: \""+method+"\"");
-		int threshold = 0;
-		switch (index) {
-			case IJ_ISODATA: return IJIsoData(histogram);
-			case HUANG: return Huang(histogram);
-			case INTERMODES: return Intermodes(histogram);
-			case ISODATA: return IsoData(histogram);
-			case LI: return Li(histogram);
-			case MAXENTROPY: return MaxEntropy(histogram);
-			case MEAN: return Mean(histogram);
-			case MINERROR: return MinErrorI(histogram);
-			case MINIMUM: return Minimum(histogram);
-			case MOMENTS: return Moments(histogram);
-			case OTSU: return Otsu(histogram);
-			case PERCENTILE: return Percentile(histogram);
-			case RENYIENTROPY: return RenyiEntropy(histogram);
-			case SHANBHAG: return Shanbhag(histogram);
-			case TRIANGLE: return Triangle(histogram);
-			case YEN: return Yen(histogram);
+		if (mStrings==null) {
+			Method[] mVals = Method.values();
+			mStrings = new String[mVals.length];
+			for (int i=0; i<mVals.length; i++)
+				mStrings[i] = mVals[i].name();
 		}
-		return -1;
+		return mStrings;
 	}
 	
-	final double log10(double x) {
-		return Math.log(x)/Math.log(10);
+	public int getThreshold(Method method, int[] histogram) {
+		int threshold = 0;
+		switch (method) {
+			case Default: threshold =  defaultIsoData(histogram); break;
+			case IJ_IsoData: threshold =  IJIsoData(histogram); break;
+			case Huang: threshold = Huang(histogram); break;
+			case Intermodes: threshold = Intermodes(histogram); break;
+			case IsoData: threshold = IsoData(histogram); break;
+			case Li: threshold = Li(histogram); break;
+			case MaxEntropy: threshold = MaxEntropy(histogram); break;
+			case Mean: threshold = Mean(histogram); break;
+			case MinError: threshold = MinErrorI(histogram); break;
+			case Minimum: threshold = Minimum(histogram); break;
+			case Moments: threshold = Moments(histogram); break;
+			case Otsu: threshold = Otsu(histogram); break;
+			case Percentile: threshold = Percentile(histogram); break;
+			case RenyiEntropy: threshold = RenyiEntropy(histogram); break;
+			case Shanbhag: threshold = Shanbhag(histogram); break;
+			case Triangle: threshold = Triangle(histogram); break;
+			case Yen: threshold = Yen(histogram); break;
+		}
+		if (threshold==-1) threshold = 0;
+		return threshold;
 	}
 
-
+	public int getThreshold(String mString, int[] histogram) {
+		// throws an exception if unknown argument
+		int index = mString.indexOf(" ");
+		if (index!=-1)
+			mString = mString.substring(0, index);
+		Method method = Method.valueOf(Method.class, mString); 
+		return getThreshold(method, histogram);
+	}
+	
 	int Huang(int [] data ) {
 		// Implements Huang's fuzzy thresholding method 
 		// Uses Shannon's entropy function (one can also use Yager's entropy function) 
@@ -55,8 +82,8 @@ public class AutoThresholder {
 		int ih, it;
 		int first_bin;
 		int last_bin;
-		int sum_pix;
-		int num_pix;
+		double sum_pix;
+		double num_pix;
 		double term;
 		double ent;  // entropy 
 		double min_ent; // min entropy 
@@ -83,16 +110,16 @@ public class AutoThresholder {
 		double [] mu_0 = new double[256];
 		sum_pix = num_pix = 0;
 		for ( ih = first_bin; ih < 256; ih++ ){
-			sum_pix += ih * data[ih];
+			sum_pix += (double)ih * data[ih];
 			num_pix += data[ih];
 			/* NUM_PIX cannot be zero ! */
-			mu_0[ih] = sum_pix / ( double ) num_pix;
+			mu_0[ih] = sum_pix / num_pix;
 		}
 
 		double [] mu_1 = new double[256];
 		sum_pix = num_pix = 0;
 		for ( ih = last_bin; ih > 0; ih-- ){
-			sum_pix += ih * data[ih];
+			sum_pix += (double)ih * data[ih];
 			num_pix += data[ih];
 			/* NUM_PIX cannot be zero ! */
 			mu_1[ih - 1] = sum_pix / ( double ) num_pix;
@@ -221,8 +248,9 @@ public class AutoThresholder {
 		// yes => exit
 		// no => increment G and repeat
 		//
-		int i, l, toth, totl, h, g=0;
-		for (i = 1; i < 256; i++){
+		int i, l, totl, g=0;
+		double toth, h;
+		for (i = 1; i < 256; i++) {
 			if (data[i] > 0){
 				g = i + 1;
 				break;
@@ -239,7 +267,7 @@ public class AutoThresholder {
 			toth = 0;
 			for (i = g + 1; i < 256; i++){
 				toth += data[i];
-				h += (data[i]*i);
+				h += ((double)data[i]*i);
 			}
 			if (totl > 0 && toth > 0){
 				l /= totl;
@@ -254,6 +282,32 @@ public class AutoThresholder {
 		return g;
 	}
 	
+	int defaultIsoData(int[] data) {
+		// This is the modified IsoData method used by the "Threshold" widget in "Default" mode
+		int n = data.length;
+		int[] data2 = new int[n];
+		int mode=0, maxCount=0;
+		for (int i=0; i<n; i++) {
+			int count = data[i];
+			data2[i] = data[i];
+			if (data2[i]>maxCount) {
+				maxCount = data2[i];
+				mode = i;
+			}
+		}
+		int maxCount2 = 0;
+		for (int i = 0; i<n; i++) {
+			if ((data2[i]>maxCount2) && (i!=mode))
+				maxCount2 = data2[i];
+		}
+		int hmax = maxCount;
+		if ((hmax>(maxCount2*2)) && (maxCount2!=0)) {
+			hmax = (int)(maxCount2 * 1.5);
+			data2[mode] = hmax;
+		}
+		return IJIsoData(data2);
+	}
+
 	int IJIsoData(int[] data) {
 		// This is the original ImageJ IsoData implementation, here for backward compatibility.
 		int level;
@@ -279,11 +333,11 @@ public class AutoThresholder {
 		do {
 			sum1=sum2=sum3=sum4=0.0;
 			for (int i=min; i<=movingIndex; i++) {
-				sum1 += i*data[i];
+				sum1 += (double)i*data[i];
 				sum2 += data[i];
 			}
 			for (int i=(movingIndex+1); i<=max; i++) {
-				sum3 += i*data[i];
+				sum3 += (double)i*data[i];
 				sum4 += data[i];
 			}			
 			result = (sum1/sum2 + sum3/sum4)/2.0;
@@ -308,12 +362,11 @@ public class AutoThresholder {
 		//    http://citeseer.ist.psu.edu/sezgin04survey.html
 		// Ported to ImageJ plugin by G.Landini from E Celebi's fourier_0.8 routines
 		int threshold;
-		int ih;
-		int num_pixels;
-		int sum_back; /* sum of the background pixels at a given threshold */
-		int sum_obj;  /* sum of the object pixels at a given threshold */
-		int num_back; /* number of background pixels at a given threshold */
-		int num_obj;  /* number of object pixels at a given threshold */
+		double num_pixels;
+		double sum_back; /* sum of the background pixels at a given threshold */
+		double sum_obj;  /* sum of the object pixels at a given threshold */
+		double num_back; /* number of background pixels at a given threshold */
+		double num_obj;  /* number of object pixels at a given threshold */
 		double old_thresh;
 		double new_thresh;
 		double mean_back; /* mean of the background pixels at a given threshold */
@@ -324,34 +377,34 @@ public class AutoThresholder {
 
 		tolerance=0.5;
 		num_pixels = 0;
-		for (ih = 0; ih < 256; ih++ ) 
+		for (int ih = 0; ih < 256; ih++ ) 
 			num_pixels += data[ih];
 
 		/* Calculate the mean gray-level */
 		mean = 0.0;
-		for ( ih = 0 + 1; ih < 256; ih++ ) //0 + 1?
-			mean += ih * data[ih];
+		for (int ih = 0 + 1; ih < 256; ih++ ) //0 + 1?
+			mean += (double)ih * data[ih];
 		mean /= num_pixels;
 		/* Initial estimate */
 		new_thresh = mean;
 
-		do{
+		do {
 			old_thresh = new_thresh;
 			threshold = (int) (old_thresh + 0.5);	/* range */
 			/* Calculate the means of background and object pixels */
 			/* Background */
 			sum_back = 0;
 			num_back = 0;
-			for ( ih = 0; ih <= threshold; ih++ ) {
-				sum_back += ih * data[ih];
+			for (int ih = 0; ih <= threshold; ih++ ) {
+				sum_back += (double)ih * data[ih];
 				num_back += data[ih];
 			}
 			mean_back = ( num_back == 0 ? 0.0 : ( sum_back / ( double ) num_back ) );
 			/* Object */
 			sum_obj = 0;
 			num_obj = 0;
-			for ( ih = threshold + 1; ih < 256; ih++ ) {
-				sum_obj += ih * data[ih];
+			for (int ih = threshold + 1; ih < 256; ih++ ) {
+				sum_obj += (double)ih * data[ih];
 				num_obj += data[ih];
 			}
 			mean_obj = ( num_obj == 0 ? 0.0 : ( sum_obj / ( double ) num_obj ) );
@@ -397,12 +450,12 @@ public class AutoThresholder {
 		double [] P1 = new double[256]; /* cumulative normalized histogram */
 		double [] P2 = new double[256]; 
 
-		int total =0;
+		double total =0;
 		for (ih = 0; ih < 256; ih++ ) 
 			total+=data[ih];
 
 		for (ih = 0; ih < 256; ih++ )
-			norm_histo[ih] = (double)data[ih]/total;
+			norm_histo[ih] = data[ih]/total;
 
 		P1[0]=norm_histo[0];
 		P2[0]=1.0-P1[0];
@@ -471,7 +524,7 @@ public class AutoThresholder {
 		double tot=0, sum=0;
 		for (int i=0; i<256; i++){
 			tot+= data[i];
-			sum+=(i*data[i]);
+			sum+=((double)i*data[i]);
 		}
 		threshold =(int) Math.floor(sum/tot);
 		return threshold;
@@ -501,7 +554,7 @@ public class AutoThresholder {
 			//The terms of the quadratic equation to be solved.
 			w0 = 1.0/sigma2-1.0/tau2;
 			w1 = mu/sigma2-nu/tau2;
-			w2 = (mu*mu)/sigma2 - (nu*nu)/tau2 + log10((sigma2*(q*q))/(tau2*(p*p)));
+			w2 = (mu*mu)/sigma2 - (nu*nu)/tau2 + Math.log10((sigma2*(q*q))/(tau2*(p*p)));
 
 			//If the next threshold would be imaginary, return with the current one.
 			sqterm = (w1*w1)-w0*w2;
@@ -613,10 +666,11 @@ public class AutoThresholder {
 			histo[i]=(double)(data[i]/total); //normalised histogram
 
 		/* Calculate the first, second, and third order moments */
-		for ( int i = 0; i < 256; i++ ){
-			m1 += i * histo[i];
-			m2 += i * i * histo[i];
-			m3 += i * i * i * histo[i];
+		for ( int i = 0; i < 256; i++ ) {
+			double di = i;
+			m1 += di * histo[i];
+			m2 += di * di * histo[i];
+			m3 += di * di * di * histo[i];
 		}
 		/* 
 		First 4 moments of the gray-level image should match the first 4 moments
@@ -648,16 +702,16 @@ public class AutoThresholder {
 		// C++ code by Jordan Bevik <Jordan.Bevic@qtiworld.com>
 		// ported to ImageJ plugin by G.Landini
 		int k,kStar;  // k = the current threshold; kStar = optimal threshold
-		int N1, N;    // N1 = # points with intensity <=k; N = total number of points
+		double N1, N;    // N1 = # points with intensity <=k; N = total number of points
 		double BCV, BCVmax; // The current Between Class Variance and maximum BCV
 		double num, denom;  // temporary bookeeping
-		int Sk;  // The total intensity for all histogram points <=k
-		int S, L=256; // The total intensity of the image
+		double Sk;  // The total intensity for all histogram points <=k
+		double S, L=256; // The total intensity of the image
 
 		// Initialize values:
 		S = N = 0;
 		for (k=0; k<L; k++){
-			S += k * data[k];	// Total histogram intensity
+			S += (double)k * data[k];	// Total histogram intensity
 			N += data[k];		// Total number of data points
 		}
 
@@ -670,7 +724,7 @@ public class AutoThresholder {
 		// Look at each possible threshold value,
 		// calculate the between-class variance, and decide if it's a max
 		for (k=1; k<L-1; k++) { // No need to check endpoints k = 0 or k = L-1
-			Sk += k * data[k];
+			Sk += (double)k * data[k];
 			N1 += data[k];
 
 			// The float casting here is to avoid compiler warning about loss of precision and
@@ -762,12 +816,12 @@ public class AutoThresholder {
 		double [] P1 = new double[256]; /* cumulative normalized histogram */
 		double [] P2 = new double[256]; 
 
-		int total =0;
+		double total =0;
 		for (ih = 0; ih < 256; ih++ ) 
 			total+=data[ih];
 
 		for (ih = 0; ih < 256; ih++ )
-			norm_histo[ih] = (double)data[ih]/total;
+			norm_histo[ih] = data[ih]/total;
 
 		P1[0]=norm_histo[0];
 		P2[0]=1.0-P1[0];
@@ -952,12 +1006,12 @@ public class AutoThresholder {
 		double [] P1 = new double[256]; /* cumulative normalized histogram */
 		double [] P2 = new double[256]; 
 
-		int total =0;
+		double total =0;
 		for (ih = 0; ih < 256; ih++ ) 
 			total+=data[ih];
 
 		for (ih = 0; ih < 256; ih++ )
-			norm_histo[ih] = (double)data[ih]/total;
+			norm_histo[ih] = data[ih]/total;
 
 		P1[0]=norm_histo[0];
 		P2[0]=1.0-P1[0];
@@ -1143,12 +1197,12 @@ public class AutoThresholder {
 		double [] P1_sq = new double[256]; 
 		double [] P2_sq = new double[256]; 
 
-		int total =0;
+		double total =0;
 		for (ih = 0; ih < 256; ih++ ) 
 			total+=data[ih];
 
 		for (ih = 0; ih < 256; ih++ )
-			norm_histo[ih] = (double)data[ih]/total;
+			norm_histo[ih] = data[ih]/total;
 
 		P1[0]=norm_histo[0];
 		for (ih = 1; ih < 256; ih++ )
