@@ -109,7 +109,13 @@ public class ByteProcessor extends ImageProcessor {
 	
 	/** Returns this image as a BufferedImage. */
 	public BufferedImage getBufferedImage() {
-		return (BufferedImage)createBufferedImage();
+		if (isDefaultLut()) {
+			BufferedImage bi = new BufferedImage(width, height, BufferedImage.TYPE_BYTE_GRAY);
+			Graphics g = bi.createGraphics();
+			g.drawImage(createImage(), 0, 0, null);
+			return bi;
+		} else
+			return (BufferedImage)createBufferedImage();
 	}
 
 	/** Returns a new, blank ByteProcessor with the specified width and height. */
@@ -158,6 +164,17 @@ public class ByteProcessor extends ImageProcessor {
         System.arraycopy(snapshotPixels,0,pixels,0,width*height);
 	}
 	
+	/** Swaps the pixel and snapshot (undo) arrays. */
+	public void swapPixelArrays() {
+		if (snapshotPixels==null) return;	
+		byte pixel;
+		for (int i=0; i<pixels.length; i++) {
+			pixel = pixels[i];
+			pixels[i] = snapshotPixels[i];
+			snapshotPixels[i] = pixel;
+		}
+	}
+
 	/** Restore pixels that are within roi but not part of mask. */
 	public void reset(ImageProcessor mask) {
 		if (mask==null || snapshotPixels==null)
@@ -193,8 +210,11 @@ public class ByteProcessor extends ImageProcessor {
 			{fill(); return;}
 		int roiWidth=this.roiWidth, roiHeight=this.roiHeight;
 		int roiX=this.roiX, roiY=this.roiY;
-		if (mask.getWidth()!=roiWidth||mask.getHeight()!=roiHeight)
-			return;
+		if (mask.getWidth()!=roiWidth||mask.getHeight()!=roiHeight) {
+			mask = getMask();
+			if (mask==null||mask.getWidth()!=roiWidth||mask.getHeight()!=roiHeight)
+				return;
+		}
 		byte[] mpixels = (byte[])mask.getPixels();
 		for (int y=roiY, my=0; y<(roiY+roiHeight); y++, my++) {
 			int i = y * width + roiX;
@@ -384,7 +404,6 @@ public class ByteProcessor extends ImageProcessor {
 			return;
 		this.min = (int)min;
 		this.max = (int)max;
-		
 		if (rLUT1==null) {
 			if (cm==null)
 				makeDefaultColorModel();
@@ -394,6 +413,8 @@ public class ByteProcessor extends ImageProcessor {
 			m.getReds(rLUT1); m.getGreens(gLUT1); m.getBlues(bLUT1); 
 			rLUT2 = new byte[256]; gLUT2 = new byte[256]; bLUT2 = new byte[256];
 		}
+		if (rLUT2==null)
+			return;
 		int index;
 		for (int i=0; i<256; i++) {
 			if (i<min) {
@@ -427,6 +448,12 @@ public class ByteProcessor extends ImageProcessor {
 		setMinAndMax(0, 255);
 	}
 
+	public void setThreshold(double minThreshold, double maxThreshold, int lutUpdate) {
+		super.setThreshold(minThreshold, maxThreshold, lutUpdate);
+		if (this.minThreshold<0.0) this.minThreshold = 0.0;
+		if (this.maxThreshold>255.0) this.maxThreshold = 255.0;
+	}
+
 	/** Copies the image contained in 'ip' to (xloc, yloc) using one of
 		the transfer modes defined in the Blitter interface. */
 	public void copyBits(ImageProcessor ip, int xloc, int yloc, int mode) {
@@ -446,60 +473,55 @@ public class ByteProcessor extends ImageProcessor {
 		}
 	}
 
-	/** 3x3 convolution contributed by Glynne Casteel. */
-	public void convolve3x3(int[] kernel) {
-		int p1, p2, p3,
-		    p4, p5, p6,
-		    p7, p8, p9;
+    public void convolve3x3(int[] kernel) {
+        int v1, v2, v3;    //input pixel values around the current pixel
+        int v4, v5, v6;
+        int v7, v8, v9;
+        int scale = 0;
 		int k1=kernel[0], k2=kernel[1], k3=kernel[2],
-		    k4=kernel[3], k5=kernel[4], k6=kernel[5],
-		    k7=kernel[6], k8=kernel[7], k9=kernel[8];
-
-		int scale = 0;
+		k4=kernel[3], k5=kernel[4], k6=kernel[5],
+		k7=kernel[6], k8=kernel[7], k9=kernel[8];
 		for (int i=0; i<kernel.length; i++)
 			scale += kernel[i];
 		if (scale==0) scale = 1;
-		int inc = roiHeight/25;
-		if (inc<1) inc = 1;
-		
-		byte[] pixels2 = (byte[])getPixelsCopy();
-		int offset, sum;
-        int rowOffset = width;
-		for (int y=yMin; y<=yMax; y++) {
-			offset = xMin + y * width;
-			p1 = 0;
-			p2 = pixels2[offset-rowOffset-1]&0xff;
-			p3 = pixels2[offset-rowOffset]&0xff;
-			p4 = 0;
-			p5 = pixels2[offset-1]&0xff;
-			p6 = pixels2[offset]&0xff;
-			p7 = 0;
-			p8 = pixels2[offset+rowOffset-1]&0xff;
-			p9 = pixels2[offset+rowOffset]&0xff;
-
-			for (int x=xMin; x<=xMax; x++) {
-				p1 = p2; p2 = p3;
-				p3 = pixels2[offset-rowOffset+1]&0xff;
-				p4 = p5; p5 = p6;
-				p6 = pixels2[offset+1]&0xff;
-				p7 = p8; p8 = p9;
-				p9 = pixels2[offset+rowOffset+1]&0xff;
-
-				sum = k1*p1 + k2*p2 + k3*p3
-				    + k4*p4 + k5*p5 + k6*p6
-				    + k7*p7 + k8*p8 + k9*p9;
-				sum /= scale;
-
-				if(sum>255) sum= 255;
-				if(sum<0) sum= 0;
-
-				pixels[offset++] = (byte)sum;
+        int inc = roiHeight/25;
+        if (inc<1) inc = 1;
+        byte[] pixels2 = (byte[])getPixelsCopy();
+        int xEnd = roiX + roiWidth;
+        int yEnd = roiY + roiHeight;
+        for (int y=roiY; y<yEnd; y++) {
+            int p  = roiX + y*width;            //points to current pixel
+            int p6 = p - (roiX>0 ? 1 : 0);      //will point to v6, currently lower
+            int p3 = p6 - (y>0 ? width : 0);    //will point to v3, currently lower
+            int p9 = p6 + (y<height-1 ? width : 0); // ...  to v9, currently lower
+            v2 = pixels2[p3]&0xff;
+            v5 = pixels2[p6]&0xff;
+            v8 = pixels2[p9]&0xff;
+            if (roiX>0) { p3++; p6++; p9++; }
+            v3 = pixels2[p3]&0xff;
+            v6 = pixels2[p6]&0xff;
+            v9 = pixels2[p9]&0xff;
+			for (int x=roiX; x<xEnd; x++,p++) {
+				if (x<width-1) { p3++; p6++; p9++; }
+				v1 = v2; v2 = v3;
+				v3 = pixels2[p3]&0xff;
+				v4 = v5; v5 = v6;
+				v6 = pixels2[p6]&0xff;
+				v7 = v8; v8 = v9;
+				v9 = pixels2[p9]&0xff;
+				int sum = k1*v1 + k2*v2 + k3*v3
+						+ k4*v4 + k5*v5 + k6*v6
+						+ k7*v7 + k8*v8 + k9*v9;
+				sum = (sum+scale/2)/scale;   //add scale/2 to round
+				if (sum>255) sum = 255;
+				if (sum<0) sum = 0;
+				pixels[p] = (byte)sum;
 			}
-			if (y%inc==0)
-				showProgress((double)(y-roiY)/roiHeight);
-		}
-		showProgress(1.0);
-	}
+            if (y%inc==0)
+                showProgress((double)(y-roiY)/roiHeight);
+        }
+        showProgress(1.0);
+    }
 
 	/** Filters using a 3x3 neighborhood. The p1, p2, etc variables, which
 		contain the values of the pixels in the neighborhood, are arranged
@@ -545,7 +567,7 @@ public class ByteProcessor extends ImageProcessor {
 
 				switch (type) {
 					case BLUR_MORE:
-						sum = (p1+p2+p3+p4+p5+p6+p7+p8+p9)/9;
+						sum = (p1+p2+p3+p4+p5+p6+p7+p8+p9+4)/9;
 						break;
 					case FIND_EDGES: // 3x3 Sobel filter
 	        			sum1 = p1 + 2*p2 + p3 - p7 - 2*p8 - p9;
@@ -622,14 +644,14 @@ public class ByteProcessor extends ImageProcessor {
 				
 				pixels[offset++] = (byte)sum;
 			}
-			if (y%inc==0)
-				showProgress((double)(y-roiY)/roiHeight);
+			//if (y%inc==0)
+			//	showProgress((double)(y-roiY)/roiHeight);
 		}
         if (xMin==1) filterEdge(type, pixels2, roiHeight, roiX, roiY, 0, 1);
         if (yMin==1) filterEdge(type, pixels2, roiWidth, roiX, roiY, 1, 0);
         if (xMax==width-2) filterEdge(type, pixels2, roiHeight, width-1, roiY, 0, 1);
         if (yMax==height-2) filterEdge(type, pixels2, roiWidth, roiX, height-1, 1, 0);
-		showProgress(1.0);
+		//showProgress(1.0);
 	}
 
 	void filterEdge(int type, byte[] pixels2, int n, int x, int y, int xinc, int yinc) {
@@ -656,7 +678,7 @@ public class ByteProcessor extends ImageProcessor {
 			}
             switch (type) {
                 case BLUR_MORE:
-                    sum = (p1+p2+p3+p4+p5+p6+p7+p8+p9)/9;
+                    sum = (p1+p2+p3+p4+p5+p6+p7+p8+p9+4)/9;
                     break;
                 case FIND_EDGES: // 3x3 Sobel filter
                     sum1 = p1 + 2*p2 + p3 - p7 - 2*p8 - p9;
