@@ -61,7 +61,8 @@ public class ColorProcessor extends ImageProcessor {
 	}
 	
 	public Image createImage() {
-		if (ij.IJ.isJava16()) return createBufferedImage();
+		if (ij.IJ.isJava16())
+			return createBufferedImage();
 		if (source==null) {
 			source = new MemoryImageSource(width, height, cm, pixels, 0, width);
 			source.setAnimated(true);
@@ -118,6 +119,11 @@ public class ColorProcessor extends ImageProcessor {
 	}
 
 
+	/** Sets the fill/draw color, where <code>color</code> is an RGB int. */
+	public void setColor(int color) {
+		fgColor = color;
+	}
+
 	/** Sets the default fill/draw value, where <code>value</code> is interpreted as an RGB int. */
 	public void setValue(double value) {
 		fgColor = (int)value;
@@ -150,7 +156,6 @@ public class ColorProcessor extends ImageProcessor {
 		setMinAndMax(min, max, 7);
 	}
 
-
 	public void setMinAndMax(double min, double max, int channels) {
 		if (max<min)
 			return;
@@ -208,6 +213,17 @@ public class ColorProcessor extends ImageProcessor {
 		}
 	}
 
+	/** Swaps the pixel and snapshot (undo) arrays. */
+	public void swapPixelArrays() {
+		if (snapshotPixels==null) return;	
+		int pixel;
+		for (int i=0; i<pixels.length; i++) {
+			pixel = pixels[i];
+			pixels[i] = snapshotPixels[i];
+			snapshotPixels[i] = pixel;
+		}
+	}
+
 	public void setSnapshotPixels(Object pixels) {
 		snapshotPixels = (int[])pixels;
 		snapshotWidth=width;
@@ -279,7 +295,7 @@ public class ColorProcessor extends ImageProcessor {
 	}
 
 	public final float getf(int x, int y) {
-		return pixels[y*width+x];
+		return getf(y*width+x);
 	}
 
 	public final void setf(int x, int y, float value) {
@@ -287,9 +303,17 @@ public class ColorProcessor extends ImageProcessor {
 	}
 
 	public final float getf(int index) {
-		return pixels[index];
+		int c = pixels[index];
+		int r = (c&0xff0000)>>16;
+		int g = (c&0xff00)>>8;
+		int b = c&0xff;
+		return (float)(r*rWeight + g*gWeight + b*bWeight);
 	}
 
+	//public final float getf(int index) {
+	//	return pixels[index];
+	//}
+
 	public final void setf(int index, float value) {
 		pixels[index] = (int)value;
 	}
@@ -456,6 +480,36 @@ public class ColorProcessor extends ImageProcessor {
 		}
 	}
 
+	/** Returns the specified plane as a byte array. */
+	public byte[] getChannel(int channel) {
+		int size = width*height;
+		byte[] bytes = new byte[size];
+		int c, r, g, b;
+		switch (channel) {
+			case 1:
+				for (int i=0; i<size; i++) {
+					c = pixels[i];
+					r = (c&0xff0000)>>16;
+					bytes[i] = (byte)r;
+				}
+				break;
+			case 2:
+				for (int i=0; i<size; i++) {
+					c = pixels[i];
+					g = (c&0xff00)>>8;
+					bytes[i] = (byte)g;
+				}
+				break;
+			case 3:
+				for (int i=0; i<size; i++) {
+					c = pixels[i];
+					b = c&0xff;
+					bytes[i] = (byte)b;
+				}
+				break;
+		}
+		return bytes;
+	}
 
 	/** Sets the current pixels from 3 byte arrays (reg, green, blue). */
 	public void setRGB(byte[] R, byte[] G, byte[] B) {
@@ -571,8 +625,7 @@ public class ColorProcessor extends ImageProcessor {
 		}
 		showProgress(1.0);
 	}
-
-
+	
 	public static final int RGB_NOISE=0, RGB_MEDIAN=1, RGB_FIND_EDGES=2,
 		RGB_ERODE=3, RGB_DILATE=4, RGB_THRESHOLD=5, RGB_ROTATE=6,
 		RGB_SCALE=7, RGB_RESIZE=8, RGB_TRANSLATE=9;
@@ -859,8 +912,8 @@ public class ColorProcessor extends ImageProcessor {
 		double xlimit = width-1.0, xlimit2 = width-1.001;
 		double ylimit = height-1.0, ylimit2 = height-1.001;
 		if (interpolationMethod==BILINEAR) {
-			if (xScale<=0.25 && yScale<=0.25)
-				return makeThumbnail(dstWidth, dstHeight, 0.6);
+			//if (xScale<=0.25 && yScale<=0.25)
+			//	return makeThumbnail(dstWidth, dstHeight, 0.6);
 			dstCenterX += xScale/2.0;
 			dstCenterY += yScale/2.0;
 		}
@@ -892,45 +945,10 @@ public class ColorProcessor extends ImageProcessor {
 		return ip2;
 	}
 	
-	/** Uses averaging to creates a new ColorProcessor containing a scaled copy 
-		of this image or selection. The amount of smoothing is determined by
-		'smoothFactor', which must be greater than zero and less than or equal 1.0
-	*/
+	/** Uses averaging to creates a new ColorProcessor containing 
+		a downsized copy  of this image or selection. */
 	public ImageProcessor makeThumbnail(int width2, int height2, double smoothFactor) {
-		ImageProcessor ip = this;
-		if (roiWidth!=width || roiHeight!=height)
-			ip = ip.crop();
-		int width = ip.getWidth();
-		int height = ip.getHeight();
-		int[] pixel = new int[3];
-		int[] sum = new int[3];
-		double xscale, yscale;
-		int w, h;
-		double product;
-		xscale = (double)width/width2;
-		yscale = (double)height/height2;
-		w = (int)(xscale*smoothFactor);
-		h = (int)(yscale*smoothFactor);
-		product = w*h;
-		ImageProcessor ip2 = ip.createProcessor(width2, height2);
-		for (int y=0; y<height2; y++) {
-			for (int x=0; x<width2; x++) {
-				for (int i=0; i<3; i++) sum[i] = 0;
-				int xbase = (int)(x*xscale);
-				int ybase = (int)(y*yscale);
-				for (int y2=0; y2<h; y2++) {
-					for (int x2=0;  x2<w; x2++) {
-						pixel = ip.getPixel(xbase+x2, ybase+y2, pixel);
-						for (int i=0; i<3; i++)
-							sum[i] += pixel[i];
-					}
-				}
-				for (int i=0; i<3; i++)
-					sum[i] = (int)(sum[i]/product+0.5);
-				ip2.putPixel(x, y, sum);
-			}
-		}
-		return ip2;
+		return resize(width2, height2, true);
 	}
 
 	/** Rotates the image or ROI 'angle' degrees clockwise.
