@@ -10,11 +10,9 @@ public class FloatProcessor extends ImageProcessor {
 
 	private float min, max, snapshotMin, snapshotMax;
 	private float[] pixels;
-	private byte[] pixels8;
+	protected byte[] pixels8;
 	private float[] snapshotPixels = null;
-	private byte[] LUT = null;
 	private float fillColor =  Float.MAX_VALUE;
-	//private float bgColor = Float.MIN_VALUE;
 	private boolean fixedScale = false;
 
 	/** Creates a new FloatProcessor using the specified pixel array and ColorModel.
@@ -27,8 +25,6 @@ public class FloatProcessor extends ImageProcessor {
 		this.pixels = pixels;
 		this.cm = cm;
 		resetRoi();
-		if (pixels!=null)
-			findMinAndMax();
 	}
 
 	/** Creates a blank FloatProcessor using the default grayscale LUT that
@@ -42,7 +38,6 @@ public class FloatProcessor extends ImageProcessor {
 		this(width, height);
 		for (int i=0; i<pixels.length; i++)
 			this.pixels[i] = (float)(pixels[i]);
-		findMinAndMax();
 	}
 	
 	/** Creates a FloatProcessor from a double array using the default grayscale LUT. */
@@ -50,10 +45,9 @@ public class FloatProcessor extends ImageProcessor {
 		this(width, height);
 		for (int i=0; i<pixels.length; i++)
 			this.pixels[i] = (float)pixels[i];
-		findMinAndMax();
 	}
 	
-	/** Creates a FloatProcessor from a float[][] array using the default LUT. */
+	/** Creates a FloatProcessor from a 2D float array using the default LUT. */
 	public FloatProcessor(float[][] array) {
 		width = array.length;
 		height = array[0].length;
@@ -65,14 +59,19 @@ public class FloatProcessor extends ImageProcessor {
 			}
 		}
 		resetRoi();
-		findMinAndMax();
 	}
 
-	/** Creates a FloatProcessor from an int[][] array. */
+	/** Creates a FloatProcessor from a 2D int array. */
 	public FloatProcessor(int[][] array) {
-		this(array.length, array[0].length);
-		setIntArray(array);
-		findMinAndMax();
+		width = array.length;
+		height = array[0].length;
+		pixels = new float[width*height];
+		int i=0;
+		for (int y=0; y<height; y++) {
+			for (int x=0; x<width; x++) {
+				pixels[i++] = (float)array[x][y];
+			}
+		}
 	}
 
 	/**
@@ -99,6 +98,7 @@ public class FloatProcessor extends ImageProcessor {
 					max = value;
 			}
 		}
+		minMaxSet = true;
 		showProgress(1.0);
 	}
 
@@ -108,12 +108,13 @@ public class FloatProcessor extends ImageProcessor {
 	resetMinAndMax() to enable auto-scaling;
 	@see ij.plugin.frame.ContrastAdjuster 
 	*/
-	public void setMinAndMax(double min, double max) {
-		if (min==0.0 && max==0.0)
+	public void setMinAndMax(double minimum, double maximum) {
+		if (minimum==0.0 && maximum==0.0)
 			{resetMinAndMax(); return;}
-		this.min = (float)min;
-		this.max = (float)max;
+		min = (float)minimum;
+		max = (float)maximum;
 		fixedScale = true;
+		minMaxSet = true;
 		resetThreshold();
 	}
 
@@ -128,11 +129,13 @@ public class FloatProcessor extends ImageProcessor {
 
 	/** Returns the smallest displayed pixel value. */
 	public double getMin() {
+		if (!minMaxSet) findMinAndMax();
 		return min;
 	}
 
 	/** Returns the largest displayed pixel value. */
 	public double getMax() {
+		if (!minMaxSet) findMinAndMax();
 		return max;
 	}
 
@@ -153,19 +156,20 @@ public class FloatProcessor extends ImageProcessor {
 		} else
 			source.newPixels();
 		lutAnimation = false;
-	    return img;
+		return img;
 	}
 	
-	byte[] create8BitImage() {
+	protected byte[] create8BitImage() {
 		// scale from float to 8-bits
 		int size = width*height;
 		if (pixels8==null)
 			pixels8 = new byte[size];
 		float value;
 		int ivalue;
-		float scale = 255f/(max-min);
+		float min2=(float)getMin(), max2=(float)getMax();
+		float scale = 255f/(max2-min2);
 		for (int i=0; i<size; i++) {
-			value = pixels[i]-min;
+			value = pixels[i]-min2;
 			if (value<0f) value = 0f;
 			ivalue = (int)((value*scale)+0.5f);
 			if (ivalue>255) ivalue = 255;
@@ -190,24 +194,25 @@ public class FloatProcessor extends ImageProcessor {
 	public void snapshot() {
 		snapshotWidth=width;
 		snapshotHeight=height;
-		snapshotMin=min;
-		snapshotMax=max;
+		snapshotMin=(float)getMin();
+		snapshotMax=(float)getMax();
 		if (snapshotPixels==null || (snapshotPixels!=null && snapshotPixels.length!=pixels.length))
 			snapshotPixels = new float[width * height];
-        System.arraycopy(pixels, 0, snapshotPixels, 0, width*height);
+		System.arraycopy(pixels, 0, snapshotPixels, 0, width*height);
 	}
 	
 	public void reset() {
 		if (snapshotPixels==null)
 			return;
-	    min=snapshotMin;
+		min=snapshotMin;
 		max=snapshotMax;
-        System.arraycopy(snapshotPixels,0,pixels,0,width*height);
+		minMaxSet = true;
+		System.arraycopy(snapshotPixels,0,pixels,0,width*height);
 	}
 	
 	public void reset(ImageProcessor mask) {
 		if (mask==null || snapshotPixels==null)
-			return;	
+			return; 
 		if (mask.getWidth()!=roiWidth||mask.getHeight()!=roiHeight)
 			throw new IllegalArgumentException(maskSizeError(mask));
 		byte[] mpixels = (byte[])mask.getPixels();
@@ -222,6 +227,17 @@ public class FloatProcessor extends ImageProcessor {
 		}
 	}
 
+	/** Swaps the pixel and snapshot (undo) arrays. */
+	public void swapPixelArrays() {
+		if (snapshotPixels==null) return;	
+		float pixel;
+		for (int i=0; i<pixels.length; i++) {
+			pixel = pixels[i];
+			pixels[i] = snapshotPixels[i];
+			snapshotPixels[i] = pixel;
+		}
+	}
+
 	public void setSnapshotPixels(Object pixels) {
 		snapshotPixels = (float[])pixels;
 		snapshotWidth=width;
@@ -274,8 +290,8 @@ public class FloatProcessor extends ImageProcessor {
 	}
 
 
-    /** Returns the value of the pixel at (x,y) in a
-    	one element int array. iArray is an optiona
+	/** Returns the value of the pixel at (x,y) in a
+		one element int array. iArray is an optiona
 		preallocated array. */
 	public int[] getPixel(int x, int y, int[] iArray) {
 		if (iArray==null) iArray = new int[1];
@@ -341,7 +357,7 @@ public class FloatProcessor extends ImageProcessor {
 			putPixel(x, y, Float.floatToIntBits(fillColor));
 	}
 
-	/**	Returns a reference to the float array containing
+	/** Returns a reference to the float array containing
 		this image's pixel data. */
 	public Object getPixels() {
 		return (Object)pixels;
@@ -358,7 +374,7 @@ public class FloatProcessor extends ImageProcessor {
 			return snapshotPixels;
 		} else {
 			float[] pixels2 = new float[width*height];
-        	System.arraycopy(pixels, 0, pixels2, 0, width*height);
+			System.arraycopy(pixels, 0, pixels2, 0, width*height);
 			return pixels2;
 		}
 	}
@@ -381,15 +397,18 @@ public class FloatProcessor extends ImageProcessor {
 
 	private void process(int op, double value) {
 		float c, v1, v2;
-		boolean resetMinMax = roiWidth==width && roiHeight==height && !(op==FILL);
+		//boolean resetMinMax = roiWidth==width && roiHeight==height && !(op==FILL);
 		c = (float)value;
+		float min2=0f, max2=0f;
+		if (op==INVERT)
+			{min2=(float)getMin(); max2=(float)getMax();}
 		for (int y=roiY; y<(roiY+roiHeight); y++) {
 			int i = y * width + roiX;
 			for (int x=roiX; x<(roiX+roiWidth); x++) {
 				v1 = pixels[i];
 				switch(op) {
 					case INVERT:
-						v2 = max - (v1 - min);
+						v2 = max2 - (v1 - min2);
 						break;
 					case FILL:
 						v2 = fillColor;
@@ -440,14 +459,12 @@ public class FloatProcessor extends ImageProcessor {
 							v2 = v1;
 						break;
 					 default:
-					 	v2 = v1;
+						v2 = v1;
 				}
 				pixels[i++] = v2;
 			}
 		}
-		if (resetMinMax)
-			findMinAndMax();
-    }
+	}
 
 	public void invert() {process(INVERT, 0.0);}
 	public void add(int value) {process(ADD, value);}
@@ -500,94 +517,94 @@ public class FloatProcessor extends ImageProcessor {
 		filter3x3(type, null);
 	}
 
-    /** 3x3 filter operations, code partly based on 3x3 convolution code
-     *  contributed by Glynne Casteel. */
-    void filter3x3(int type, int[] kernel) {
-		float v1, v2, v3;           //input pixel values around the current pixel
-        float v4, v5, v6;
-        float v7, v8, v9;
-        float k1=0f, k2=0f, k3=0f;  //kernel values (used for CONVOLVE only)
-        float k4=0f, k5=0f, k6=0f;
-        float k7=0f, k8=0f, k9=0f;
-        float scale = 0f;
-        if (type==CONVOLVE) {
-            k1=kernel[0]; k2=kernel[1]; k3=kernel[2];
-            k4=kernel[3]; k5=kernel[4]; k6=kernel[5];
-		    k7=kernel[6]; k8=kernel[7]; k9=kernel[8];
-    		for (int i=0; i<kernel.length; i++)
-    			scale += kernel[i];
-    		if (scale==0) scale = 1f;
-            scale = 1f/scale; //multiplication factor (multiply is faster than divide)
-        }
+	/** 3x3 filter operations, code partly based on 3x3 convolution code
+	 *	contributed by Glynne Casteel. */
+	void filter3x3(int type, int[] kernel) {
+		float v1, v2, v3;			//input pixel values around the current pixel
+		float v4, v5, v6;
+		float v7, v8, v9;
+		float k1=0f, k2=0f, k3=0f;	//kernel values (used for CONVOLVE only)
+		float k4=0f, k5=0f, k6=0f;
+		float k7=0f, k8=0f, k9=0f;
+		float scale = 0f;
+		if (type==CONVOLVE) {
+			k1=kernel[0]; k2=kernel[1]; k3=kernel[2];
+			k4=kernel[3]; k5=kernel[4]; k6=kernel[5];
+			k7=kernel[6]; k8=kernel[7]; k9=kernel[8];
+			for (int i=0; i<kernel.length; i++)
+				scale += kernel[i];
+			if (scale==0) scale = 1f;
+			scale = 1f/scale; //multiplication factor (multiply is faster than divide)
+		}
 		int inc = roiHeight/25;
 		if (inc<1) inc = 1;
 		
 		float[] pixels2 = (float[])getPixelsCopy();
 		//float[] pixels2 = (float[])getPixelsCopy();
-        int xEnd = roiX + roiWidth;
-        int yEnd = roiY + roiHeight;
+		int xEnd = roiX + roiWidth;
+		int yEnd = roiY + roiHeight;
 		for (int y=roiY; y<yEnd; y++) {
-			int p  = roiX + y*width;            //points to current pixel
-            int p6 = p - (roiX>0 ? 1 : 0);      //will point to v6, currently lower
-            int p3 = p6 - (y>0 ? width : 0);    //will point to v3, currently lower
-            int p9 = p6 + (y<height-1 ? width : 0); // ...  to v9, currently lower
-            v2 = pixels2[p3];
-            v5 = pixels2[p6];
-            v8 = pixels2[p9];
-            if (roiX>0) { p3++; p6++; p9++; }
-            v3 = pixels2[p3];
-            v6 = pixels2[p6];
-            v9 = pixels2[p9];
+			int p  = roiX + y*width;			//points to current pixel
+			int p6 = p - (roiX>0 ? 1 : 0);		//will point to v6, currently lower
+			int p3 = p6 - (y>0 ? width : 0);	//will point to v3, currently lower
+			int p9 = p6 + (y<height-1 ? width : 0); // ...	to v9, currently lower
+			v2 = pixels2[p3];
+			v5 = pixels2[p6];
+			v8 = pixels2[p9];
+			if (roiX>0) { p3++; p6++; p9++; }
+			v3 = pixels2[p3];
+			v6 = pixels2[p6];
+			v9 = pixels2[p9];
 
-            switch (type) {
-                case BLUR_MORE:
-    			for (int x=roiX; x<xEnd; x++,p++) {
-                    if (x<width-1) { p3++; p6++; p9++; }
-    				v1 = v2; v2 = v3;
-    				v3 = pixels2[p3];
-    				v4 = v5; v5 = v6;
-    				v6 = pixels2[p6];
-    				v7 = v8; v8 = v9;
-    				v9 = pixels2[p9];
-                    pixels[p] = (v1+v2+v3+v4+v5+v6+v7+v8+v9)*0.11111111f; //0.111... = 1/9
-                }
-                break;
-                case FIND_EDGES:
-    			for (int x=roiX; x<xEnd; x++,p++) {
-                    if (x<width-1) { p3++; p6++; p9++; }
-    				v1 = v2; v2 = v3;
-    				v3 = pixels2[p3];
-    				v4 = v5; v5 = v6;
-    				v6 = pixels2[p6];
-    				v7 = v8; v8 = v9;
-    				v9 = pixels2[p9];
-                    float sum1 = v1 + 2*v2 + v3 - v7 - 2*v8 - v9;
-                    float sum2 = v1  + 2*v4 + v7 - v3 - 2*v6 - v9;
-                    pixels[p] = (float)Math.sqrt(sum1*sum1 + sum2*sum2);
-                }
-                break;
-                case CONVOLVE:
-    			for (int x=roiX; x<xEnd; x++,p++) {
-                    if (x<width-1) { p3++; p6++; p9++; }
-    				v1 = v2; v2 = v3;
-    				v3 = pixels2[p3];
-    				v4 = v5; v5 = v6;
-    				v6 = pixels2[p6];
-    				v7 = v8; v8 = v9;
-    				v9 = pixels2[p9];
-                    float sum = k1*v1 + k2*v2 + k3*v3
-                              + k4*v4 + k5*v5 + k6*v6
-                              + k7*v7 + k8*v8 + k9*v9;
-                    sum *= scale;
-                    pixels[p] = sum;
-                }
-                break;
+			switch (type) {
+				case BLUR_MORE:
+				for (int x=roiX; x<xEnd; x++,p++) {
+					if (x<width-1) { p3++; p6++; p9++; }
+					v1 = v2; v2 = v3;
+					v3 = pixels2[p3];
+					v4 = v5; v5 = v6;
+					v6 = pixels2[p6];
+					v7 = v8; v8 = v9;
+					v9 = pixels2[p9];
+					pixels[p] = (v1+v2+v3+v4+v5+v6+v7+v8+v9)*0.11111111f; //0.111... = 1/9
+				}
+				break;
+				case FIND_EDGES:
+				for (int x=roiX; x<xEnd; x++,p++) {
+					if (x<width-1) { p3++; p6++; p9++; }
+					v1 = v2; v2 = v3;
+					v3 = pixels2[p3];
+					v4 = v5; v5 = v6;
+					v6 = pixels2[p6];
+					v7 = v8; v8 = v9;
+					v9 = pixels2[p9];
+					float sum1 = v1 + 2*v2 + v3 - v7 - 2*v8 - v9;
+					float sum2 = v1	 + 2*v4 + v7 - v3 - 2*v6 - v9;
+					pixels[p] = (float)Math.sqrt(sum1*sum1 + sum2*sum2);
+				}
+				break;
+				case CONVOLVE:
+				for (int x=roiX; x<xEnd; x++,p++) {
+					if (x<width-1) { p3++; p6++; p9++; }
+					v1 = v2; v2 = v3;
+					v3 = pixels2[p3];
+					v4 = v5; v5 = v6;
+					v6 = pixels2[p6];
+					v7 = v8; v8 = v9;
+					v9 = pixels2[p9];
+					float sum = k1*v1 + k2*v2 + k3*v3
+							  + k4*v4 + k5*v5 + k6*v6
+							  + k7*v7 + k8*v8 + k9*v9;
+					sum *= scale;
+					pixels[p] = sum;
+				}
+				break;
 			}
 			if (y%inc==0)
 				showProgress((double)(y-roiY)/roiHeight);
 		}
 		showProgress(1.0);
-    }
+	}
 
 	/** Rotates the image or ROI 'angle' degrees clockwise.
 		@see ImageProcessor#setInterpolate
@@ -670,7 +687,7 @@ public class FloatProcessor extends ImageProcessor {
 		}
 	}
 	
-    public void noise(double range) {
+	public void noise(double range) {
 		Random rnd=new Random();
 		for (int y=roiY; y<(roiY+roiHeight); y++) {
 			int i = y * width + roiX;
@@ -681,7 +698,7 @@ public class FloatProcessor extends ImageProcessor {
 			}
 		}
 		resetMinAndMax();
-    }
+	}
 
 	public ImageProcessor crop() {
 		ImageProcessor ip2 = createProcessor(roiWidth, roiHeight);
@@ -692,7 +709,7 @@ public class FloatProcessor extends ImageProcessor {
 			for (int xs=0; xs<roiWidth; xs++)
 				pixels2[offset1++] = pixels[offset2++];
 		}
-        return ip2;
+		return ip2;
 	}
 	
 	/** Returns a duplicate of this image. */ 
@@ -758,7 +775,7 @@ public class FloatProcessor extends ImageProcessor {
 					xs = (x-xCenter)/xScale + xCenter;
 					xsi = (int)xs;
 					if (checkCoordinates && ((xsi<xmin) || (xsi>xmax) || (ysi<ymin) || (ysi>ymax)))
-						pixels[index1++] = (float)min;
+						pixels[index1++] = (float)getMin();
 					else {
 						if (interpolationMethod==BILINEAR) {
 							if (xs<0.0) xs = 0.0;
@@ -842,6 +859,46 @@ public class FloatProcessor extends ImageProcessor {
 		showProgress(1.0);
 		return ip2;
 	}
+	
+	FloatProcessor downsize(int dstWidth, int dstHeight) {
+		FloatProcessor ip2 = this;
+		if (dstWidth<roiWidth) {	  //downsizing in x
+			ip2 = ip2.downsize1D(dstWidth, roiHeight, true);
+			ip2.setRoi(0, 0, dstWidth, roiHeight);	//prepare roi for resizing in y
+		}
+		if (dstHeight<roiHeight)	  //downsizing in y
+			ip2 = ip2.downsize1D(ip2.getRoi().width, dstHeight, false);
+		if (ip2.getWidth()!=dstWidth || ip2.getHeight()!=dstHeight)
+			ip2 = (FloatProcessor)ip2.resize(dstWidth, dstHeight);	//do any upsizing if required
+		return ip2;
+	}
+	
+	// Downsizing in one direction.
+	private FloatProcessor downsize1D(int dstWidth, int dstHeight, boolean xDirection) {
+		int srcPointInc = xDirection ? 1 : width;	//increment of array index for next point along direction
+		int srcLineInc = xDirection ? width : 1;	//increment of array index for next line to be downscaled
+		int dstPointInc = xDirection ? 1 : dstWidth;
+		int dstLineInc = xDirection ? dstWidth : 1;
+		int srcLine0 = xDirection ? roiY : roiX;
+		int dstLines = xDirection ? dstHeight : dstWidth;
+		DownsizeTable dt = xDirection ?
+			new DownsizeTable(getWidth(), roiX, roiWidth, dstWidth, interpolationMethod) : 
+			new DownsizeTable(getHeight(), roiY, roiHeight, dstHeight, interpolationMethod);
+		FloatProcessor ip2 = (FloatProcessor)createProcessor(dstWidth, dstHeight);
+		float[] pixels = (float[])getPixels();
+		float[] pixels2 = (float[])ip2.getPixels();
+		for (int srcLine=srcLine0, dstLine=0; dstLine<dstLines; srcLine++,dstLine++) {
+			int dstLineOffset = dstLine * dstLineInc;
+			int tablePointer = 0;
+			for (int srcPoint=dt.srcStart, p=srcPoint*srcPointInc+srcLine*srcLineInc;
+			srcPoint<=dt.srcEnd; srcPoint++, p+=srcPointInc) {
+				float v = pixels[p];
+				for (int i=0; i<dt.kernelSize; i++, tablePointer++)
+				pixels2[dstLineOffset+dt.indices[tablePointer]*dstPointInc] += v * dt.weights[tablePointer];
+			}
+		}
+		return ip2;
+	}
 
 	/** This method is from Chapter 16 of "Digital Image Processing:
 		An Algorithmic Introduction Using Java" by Burger and Burge
@@ -866,6 +923,7 @@ public class FloatProcessor extends ImageProcessor {
 	
 	/** Sets the foreground fill/draw color. */
 	public void setColor(Color color) {
+		drawingColor = color;
 		int bestIndex = getBestIndex(color);
 		if (bestIndex>0 && getMin()==0.0 && getMax()==0.0) {
 			fillColor = bestIndex;
@@ -873,7 +931,7 @@ public class FloatProcessor extends ImageProcessor {
 		} else if (bestIndex==0 && getMin()>0.0 && (color.getRGB()&0xffffff)==0)
 			fillColor = 0f;
 		else
-			fillColor = (float)(min + (max-min)*(bestIndex/255.0));
+			fillColor = (float)(getMin() + (getMax()-getMin())*(bestIndex/255.0));
 	}
 	
 	/** Sets the default fill/draw value. */
@@ -893,9 +951,9 @@ public class FloatProcessor extends ImageProcessor {
 	public void setThreshold(double minThreshold, double maxThreshold, int lutUpdate) {
 		if (minThreshold==NO_THRESHOLD)
 			{resetThreshold(); return;}
-		if (max>min) {
-			double minT = Math.round(((minThreshold-min)/(max-min))*255.0);
-			double maxT = Math.round(((maxThreshold-min)/(max-min))*255.0);
+		if (getMax()>getMin()) {
+			double minT = Math.round(((minThreshold-getMin())/(getMax()-getMin()))*255.0);
+			double maxT = Math.round(((maxThreshold-getMin())/(getMax()-getMin()))*255.0);
 			super.setThreshold(minT, maxT, lutUpdate); // update LUT
 		} else
 			super.resetThreshold();
@@ -923,8 +981,8 @@ public class FloatProcessor extends ImageProcessor {
 	public void dilate() {}
 
 	/** Returns this FloatProcessor.
-	*  @param channelNumber   Ignored (needed for compatibility with ColorProcessor.toFloat)
-	*  @param fp              Ignored (needed for compatibility with the other ImageProcessor types).
+	*  @param channelNumber	  Ignored (needed for compatibility with ColorProcessor.toFloat)
+	*  @param fp			  Ignored (needed for compatibility with the other ImageProcessor types).
 	*  @return This FloatProcessor
 	*/
 	public FloatProcessor toFloat(int channelNumber, FloatProcessor fp) {
@@ -932,9 +990,9 @@ public class FloatProcessor extends ImageProcessor {
 	}
 	
 	/** Sets the pixels, and min&max values from a FloatProcessor.
-	*  Also the  values are taken from the FloatProcessor.
-	*  @param channelNumber   Ignored (needed for compatibility with ColorProcessor.toFloat)
-	*  @param fp              The FloatProcessor where the image data are read from.
+	*  Also the	 values are taken from the FloatProcessor.
+	*  @param channelNumber	  Ignored (needed for compatibility with ColorProcessor.toFloat)
+	*  @param fp			  The FloatProcessor where the image data are read from.
 	*/
 	public void setPixels(int channelNumber, FloatProcessor fp) {
 		if (fp.getPixels() != getPixels())
