@@ -5,18 +5,25 @@ import java.awt.image.*;
 import ij.gui.*;
 import ij.util.*;
 import ij.plugin.filter.GaussianBlur;
+import ij.process.AutoThresholder.Method;
 import ij.gui.Roi;
 import ij.gui.ShapeRoi;
+import ij.gui.Overlay;
+import ij.Prefs;
 
 /**
 This abstract class is the superclass for classes that process
 the four data types (byte, short, float and RGB) supported by ImageJ.
+An ImageProcessor contains the pixel data of a 2D image and
+some basic methods to manipulate it.
 @see ByteProcessor
 @see ShortProcessor
 @see FloatProcessor
 @see ColorProcessor
+@see ij.ImagePlus
+@see ij.ImageStack
 */
-public abstract class ImageProcessor extends Object {
+public abstract class ImageProcessor implements Cloneable {
 
 	/** Value of pixels included in masks. */
 	public static final int BLACK = 0xFF000000;
@@ -40,10 +47,10 @@ public abstract class ImageProcessor extends Object {
 	/** Interpolation methods */
 	public static final int NEAREST_NEIGHBOR=0, NONE=0, BILINEAR=1, BICUBIC=2;
 
+	public static final int BLUR_MORE=0, FIND_EDGES=1, MEDIAN_FILTER=2, MIN=3, MAX=4, CONVOLVE=5;
 	static public final int RED_LUT=0, BLACK_AND_WHITE_LUT=1, NO_LUT_UPDATE=2, OVER_UNDER_LUT=3;
 	static final int INVERT=0, FILL=1, ADD=2, MULT=3, AND=4, OR=5,
 		XOR=6, GAMMA=7, LOG=8, MINIMUM=9, MAXIMUM=10, SQR=11, SQRT=12, EXP=13, ABS=14;
-	static final int BLUR_MORE=0, FIND_EDGES=1, MEDIAN_FILTER=2, MIN=3, MAX=4, CONVOLVE=5;
 	static final String WRONG_LENGTH = "width*height!=pixels.length";
 	
 	int fgColor = 0;
@@ -58,6 +65,7 @@ public abstract class ImageProcessor extends Object {
 	private static int overRed, overGreen=255, overBlue;
 	private static int underRed, underGreen, underBlue=255;
 	private static boolean useBicubic;
+	private int sliceNumber;
 		
     ProgressBar progressBar;
 	protected int width, snapshotWidth;
@@ -90,14 +98,14 @@ public abstract class ImageProcessor extends Object {
 	protected ColorModel cm2;
 	protected SampleModel sampleModel;
 	protected static IndexColorModel defaultColorModel;
-
+	protected boolean minMaxSet;
 		
 	protected void showProgress(double percentDone) {
 		if (progressBar!=null)
         	progressBar.show(percentDone);
 	}
 
-	// Obsolete
+	/** @deprecated */
 	protected void hideProgress() {
 		showProgress(1.0);
 	}
@@ -296,7 +304,8 @@ public abstract class ImageProcessor extends Object {
 			stdDev = 0.0;
 		boolean isPseudoColor = stdDev<20.0;
 		if ((int)stdDev==67) isPseudoColor = true; // "3-3-2 RGB" LUT
-		//ij.IJ.log("isPseudoColorLut: "+(isPseudoColor) + " " + stdDev);
+		if (ij.IJ.debugMode)
+			ij.IJ.log("isPseudoColorLut: "+(isPseudoColor) + " " + stdDev);
 		return isPseudoColor;
 	}
 	
@@ -330,15 +339,15 @@ public abstract class ImageProcessor extends Object {
 		value closest to the specified color. */
 	public abstract void setColor(Color color);
 
-	/** Obsolete (use setValue) */
+	/** Sets the default fill/draw value. Use setValue() with float images. */
 	public void setColor(int value) {
-		fgColor = value;
+		setValue(value);
 	}
 
 	/** Sets the default fill/draw value. */
 	public abstract void setValue(double value);
 
-	/** Sets the background fill value used by the rotate(), scale() and translate() methods. */
+	/** Sets the background fill value used by the rotate() and scale() methods. */
 	public abstract void setBackgroundValue(double value);
 
 	/** Returns the background fill value. */
@@ -366,7 +375,7 @@ public abstract class ImageProcessor extends Object {
 		can be RED_LUT, BLACK_AND_WHITE_LUT, OVER_UNDER_LUT or NO_LUT_UPDATE.
 		Thresholding of RGB images is not supported. */
 	public void setThreshold(double minThreshold, double maxThreshold, int lutUpdate) {
-		//ij.IJ.write("setThreshold: "+" "+minThreshold+" "+maxThreshold+" "+lutUpdate);
+		//ij.IJ.log("setThreshold: "+" "+minThreshold+" "+maxThreshold+" "+lutUpdate);
 		if (this instanceof ColorProcessor)
 			return;
 		this.minThreshold = minThreshold;
@@ -404,19 +413,36 @@ public abstract class ImageProcessor extends Object {
 					bLUT2[i] = bLUT1[i];
 				}
 			}
-		else if (lutUpdate==BLACK_AND_WHITE_LUT)
+
+		else if (lutUpdate==BLACK_AND_WHITE_LUT) {
+			// updated in v1.43i by Gabriel Lindini to use blackBackground setting
+
+			byte  foreground = Prefs.blackBackground?(byte)255:(byte)0;
+
+			byte background = (byte)(255 - foreground);
+
 			for (int i=0; i<256; i++) {
+
 				if (i>=t1 && i<=t2) {
-					rLUT2[i] = (byte)0;
-					gLUT2[i] = (byte)0;
-					bLUT2[i] = (byte)0;
+
+					rLUT2[i] = foreground;
+
+					gLUT2[i] = foreground;
+
+					bLUT2[i] = foreground;
+
 				} else {
-					rLUT2[i] = (byte)255;
-					gLUT2[i] = (byte)255;
-					bLUT2[i] = (byte)255;
+
+					rLUT2[i] = background;
+
+					gLUT2[i] =background;
+
+					bLUT2[i] =background;
+
 				}
+
 			}
-		else
+		} else {
 			for (int i=0; i<256; i++) {
 				if (i>=t1 && i<=t2) {
 					rLUT2[i] = rLUT1[i];
@@ -433,29 +459,42 @@ public abstract class ImageProcessor extends Object {
 					bLUT2[i] = (byte)underBlue;
 				}
 			}
+		}
 
 		cm = new IndexColorModel(8, 256, rLUT2, gLUT2, bLUT2);
 		newPixels = true;
 		source = null;
 	}
 	
-	public void setAutoThreshold(String method) {
-		if (method==null)
+	public void setAutoThreshold(String mString) {
+		if (mString==null)
 			throw new IllegalArgumentException("Null method");
-		boolean darkBackground = method.indexOf("dark")!=-1;
-		int index = method.indexOf(" ");
+		boolean darkBackground = mString.indexOf("dark")!=-1;
+		int index = mString.indexOf(" ");
 		if (index!=-1)
-			method = method.substring(0, index);
-		setAutoThreshold(method, darkBackground, RED_LUT);
+			mString = mString.substring(0, index);
+		setAutoThreshold(mString, darkBackground, RED_LUT);
 	}
 	
-	public void setAutoThreshold(String method, boolean darkBackground, int lutUpdate) {
+	public void setAutoThreshold(String mString, boolean darkBackground, int lutUpdate) {
+		Method m = null;
+		try {
+			m = Method.valueOf(Method.class, mString);
+		} catch(Exception e) {
+			m = null;
+		}
+		if (m==null)
+			throw new IllegalArgumentException("Invalid method (\""+mString+"\")");
+		setAutoThreshold(m, darkBackground, lutUpdate);
+	}
+
+	public void setAutoThreshold(Method method, boolean darkBackground) {
+		setAutoThreshold(method, darkBackground, RED_LUT);
+	}
+
+	public void setAutoThreshold(Method method, boolean darkBackground, int lutUpdate) {
 		if (method==null || (this instanceof ColorProcessor))
 			return;
-		if (method.equals("Default")) {
-			setAutoThreshold(ISODATA2, lutUpdate);
-			return;
-		}
 		double min=0.0, max=0.0;
 		boolean notByteData = !(this instanceof ByteProcessor);
 		ImageProcessor ip2 = this;
@@ -468,9 +507,7 @@ public abstract class ImageProcessor extends Object {
 			ip2.setMask(mask);
 			ip2.setRoi(rect);	
 		}
-		int options = ij.measure.Measurements.AREA+ ij.measure.Measurements.MIN_MAX+ ij.measure.Measurements.MODE;
-		ImageStatistics stats = ImageStatistics.getStatistics(ip2, options, null);
-		int[] histogram = stats.histogram;
+		ImageStatistics stats = ip2.getStatistics();
 		AutoThresholder thresholder = new AutoThresholder();
 		int threshold = thresholder.getThreshold(method, stats.histogram);
 		double lower, upper;
@@ -478,13 +515,14 @@ public abstract class ImageProcessor extends Object {
 			if (isInvertedLut())
 				{lower=0.0; upper=threshold;}
 			else
-				{lower=threshold; upper=255.0;}
+				{lower=threshold+1; upper=255.0;}
 		} else {
 			if (isInvertedLut())
-				{lower=threshold; upper=255.0;}
+				{lower=threshold+1; upper=255.0;}
 			else
 				{lower=0.0; upper=threshold;}
 		}
+		if (lower>255) lower = 255;
 		if (notByteData) {
 			if (max>min) {
 				lower = min + (lower/255.0)*(max-min);
@@ -493,8 +531,6 @@ public abstract class ImageProcessor extends Object {
 				lower = upper = min;
 		}
 		setThreshold(lower, upper, lutUpdate);
-		if (notByteData && lutUpdate!=NO_LUT_UPDATE)
-			setLutAnimation(true);
 	}
 
 	/** Automatically sets the lower and upper threshold levels, where 'method'
@@ -518,8 +554,7 @@ public abstract class ImageProcessor extends Object {
 			ip2.setMask(mask);
 			ip2.setRoi(rect);	
 		}
-		int options = ij.measure.Measurements.AREA+ ij.measure.Measurements.MIN_MAX+ ij.measure.Measurements.MODE;
-		ImageStatistics stats = ImageStatistics.getStatistics(ip2, options, null);
+		ImageStatistics stats = ip2.getStatistics();
 		int[] histogram = stats.histogram;
 		int originalModeCount = histogram[stats.mode];
 		if (method==ISODATA2) {
@@ -573,8 +608,8 @@ public abstract class ImageProcessor extends Object {
 				lower = upper = min;
 		}
 		setThreshold(lower, upper, lutUpdate);
-		if (notByteData && lutUpdate!=NO_LUT_UPDATE)
-			setLutAnimation(true);
+		//if (notByteData && lutUpdate!=NO_LUT_UPDATE)
+		//	setLutAnimation(true);
 	}
 
 	/** Disables thresholding. */
@@ -607,8 +642,27 @@ public abstract class ImageProcessor extends Object {
 		return lutUpdateMode;
 	}
 
+	/* Sets the threshold levels (non-visible) of an 8-bit mask based on
+		the state of Prefs.blackBackground and isInvertedLut().
+		@see ImageProcessor#resetBinaryThreshold	
+	*/
+	public void setBinaryThreshold() {
+		//ij.IJ.log("setMaskThreshold1");
+		if (!(this instanceof ByteProcessor)) return;
+		double t1=255.0, t2=255.0;
+		boolean invertedLut = isInvertedLut();
+		if ((invertedLut&&ij.Prefs.blackBackground) || (!invertedLut&&!ij.Prefs.blackBackground)) {
+			t1 = 0.0;
+			t2 = 0.0;
+		}
+		//ij.IJ.log("setMaskThreshold2 "+t1+" "+t2);
+		setThreshold(t1, t2, ImageProcessor.NO_LUT_UPDATE);
+	}
+
 	/** Resets the threshold if minThreshold=maxThreshold and lutUpdateMode=NO_LUT_UPDATE. 
-		This removes the invisible threshold set by the MakeBinary and Convert to Mask commands.*/
+		This removes the invisible threshold set by the MakeBinary and Convert to Mask commands.
+		@see ImageProcessor#setBinaryThreshold	
+	*/
 	public void resetBinaryThreshold() {
 		if (minThreshold==maxThreshold && lutUpdateMode==NO_LUT_UPDATE)
 			resetThreshold();
@@ -745,7 +799,7 @@ public abstract class ImageProcessor extends Object {
 		return mask;
 	}
 
-	/** Returns the mask byte array, or null if there is no mask. */
+	/** Returns a reference to the mask pixel array, or null if there is no mask. */
 	public byte[] getMaskArray() {
 		return mask!=null?(byte[])mask.getPixels():null;
 	}
@@ -774,6 +828,11 @@ public abstract class ImageProcessor extends Object {
 		interpolate = method!=NONE?true:false;
 	}
 	
+	/** Returns the current interpolation method (NONE, BILINEAR or BICUBIC). */
+	public int getInterpolationMethod() {
+		return interpolationMethod;
+	}
+	
 	public static String[] getInterpolationMethods() {
 		if (interpolationMethods==null)
 			interpolationMethods = new String[] {"None", "Bilinear", "Bicubic"};
@@ -785,7 +844,7 @@ public abstract class ImageProcessor extends Object {
 		return interpolate;
 	}
 
-	/** Obsolete. */
+	/** @deprecated */
 	public boolean isKillable() {
 		return false;
 	}
@@ -873,7 +932,8 @@ public abstract class ImageProcessor extends Object {
 		int n = (int)Math.round(Math.sqrt(dx*dx + dy*dy));
 		double xinc = dx/n;
 		double yinc = dy/n;
-		n++;
+		if (!((xinc==0&&n==height) || (yinc==0&&n==width)))
+			n++;
 		double[] data = new double[n];
 		double rx = x1;
 		double ry = y1;
@@ -952,16 +1012,16 @@ public abstract class ImageProcessor extends Object {
 		int n = absdy>absdx?absdy:absdx;
 		double xinc = (double)dx/n;
 		double yinc = (double)dy/n;
-		double x = cx<0?cx-0.5:cx+0.5;
-		double y = cy<0?cy-0.5:cy+0.5;
+		double x = cx;
+		double y = cy;
 		n++;
 		cx = x2; cy = y2;
 		if (n>1000000) return;
 		do {
 			if (lineWidth==1)
-				drawPixel((int)x, (int)y);
+				drawPixel((int)Math.round(x), (int)Math.round(y));
 			else if (lineWidth==2)
-				drawDot2((int)x, (int)y);
+				drawDot2((int)Math.round(x), (int)Math.round(y));
 			else
 				drawDot((int)x, (int)y);
 			x += xinc;
@@ -1024,7 +1084,7 @@ public abstract class ImageProcessor extends Object {
 		resetRoi();
 	}
 
-	/** Obsolete */
+	/** @deprecated */
 	public void drawDot2(int x, int y) {
 		drawPixel(x, y);
 		drawPixel(x-1, y);
@@ -1152,6 +1212,40 @@ public abstract class ImageProcessor extends Object {
 		drawString(s);
 	}
 
+	/** Draws a string at the specified location with a filled background.
+		A JavaScript example is available at
+			http://imagej.nih.gov/ij/macros/js/DrawTextWithBackground.js
+	*/
+	public void drawString(String s, int x, int y, Color background) {
+		Color foreground = drawingColor;
+		FontMetrics metrics = getFontMetrics();
+		int w = 0;
+		int h = metrics.getAscent() + metrics.getDescent();
+		int y2 = y;
+		if (s.indexOf("\n")!=-1) {
+			String[] s2 = Tools.split(s, "\n");
+			for (int i=0; i<s2.length; i++) {
+				int w2 = getStringWidth(s2[i]);
+				if (w2>w) w = w2;
+			}
+			int h2 = metrics.getHeight();
+			y2 += h2*(s2.length-1);
+			h += h2*(s2.length-1);
+		} else
+			w = getStringWidth(s);
+		int x2 = x;
+		if (justification==CENTER_JUSTIFY)
+			x2 -= w/2;
+		else if (justification==RIGHT_JUSTIFY)
+			x2 -= w;
+		setColor(background);
+		setRoi(x2, y2-h, w, h);
+		fill();
+		resetRoi();
+		setColor(foreground);
+		drawString(s, x, y);
+	}
+
 	/** Sets the justification used by drawString(), where <code>justification</code>
 		is CENTER_JUSTIFY, RIGHT_JUSTIFY or LEFT_JUSTIFY. The default is LEFT_JUSTIFY. */
 	public void setJustification(int justification) {
@@ -1166,7 +1260,7 @@ public abstract class ImageProcessor extends Object {
 	}
 	
 	/** Specifies whether or not text is drawn using antialiasing. Antialiased
-		test requires Java 2 and an 8 bit or RGB image. Antialiasing does not
+		test requires an 8 bit or RGB image. Antialiasing does not
 		work with 8-bit images that are not using 0-255 display range. */
 	public void setAntialiasedText(boolean antialiasedText) {
 		if (antialiasedText && (((this instanceof ByteProcessor)&&getMin()==0.0&&getMax()==255.0) || (this instanceof ColorProcessor)))
@@ -1174,7 +1268,7 @@ public abstract class ImageProcessor extends Object {
 		else
 			this.antialiasedText = false;
 	}
-
+	
 	/** Returns the width in pixels of the specified string. */
 	public int getStringWidth(String s) {
 		setupFontMetrics();
@@ -1194,6 +1288,12 @@ public abstract class ImageProcessor extends Object {
 		return w;
 	}
 	
+	/** Returns the current font. */
+	public Font getFont() {
+		setupFontMetrics();
+		return font;
+	}
+
 	/** Returns the current FontMetrics. */
 	public FontMetrics getFontMetrics() {
 		setupFontMetrics();
@@ -1279,21 +1379,33 @@ public abstract class ImageProcessor extends Object {
 		return ("ip[width="+width+", height="+height+", min="+getMin()+", max="+getMax()+"]");
 	}
 
-	/**	Fills the image or ROI bounding rectangle with the current fill/draw value. Use
-	*	fill(mask) to fill non-rectangular selections.
-	*	@see ImageProcessor fill(Roi)
+	/** Fills the image or ROI bounding rectangle with the current fill/draw value. Use
+	*	fill(Roi) or fill(ip.getMask()) to fill non-rectangular selections.
+	*	@see #setColor(Color)
+	*	@see #setValue(double)
+	*	@see #fill(Roi)
 	*/
 	public void fill() {
 		process(FILL, 0.0);
 	}
 
-	/** Fills pixels that are within the ROI and part of the mask
-		(i.e. pixels that have a value=BLACK in the mask array). 
-		Throws and IllegalArgumentException if the mask is null or
-		the size of the mask is not the same as the size of the ROI. */
+	/** Fills pixels that are within the ROI bounding rectangle and part of 
+	*	the mask (i.e. pixels that have a value=BLACK in the mask array).
+	*	Use ip.getMask() to acquire the mask. 
+	*	Throws and IllegalArgumentException if the mask is null or
+	*	the size of the mask is not the same as the size of the ROI.
+	*	@see #setColor(Color)
+	*	@see #setValue(double)
+	*	@see #getMask
+	*	@see #fill(Roi)
+	*/
 	public abstract void fill(ImageProcessor mask);
 
-	/** Fills an Roi. */
+	/**	 Fills the ROI with the current fill/draw value. 
+	*	@see #setColor(Color)
+	*	@see #setValue(double)
+	*	@see #fill(Roi)
+	*/
 	public void fill(Roi roi) {
 		ImageProcessor m = getMask();
 		Rectangle r = getRoi();
@@ -1320,11 +1432,45 @@ public abstract class ImageProcessor extends Object {
 		setRoi(r);
 	}
 
-	/** Draws an Roi. */
+	/** Draws the specified ROI on this image using the line
+		width and color defined by ip.setLineWidth() and ip.setColor().
+		@see ImageProcessor#drawRoi
+	*/
 	public void draw(Roi roi) {
 		roi.drawPixels(this);
 	}
 
+	/** Draws the specified ROI on this image using the stroke
+		width, stroke color and fill color defined by roi.setStrokeWidth,
+		roi.setStrokeColor() and roi.setFillColor(). Works best with RGB
+		images. Does not work with 16-bit and float images.
+		Requires Java 1.6.
+		@see ImageProcessor#draw
+		@see ImageProcessor#drawOverlay
+	*/
+	public void drawRoi(Roi roi) {
+		Image img = createImage();
+		Graphics g = img.getGraphics();
+		ij.ImagePlus imp = roi.getImage();
+		if (imp!=null) {
+			roi.setImage(null);
+			roi.drawOverlay(g);
+			roi.setImage(imp);
+		} else
+			roi.drawOverlay(g);
+	}
+
+	/** Draws the specified Overlay on this image. Works best
+		with RGB images. Does not work with 16-bit and float 
+		images. Requires Java 1.6.
+		@see ImageProcessor#drawRoi
+	*/
+	public void drawOverlay(Overlay overlay) {
+		 Roi[] rois = overlay.toArray();
+		 for (int i=0; i<rois.length; i++)
+		 	drawRoi(rois[i]);
+	}
+
 	/** Set a lookup table used by getPixelValue(), getLine() and
 		convertToFloat() to calibrate pixel values. The length of
 		the table must be 256 for byte images and 65536 for short
@@ -1414,6 +1560,11 @@ public abstract class ImageProcessor extends Object {
 
 	public abstract void setf(int index, float value);
 
+	/** Returns a copy of the pixel data as a 2D int array with
+		dimensions [x=0..width-1][y=0..height-1]. With RGB
+		images, the returned values are in packed ARGB format.
+		With float images, the returned values must be converted
+		to float using Float.intBitsToFloat(). */
 	public int[][] getIntArray() {
 		int[][] a = new int [width][height];
 		for(int y=0; y<height; y++) {
@@ -1423,6 +1574,7 @@ public abstract class ImageProcessor extends Object {
 		return a; 
 	}
 
+	/** Replaces the pixel data with contents of the specified 2D int array. */
 	public void setIntArray(int[][] a) {
 		for(int y=0; y<height; y++) {
 			for(int x=0; x<width; x++)
@@ -1430,6 +1582,8 @@ public abstract class ImageProcessor extends Object {
 		}
 	}
 
+	/** Returns a copy of the pixel data as a 2D float 
+		array with dimensions [x=0..width-1][y=0..height-1]. */
 	public float[][] getFloatArray() {
 		float[][] a = new float[width][height];
 		for(int y=0; y<height; y++) {
@@ -1439,6 +1593,7 @@ public abstract class ImageProcessor extends Object {
 		return a; 
 	}
 
+	/** Replaces the pixel data with contents of the specified 2D float array. */
 	public void setFloatArray(float[][] a) {
 		for(int y=0; y<height; y++) {
 			for(int x=0; x<width; x++)
@@ -1446,6 +1601,27 @@ public abstract class ImageProcessor extends Object {
 		}
 	}
 	
+	/** Experimental */
+	public void getNeighborhood(int x, int y, double[][] arr) {
+		int nx=arr.length;
+		int ny=arr[0].length;
+		int nx2 = (nx-1)/2;
+		int ny2 = (ny-1)/2;
+	 	if (x>=nx2 && y>=ny2 && x<width-nx2-1 && y<height-ny2-1) { 
+			int index = (y-ny2)*width + (x-nx2);
+			for (int y2=0; y2<ny; y2++) {
+	 			for (int x2=0; x2<nx; x2++)
+					arr[x2][y2] = getf(index++);			
+				index += (width - nx);
+			}	
+		} else {
+			for (int y2=0; y2<ny; y2++) {
+	 			for (int x2=0; x2<nx; x2++)
+					arr[x2][y2] = getPixelValue(x2, y2);			
+			}	
+		}
+	}
+
     /** Returns the samples for the pixel at (x,y) in an int array.
     	RGB pixels have three samples, all others have one.
 		Returns zeros if the the coordinates are not in bounds.
@@ -1478,7 +1654,7 @@ public abstract class ImageProcessor extends Object {
 		if (useBicubic)
 			return getBicubicInterpolatedPixel(x, y, this);
 		if (x<0.0 || x>=width-1.0 || y<0.0 || y>=height-1.0) {
-			if (x<-1.0 || x>=width || y<=1.0 || y>=height)
+			if (x<-1.0 || x>=width || y<-1.0 || y>=height)
 				return 0.0;
 			else
 				return getInterpolatedEdgeValue(x, y);
@@ -1531,7 +1707,7 @@ public abstract class ImageProcessor extends Object {
 	}
 	
 	static final double a = 0.5; // Catmull-Rom interpolation
-	final double cubic(double x) {
+	public static final double cubic(double x) {
 		if (x < 0.0) x = -x;
 		double z = 0.0;
 		if (x < 1.0) 
@@ -1622,6 +1798,11 @@ public abstract class ImageProcessor extends Object {
 	/** Adds 'value' to each pixel in the image or ROI. */
 	public void add(double value) {process(ADD, value);}
 	
+	/** Subtracts 'value' from each pixel in the image or ROI. */
+	public void subtract(double value) {
+		add(-value);
+	}
+	
 	/** Multiplies each pixel in the image or ROI by 'value'. */
 	public void multiply(double value) {process(MULT, value);}
 	
@@ -1683,6 +1864,9 @@ public abstract class ImageProcessor extends Object {
 	/** Restores the pixel data from the snapshot (undo) buffer. */
 	public abstract void reset();
 	
+	/** Swaps the pixel and snapshot (undo) buffers. */
+	public abstract void swapPixelArrays();
+
 	/** Restores pixels from the snapshot buffer that are 
 		within the rectangular roi but not part of the mask. */
 	public abstract void reset(ImageProcessor mask);
@@ -1697,8 +1881,8 @@ public abstract class ImageProcessor extends Object {
 		3x3 integer convolution kernel. */
 	public abstract void convolve3x3(int[] kernel);
 	
-	/** A 3x3 filter operation, where the argument (BLUR_MORE, 
-		FIND_EDGES, etc.) determines the filter type. */
+	/** A 3x3 filter operation, where the argument (BLUR_MORE,  FIND_EDGES, 
+	     MEDIAN_FILTER, MIN or MAX) determines the filter type. */
 	public abstract void filter(int type);
 	
 	/** A 3x3 median filter. Requires 8-bit or RGB image. */
@@ -1738,6 +1922,36 @@ public abstract class ImageProcessor extends Object {
 		return resize(dstWidth, (int)(dstWidth*((double)roiHeight/roiWidth)));
 	}
 
+	/** Creates a new ImageProcessor containing a scaled copy of this image or ROI.
+		@param dstWidth   Image width of the resulting ImageProcessor
+		@param dstHeight  Image height of the resulting ImageProcessor
+		@param useAverging  True means that the averaging occurs to avoid
+			aliasing artifacts; the kernel shape for averaging is determined by 
+			the interpolationMethod. False if subsampling without any averaging  
+			should be used on downsizing.  Has no effect on upsizing.
+		@ImageProcessor#setInterpolationMethod for setting the interpolation method
+		@author Michael Schmid
+	*/
+	public ImageProcessor resize(int dstWidth, int dstHeight, boolean useAverging) {
+		Rectangle r = getRoi();
+		int rWidth = r.width;
+		int rHeight = r.height;
+		if ((dstWidth>=rWidth && dstHeight>=rHeight) || !useAverging)
+			return resize(dstWidth, dstHeight);  //upsizing or downsizing without averaging
+		else {  //downsizing with averaging in at least one direction: convert to float
+			ImageProcessor ip2 = createProcessor(dstWidth, dstHeight);
+			FloatProcessor fp = null;
+			for (int channelNumber=0; channelNumber<getNChannels(); channelNumber++) {
+				fp = toFloat(channelNumber, fp);
+				fp.setInterpolationMethod(interpolationMethod);
+				fp.setRoi(getRoi());
+				FloatProcessor fp2 = fp.downsize(dstWidth, dstHeight);
+				ip2.setPixels(channelNumber, fp2);
+			}
+			return ip2;
+		}
+	}
+
 	/** Rotates the image or selection 'angle' degrees clockwise.
 		@see ImageProcessor#setInterpolate
 	*/
@@ -1750,6 +1964,7 @@ public abstract class ImageProcessor extends Object {
 	*/
   	public void translate(double xOffset, double yOffset) {
   		ImageProcessor ip2 = this.duplicate();
+  		ip2.setBackgroundValue(0.0);
 		boolean integerOffsets = xOffset==(int)xOffset && yOffset==(int)yOffset;
   		if (integerOffsets || interpolationMethod==NONE) {
 			for (int y=roiY; y<(roiY + roiHeight); y++) {
@@ -1770,7 +1985,10 @@ public abstract class ImageProcessor extends Object {
 		} 
   	}
   	
-  	/** Obsolete; replaced by translate(x,y). */
+	/**
+	* @deprecated
+	* replaced by translate(x,y)
+	*/
   	public void translate(int xOffset, int yOffset, boolean eraseBackground) {
 		translate(xOffset, yOffset);
   	}
@@ -1884,11 +2102,11 @@ public abstract class ImageProcessor extends Object {
 		do {
 			sum1=sum2=sum3=sum4=0.0;
 			for (int i=min; i<=movingIndex; i++) {
-				sum1 += i*histogram[i];
+				sum1 += (double)i*histogram[i];
 				sum2 += histogram[i];
 			}
 			for (int i=(movingIndex+1); i<=max; i++) {
-				sum3 += i*histogram[i];
+				sum3 += (double)i*histogram[i];
 				sum4 += histogram[i];
 			}			
 			result = (sum1/sum2 + sum3/sum4)/2.0;
@@ -2090,5 +2308,34 @@ public abstract class ImageProcessor extends Object {
 	public static void setUseBicubic(boolean b) {
 		useBicubic = b;
 	}
+	
+	/* Calculates and returns statistics (area, mean, std-dev, mode, min, max,
+		centroid, center of mass, 256 bin histogram) for this image or ROI. */
+	public ImageStatistics getStatistics() {
+		// 127 = AREA+MEAN+STD_DEV+MODE+MIN_MAX+CENTROID+CENTER_OF_MASS
+		return ImageStatistics.getStatistics(this, 127, null);
+	}
+	
+	/* Returns the PlugInFilter slice number. */
+	public int getSliceNumber() {
+		if (sliceNumber<1)
+			return 1;
+		else
+			return sliceNumber;
+	}
+
+	/** PlugInFilterRunner uses this method to set the slice number. */
+	public void setSliceNumber(int slice) {
+		sliceNumber = slice;
+	}
+	
+	/** Returns a shallow copy of this ImageProcessor. */
+	public synchronized Object clone() {
+		try {
+			return super.clone();
+		} catch (CloneNotSupportedException e) {
+			return null;
+		}
+	}
 
 }
