@@ -11,6 +11,7 @@ import ij.io.SaveDialog;
 import ij.measure.ResultsTable;
 import ij.util.Tools;
 import ij.plugin.frame.Recorder;
+import ij.gui.GenericDialog;
 
 
 /**
@@ -85,6 +86,7 @@ public class TextPanel extends Panel implements AdjustmentListener,
 			addPopupItem("Summarize");
 			addPopupItem("Distribution...");
 			addPopupItem("Set Measurements...");
+			addPopupItem("Rename...");
 			addPopupItem("Duplicate...");
 		}
 	}
@@ -254,10 +256,10 @@ public class TextPanel extends Panel implements AdjustmentListener,
 	}
 
 	synchronized void adjustHScroll() {
-		if(iRowHeight==0) return;
+		if (iRowHeight==0) return;
 		Dimension d = tc.getSize();
 		int w=0;
-		for(int i=0;i<iColCount;i++)
+		for (int i=0; i<iColCount; i++)
 			w+=iColWidth[i];
 		iGridWidth=w;
 		sbHoriz.setValues(iX,d.width,0,iGridWidth);
@@ -369,18 +371,33 @@ public class TextPanel extends Panel implements AdjustmentListener,
 		}
 	}
 
+	private void scroll(int inc) {
+		synchronized(this) {
+			sbVert.setValue(sbVert.getValue()+inc);
+			iY=iRowHeight*sbVert.getValue();
+			tc.repaint();
+		}
+	}
+
 	/** Unused keyPressed and keyTyped events will be passed to 'listener'.*/
 	public void addKeyListener(KeyListener listener) {
 		keyListener = listener;
 	}
 	
+	public void addMouseListener(MouseListener listener) {
+		tc.addMouseListener(listener);
+	}
+
 	public void keyPressed(KeyEvent e) {
 		int key = e.getKeyCode();
 		if (key==KeyEvent.VK_BACK_SPACE)
 			clearSelection();
+		else if (key==KeyEvent.VK_UP)
+			scroll(-1);
+		else if (key==KeyEvent.VK_DOWN)
+			scroll(1);
 		else if (keyListener!=null&&key!=KeyEvent.VK_S&& key!=KeyEvent.VK_C && key!=KeyEvent.VK_X&& key!=KeyEvent.VK_A)
 			keyListener.keyPressed(e);
-		
 	}
 	
 	public void keyReleased (KeyEvent e) {}
@@ -408,6 +425,8 @@ public class TextPanel extends Panel implements AdjustmentListener,
 			clearSelection();
 		else if (cmd.equals("Select All"))
 			selectAll();
+		else if (cmd.equals("Rename..."))
+			rename(null);
 		else if (cmd.equals("Duplicate..."))
 			duplicate();
 		else if (cmd.equals("Summarize"))
@@ -424,6 +443,47 @@ public class TextPanel extends Panel implements AdjustmentListener,
  	
  	public void lostOwnership (Clipboard clip, Transferable cont) {}
 
+	void rename(String title2) {
+		if (rt==null) return;
+		if (title2!=null && title2.equals(""))
+			title2 = null;
+		Component comp = getParent();
+		if (comp==null || !(comp instanceof TextWindow))
+			return;
+		TextWindow tw = (TextWindow)comp;
+		if (title2==null) {
+			GenericDialog gd = new GenericDialog("Rename", tw);
+			gd.addStringField("Title:", "Results2", 20);
+			gd.showDialog();
+			if (gd.wasCanceled())
+				return;
+			title2 = gd.getNextString();
+		}
+		String title1 = title;
+		if (title!=null && title.equals("Results")) {
+			IJ.setTextPanel(null);
+			Analyzer.setUnsavedMeasurements(false);
+			Analyzer.setResultsTable(null);
+			Analyzer.resetCounter();
+		}
+		if (title2.equals("Results")) {
+			//tw.setVisible(false);
+			tw.dispose();
+			WindowManager.removeWindow(tw);
+			flush();
+			rt.show("Results");
+		} else {
+			tw.setTitle(title2);
+			int mbSize = tw.mb!=null?tw.mb.getMenuCount():0;
+			if (mbSize>0 && tw.mb.getMenu(mbSize-1).getLabel().equals("Results"))
+				tw.mb.remove(mbSize-1);
+			title = title2;
+		}
+		Menus.updateWindowMenuItem(title1, title2);
+		if (Recorder.record)
+			Recorder.recordString("IJ.renameResults(\""+title2+"\");\n");
+	}
+
 	void duplicate() {
 		if (rt==null) return;
 		ResultsTable rt2 = (ResultsTable)rt.clone();
@@ -559,6 +619,8 @@ public class TextPanel extends Panel implements AdjustmentListener,
 				IJ.error("Selection required");
 			return;
 		}
+		if (Recorder.record)
+			Recorder.recordString("IJ.deleteRows("+selStart+", "+selEnd+");\n");
 		if (selStart==0 && selEnd==(iRowCount-1)) {
 			vData.removeAllElements();
 			iRowCount = 0;
@@ -625,7 +687,7 @@ public class TextPanel extends Panel implements AdjustmentListener,
 			tc.repaint();
 	}
 	
-	/** Creates a selection. */
+	/** Creates a selection and insures that it is visible. */
 	public void setSelection (int startLine, int endLine) {
 		if (startLine>endLine) endLine = startLine;
 		if (startLine<0) startLine = 0;
@@ -635,8 +697,21 @@ public class TextPanel extends Panel implements AdjustmentListener,
 		selOrigin = startLine;
 		selStart = startLine;
 		selEnd = endLine;
+		int vstart = sbVert.getValue();
+		int visible = sbVert.getVisibleAmount()-1;
+		if (startLine<vstart) {
+			sbVert.setValue(startLine);
+			iY=iRowHeight*startLine;
+		} else if (endLine>=vstart+visible) {
+			vstart = endLine - visible + 1;
+			if (vstart<0) vstart = 0;
+			sbVert.setValue(vstart);
+			iY=iRowHeight*vstart;
+		}
 		tc.repaint();
 	}
+	
+	
 
 	/** Writes all the text in this TextPanel to a file. */
 	public void save(PrintWriter pw) {
@@ -658,33 +733,58 @@ public class TextPanel extends Panel implements AdjustmentListener,
 		'false' if the user cancels the save as dialog.*/
 	public boolean saveAs(String path) {
 		boolean isResults = IJ.isResultsWindow() && IJ.getTextPanel()==this;
-		if (path.equals("")) {
-			IJ.wait(10);
-			boolean hasHeadings = !getColumnHeadings().equals("");
-			String ext = isResults||hasHeadings?Prefs.get("options.ext", ".xls"):".txt";
-			SaveDialog sd = new SaveDialog("Save as Text", title, ext);
-			String file = sd.getFileName();
-			if (file == null) return false;
-			path = sd.getDirectory() + file;
+		boolean summarized = false;
+		if (isResults) {
+			String lastLine = iRowCount>=2?getLine(iRowCount-2):null;
+			summarized = lastLine!=null && lastLine.startsWith("Max");
 		}
-		PrintWriter pw = null;
-		try {
-			FileOutputStream fos = new FileOutputStream(path);
-			BufferedOutputStream bos = new BufferedOutputStream(fos);
-			pw = new PrintWriter(bos);
+		if (rt!=null && rt.getCounter()!=0 && !summarized) {
+			if (path==null || path.equals("")) {
+				IJ.wait(10);
+				String name = isResults?"Results":title;
+				SaveDialog sd = new SaveDialog("Save Results", name, Prefs.get("options.ext", ".xls"));
+				String file = sd.getFileName();
+				if (file==null) return false;
+				path = sd.getDirectory() + file;
+			}
+			try {
+				rt.saveAs(path);
+			} catch (IOException e) {
+				IJ.error(""+e);
+			}
+		} else {
+			if (path.equals("")) {
+				IJ.wait(10);
+				boolean hasHeadings = !getColumnHeadings().equals("");
+				String ext = isResults||hasHeadings?Prefs.get("options.ext", ".xls"):".txt";
+				if (ext.equals(".csv")) ext = ".txt";
+				SaveDialog sd = new SaveDialog("Save as Text", title, ext);
+				String file = sd.getFileName();
+				if (file == null) return false;
+				path = sd.getDirectory() + file;
+			}
+			PrintWriter pw = null;
+			try {
+				FileOutputStream fos = new FileOutputStream(path);
+				BufferedOutputStream bos = new BufferedOutputStream(fos);
+				pw = new PrintWriter(bos);
+			}
+			catch (IOException e) {
+				//IJ.write("" + e);
+				return true;
+			}
+			save(pw);
+			pw.close();
 		}
-		catch (IOException e) {
-			//IJ.write("" + e);
-			return true;
-		}
-		save(pw);
-		pw.close();
 		if (isResults) {
 			Analyzer.setUnsavedMeasurements(false);
-			if (Recorder.record)
-				Recorder.record("saveAs", "Measurements", path);
+			if (Recorder.record && !IJ.isMacro())
+				Recorder.record("saveAs", "Results", path);
+		} else if (rt!=null) {
+			if (Recorder.record && !IJ.isMacro())
+				Recorder.record("saveAs", "Results", path);
 		} else {
-			if (Recorder.record)
+			if (Recorder.record && !IJ.isMacro())
 				Recorder.record("saveAs", "Text", path);
 		}
 		IJ.showStatus("");
@@ -752,7 +852,21 @@ public class TextPanel extends Panel implements AdjustmentListener,
 	public void setResultsTable(ResultsTable rt) {
 		this.rt = rt;
 	}
+	
+	/** Returns the ResultsTable associated with this TextPanel, or null. */
+	public ResultsTable getResultsTable() {
+		return rt;
+	}
 
+	public void scrollToTop() {
+		sbVert.setValue(0);
+		iY = 0;
+		for(int i=0;i<iColCount;i++)
+			tc.calcAutoWidth(i);
+		adjustHScroll();
+		tc.repaint();
+	}
+        
 	void flush() {
 		if (vData!=null)
 			vData.removeAllElements();
